"""
CorpusPlayer.  Media Player for Superkaramba v0.35
Copyright (C) 2004  V.F.A. de Antoni

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact:
   vdeantoni@inicia.es

Data provided by:

                                                Music Charts:
                                                         Veronica's Top40 (1956 - 2003)
                                                                     - http://www.walravens.info
                                                         Acclaimed Music Charts (1945 - 2003)
                                                                     - http://hem.bredband.net/b132682
                                                         Gracenote
                                                                     - http://www.gracenote.com


"""
import karamba
import os
import re
import sys
import string
#import urllib
try:
   import xmms
except:
   print "WARNING: Could not load xmmspy python module. This theme requires it to run.\n\nxmmspy can be downloaded from:\n\t- http://www.python.org/pypi?:action=display&name=xmmspy&version=0.4\n"
   print "::CLOSE"
   exit

try:
   import eyeD3
   id3_method = "eyed3"
except:
   print "\nNOTE: Could not load eyeD3-python module .  Will use 'id3' and 'id3info' shell utilities instead.\n\neyeD3 can be downloaded from:\n\t- http://eyed3.nicfit.net/\n"
   try:
      import id3
      id3_method = "id3"
   except:
      id3_method = "bash"
      print "\nNOTE: Could not load PyID3-python module .  Will use 'id3' and 'id3info' shell utilities instead.\n\nPyID3 can be downloaded from:\n\t- http://www.python.org/pypi?:action=display&name=PyID3&version=0.5.5\n"

try:
   import ogg.vorbis
   ogg_method = 'pyvorbis'
except:
   ogg_method = 'bash'
   print "\nNOTE: Could not load pyvorbis-module .  Will use 'ogginfo' shell utility instead.\n\npyvorbis can be downloaded from:\n\t- http://www.andrewchatham.com/pyogg/\n"



#################### Locale setting #########################
##
## ('ascii', 'latin-1', 'iso-8859-1', 'iso-8859-2', 'iso-8859-15', 'utf-8', 'utf-16')
##
locale = 'iso 8859-15'
#######################################################


###### Catching & dumping error and status messages to InfoTerminal ######
##
## Dump action sets the action to be performed:
##
## 0  -  disabled
## 1  -  write to InfoTerminal
## 2  -  append to InfoTerminal (default)
##
dump_action = 2
#######################################################


#################### Custom options ########################
##
## Leave this to '0' otherwise you will get options that you can't use!
##    If you know python and bash scripting a bit you might change it to '1'
##    to get the options work for you.  Just adjust some scripts and point
##    the sources and targets to yours
##
custom = 0

#######################################################


################## Configuration defaults ######################
##
## Path to where the theme modules are located ( ../theme/path/modules/ )
## NOTE  add trailing slash ( / )
theme_path = ""

albumart_dir = ""

## Chart provider
##    'veronica_src'    -  Veronica Top40
##    'acclaimed_src'  -  Acclaimed Music Charts
##    'local_src'          -  Local MySQL or text
##                              ('custom' needs to be set to '1' for this
##                                and have adjusted the required code!)
chart_src = "veronica_src"

## ID3-tag source to read from
##    'netjuke'       -  getting tags from Netjuke's DB
##    'internal_v1' -  getting rags from file (ID3v1)
##    'internal_v2' -  getting rags from file (ID3v2)
id3_src = "internal_v2"

## Set path to where your music is stored ( /path/to/music/ )
## NOTE  if you prefer to set it here you must not forget
##          to add a trailing slash ( / )
local_path = "not set"

## Set your music server URL ( http://your.music.server/music/path/ )
##    this is needed when XMMS gets the URL instead of its path.
## NOTE  add trailing slash ( / )
server_path = "not set"

## Calendar export format
##    'Short'   -   Fancy Calendar output format (default)
##     'Text'    -   shows calendar with a lot of irrelevant data and takes up space
## NOTE  Change to "Text" if the current "Short" does not show anything on the InfoTerminal
cal_export_type = "Short"
cal_path = "not set"
csv_path = "not set"
log_file = "not set"

## Set time format for in clock display
##    'hhmmsstz'    -     hh:mm:ss Timezone     |     'hhmmsstzus'     -     hh:mm:ss Timezone AM/PM
##    'hhmmss'       -    hh:mm:ss                     |     'hhmmssus'        -     hh:mm:ss AM/PM
##    'hhmm'          -    hh:mm                          |     'hhmmus'            -     hh:mm AM/PM
time_format = "hhmmsstz"



######################## Resets ##########################
##
## DO NOT change these unless you know what you're doing
## Some checks might otherwise not be run

## Sets the default display to Player
display_view = 0
## Set config flag to make it read the theme configuration file
## (~/.superkaramba/Just-a-Player.rc)
config_state = 1

id3_src_txt = ""
id3genre = ""
isstreaming = ""
xmms_playtime = 0
time_eventscnt = 0
allday_eventcnt = 0
cur_cal_mod = 0
last_cal_mod = 1
sidemenu_xpos = 310


#######################################################

######################## DONE ##########################


def initWidget(widget):

   global dump_action, config_state, theme_init, info_dump_path, theme_path
   try:
      test_theme_init = karamba.readConfigEntry(widget, 'theme_init')
      if (test_theme_init == None):
         dumpInfo("\n... writing configuration file ...\n")
         ## Theme is run for the first time or the theme configuration file (*.rc) can't be located
         theme_init = "pass"
         karamba.writeConfigEntry(widget, 'theme_init', theme_init)
         theme_path = karamba.getThemePath(widget)
         ## Set config flag to 3 to get things set up for the first time
         config_state = 3
         dumpInfo("\ni... theme run for the first time ...\n\n")
   except:
         theme_init = "error"
         theme_path = "error"
         dumpInfo("\n!   ERROR determining path to theme\n")
   info_dump_path = karamba.getThemePath(widget) + "../data/info_dump"
   try:
      f = open(info_dump_path,"w")
      f.write('')
      f.close()
   except:
      pass
   dumpInfo("\n... initializing theme ...\n")
   try:
      dumpInfo("\n... getting config defaults ...\n")
      setDefaults(widget)
   except:
      dumpInfo("\n!   ERROR getting defaults")
   try:
      SetMenu(widget)
      dumpInfo("\n... menu items have been set\n")
   except:
      dumpInfo("\n!   ERROR setting menu\n")

   initGUI(widget)


def widgetUpdated(widget):

   global dump_action, last_cal_mod, time_eventscnt, allday_eventcnt

   MonitorCalendar(widget)
   if (cur_cal_mod != last_cal_mod):
      last_cal_mod = cur_cal_mod
      try:
         pattern = "\[0-9\]\[0-9\]:\[0-9\]"
         tec = os.popen("konsolekalendar --view --export-type \"Short\" --file \"" + cal_path + "\" | grep -ce " + pattern)
         time_eventscnt = tec.read()
         time_eventscnt = time_eventscnt.replace('\n','')
         time_eventscnt = "appointments  (" + str(time_eventscnt) + ")"
         aec = os.popen("konsolekalendar --view --export-type \"Short\" --file \"" + cal_path + "\" | grep -c 'all day'")
         allday_eventcnt = aec.read()
         allday_eventcnt = allday_eventcnt.replace('\n','')
         allday_eventcnt = "all day events  (" + str(allday_eventcnt) + ")"
      except:
         time_eventscnt = "n.a."
         allday_eventcnt = "n.a."

   updateGUI(widget)


def initGUI(widget):

  global local_path, btn_reverse, btn_play, btn_pause, btn_stop, btn_forward, lbl_id3artist, lbl_id3title, lbl_id3comments, lbl_id3album, lbl_id3genre, lbl_id3year, btn_power_on, btn_power_off, xmms_status, lbl_clock_center, lbl_clock_top, lbl_clock_bottom1, lbl_clock_bottom2, lbl_clock_bottom3, lbl_play_time, lbl_track_info, btn_xmms_main, btn_xmms_eq, btn_xmms_pl, btn_xmms_eject, btn_xmms_repeat, btn_xmms_shuffle, btn_xmms_config, btn_xmms_exit, btn_search_charts, btn_id3_edit, btn_add_fav, btn_match_charts, time_display_stat, btn_refresh, btn_netjuke_edit, main_stat, current_track_url, track_url, btn_toggle_display, resized, tip_id3artist, lbl_paths_config, ext_genre_lbl, id3_src, lbl_source_info, db_init, display_view, btn_extend, img_netjuke, config_state, img_mysqlbd_stat, netjukedb_status, btn_getconfig, playermenu, configmenu, clockmenu, lbl_event_cnt, lbl_allday_cnt, btn_sidemenu, iconPlayer, iconPIM, iconNetjuke, iconCharts, iconWeb, iconTerm, iconGracenote, img_netjuke2, img_mysqlbd_stat2, sidemenu, iconFav


  dumpInfo("\n... initializing display ...\n")

  playermenu = 0
  configmenu = 0
  clockmenu = 0

  create_bar(widget)

  karamba.resizeWidget(widget, 310, 160)
  resized = 0
  sidemenu = 0
  display_view = 0

  time_display_stat = "playing"
  main_stat = 0
  current_track_url = "clear"
  track_url = "none"

  iconPlayer = theme_path + "../icons/player.png"
  iconNetjuke = theme_path + "../icons/netjuke.png"
  iconCharts = theme_path + "../icons/hits.png"
  iconPIM = theme_path + "../icons/contact.png"
  iconWeb = theme_path + "../icons/globe.png"
  iconTerm = theme_path + "../icons/term.png"
  iconGracenote = theme_path + "../icons/gracenote.png"
  iconFav = theme_path + "../images/fav_status.png"

  lbl_track_info = karamba.createText(widget, 33, 50, 350, 40, "")
  lbl_source_info = karamba.createText(widget, 33, 98, 350, 60, "")
  lbl_paths_config = karamba.createText(widget, 35, 180, 380, 200, "" )

  btn_toggle_display = karamba.createImage(widget, 5, 20, "../icons/toggle_display.png")
  karamba.attachClickArea(widget, btn_toggle_display)

  btn_getconfig = karamba.createImage(widget, 277, 110, "../icons/help.png")
  karamba.attachClickArea(widget, btn_getconfig)
  karamba.hideImage(widget, btn_getconfig)

  img_display = karamba.createImage(widget, 10, 40, "../images/background_trans_blue.png")

  btn_power_on = karamba.createImage(widget, 5, 20, "../images/btn_power_on.png")
  karamba.attachClickArea(widget, btn_power_on)

  lbl_clock_top = karamba.createText(widget, 24, 45, 300, 40, "")
  lbl_clock_center = karamba.createText(widget, 24, 70, 324, 40, "")
  lbl_clock_bottom1 = karamba.createText(widget, 30, 115, 50, 40, "")
  lbl_clock_bottom2 = karamba.createText(widget, 100, 115, 150, 40, "")
  lbl_clock_bottom3 = karamba.createText(widget, 255, 115, 50, 40, "")
  lbl_event_cnt = karamba.createText(widget, 70, 100, 100, 15, "")
  lbl_allday_cnt = karamba.createText(widget, 180, 100, 100, 15, "")

  # XMMS control button display
  btn_reverse = karamba.createImage(widget, 30, 115, "../images/btn_reverse.png")
  karamba.resizeImage(widget, btn_reverse, 16, 17)
  btn_play = karamba.createImage(widget, 54, 115, "../images/btn_play.png")
  karamba.resizeImage(widget, btn_play, 16, 17)
  btn_pause = karamba.createImage(widget, 78, 116, "../images/btn_pause.png")
  karamba.resizeImage(widget, btn_pause, 16, 15)
  btn_stop = karamba.createImage(widget, 101, 115, "../images/btn_stop.png")
  karamba.resizeImage(widget, btn_stop, 16, 16)
  btn_forward = karamba.createImage(widget, 125, 115, "../images/btn_forward.png")
  karamba.resizeImage(widget, btn_forward, 16, 17)

  # Misc buttons (right)
  btn_sidemenu = karamba.createImage(widget, 285, 44, "../icons/config.png")
  karamba.attachClickArea(widget, btn_sidemenu)
  btn_match_charts = karamba.createText(widget, 310, 42, 60, 10, "find match")
  btn_id3_edit = karamba.createText(widget, 310, 55, 60, 10, "Rolling Stone")
  btn_netjuke_edit = karamba.createText(widget, 310, 68, 60, 10, "-----")
  btn_add_fav = karamba.createText(widget, 310, 81, 60, 10, "option menu")
  btn_search_charts = karamba.createText(widget, 310, 94, 60, 10, "search charts")
  btn_refresh = karamba.createText(widget, 310, 107, 60, 10, "refresh")
  btn_extend = karamba.createText(widget, 310, 120, 60, 10, "")

  btn_xmms_main = karamba.createText(widget, 35, 28, 30, 15, "Main")
  btn_xmms_eq = karamba.createText(widget, 68, 28, 30, 15, "EQ")
  btn_xmms_pl = karamba.createText(widget, 92, 28, 30, 15, "PL")
  btn_xmms_eject = karamba.createText(widget, 113, 28, 30, 15, "Eject")
  btn_xmms_repeat = karamba.createText(widget, 148, 28, 38, 15, "Repeat")
  btn_xmms_shuffle = karamba.createText(widget, 190, 28, 38, 15, "Shuffle")
  btn_xmms_config = karamba.createText(widget, 233, 28, 30, 15, "Config")
  btn_xmms_exit = karamba.createText(widget, 270, 28, 30, 15, "Exit")

  lbl_play_time = karamba.createText(widget, 110, 140, 100, 20, "")

  lbl_id3artist = karamba.createText(widget, 45, 45, 270, 20, "Loading ...")
  tip_id3artist = karamba.createImage(widget, 45, 45, "../images/trans_bar_artist.png")
  lbl_id3title = karamba.createText(widget, 30, 62, 300, 20, "")
  lbl_id3comments = karamba.createText(widget, 50, 79, 250, 20, "")
  lbl_id3album = karamba.createText(widget, 30, 98, 270, 15, "")
  lbl_id3genre = karamba.createText(widget, 160, 115, 100, 20, "")
  lbl_id3year = karamba.createText(widget, 210, 115, 90, 20, "")

  dumpInfo("\n... setting font properties ...\n")
  SetTextFont(widget)

  ResetLabels(widget)
  dumpInfo("\n...    J u s t - a - P l a y e r    l o a d e d    ...\n")

def ResetLabels(widget):

  karamba.hideImage(widget, progBar)

  karamba.hideImage(widget, btn_reverse)
  karamba.hideImage(widget, btn_play)
  karamba.hideImage(widget, btn_pause)
  karamba.hideImage(widget, btn_stop)
  karamba.hideImage(widget, btn_forward)
  karamba.hideImage(widget, btn_toggle_display)

  karamba.changeText(widget, btn_xmms_main, "")
  karamba.changeText(widget, btn_xmms_eq, "")
  karamba.changeText(widget, btn_xmms_pl, "")
  karamba.changeText(widget, btn_xmms_eject, "")
  karamba.changeText(widget, btn_xmms_repeat, "")
  karamba.changeText(widget, btn_xmms_shuffle, "")
  karamba.changeText(widget, btn_xmms_config, "")
  karamba.changeText(widget, btn_xmms_exit, "")

  karamba.hideImage(widget, btn_sidemenu)
  karamba.changeText(widget, btn_match_charts, "")
  karamba.changeText(widget, btn_id3_edit, "")
  karamba.changeText(widget, btn_netjuke_edit, "")
  karamba.changeText(widget, btn_add_fav, "")
  karamba.changeText(widget, btn_search_charts, "")
  karamba.changeText(widget, btn_refresh, "")
  karamba.changeText(widget, btn_extend, "")

  karamba.changeText(widget, lbl_play_time, "")
  karamba.changeText(widget, lbl_track_info, "")
  karamba.changeText(widget, lbl_source_info, "")

  karamba.changeText(widget, lbl_id3comments, "")
  karamba.changeText(widget, lbl_id3title, "")
  karamba.changeText(widget, lbl_id3artist, "")
  karamba.changeText(widget, lbl_id3album, "")
  karamba.changeText(widget, lbl_id3year, "")
  karamba.changeText(widget, lbl_id3genre, "")
  karamba.changeText(widget, lbl_paths_config, "")

  karamba.addImageTooltip(widget, tip_id3artist,"")
  karamba.addImageTooltip(widget, btn_toggle_display,"")
  karamba.addImageTooltip(widget, btn_sidemenu,"")
  karamba.addImageTooltip(widget, btn_getconfig,"")

  return

# Check the current state, and display the correct buttons.
def updateGUI(widget):

  global netjuke_path, xmms_status, current_track_url, artist_tooltip, id3_src, id3_src_txt, media_type_txt, chart_src_txt, resized, vars_msg, sidemenu, dump_action, albumart_lbl, taglist, qartist, id3artist_f

  vars_msg = "\n*******************************************\n\tCurrent values of variables\n*******************************************\n"
  vars_msg = vars_msg + "\nID3 source\t\t: " + str(id3_src) + "\nCharts provider\t\t: " + str(chart_src)

  xmms_status = xmms.control.is_running()

  syslog = karamba.readMenuConfigOption(widget, "syslog_state")
  if syslog == 1:
      try:
         os.system(theme_path + "../scripts/LogMonitor.sh \"" + log_file + "\" \"" + info_dump_path + "\"")
      except:
         dump_action = 1
         dumpInfo("\n \n\t*** not getting log data ***\n")

  if xmms_status == 0:
      DrawStandby(widget)
  else:

    if display_view == 0:
         if sidemenu == 1:
            karamba.resizeWidget(widget, 400, 160)
         else:
            karamba.resizeWidget(widget, 310, 160)
         resized = 0
         karamba.redrawWidget(widget)

    if display_view == 2:
         if sidemenu == 1:
            karamba.resizeWidget(widget, 310, 160)
            resized = 0
            karamba.redrawWidget(widget)

    if display_view == 2:
         karamba.hideImage(widget, btn_sidemenu)
    else:
         karamba.showImage(widget, btn_sidemenu)

    karamba.redrawWidget(widget)
    karamba.changeText(widget, btn_match_charts, "find match")
    karamba.changeText(widget, btn_id3_edit, "Rolling Stone")
    karamba.changeText(widget, btn_netjuke_edit, "-----")
    karamba.changeText(widget, btn_add_fav, "option menu")
    karamba.changeText(widget, btn_search_charts, "search charts")
    karamba.changeText(widget, btn_refresh, "refresh")
    karamba.changeText(widget, btn_extend, "")

    karamba.changeText(widget, lbl_clock_top, "")
    karamba.changeText(widget, lbl_clock_center, "")
    karamba.changeText(widget, lbl_clock_bottom1, "")
    karamba.changeText(widget, lbl_clock_bottom2, "")
    karamba.changeText(widget, lbl_clock_bottom3, "")
    karamba.changeText(widget, lbl_event_cnt, "")
    karamba.changeText(widget, lbl_allday_cnt, "")

    karamba.showImage(widget, btn_toggle_display)
    karamba.addImageTooltip(widget, btn_toggle_display,"Toggle Display")

    karamba.showImage(widget, progBar)
    if main_stat == 1:
        xmms.control.main_win_toggle(1)
    else:
        xmms.control.main_win_toggle(0)

    karamba.hideImage(widget, btn_power_on)
    karamba.showImage(widget, btn_stop)
    karamba.showImage(widget, btn_forward)
    karamba.showImage(widget, btn_reverse)
    karamba.showImage(widget, btn_pause)
    karamba.showImage(widget, btn_play)

    karamba.changeText(widget, btn_xmms_main, "Main")
    karamba.changeText(widget, btn_xmms_eq, "EQ")
    karamba.changeText(widget, btn_xmms_pl, "PL")
    karamba.changeText(widget, btn_xmms_eject, "Eject")
    karamba.changeText(widget, btn_xmms_repeat, "Repeat")
    karamba.changeText(widget, btn_xmms_shuffle, "Shuffle")
    karamba.changeText(widget, btn_xmms_config, "Config")
    karamba.changeText(widget, btn_xmms_exit, "Exit")

    if not xmms.control.is_main_win():
        karamba.changeTextColor(widget, btn_xmms_main, 255, 255, 255)
        karamba.changeTextShadow(widget, btn_xmms_main, 1)
    else:
        karamba.changeTextColor(widget, btn_xmms_main, 156, 255, 253)
        karamba.changeTextShadow(widget, btn_xmms_main, -1)

    if not xmms.control.is_eq_win():
        karamba.changeTextColor(widget, btn_xmms_eq, 255, 255, 255)
        karamba.changeTextShadow(widget, btn_xmms_eq, 1)
    else:
        karamba.changeTextColor(widget, btn_xmms_eq, 156, 255, 253)
        karamba.changeTextShadow(widget, btn_xmms_eq, -1)

    if not xmms.control.is_pl_win():
        karamba.changeTextColor(widget, btn_xmms_pl, 255, 255, 255)
        karamba.changeTextShadow(widget, btn_xmms_pl, 1)
    else:
        karamba.changeTextColor(widget, btn_xmms_pl, 156, 255, 253)
        karamba.changeTextShadow(widget, btn_xmms_pl, -1)

    if not xmms.control.is_shuffle():
        karamba.changeTextColor(widget, btn_xmms_shuffle, 255, 255, 255)
        karamba.changeTextShadow(widget, btn_xmms_shuffle, 1)
    else:
        karamba.changeTextColor(widget, btn_xmms_shuffle, 156, 255, 253)
        karamba.changeTextShadow(widget, btn_xmms_shuffle, -1)

    if not xmms.control.is_repeat():
        karamba.changeTextColor(widget, btn_xmms_repeat, 255, 255, 255)
        karamba.changeTextShadow(widget, btn_xmms_repeat, 1)
    else:
        karamba.changeTextColor(widget, btn_xmms_repeat, 156, 255, 253)
        karamba.changeTextShadow(widget, btn_xmms_repeat, -1)

    if display_view == 1:
      ## Configuration display
      DrawConfig(widget)
      karamba.redrawWidget(widget)
    elif display_view == 2:
      ## Clock display
      DrawClock(widget)
      karamba.changeText(widget, lbl_track_info, "")
      karamba.changeText(widget, lbl_source_info, "")
      karamba.hideImage(widget, btn_getconfig)
    else:
      ## Player display
      karamba.addImageTooltip(widget, btn_sidemenu,"Show menu")
      karamba.changeText(widget, lbl_track_info, "")
      karamba.changeText(widget, lbl_source_info, "")
      karamba.changeText(widget, lbl_source_info, "")
      karamba.changeText(widget, lbl_paths_config, "")
      karamba.redrawWidget(widget)
      try:
         GetTrackURL(widget)
      except:
         pass
      if current_track_url != track_url:
            current_track_url = track_url
            karamba.resizeImage(widget, progBar, 0, 16)
            karamba.changeText(widget, lbl_id3title, "reading ...")
            karamba.changeText(widget, lbl_id3comments, "")
            karamba.changeText(widget, lbl_id3artist, "")
            karamba.changeText(widget, lbl_id3album, "")
            karamba.changeText(widget, lbl_id3genre, "")
            karamba.changeText(widget, lbl_id3year, "")
            karamba.redrawWidget(widget)
            GetTrackTags(widget)
            try:
               test = str(taglist['artist']) + 'testme'
            except:
               dumpInfo('test 1 failed - getting tags from script')
               GetTagFromScript(widget)
               test = 'passed'
            if not test:
               dumpInfo('test 2 failed - getting tags from script')
               GetTagFromScript(widget)
            if taglist['comments'] == '' or taglist['comments'] == 'None':
               taglist['comments'] = ''
               ypos = 72
            else:
               ypos = 62
            karamba.changeText(widget, lbl_id3comments, taglist['comments'])
            karamba.moveText(widget, lbl_id3title, 24, ypos)
            id3title_c = string.center('"' + taglist['title'] + '"', 43)
            karamba.changeText(widget, lbl_id3title, id3title_c)
            karamba.redrawWidget(widget)
            id3artist_f = str(taglist['artist'])
            if re.search(', The', id3artist_f):
               try:
                  id3artist_f = 'The ' + str(re.sub(', The', '', id3artist_f))
               except:
                  pass
            qartist = (re.sub('( )|(,)|(\.)|(-)|(!)|(&)|(\')|(\")|(\\\\)','', id3artist_f)).lower()
            karamba.changeText(widget, lbl_id3artist, id3artist_f)
            if taglist['artist'] == "-":
               current_track_url = "reset_me"
               config_state = 1
            karamba.changeText(widget, lbl_id3album, taglist['album'])
            karamba.changeText(widget, lbl_id3genre, taglist['genre'].center(15))
            year_r = "(" + str(taglist['year']) + ")"
            year_r = year_r.rjust(14)
            karamba.changeText(widget, lbl_id3year, year_r)
            #karamba.moveText(widget, lbl_id3year, 180  + len(str(taglist['genre']))*5, 115)
            karamba.redrawWidget(widget)
            artist_tooltip = str(file_name)
            karamba.addImageTooltip(widget, tip_id3artist, artist_tooltip)

    karamba.redrawWidget(widget)
    if xmms.control.is_running():
        ## Getting playing and remaining time
        GetPlaytime(widget, time_display_stat)
        ## Center track time diplay
        if time_display_stat == "remain":
            karamba.moveText(widget, lbl_play_time, 110, 140)
        elif time_display_stat == "playing":
            karamba.moveText(widget, lbl_play_time, 129, 140)
        karamba.changeText(widget, lbl_play_time, xmms_playtime)

    karamba.redrawWidget(widget)
    ## Setting the state of the control buttons
    #
    # When xmms is playing,
    if xmms.control.is_playing() and not xmms.control.is_paused() and display_view == 0:
         # Illuminate control buttons.
         karamba.removeImageEffects(widget, btn_reverse)
         karamba.changeImageIntensity(widget, btn_play, -0.5, 600)
         karamba.removeImageEffects(widget, btn_pause)
         karamba.removeImageEffects(widget, btn_stop)
         karamba.removeImageEffects(widget, btn_forward)
    # When xmms is paused,
    elif xmms.control.is_playing() and xmms.control.is_paused() and display_view == 0:
         # Dim all control buttons but flash Play and Pause at an interval of '600'
         karamba.changeImageIntensity(widget, btn_reverse, -0.5, 0)
         karamba.changeImageIntensity(widget, btn_play, -0.5, 600)
         karamba.changeImageIntensity(widget, btn_pause, -0.5, 600)
         karamba.changeImageIntensity(widget, btn_stop, -0.5, 0)
         karamba.changeImageIntensity(widget, btn_forward, -0.5, 0)
    # When xmms running in Stand-By,
    elif xmms.control.is_running() and not xmms.control.is_playing() and not xmms.control.is_paused() and display_view == 0:
         # Dim all control buttons
         karamba.changeImageIntensity(widget, btn_reverse, -0.5, 0)
         karamba.changeImageIntensity(widget, btn_play, -0.5, 0)
         karamba.changeImageIntensity(widget, btn_pause, -0.5, 0)
         karamba.changeImageIntensity(widget, btn_stop, -0.5, 0)
         karamba.changeImageIntensity(widget, btn_forward, -0.5, 0)
    # Otherwise,
    else:
      # Dim all control buttons
      if display_view == 0:
         karamba.changeImageIntensity(widget, btn_reverse, -0.5, 0)
         karamba.changeImageIntensity(widget, btn_play, -0.5, 0)
         karamba.changeImageIntensity(widget, btn_pause, -0.5, 0)
         karamba.changeImageIntensity(widget, btn_stop, -0.5, 0)
         karamba.changeImageIntensity(widget, btn_forward, -0.5, 0)

    if display_view == 0:
      progress_bar(widget)

    karamba.redrawWidget(widget)


## Getting path of currently selected / played track
## When XMMS returns an URL it will try to replace
## the server URL with the path you have set in
## the config as 'Server path' and 'Local path'
## When it is unable to get the path to the track
## it will return the 'isstreaming' flag as 'http'
## In which case either the tags won't be shown
## but instead telling you what's going on.
def GetTrackURL(widget):

   global track_url, file_name, track_path, media_type, isstreaming
   isstreaming = "-"
   track_url = "-"
   track_path = "-"
   media_type = "-"
   file_name = "-"
   if xmms_status != 0:
      try:
         x = xmms.control.get_playlist_pos()
         track_url = xmms.control.get_playlist_file(x)
         track_url_set = track_url.split("/")
         isstreaming = track_url_set[0]
         file_name = str(track_url_set[-1:])
         media_type_set = track_url.split('.')
         media_type = str(media_type_set[-1])
         if isstreaming == "http:":
            isstreaming = "http"
            track_path = re.sub(server_path, local_path, track_url)
         else:
            isstreaming = "file"
            track_path = track_url
      except:
         pass
      return track_path



def GetPlaytime(widget, time_display_stat):

   global id3track_time_str, xmms_playtime, xmms_trackinfo, id3_src

   try:
      x = xmms.control.get_playlist_pos()
      id3track_time = xmms.control.get_playlist_time(x)
      id3track_time = int(id3track_time)/1000
   except:
      fp = os.popen("xmmsctrl print %S")
      id3track_time = int(fp.read())

   id3track_time_min = int(id3track_time / 60)
   id3track_time_sec = int(id3track_time - (int(id3track_time_min) * 60))
   if id3track_time_sec < 10:
      id3track_time_sec = str(id3track_time_sec)
      id3track_time_sec = "0" + id3track_time_sec

   fp = os.popen("xmmsctrl gettime")
   play_time_MS = float(fp.read())
   play_time = int(play_time_MS)
   play_time_min = int(play_time_MS / 60)
   play_time_sec = int(play_time_MS - (int(play_time_min) * 60))
   if play_time_sec < 10:
      play_time_sec = str(play_time_sec)
      play_time_sec = "0" + play_time_sec

   if play_time_min < 10:
      play_time_min = str(play_time_min)
      play_time_min = "0" + play_time_min

   remaining_time = int(id3track_time) - int(play_time)
   remaining_time_min =  int(remaining_time / 60)
   remaining_time_sec = int(remaining_time - (int(remaining_time_min) * 60))
   if remaining_time_sec < 10:
      remaining_time_sec = str(remaining_time_sec)
      remaining_time_sec = "0" + remaining_time_sec

   if remaining_time_min < 10:
      remaining_time_min = str(remaining_time_min)
      remaining_time_min = "0" + remaining_time_min

   id3track_time_str =  str(id3track_time_min) + ":" + str(id3track_time_sec)
   play_time_str = str(play_time_min) + " : " + str(play_time_sec)
   remain_time_str = str(remaining_time_min) + " : " + str(remaining_time_sec)

   if time_display_stat  == "remain":
      xmms_playtime = "- " + remain_time_str
   else:
      xmms_playtime = play_time_str



#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget, netjukedb
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):

    global dump_action, current_track_url, time_display_stat, main_stat, display_view, resized, playermenu, tvguide_nl, pimsearch, dumpid3v1, dumpid3v2, dumpsyslog, searchveronica, searchacclaimed, searchgracenote, playermenusep, rollingstone


    # XMMS Main Window button
    if button == 1 and 35 < x < 60 and 28 < y < 38 and xmms.control.is_running():

        if xmms.control.is_main_win():
            xmms.control.main_win_toggle(0)
            main_stat = 0
        else:
            xmms.control.main_win_toggle(1)
            main_stat = 1

    # XMMS EQ button
    if button == 1 and 68 < x < 82 and 28 < y < 38 and xmms.control.is_running():

        if xmms.control.is_eq_win():
            xmms.control.eq_win_toggle(0)
        else:
            xmms.control.eq_win_toggle(1)

    # XMMS PL button
    if button == 1 and 92 < x < 104 and 28 < y < 38 and xmms.control.is_running():

        if xmms.control.is_pl_win():
            xmms.control.pl_win_toggle(0)
        else:
            xmms.control.pl_win_toggle(1)

    # XMMS Eject button
    if button == 1 and 113 < x < 136 and 28 < y < 38 and xmms.control.is_running():

        xmms.control.eject()

    # XMMS Repeat button
    if button == 1 and 148 < x < 178 and 28 < y < 38 and xmms.control.is_running():

        xmms.control.toggle_repeat()

    # XMMS Shuffle button
    if button == 1 and 190 < x < 220 and 28 < y < 38 and xmms.control.is_running():

        xmms.control.toggle_shuffle()

    # XMMS Config button
    if button == 1 and 233 < x < 262 and 28 < y < 38 and xmms.control.is_running():
        xmms.control.show_prefs_box()

    # XMMS Power ON button (MAIN)
    if button == 1 and 5 < x < 17 and 20 < y < 32 and not xmms.control.is_running():

         karamba.executeInteractive(widget, ["xmms"])
         dump_action = 1
         dumpInfo("... starting XMMS\n")
         current_track_url = "playme"
         display_view = 0
         updateGUI(widget)

    # XMMS Power OFF button (Toolbar)
    if button == 1 and 270 < x < 300 and 28 < y < 38:

         dump_action = 1
         dumpInfo("... XMMS has exited\n")
         karamba.resizeWidget(widget, 400, 160)
         resized = 0
         xmms.control.quit()
         updateGUI(widget)
         time.sleep(1)

    # Back button
    if button == 1 and 30 < x < 50 and 115 < y < 135 and display_view == 0:

        xmms.control.playlist_prev()
        karamba.resizeImage(widget, progBar, 0, 16)

    # Play/Pause button
    if button == 1 and 54 < x < 74 and 115 < y < 135 and xmms.control.is_running() and display_view == 0:

            if xmms.control.is_playing():
                  xmms.control.pause()
            else:
                  xmms.control.play()
                  current_track_url = "reset_me"
                  updateGUI(widget)

    # Pause button
    if button == 1 and 77 < x < 97 and 116 < y < 136 and xmms.control.is_playing() and display_view == 0:

        xmms.control.pause()

    # Stop button
    if button == 1 and 100 < x < 120 and 115 < y < 135 and xmms.control.is_playing() and display_view == 0:

        xmms.control.stop()
        karamba.resizeImage(widget, progBar, 0, 16)

    # Next button
    if button == 1 and 125 < x < 145 and 115 < y < 135 and display_view == 0:

        xmms.control.playlist_next()
        karamba.resizeImage(widget, progBar, 0, 16)


    # Play Time Display Toggle
    if button == 1 and 110 < x < 170 and 140 < y < 155 and xmms.control.is_running():

        if time_display_stat == "remain":
                  time_display_stat = "playing"
        else:
                  time_display_stat = "remain"

        updateGUI(widget)


    # Go 30 secs back.
    if button == 5 and xmms.control.is_playing() and 30 < x < 50 and 115 < y < 135 and display_view == 0:

        xmms.control.jump_to_time(xmms.control.get_output_time() - 30000)

    # Skip 30 secs.
    if button == 4 and xmms.control.is_playing() and 125 < x < 145 and 115 < y < 135 and display_view == 0:

        xmms.control.jump_to_time(xmms.control.get_output_time() + 30000)

    # Go 10 secs back. (Time Display)
    if button == 5 and xmms.control.is_playing() and 110 < x < 170 and 140 < y < 155 and display_view == 0:

        xmms.control.jump_to_time(xmms.control.get_output_time() - 10000)

    # Skip 10 secs. (Time Display)
    if button == 4 and xmms.control.is_playing() and 110 < x < 170 and 140 < y < 155 and display_view == 0:

        xmms.control.jump_to_time(xmms.control.get_output_time() + 10000)


    # Match current Artsist with set Music Chart
    if button == 1 and xmms.control.is_playing() and 50 < x < 100 and 45 < y < 60 and display_view == 0:

        if chart_src == "local_src" and custom == 1:
             os.system(theme_path + "../scripts/search_local_charts_by_artist.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\"")
        elif chart_src == "veronica_src":
             os.system(theme_path + "../scripts/search_veronica_charts_by_artist.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\"")
        elif chart_src == "acclaimed_src":
             os.system(theme_path + "../scripts/search_acclaimed_charts_by_artist.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\"")
        else:
             os.system(theme_path + "../scripts/search_veronica_charts_by_artist.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\"")

    # Match current Title with set Music Chart
    if button == 1 and xmms.control.is_playing() and 75 < x < 250 and 65 < y < 90 and display_view == 0:

        if chart_src == "local_src" and custom == 1:
             os.system(theme_path + "../scripts/search_local_charts_by_title.sh \"" + taglist['title'] + "\" \"" + info_dump_path + "\"")
        elif chart_src == "veronica_src":
             os.system(theme_path + "../scripts/search_veronica_charts_by_title.sh \"" + taglist['title'] + "\" \"" + info_dump_path + "\"")
        elif chart_src == "acclaimed_src":
             os.system(theme_path + "../scripts/search_veronica_charts_by_title.sh \"" + taglist['title'] + "\" \"" + info_dump_path + "\"")
        else:
             os.system(theme_path + "../scripts/search_veronica_charts_by_title.sh \"" + taglist['title'] + "\" \"" + info_dump_path + "\"")

    # Match current Album with set Music Chart
    if button == 1 and xmms.control.is_playing() and 30 < x < 100 and 100 < y < 115 and display_view == 0:

        if chart_src == "local_src":
           scharts_cmd = theme_path + "../scripts/search_local_charts_by_album.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\" \"" + taglist['album'] + "\""
        else:
           scharts_cmd = theme_path + "../scripts/search_gracenote-albums_by_artist+track.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\" \"" + taglist['title'] + "\""

        os.system(scharts_cmd)

    # Match current Year with set Music Chart (List all releases from current Year)
    if button == 1 and xmms.control.is_playing() and 260 < x < 300 and 115 < y < 130 and display_view == 0:

        if chart_src == "local_src" and custom == 1:
             os.system(theme_path + "../scripts/etc/__xmms-ext_artist-by-year-charts.sh \"" + taglist['year'] + "\" \"" + info_dump_path + "\"")
        elif chart_src == "veronica_src":
             os.system(theme_path + "../scripts/search_veronica_charts_by_year.sh \"" + taglist['year'] + "\" \"" + info_dump_path + "\"")
        elif chart_src == "acclaimed_src":
             os.system(theme_path + "../scripts/search_acclaimed_charts_by_year.sh \"" + taglist['year'] + "\" \"" + info_dump_path + "\"")
        else:
             os.system(theme_path + "../scripts/search_veronica_charts_by_year.sh \"" + taglist['year'] + "\" \"" + info_dump_path + "\"")

    # Match current Genre with Acclaimed Music Charts
    #if button == 1 and xmms.control.is_playing() and 170 < x < 210 and 115 < y < 130 and display_view == 0:

        #if custom == 1:
           #scharts_cmd = theme_path + "../scripts/__search_acclaimed_charts_by_genre+free.sh \"" + taglist['genre'] + "\" \"" + info_dump_path + "\""
        #else:
           #scharts_cmd = theme_path + "../scripts/__search_acclaimed_charts_by_genre.sh \"" + taglist['genre'] + "\" \"" + info_dump_path + "\""

        #os.system(scharts_cmd)

    # Match current track info (Artist + Title) with set Music Chart
    if button == 1 and xmms.control.is_playing() and 310 < x < 360 and 42 < y < 52 and display_view == 0:

        if chart_src == "local_src" and custom == 1:
             os.system(theme_path + "../scripts/search_local_charts_match_artist+title.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\" \"" + taglist['title'] + "\"")
        elif chart_src == "veronica_src":
             os.system(theme_path + "../scripts/search_veronica_charts_by_artist+free.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\"")
        elif chart_src == "acclaimed_src":
             #os.system(theme_path + "../scripts/__search_acclaimed_charts_by_artist+free.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\"")
             os.system(theme_path + "../scripts/search_veronica_charts_by_artist+free.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\"")
        else:
             os.system(theme_path + "../scripts/search_veronica_charts_by_artist+free.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\"")

    # Goto RollingStone.com
    if button == 1 and xmms.control.is_playing() and 310 < x < 360 and 55 < y < 65 and display_view == 0:
        os.system("kfmclient exec \"http://www.rollingstone.com/" + qartist + "\"")


    # Option Menu
    if button == 1 and xmms.control.is_running() and 310 < x < 365 and 81 < y < 94 and display_view == 0:

        #searchacclaimed = karamba.addMenuItem(widget, playermenu, "Search Acclaimed Charts", iconCharts)
        playermenu = karamba.createMenu(widget)
        pimsearch = karamba.addMenuItem(widget, playermenu, "Search Kaddressbook (CSV)", iconPIM)
        searchveronica = karamba.addMenuItem(widget, playermenu, "Search Veronica Top40 Charts", iconCharts)
        searchgracenote = karamba.addMenuItem(widget, playermenu, "Search Gracenote", iconGracenote)
        rollingstone = karamba.addMenuItem(widget, playermenu, "Find artist at Rolling Stone Magazine", iconWeb)
        dumpid3v1 = karamba.addMenuItem(widget, playermenu, "Dump ID3v1 tags", iconPlayer)
        dumpid3v2 = karamba.addMenuItem(widget, playermenu, "Dump ID3v2 tags", iconPlayer)
        karamba.addMenuSeparator(widget, playermenu)
        dumpsyslog = karamba.addMenuItem(widget, playermenu, "Dump system messages", iconTerm)
        tvguide_nl = karamba.addMenuItem(widget, playermenu, "What's on TV (Dutch)", iconWeb)
        karamba.popupMenu(widget, playermenu, 370, 20)

    # Search set Music Chart provider
    if button == 1 and xmms.control.is_playing() and 310 < x < 365 and 94 < y < 104 and display_view == 0:

        if chart_src == "local_src" and custom == 1:
             os.system(theme_path + "../scripts/search_local_charts_main.sh \"" + taglist['artist'] + "\" \"" + taglist['title']  + "\" \"" + info_dump_path + "\"")
        elif chart_src == "veronica_src":
             os.system(theme_path + "../scripts/search_veronica_charts_main.sh \"" + taglist['artist'] + "\" \"" + taglist['title']  + "\" \"" + info_dump_path + "\" \"" + iconCharts + "\"")
        elif chart_src == "acclaimed_src":
             #os.system(theme_path + "../scripts/search_acclaimed_charts_main.sh \"" + taglist['artist'] + "\" \"" + taglist['title']  + "\" \"" + info_dump_path + "\" \"" + taglist['album'] + "\"")
             os.system(theme_path + "../scripts/search_veronica_charts_main.sh \"" + taglist['artist'] + "\" \"" + taglist['title']  + "\" \"" + info_dump_path + "\"")
        else:
             os.system(theme_path + "../scripts/search_veronica_charts_main.sh \"" + taglist['artist'] + "\" \"" + taglist['title']  + "\" \"" + info_dump_path + "\" \"" + iconCharts + "\"")

    # Refresh display
    if button == 1 and xmms.control.is_running() and 310 < x < 345 and 107 < y < 117:

        current_track_url = "reset_me"
        config_state = 1
        updateGUI(widget)

    # Extend Module
    if button == 1 and xmms.control.is_running() and 310 < x < 345 and 122 < y < 135 and display_view == 0:

       pass


    # Show events for Today
    if button == 1 and 24 < x < 100 and 45 < y < 60 and (display_view == 2 or xmms_status == 0):
       if cal_path != "not set":
          os.system(theme_path + "../scripts/pim_show-today.sh \"" + cal_path + "\" \"" + cal_export_type + "\" \"" + info_dump_path + "\"")
       else:
          dump_action = 1
          dumpInfo("\nPath to the Calendar resource-file (*.ics) is not set or is unknown.\n \nUse the theme configuration menu to set its path.")

    # Show events for Next 7 days (Week)
    if button == 1 and 230 < x < 290 and 45 < y < 60 and (display_view == 2 or xmms_status == 0):
       if cal_path != "not set":
          os.system(theme_path + "../scripts/pim_show-week.sh \"" + cal_path + "\" \"" + cal_export_type + "\" \"" + info_dump_path + "\"")
       else:
          dump_action = 1
          dumpInfo("\nPath to the Calendar resource-file (*.ics) is not set or is unknown.\n \nUse the theme configuration menu to set its path.")

    # Show events for next 28 days (Month)
    if button == 1 and 100 < x < 200 and 115 < y < 130 and (display_view == 2 or xmms_status == 0):
       if cal_path != "not set":
          os.system(theme_path + "../scripts/pim_show-month.sh \"" + cal_path + "\" \"" + cal_export_type + "\" \"" + info_dump_path + "\"")
       else:
          dump_action = 1
          dumpInfo("\nPath to the Calendar resource-file (*.ics) is not set or is unknown.\n \nUse the theme configuration menu to set its path.")

    # Show events for the next (input) days
    if button == 1 and 255 < x < 290 and 115 < y < 130 and (display_view == 2 or xmms_status == 0):
       if cal_path != "not set":
          os.system(theme_path + "../scripts/pim_show-free.sh \"" + cal_path + "\" \"" + cal_export_type + "\" \"" + info_dump_path + "\"")
       else:
          dump_action = 1
          dumpInfo("\nPath to the Calendar resource-file (*.ics) is not set or is unknown.\n \nUse the theme configuration menu to set its path.")


#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves

    pass

#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):

   global dump_action

   ## Option Menu actions
   if menu == playermenu:
      if id == tvguide_nl:
         os.system(theme_path + "../scripts/etc/tv_gids_online--dutch-tv--.sh \"" + info_dump_path + "\"")

      elif id == pimsearch:
         if csv_path != "not set":
            os.system(theme_path + "../scripts/pim_search-contacts.sh \"" + csv_path + "\" \"" + info_dump_path + "\"")
         else:
            dump_action = 1
            dumpInfo("\n\nPath to the Kaddressbook CSV-file (*.csv) is not set or is unknown.\n\nUse the theme configuration menu to set its path.\n")

      elif id == dumpid3v1:
         os.system(theme_path + "../scripts/etc/list_id3v1_info.sh \"" + media_type + "\" \"" + track_path + "\" \"" + info_dump_path + "\"")

      elif id == dumpid3v2:
         os.system(theme_path + "../scripts/etc/list_id3v2_info.sh \"" + media_type + "\" \"" + track_path + "\" \"" + info_dump_path + "\"")

      elif id == dumpsyslog:
         os.system(theme_path + "../scripts/CatMessages.sh \"" + info_dump_path + "\"")

      elif id == searchveronica:
         os.system(theme_path + "../scripts/search_veronica_charts_main.sh \"" + taglist['artist'] + "\" \"" + taglist['title']  + "\" \"" + info_dump_path + "\" \"" + iconCharts + "\"")

      elif id == searchgracenote:
         os.system(theme_path + "../scripts/search_gracenote_main.sh \"" + taglist['artist'] + "\" \"" + info_dump_path + "\" \"" + taglist['title']  + "\" \"" + taglist['album']  + "\" \"" + iconGracenote + "\"")

      elif id == rollingstone:
         os.system("kfmclient exec \"http://www.rollingstone.com/" + qartist + "\"")




#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
   global chart_srcpid, id3_srcpid, local_pathpid, server_pathpid, cal_pathpid, csv_pathpid, id3_src_txt, time_formatpid, syslog_statepid, localepid, cal_export_typepid, log_filepid
   global dump_action, syslog, syslog_state

   chart_srcpid = None
   id3_srcpid = None
   local_pathpid = None
   server_pathpid = None
   cal_pathpid = None
   csv_pathpid = None
   time_formatpid = None
   syslog_statepid = None
   localepid = None
   cal_export_typepid = None
   log_filepid = None

   chart_src_cmd = ["kdialog", "--miniicon", iconCharts, "--title", "Charts Provider selection", "--radiolist", "Please select a Charts provider to use:", "veronica_src", "Veronica Top40", "off", "acclaimed_src", "Acclaimed Music", "off"]

   id3_src_cmd = ["kdialog", "--miniicon", iconPlayer, "--title", "ID3 Tag Source selection", "--radiolist", "Please select a source to read ID3 tags from:", "internal_v1", "ID3 Tag v1", "off", "internal_v2", "ID3 Tag v2", "off"]

   local_path_cmd = ["kdialog", "--miniicon", iconPlayer, "--title", "Select local media path", "--getexistingdirectory", "."]
   servpath_cmd = ["kdialog", "--miniicon", iconPlayer, "--inputbox", "Enter server path:\n(eg. http://xxx.xxx.xxx/music/)\nInclude a trailing slash !", "http://"]
   cal_path_cmd = ["kdialog", "--miniicon", iconPlayer, "--title", "Select your default Kalendar resource-file.\nNo writing will be done.  Just for showing you your events from the Clock Display", "--getopenfilename", "."]

   time_format_cmd = ["kdialog", "--miniicon", iconPlayer, "--title", "Time & Date", "--radiolist", "Select a format for time display:", "hhmmsstz", "24h HH:MM:SS Timezone", "off", "hhmmss", "24h HH:MM:SS", "off", "hhmm", "24h HH:MM", "off", "hhmmsstzus", "12h HH:MM:SS AM/PM TZ", "off", "hhmmssus", "12h HH:MM:SS AM/PM", "off",  "hhmmus", "12h HH:MM AM/PM", "off", "default", "12h HH:MM:SS AM/PM (default)", "off"]

   csv_path_cmd = ["kdialog", "--miniicon", iconPlayer, "--title", "Select the CSV export file from Kaddressbook.\nYou will have to create one first from Kaddressbook itself.", "--getopenfilename", "."]

   locale_cmd = ["kdialog", "--miniicon", iconPlayer, "--title", "Locale selection", "--radiolist", "Please select an encoding:", "ascii", "ASCII", "off", "latin-1", "Latin 1", "off", "iso-8859-1", "ISO 8859-1", "off", "iso-8859-2", "ISO 8859-2", "off", "iso-8859-15", "ISO 8859-15", "off", "utf-8", "UTF-8", "off", "utf-16", "UTF-16", "off"]

   cal_export_type_cmd = ["kdialog", "--miniicon", iconPlayer, "--title", "Calendar output format selection", "--combobox", "Select the format output for konsolekalendar:", "Short", "Text"]

   log_file_cmd = ["kdialog", "--miniicon", iconPlayer, "--title", "Select a log-file to monitor.\nYou must have read-access to this file.", "--getopenfilename", "."]

   if (key == "chart_src"):
      karamba.setMenuConfigOption(widget, "chart_src", 0)
      chart_srcpid = karamba.executeInteractive(widget, chart_src_cmd)

   if (key == "id3_src"):
      karamba.setMenuConfigOption(widget, "id3_src", 0)
      id3_srcpid = karamba.executeInteractive(widget, id3_src_cmd)
      id3_src_txt = ""

   if (key == "local_path"):
      karamba.setMenuConfigOption(widget, "local_path", 0)
      local_pathpid = karamba.executeInteractive(widget, local_path_cmd)

   if (key == "server_path"):
      karamba.setMenuConfigOption(widget, "server_path", 0)
      server_pathpid = karamba.executeInteractive(widget, servpath_cmd)

   if (key == "cal_path"):
      karamba.setMenuConfigOption(widget, "cal_path", 0)
      cal_pathpid = karamba.executeInteractive(widget, cal_path_cmd)

   if (key == "time_format"):
      karamba.setMenuConfigOption(widget, "time_format", 0)
      time_formatpid = karamba.executeInteractive(widget, time_format_cmd)

   if (key == "csv_path"):
      karamba.setMenuConfigOption(widget, "csv_path", 0)
      csv_pathpid = karamba.executeInteractive(widget, csv_path_cmd)

   if (key == "syslog_state"):
      try:
         syslog_state = karamba.readMenuConfigOption(widget, "syslog_state")
         if syslog_state == 1:
            syslog = 1
         else:
            syslog = 0
      except:
            syslog = 0
      return syslog

   if (key == "locale"):
      karamba.setMenuConfigOption(widget, "locale", 0)
      localepid = karamba.executeInteractive(widget, locale_cmd)

   if (key == "cal_export_type"):
      karamba.setMenuConfigOption(widget, "cal_export_type", 0)
      cal_export_typepid = karamba.executeInteractive(widget, cal_export_type_cmd)

   if (key == "log_file"):
      karamba.setMenuConfigOption(widget, "log_file", 0)
      log_filepid = karamba.executeInteractive(widget, log_file_cmd)

   updateGUI(widget)



#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):

   global display_view, current_track_url, resized, dump_action, playermenu, configmenu, clockmenu, sidemenu, resized

   if (meter == btn_toggle_display) and (button == 1):

      if display_view == 0:
         display_view = 1
         karamba.resizeWidget(widget, 310, 160)
         #dumpInfo("\nChanging to Configuration display (1)")
         current_track_url = "resetme00"
      elif display_view == 1:
         display_view = 2
         karamba.resizeWidget(widget, 310, 160)
         karamba.addImageTooltip(widget, btn_getconfig,"")
         #dumpInfo("\nChanging to Clock display (2)")
         current_track_url = "resetme01"
      elif display_view == 2:
         karamba.resizeWidget(widget, 310, 160)
         display_view = 0
         #dumpInfo("\nChanging to Player display (0)")
         current_track_url = "resetme02"
      else:
         display_view = 0
         dumpInfo("\nReverting to default display (0)")
         current_track_url = "resetme03"

      karamba.redrawWidget(widget)
      updateGUI(widget)

   if (meter == btn_getconfig) and (button == 1) and display_view == 1:
      dump_action = 1
      dumpInfo(vars_msg)

   if (meter == btn_sidemenu) and (button == 1) and display_view == 0:

        if sidemenu == 0:
           if resized == 1 and display_view == 0:
              karamba.resizeWidget(widget, 400, 460)
           else:
              karamba.resizeWidget(widget, 400, 160)
           sidemenu = 1
        else:
           if resized == 1 and display_view == 0:
              karamba.resizeWidget(widget, 310, 460)
           else:
              karamba.resizeWidget(widget, 310, 160)
           sidemenu = 0

        karamba.redrawWidget(widget)
        updateGUI(widget)

   if (meter == btn_sidemenu) and (button == 1) and display_view == 1:

        if resized == 0:
             karamba.resizeWidget(widget, 310, 460)
             resized = 1
        else:
             karamba.resizeWidget(widget, 310, 160)
             resized = 0

        karamba.redrawWidget(widget)
        updateGUI(widget)

def create_bar(widget):
   global progBar
   progBar = karamba.createImage(widget, 32, 132, "../images/progess_slider_blue-trans.png")

#updates progress bar to be the right size
def progress_bar(widget):
   global barPercent
   global playingTime
   global currTime

   fp = os.popen("xmmsctrl gettime")
   currTime = float(fp.read())
   try:
      playingTime = float(xmms.control.get_playlist_time(x)/1000)
   except:
      fp = os.popen("xmmsctrl print %S")
      playingTime = float(fp.read())

   try:
      barPercent = int((currTime / playingTime) * 100)
      barWidth = int((currTime / playingTime) * 267)
   except:
      barPercent = int(0)
      barWidth = int(0)

   karamba.resizeImage(widget, progBar, barWidth, 2)

   if barPercent == 100 :
      karamba.resizeImage(widget, progBar, 0, 2)



def setDefaults(widget):

   global dump_action, statmsg, config_state, chart_src, id3_src, local_path, theme_path, current_track_url, server_path, cal_path, csv_path, time_format, syslog, syslog_state, locale, cal_export_type, log_file
   statmsg = ""
   if config_state == 3:
      statmsg = statmsg + "\n... initial setup passed.\n \n \n\t\t******\n\nUse the configuration menu to customize\n \n\t\t******\n \n \n"
      dumpInfo("\nCurrent settings:\n \nDefault locale\t\t: " + str(locale) + "\nID3 source\t\t: " + str(id3_src) + "\nCharts provider\t\t: " + str(chart_src) + "\n---\nLocal path\t\t: " + str(local_path) + "\nServer path\t\t: " + str(server_path) + "\n\n\nGeneral seetings:\n\nTime format\t\t: " + str(time_format) + "\nKaddress CSV-file path\t: " + str(csv_path) + "\nCalendar path\t\t: " + str(cal_path) + "\nCalendar output format\t: " + str(cal_export_type) + "\nLog-file\t\t: " + str(log_file))
   elif config_state == 1:
      statmsg = statmsg + "\n\nReading configuration ...\n\n"
      theme_path = karamba.getThemePath(widget)
      test_theme_path = karamba.readConfigEntry(widget, 'theme_path')
      if (str(test_theme_path) != str(theme_path)):
         karamba.writeConfigEntry(widget, 'theme_path', theme_path)
         statmsg = statmsg + "\nTheme path\t\t: " + str(theme_path)
      else:
         statmsg = statmsg + "\nTheme path\t\t: not changed"

      test_cal_export_type = karamba.readConfigEntry(widget, 'cal_export_type')
      if (test_cal_export_type != None):
         cal_export_type = test_cal_export_type
         statmsg = statmsg + "\nCalendar output format\t: " + str(cal_export_type)
      else:
         statmsg = statmsg + "\nCalendar output format\t: not set"

      test_log_file = karamba.readConfigEntry(widget, 'log_file')
      if (test_log_file != None):
         log_file = test_log_file
         statmsg = statmsg + "\nLog-file\t\t: " + str(log_file)
      else:
         statmsg = statmsg + "\nLog-file\t\t: not set"

      test_chart_src = karamba.readConfigEntry(widget, 'chart_src')
      if (test_chart_src != None):
         chart_src = test_chart_src
         statmsg = statmsg + "\nChart provider\t\t: " + str(chart_src)
      else:
         statmsg = statmsg + "\nChart provider\t\t: not set"

      test_id3_src = karamba.readConfigEntry(widget, 'id3_src')
      if (test_id3_src != None):
         id3_src = test_id3_src
         statmsg = statmsg + "\nID3 source\t\t: " + str(id3_src)
      else:
         statmsg = statmsg + "\nID3 source\t\t: not set"

      test_cal_path = karamba.readConfigEntry(widget, 'cal_path')
      if (test_cal_path != None):
         cal_path = test_cal_path
         statmsg = statmsg + "\nCalendar path\t\t: " + str(cal_path)
      else:
         statmsg = statmsg + "\nCalendar path\t\t: not set"

      test_time_format = karamba.readConfigEntry(widget, 'time_format')
      if (test_time_format != None):
         time_format = test_time_format
         statmsg = statmsg + "\nTime format\t\t: " + str(time_format)
      else:
         statmsg = statmsg + "\nTime format\t\t: not set"

      test_csv_path = karamba.readConfigEntry(widget, 'csv_path')
      if (test_csv_path != None):
         csv_path = test_csv_path
         statmsg = statmsg + "\nKaddress CSV path\t: " + str(csv_path)
      else:
         statmsg = statmsg + "\nKaddress CSV path\t: not set"

      test_local_path = karamba.readConfigEntry(widget, 'local_path')
      if (test_local_path != None):
         local_path = test_local_path
         statmsg = statmsg + "\nLocal path\t\t: " + str(local_path)
      else:
         statmsg = statmsg + "\nLocal path\t\t: not set"

      syslog_state = karamba.readMenuConfigOption(widget, "syslog_state")
      if syslog_state == 1:
         syslog = 1
      else:
         syslog = 0

      test_server_path = karamba.readConfigEntry(widget, 'server_path')
      if (test_server_path != None):
         server_path = test_server_path
         statmsg = statmsg + "\nServer path\t\t: " + str(server_path)
      else:
         statmsg = statmsg + "\nServer path\t\t: not set"

      test_locale = karamba.readConfigEntry(widget, 'locale')
      if (test_locale != None):
         locale = test_locale
         statmsg = statmsg + "\nDefault locale\t\t: " + str(locale)
      else:
         statmsg = statmsg + "\nDefault locale\t\t: not set"
   elif config_state == 2:
      dump_action = 1
      dumpInfo("\n... configuration has changed\n \nCurrent settings:\n \nDefault locale\t\t: " + str(locale) + "\nID3 source\t\t: " + str(id3_src) + "\nCharts provider\t\t: " + str(chart_src) + "\n---\nLocal path\t\t: " + str(local_path) + "\nServer path\t\t: " + str(server_path) + "\n\n\nGeneral seetings:\n\nTime format\t\t: " + str(time_format) + "\nKaddress CSV-file path\t: " + str(csv_path) + "\nCalendar path\t\t: " + str(cal_path) + "\nCalendar output format\t: " + str(cal_export_type) + "\nLog-file\t\t: " + str(log_file))

   config_state = 0
   statmsg = statmsg + "\n \n... new settings have been adjusted\n"
   dumpInfo(statmsg)
   current_track_url = "resetme11"


#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):

   global chart_srcpid, id3_srcpid, local_pathpid, server_pathpid, cal_pathpid, csv_pathpid, time_formatpid, syslog_statepid, localepid, log_filepid, cal_export_typepid
   global config_state, chart_src, id3_src, local_path, server_path, cal_path, csv_path, time_format, locale, log_file, cal_export_type

   output = output.replace('\n','')

   if pid == chart_srcpid:
      karamba.writeConfigEntry(widget, 'chart_src', output)
      chart_src = output
      config_state = 2
      setDefaults(widget)

   elif pid == id3_srcpid:
      karamba.writeConfigEntry(widget, 'id3_src', output)
      id3_src = output
      config_state = 2
      setDefaults(widget)

   elif pid == local_pathpid:
      karamba.writeConfigEntry(widget, 'local_path', (str(output) + "/"))
      local_path = str(output) + "/"
      config_state = 2
      setDefaults(widget)

   elif pid == server_pathpid:
      karamba.writeConfigEntry(widget, 'server_path', output)
      server_path = output
      config_state = 2
      setDefaults(widget)

   elif pid == cal_pathpid:
      karamba.writeConfigEntry(widget, 'cal_path', output)
      cal_path = output
      config_state = 2
      setDefaults(widget)

   elif pid == time_formatpid:
      karamba.writeConfigEntry(widget, 'time_format', output)
      time_format = output
      config_state = 2
      setDefaults(widget)

   elif pid == csv_pathpid:
      karamba.writeConfigEntry(widget, 'csv_path', output)
      csv_path = output
      config_state = 2
      setDefaults(widget)

   elif pid == syslog_statepid:
      config_state = 2
      setDefaults(widget)

   elif pid == localepid:
      karamba.writeConfigEntry(widget, 'locale', output)
      locale = output
      config_state = 2
      setDefaults(widget)

   elif pid == log_filepid:
      karamba.writeConfigEntry(widget, 'log_file', output)
      log_file = output
      config_state = 2
      setDefaults(widget)

   elif pid == cal_export_typepid:
      karamba.writeConfigEntry(widget, 'cal_export_type', output)
      cal_export_type = output
      config_state = 2
      setDefaults(widget)


def DrawClock(widget):

   karamba.hideImage(widget, progBar)
   karamba.hideImage(widget, btn_power_on)
   karamba.hideImage(widget, btn_reverse)
   karamba.hideImage(widget, btn_play)
   karamba.hideImage(widget, btn_pause)
   karamba.hideImage(widget, btn_stop)
   karamba.hideImage(widget, btn_forward)

   karamba.changeText(widget, lbl_id3comments, "")
   karamba.changeText(widget, lbl_id3title, "")
   karamba.changeText(widget, lbl_id3artist, "")
   karamba.changeText(widget, lbl_id3album, "")
   karamba.changeText(widget, lbl_id3year, "")
   karamba.changeText(widget, lbl_id3genre, "")

   karamba.addImageTooltip(widget, btn_sidemenu,"")
   karamba.addImageTooltip(widget, tip_id3artist,"")

   tt = os.popen("/bin/date +' day %j%t%t         week %V'")
   time_top = tt.read()
   if time_format == "hhmmsstz":
      tc = os.popen("/bin/date +'%H : %M : %S   %Z'")
   elif time_format == "hhmmss":
      tc = os.popen("/bin/date +'%H : %M : %S'")
   elif time_format == "hhmm":
      tc = os.popen("/bin/date +'%H : %M'")
   elif time_format == "hhmmsstzus":
      tc = os.popen("/bin/date +'%l : %M : %S   %Z'")
   elif time_format == "hhmmssus":
      tc = os.popen("/bin/date +'%l : %M : %S   %p'")
   elif time_format == "hhmmus":
      tc = os.popen("/bin/date +'%l : %M   %p'")
   else:
      tc = os.popen("/bin/date +'%r'")

   time_center = tc.read()
   tb1 = os.popen("/bin/date +'%a.'")
   time_bottom1 = tb1.read()
   tb2 = os.popen("/bin/date +'%d  %B'")
   time_bottom2 = tb2.read()
   tb3 = os.popen("/bin/date +'%Y'")
   time_bottom3 = tb3.read()

   time_center = string.center(time_center, 27)
   karamba.changeText(widget, lbl_clock_top, time_top)
   karamba.changeText(widget, lbl_clock_center, time_center)
   karamba.changeText(widget, lbl_clock_bottom1, time_bottom1)
   karamba.changeText(widget, lbl_clock_bottom2, time_bottom2)
   karamba.moveText(widget, lbl_clock_bottom2, 198 - ((len(time_bottom2)*15)/2), 115)
   karamba.changeText(widget, lbl_clock_bottom3, time_bottom3)
   karamba.changeText(widget, lbl_event_cnt, time_eventscnt)
   karamba.changeText(widget, lbl_allday_cnt, allday_eventcnt)

   #return

def DrawConfig(widget):

   global netjuke_enable, constatus, current_track_url

   karamba.showImage(widget, btn_getconfig)

   karamba.hideImage(widget, progBar)
   karamba.hideImage(widget, btn_power_on)
   karamba.hideImage(widget, btn_reverse)
   karamba.hideImage(widget, btn_play)
   karamba.hideImage(widget, btn_pause)
   karamba.hideImage(widget, btn_stop)
   karamba.hideImage(widget, btn_forward)

   karamba.changeText(widget, lbl_id3comments, "")
   karamba.changeText(widget, lbl_id3title, "")
   karamba.changeText(widget, lbl_id3artist, "")
   karamba.addImageTooltip(widget, tip_id3artist,"")
   karamba.changeText(widget, lbl_id3album, "")
   karamba.changeText(widget, lbl_id3year, "")
   karamba.changeText(widget, lbl_id3genre, "")

   karamba.addImageTooltip(widget, btn_sidemenu,"Show more")
   karamba.addImageTooltip(widget, btn_getconfig,"Check and list settings of current configuration")
   karamba.addImageTooltip(widget, tip_id3artist,"")

   GetTrackInfo(widget)
   karamba.changeText(widget, lbl_track_info, track_info_txt)
   karamba.redrawWidget(widget)
   GetConfigInfo(widget)
   karamba.changeText(widget, lbl_source_info, config_info_txt)
   karamba.changeText(widget, lbl_paths_config, paths_config_txt)
   karamba.redrawWidget(widget)

   return


def GetID3tags(widget, tag_item):

   global tag_item_info

   ## Set the way ID3-tags are read
   #     0  - external (from seperate scripts)
   #     1  - external (from single script)
   readtype = 1
   tag_item_info = ""
   #dumpInfo("\nReading ID3-tags using id3info/ogginfo\n")

   if readtype == 0:
      if id3_src == "internal_v1":
         x = os.popen(theme_path + "../scripts/tag_" + tag_item + "_id3v1.sh " + local_path + " " + id3_src)
      else:
         x = os.popen(theme_path + "../scripts/tag_" + tag_item + "_id3v2.sh " + local_path + " " + id3_src)

      tag_item_info = x.read()
      tag_item_info = tag_item_info.replace('\n','')
      return tag_item_info

   else:
      x = os.popen(theme_path + "../scripts/get_id3tag.sh " + track_path + " " + id3_src + " " + tag_item + " " + media_type)
      tag_item_info = x.read()
      tag_item_info = tag_item_info.replace('\n','')
      return tag_item_info


def GetConfigInfo(widget):

   global config_info_txt, paths_config_txt

   if id3_src == "internal_v1":
      id3_src_txt = "ID3 v1"
   elif id3_src == "internal_v2":
      id3_src_txt = "ID3 v2"
   else:
      id3_src_txt = "unknown"

   if chart_src == "veronica_src":
      chart_src_txt = "Veronica Top40 ('65-'03)"
   elif chart_src == "acclaimed_src":
      chart_src_txt = "Acclaimed Music ('40-'03)"
   elif chart_src == "local_src":
      chart_src_txt = "Custom (local) DB"
   else:
      chart_src_txt = "unknown"

   if config_state == 1:
      current_track_url = "resetme"
      updateGUI(widget)

   try:
      paths_config_txt = "Default locale :\n     " +  str(locale) + "\n\nLocal path :\n     " +  str(local_path) + "\nServer path :\n     " +  str(server_path) + "\nCalendar path :\n     " +  str(cal_path) + "\nKaddressbook CSV-export path :\n     " +  str(csv_path)
      config_info_txt = "ID3 source\t" + ": " + str(id3_src_txt) + "\nCharts provider\t" + ": " + str(chart_src_txt)
   except:
      paths_config_txt = "Default locale\t :not set\n\nLocal path\t: not set\nServer path\t: not set\nCalendar path\t: not set\nKaddressbook CSV-export path\t: not set"
      config_info_txt = "ID3 source\t: unknown\nCharts provider\t: unknown"

   return

def GetTrackInfo(widget):

   global track_info_txt

   media_type_txt = "n.a."
   try:
      GetTrackURL(widget)
      if media_type == "ogg":
         media_type_txt = "Ogg Vorbis"
      elif media_type == "mp3":
         media_type_txt = "MPEG-3"
      else:
         media_type_txt = "unknown"

      if isstreaming == 'http':
         streaming_txt = ' (streaming)'
      elif isstreaming == 'file':
         streaming_txt = ''
      else:
         streaming_txt = ''

      xmms_trackinfo = xmms.control.get_info()
      xmms_trackinfo_bitr = str(int(xmms_trackinfo[0])/1000)
      xmms_trackinfo_splr = str(int(xmms_trackinfo[1])/1000)

      if str(int(xmms_trackinfo[2])) == "1":
         xmms_trackinfo_chnls = "Mono"
      else:
         xmms_trackinfo_chnls = "Stereo"

      track_info_txt = "Playing stats      : " + xmms_trackinfo_bitr + " Kbps  " + xmms_trackinfo_splr + " KHz  " + xmms_trackinfo_chnls + "\nMedia Type        : " + str(media_type_txt) + str(streaming_txt) + "\nEncoding            : " + (str(locale)).upper()
      return track_info_txt
   except:
      track_info_txt = "Playing stats      : -- Kbps   -- KHz   --\nMedia Type        : --\nEncoding        : --"
      return track_info_txt


#######################################################
##       Catching & dumping error and status messages to InfoTerminal
##
## Dump action sets the action to be performed:
##
## 0  -  disabled
## 1  -  write to InfoTerminal
## 2  -  append to InfoTerminal (default)
##
#######################################################
def dumpInfo(msg_txt):

   global dump_action

   filename = theme_path + "../data/info_dump"
   ## Default (append)
   #dump_action = 2
   ## Append message to InfoTerminal
   if dump_action == 2:
      try:
         f = open(filename,"a")
         f.write(msg_txt)
         f.close()
      except:
         pass
   ## Write message to InfoTerminal (overwrite)
   elif dump_action == 1:
      try:
         f = open(filename,"w")
         f.write(msg_txt)
         f.close()
      except:
         pass
   ## Write message to InfoTerminal (overwrite)
   elif dump_action == 0:
      pass

   ## Reset to default (append)
   dump_action = 2

def SetTextFont(widget):

  karamba.changeTextSize(widget, lbl_source_info, 9)
  karamba.changeTextFont(widget, lbl_source_info, "Sans")
  karamba.changeTextColor(widget, lbl_source_info, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_source_info, 0)

  karamba.changeTextSize(widget, lbl_track_info, 10)
  karamba.changeTextFont(widget, lbl_track_info, "Sans")
  karamba.changeTextColor(widget, lbl_track_info, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_track_info, 1)

  karamba.changeTextSize(widget, btn_xmms_main, 8)
  karamba.changeTextFont(widget, btn_xmms_main, "Neuropol")
  karamba.changeTextColor(widget, btn_xmms_main, 255, 255, 255)
  karamba.changeTextShadow(widget, btn_xmms_main, 1)

  karamba.changeTextSize(widget, btn_xmms_eq, 8)
  karamba.changeTextFont(widget, btn_xmms_eq, "Neuropol")
  karamba.changeTextColor(widget, btn_xmms_eq, 255, 255, 255)
  karamba.changeTextShadow(widget, btn_xmms_eq, 1)

  karamba.changeTextSize(widget, btn_xmms_pl, 8)
  karamba.changeTextFont(widget, btn_xmms_pl, "Neuropol")
  karamba.changeTextColor(widget, btn_xmms_pl, 255, 255, 255)
  karamba.changeTextShadow(widget, btn_xmms_pl, 1)

  karamba.changeTextSize(widget, btn_xmms_eject, 8)
  karamba.changeTextFont(widget, btn_xmms_eject, "Neuropol")
  karamba.changeTextColor(widget, btn_xmms_eject, 255, 255, 255)
  karamba.changeTextShadow(widget, btn_xmms_eject, 1)

  karamba.changeTextSize(widget, btn_xmms_repeat, 8)
  karamba.changeTextFont(widget, btn_xmms_repeat, "Neuropol")
  karamba.changeTextColor(widget, btn_xmms_repeat, 255, 255, 255)
  karamba.changeTextShadow(widget, btn_xmms_repeat, 1)

  karamba.changeTextSize(widget, btn_xmms_shuffle, 8)
  karamba.changeTextFont(widget, btn_xmms_shuffle, "Neuropol")
  karamba.changeTextColor(widget, btn_xmms_shuffle, 255, 255, 255)
  karamba.changeTextShadow(widget, btn_xmms_shuffle, 1)

  karamba.changeTextSize(widget, btn_xmms_config, 8)
  karamba.changeTextFont(widget, btn_xmms_config, "Neuropol")
  karamba.changeTextColor(widget, btn_xmms_config, 255, 255, 255)
  karamba.changeTextShadow(widget, btn_xmms_config, 1)

  karamba.changeTextSize(widget, btn_xmms_exit, 8)
  karamba.changeTextFont(widget, btn_xmms_exit, "Neuropol")
  karamba.changeTextColor(widget, btn_xmms_exit, 255, 113, 74)
  karamba.changeTextShadow(widget, btn_xmms_exit, 1)

  # Misc buttons (right))

  karamba.changeTextSize(widget, btn_match_charts, 8)
  karamba.changeTextFont(widget, btn_match_charts, "Sans")
  karamba.changeTextColor(widget, btn_match_charts, 204,229,255)
  karamba.changeTextShadow(widget, btn_match_charts, 1)

  karamba.changeTextSize(widget, btn_id3_edit, 8)
  karamba.changeTextFont(widget, btn_id3_edit, "Sans")
  karamba.changeTextColor(widget, btn_id3_edit, 204,229,255)
  karamba.changeTextShadow(widget, btn_id3_edit, 1)

  karamba.changeTextSize(widget, btn_netjuke_edit, 8)
  karamba.changeTextFont(widget, btn_netjuke_edit, "Sans")
  karamba.changeTextColor(widget, btn_netjuke_edit, 204,229,255)
  karamba.changeTextShadow(widget, btn_netjuke_edit, 1)

  karamba.changeTextSize(widget, btn_add_fav, 8)
  karamba.changeTextFont(widget, btn_add_fav, "Sans")
  karamba.changeTextColor(widget, btn_add_fav, 204,229,255)
  karamba.changeTextShadow(widget, btn_add_fav, 1)

  karamba.changeTextSize(widget, btn_search_charts, 8)
  karamba.changeTextFont(widget, btn_search_charts, "Sans")
  karamba.changeTextColor(widget, btn_search_charts, 204,229,255)
  karamba.changeTextShadow(widget, btn_search_charts, 1)

  karamba.changeTextSize(widget, btn_refresh, 8)
  karamba.changeTextFont(widget, btn_refresh, "Sans")
  karamba.changeTextColor(widget, btn_refresh, 204,229,255)
  karamba.changeTextShadow(widget, btn_refresh, 1)

  karamba.changeTextSize(widget, btn_extend, 8)
  karamba.changeTextFont(widget, btn_extend, "Sans")
  karamba.changeTextColor(widget, btn_extend, 204,229,255)
  karamba.changeTextShadow(widget, btn_extend, 1)
  karamba.changeTextSize(widget, lbl_clock_top, 10)
  karamba.changeTextFont(widget, lbl_clock_top, "911 Porscha")
  karamba.changeTextColor(widget, lbl_clock_top, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_clock_top, 0)

  karamba.changeTextSize(widget, lbl_clock_center, 24)
  karamba.changeTextFont(widget, lbl_clock_center, "Digital Readout Thick Upright")
  karamba.changeTextColor(widget, lbl_clock_center, 181, 234, 255)
  karamba.changeTextShadow(widget, lbl_clock_center, 0)

  karamba.changeTextSize(widget, lbl_clock_bottom1, 10)
  karamba.changeTextFont(widget, lbl_clock_bottom1, "911 Porscha")
  karamba.changeTextColor(widget, lbl_clock_bottom1, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_clock_bottom1, 0)

  karamba.changeTextSize(widget, lbl_clock_bottom2, 10)
  karamba.changeTextFont(widget, lbl_clock_bottom2, "911 Porscha")
  karamba.changeTextColor(widget, lbl_clock_bottom2, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_clock_bottom2, 0)

  karamba.changeTextSize(widget, lbl_clock_bottom3, 10)
  karamba.changeTextFont(widget, lbl_clock_bottom3, "911 Porscha")
  karamba.changeTextColor(widget, lbl_clock_bottom3, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_clock_bottom3, 0)

  karamba.changeTextSize(widget, lbl_event_cnt, 8)
  karamba.changeTextFont(widget, lbl_event_cnt, "Sans")
  karamba.changeTextColor(widget, lbl_event_cnt, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_event_cnt, 0)

  karamba.changeTextSize(widget, lbl_allday_cnt, 8)
  karamba.changeTextFont(widget, lbl_allday_cnt, "Sans")
  karamba.changeTextColor(widget, lbl_allday_cnt, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_allday_cnt, 0)

  karamba.changeTextSize(widget, lbl_play_time, 14)
  karamba.changeTextFont(widget, lbl_play_time, "Zygo")
  karamba.changeTextColor(widget, lbl_play_time, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_play_time, 0)

  karamba.changeTextSize(widget, lbl_id3title, 14)
  karamba.changeTextFont(widget, lbl_id3title, "FrizQuadrata")
  karamba.changeTextColor(widget, lbl_id3title, 124, 211, 255)
  karamba.changeTextShadow(widget, lbl_id3title, 1)

  karamba.changeTextSize(widget, lbl_id3artist, 14)
  karamba.changeTextFont(widget, lbl_id3artist, "FrizQuadrata")
  karamba.changeTextColor(widget, lbl_id3artist, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_id3artist, 1)

  karamba.changeTextSize(widget, lbl_id3comments, 12)
  karamba.changeTextFont(widget, lbl_id3comments, "TempoFontItalic")
  karamba.changeTextColor(widget, lbl_id3comments, 193, 234, 255)
  karamba.changeTextShadow(widget, lbl_id3comments, 1)

  karamba.changeTextSize(widget, lbl_id3album, 12)
  karamba.changeTextFont(widget, lbl_id3album, "FrizQuadrata")
  karamba.changeTextColor(widget, lbl_id3album, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_id3album, 1)

  karamba.changeTextSize(widget, lbl_id3genre, 13)
  karamba.changeTextFont(widget, lbl_id3genre, "FrizQuadrata")
  karamba.changeTextColor(widget, lbl_id3genre, 193, 234, 255)
  karamba.changeTextShadow(widget, lbl_id3genre, 1)

  karamba.changeTextSize(widget, lbl_id3year, 13)
  karamba.changeTextFont(widget, lbl_id3year, "FrizQuadrata")
  karamba.changeTextColor(widget, lbl_id3year, 193, 234, 255)
  karamba.changeTextShadow(widget, lbl_id3year, 1)

  karamba.changeTextSize(widget, lbl_paths_config, 9)
  karamba.changeTextFont(widget, lbl_paths_config, "Sans")
  karamba.changeTextColor(widget, lbl_paths_config, 255, 255, 255)
  karamba.changeTextShadow(widget, lbl_paths_config, 0)


def SetMenu(widget):

   karamba.addMenuConfigOption(widget, 'locale', "Select default encoding")
   karamba.setMenuConfigOption(widget, 'locale', 0)
   karamba.addMenuConfigOption(widget, 'id3_src', "Select ID3 source")
   karamba.setMenuConfigOption(widget, 'id3_src', 0)
   karamba.addMenuConfigOption(widget, 'chart_src', "Select Charts provider")
   karamba.setMenuConfigOption(widget, 'chart_src', 0)
   karamba.addMenuConfigOption(widget, 'local_path', "Local media path")
   karamba.setMenuConfigOption(widget, 'local_path', 0)
   karamba.addMenuConfigOption(widget, 'server_path', "Server media path")
   karamba.setMenuConfigOption(widget, 'server_path', 0)
   karamba.addMenuConfigOption(widget, 'cal_path', "Calendar path")
   karamba.setMenuConfigOption(widget, 'cal_path', 0)
   karamba.addMenuConfigOption(widget, 'cal_export_type', "Calendar output format")
   karamba.setMenuConfigOption(widget, 'cal_export_type', 0)
   karamba.addMenuConfigOption(widget, 'csv_path', "Kaddressbook CSV-export path")
   karamba.setMenuConfigOption(widget, 'csv_path', 0)
   karamba.addMenuConfigOption(widget, 'time_format', "Select time format")
   karamba.setMenuConfigOption(widget, 'time_format', 0)
   karamba.addMenuConfigOption(widget, 'log_file', "Select Log-file")
   karamba.setMenuConfigOption(widget, 'log_file', 0)
   karamba.addMenuConfigOption(widget, 'syslog_state', "Dump Log-file to InfoTerminal")


def GetTrackTags(widget):

   if isstreaming == "http" and local_path == "not set":
      karamba.changeText(widget, lbl_id3artist, "streaming audio ...")
      karamba.changeText(widget, lbl_id3title, "local path to media root not set")
      karamba.moveText(widget, lbl_id3title, 110, 72)
      karamba.changeText(widget, lbl_id3comments, "")
      karamba.changeText(widget, lbl_id3album, "ID3 tags unavailable")
      karamba.changeText(widget, lbl_id3genre, "")
      karamba.changeText(widget, lbl_id3year, "")
      return
   else:
      GetFileTags(widget)
      return

def MonitorCalendar(widget):

   global cur_cal_mod, last_cal_mod

   if cal_path != "not set":
      try:
         cur_cal_mod = os.path.getmtime(cal_path)
         if cur_cal_mod != last_cal_mod:
            last_cal_mod = 0
            return last_cal_mod
      except:
         cur_cal_mod = 0
         last_cal_mod = 0

def GetFileTags(widget):

   global taglist, dump_action
   #### Ogg Vorbis audio
   if media_type == "ogg":
      ## Using pyvorbis
      if ogg_method == "pyvorbis":
         try:
            vf = ogg.vorbis.VorbisFile(track_path)
            tag = vf.comment()
            tag.items()
            uartist = (tag.items()[3])[1]
            utitle = (tag.items()[4])[1]
            ucomm = (tag.items()[5])[1]
            ualbum = (tag.items()[0])[1]
            try:
               ugenre = (tag.items()[7])[1]
            except:
               ugenre = 'unknown'
            uyear = (tag.items()[6])[1]
         except:
            GetTagFromScript(widget)
            return taglist
      ## Using Bash-script
      elif ogg_method == "bash":
         GetTagFromScript(widget)
         return taglist

   #### MPEG-3 audio
   elif media_type == "mp3":
      ## Using id3
      if id3_method == "id3":
         if id3_src == "internal_v1":
            try:
               id3v1 = id3.ID3v1(track_path)
               uartist = id3v1.artist
               utitle = id3v1.title
               ucomm = id3v1.comment
               ualbum = id3v1.album
               try:
                  ugenre = id3v1.genre
               except:
                  ugenre = 'unknown'
               uyear = id3v1.year
            except:
               GetTagFromScript(widget)
               return taglist
         elif id3_src == "internal_v2":
            try:
               id3v2 = id3.ID3v2(track_path)
               for frame in id3v2.frames:
                  if frame.id == 'TPE1':
                     uartist = frame.value
                  elif frame.id == 'TIT2':
                     utitle = frame.value
                  elif frame.id == 'COMM':
                     ucomm = frame.value
                  elif frame.id == 'TALB':
                     ualbum = frame.value
                  elif frame.id == 'TCON':
                     try:
                        ugenre = frame.value
                     except:
                        ugenre = 'unknown'
                  elif frame.id == 'TYER':
                     uyear = frame.value
            except:
               GetTagFromScript(widget)
               return taglist
      ## Using eyeD3
      elif id3_method == "eyed3":
         try:
            tag = eyeD3.Tag()
            if id3_src == 'internal_v1':
               tag.link(track_path, eyeD3.ID3_V1)
            elif id3_src == 'internal_v2':
               tag.link(track_path, eyeD3.ID3_V2)
            uartist = tag.getArtist()
            utitle = tag.getTitle()
            ucomm = tag.getComment()
            ualbum = tag.getAlbum()
            try:
               ugenre = tag.getGenre()
            except:
               ugenre = 'unknown'
            uyear = tag.getYear()
         except:
            GetTagFromScript(widget)
            return taglist
      ## Using Bash-script
      elif id3_method == "bash":
         GetTagFromScript(widget)
         return taglist

   if uartist:
      artist = (uartist).encode(locale)
   else:
      artist = ''
   if utitle:
      title = (utitle).encode(locale)
   else:
      title = ''
   if ucomm:
      comm = (ucomm).encode(locale)
   else:
      comm = ''
   if ualbum:
      album = (ualbum).encode(locale)
   else:
      album = ''
   if ugenre:
      try:
         genre = (ugenre).encode(locale)
      except:
         genre = 'unknown'
   else:
      genre = 'no genre'
   if uyear:
      year = (uyear).encode(locale)
   else:
      year = ''
   taglist = {'artist': artist,
                  'title': title,
                  'comments': comm,
                  'album': album,
                  'genre': genre,
                  'year': year}
   return taglist


def GetTagFromScript(widget):

   global taglist
   taglist = {'artist': GetID3tags(widget, "artist"),
                  'title': GetID3tags(widget, "title"),
                  'comments': GetID3tags(widget, "comments"),
                  'album': GetID3tags(widget, "album"),
                  'genre': GetID3tags(widget, "genre"),
                  'year': GetID3tags(widget, "year")}
   return taglist

def DrawStandby(widget):

   global time_top, time_center, time_bottom1, time_bottom2, time_bottom3, btn_toggle_display

   karamba.showImage(widget, btn_power_on)
   karamba.hideImage(widget, btn_sidemenu)

   tt = os.popen("/bin/date +' day %j%t%t         week %V'")
   time_top = tt.read()
   if time_format == "hhmmsstz":
      tc = os.popen("/bin/date +'%H : %M : %S   %Z'")
   elif time_format == "hhmmss":
      tc = os.popen("/bin/date +'%H : %M : %S'")
   elif time_format == "hhmm":
      tc = os.popen("/bin/date +'%H : %M'")
   elif time_format == "hhmmsstzus":
      tc = os.popen("/bin/date +'%l : %M : %S   %Z'")
   elif time_format == "hhmmssus":
      tc = os.popen("/bin/date +'%l : %M : %S   %p'")
   elif time_format == "hhmmus":
      tc = os.popen("/bin/date +'%l : %M   %p'")
   else:
      tc = os.popen("/bin/date +'%r'")

   time_center = tc.read()
   tb1 = os.popen("/bin/date +'%a.'")
   time_bottom1 = tb1.read()
   tb2 = os.popen("/bin/date +'%d  %B'")
   time_bottom2 = tb2.read()
   tb3 = os.popen("/bin/date +'%Y'")
   time_bottom3 = tb3.read()

   time_center = string.center(time_center, 27)
   karamba.changeText(widget, lbl_clock_top, time_top)
   karamba.changeText(widget, lbl_clock_center, time_center)
   karamba.changeText(widget, lbl_clock_bottom1, time_bottom1)
   karamba.changeText(widget, lbl_clock_bottom2, time_bottom2)
   karamba.moveText(widget, lbl_clock_bottom2, 198 - ((len(time_bottom2)*15)/2), 115)
   karamba.changeText(widget, lbl_clock_bottom3, time_bottom3)
   karamba.changeText(widget, lbl_event_cnt, time_eventscnt)
   karamba.changeText(widget, lbl_allday_cnt, allday_eventcnt)

   ResetLabels(widget)
   karamba.addImageTooltip(widget, btn_toggle_display,"Power On")


# This will be printed when the widget loads.
print "CorpusPlayer loaded"
