# gTaskBar Version 0.2

#this import statement allows access to the karamba functions
import karamba
import string
import os

#############################################
# PARAMETERS (EDIT THESE!)

#b
# iconSize:
#
#   Size of the icons you want to use
#
iconSize = 22

#
# iconRoot:
#
#   Root directory of the icon scheme you want to use.
#   Omit trailing slash.
#
#   Note: this script expects to find directories
#         "16x16/apps", "32x32/apps", etc., below the
#         directory you specify here
#
iconRoot = "/usr/share/icons/crystalsvg"


#
# leftendFile:
#
#   The image file to place at the left end of all the task icons
#
leftendFile = "pics/bar_leftend_32x36.png"

#
# rightendFile:
#
#   The image file to place at the right end of all the task icons
#
rightendFile = "pics/bar_rightend_32x36.png"

#
# middleFile:
#
#   The image file to place behind each of the task icons
#
middleFile = "pics/bar_middle_36x36.png"

# Notes about the image files:
#    (1) It's probably best to use leftend/rightend/middle image files
#    that are all the same height.  I haven't tested the script
#    with image files that are different heights...
#    (2) They CAN be all different widths.
#    (3) Make sure your middle image file is at least as big as your
#    iconSize.  If you want the icons spaced out a little, use a middle
#    image file that is a few pixels wider than your iconSize


#
# emptyPosition:
#
#   Where the top left of the widget should be when
#     there are no icons
#
#   Note: It's important to set this properly even if you've
#         set hideWhenEmpty below, since these coordinates
#         determine how the widget is moved around as it grows
#
emptyPosition = (110,972)

#
# growDirection:
#
#   Set whether the widget will increase size to the left,
#     to the right, or in both directions from the center as
#     new tasks appear
#
#   growDirection: 0 = left, 1 = right, 2 = from center
#
growDirection = 2

#
# hideWhenEmpty:
#
#   Move the widget offscreen if there are no tasks
#
#   hideWhenEmpty: 0 = no, 1 = yes
#
hideWhenEmpty = 0



# END OF PARAMETERS
#############################################
# don't edit below here...

taskGroupList = []
leftend = 0
rightend = 0
middle = []
maxHeight = 0
leftendSize = [0,0]
rightendSize = [0,0]
middleSize = [0,0]
iconOffset = [0,0]

class aTask:
	def __init__(self):
		self.id = 0
		self.name = ""
		self.menuID = 0

class aTaskGroup:
	def __init__(self):
		self.taskList = []
		self.appName = ""
		self.iconPath = ""
		self.iconRef = 0
		self.clickPos = -1
		self.grayed = 0
		self.menu = 0
		self.minimizeID = 0

#############################################
def getTaskAppIconName(nameString):
	# returns (taskName, appName, iconPath)
	global iconRoot
	global iconSize

	splitName = string.split(nameString, " - ")
	if(len(splitName) > 1):
		tmp = ( string.join(splitName[0:len(splitName)-1]," - "), splitName[len(splitName)-1])
	else:
		tmp = (splitName[0], splitName[0])
	appNameSplit = string.split(tmp[1])

	foundIconAt = -1
	iconPath = ""
	for i in range(len(appNameSplit)):
		iconPath = iconRoot + "/" + `iconSize` + "x" + `iconSize` + "/apps/" + string.lower(appNameSplit[i]) + ".png"
		if (os.path.exists(iconPath)):
			foundIconAt = i
			break

	appName = ""
	if (foundIconAt >= 0):
		appName = appNameSplit[foundIconAt]
	else:
		appName = tmp[1]
		iconPath = iconRoot + "/" + `iconSize` + "x" + `iconSize` + "/apps/x.png"
		if(not os.path.exists(iconPath)):
			iconPath = "pics/" + `iconSize` + "x" + `iconSize` + "/x.png"

	return (tmp[0], appName, iconPath)


#############################################
def createNewTaskGroup(widget, appName, iconPath):
	global taskGroupList
	global middle
	global rightend
	global leftendSize
	global rightendSize
	global middleSize
	global middleFile
	global maxHeight
	global growDirection
	global emptyPosition
	global iconOffset

	newTaskGroup = aTaskGroup()
	newTaskGroup.appName = appName
	karamba.resizeWidget(widget, leftendSize[0]+rightendSize[0]+middleSize[0]*(len(taskGroupList)+1), maxHeight)
	karamba.moveImage(widget, rightend, leftendSize[0]+middleSize[0]*(len(taskGroupList)+1), 0)
	if(growDirection == 0):
		karamba.moveWidget(widget, emptyPosition[0]-middleSize[0]*(len(taskGroupList)+1), emptyPosition[1])
	elif(growDirection == 2):
		karamba.moveWidget(widget, emptyPosition[0]-(middleSize[0]*(len(taskGroupList)+1))/2, emptyPosition[1])
	tmp = karamba.createImage(widget, leftendSize[0]+middleSize[0]*len(taskGroupList), 0, middleFile)
	middle.append(tmp)
	newTaskGroup.iconRef = karamba.createImage(widget, leftendSize[0]+middleSize[0]*len(taskGroupList)+iconOffset[0], iconOffset[1], iconPath)
	newTaskGroup.iconPath = iconPath
	karamba.addImageTooltip(widget, newTaskGroup.iconRef, appName)
	newTaskGroup.menu = karamba.createMenu(widget)
	newTaskGroup.minimizeID = karamba.addMenuItem(widget, newTaskGroup.menu, "Minimize All", newTaskGroup.iconPath)
	taskGroupList.append(newTaskGroup)



#############################################
def deleteTaskGroup(widget, index):
	global taskGroupList
	global middle
	global rightend
	global leftendSize
	global rightendSize
	global middleSize
	global maxHeight
	global growDirection
	global emptyPosition
	global iconOffset

	lastMiddle = middle.pop()
	karamba.deleteImage(widget, lastMiddle)
	karamba.deleteImage(widget, taskGroupList[index].iconRef)
	karamba.deleteMenu(widget, taskGroupList[index].menu)
	taskGroupList.pop(index)
	for i in range(len(taskGroupList)-index):
		karamba.moveImage(widget, taskGroupList[i+index].iconRef,leftendSize[0]+middleSize[0]*(i+index)+iconOffset[0], iconOffset[1])
	karamba.moveImage(widget, rightend, leftendSize[0]+middleSize[0]*len(taskGroupList), 0)
	karamba.resizeWidget(widget, leftendSize[0]+rightendSize[0]+middleSize[0]*len(taskGroupList), maxHeight)
	if (len(taskGroupList) == 0) and (hideWhenEmpty):
		karamba.moveWidget(widget, emptyPosition[0], 0 - maxHeight)
	else:
		if(growDirection == 0):
			karamba.moveWidget(widget, emptyPosition[0]-middleSize[0]*len(taskGroupList), emptyPosition[1])
		elif(growDirection == 2):
			karamba.moveWidget(widget, emptyPosition[0]-(middleSize[0]*len(taskGroupList))/2, emptyPosition[1])


#############################################
#this is called when you widget is initialized
def initWidget(widget):
	global iconRoot
	global iconSize
	global taskGroupList
	global maxHeight
	global leftend
	global rightend
	global leftendSize
	global rightendSize
	global middleSize
	global leftendFile
	global rightendFile
	global middleFile
	global iconOffset

	# preliminaries:
	leftend = karamba.createImage(widget, 0, 0, leftendFile)
	rightend = karamba.createImage(widget, 0, 0, rightendFile)
	tmp = karamba.createImage(widget, 0, 0, middleFile)
	leftendSize[0] = karamba.getImageWidth(widget, leftend)
	leftendSize[1] = karamba.getImageHeight(widget, leftend)
	rightendSize[0] = karamba.getImageWidth(widget, rightend)
	rightendSize[1] = karamba.getImageHeight(widget, rightend)
	middleSize[0] = karamba.getImageWidth(widget, tmp)
	middleSize[1] = karamba.getImageHeight(widget, tmp)
	karamba.deleteImage(widget, tmp)
	maxHeight = leftendSize[1]
	if(rightendSize[1] > maxHeight):
		maxHeight = rightendSize[1]
	if(middleSize[1] > maxHeight):
		maxHeight = middleSize[1]
	for i in range(2):
		iconOffset[i] = middleSize[i]/2 - iconSize/2

	karamba.resizeWidget(widget, leftendSize[0]+rightendSize[0], maxHeight)
	karamba.moveImage(widget, rightend, leftendSize[0], 0)

	# get tasks and create task groups and icons
	taskIDs = karamba.getTaskList(widget)
	taskNames = karamba.getTaskNames(widget)

	for i in range(len(taskIDs)):
		if(i == 0):
			(n, a, icon) = getTaskAppIconName(taskNames[0])
			createNewTaskGroup(widget, a, icon)
			newTask = aTask()
			newTask.id = taskIDs[0]
			newTask.name = n
			newTask.menuID = karamba.addMenuItem(widget, taskGroupList[0].menu, newTask.name, taskGroupList[0].iconPath)
			taskGroupList[0].taskList.append(newTask)
		else:
			(n, a, icon) = getTaskAppIconName(taskNames[i])
			newTask = aTask()
			newTask.id = taskIDs[i]
			newTask.name = n
			foundAppAt = -1
			for j in range(len(taskGroupList)):
				if(string.lower(a) == string.lower(taskGroupList[j].appName)):
					foundAppAt = j
					break
			if (foundAppAt < 0):
				createNewTaskGroup(widget, a, icon)
				foundAppAt = len(taskGroupList) - 1
			newTask.menuID = karamba.addMenuItem(widget, taskGroupList[foundAppAt].menu, newTask.name, taskGroupList[foundAppAt].iconPath)
			taskGroupList[foundAppAt].taskList.append(newTask)



#############################################
#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
	global taskGroupList
	global middle

	taskIDs = karamba.getTaskList(widget)
	taskNames = karamba.getTaskNames(widget)

	# get rid of task groups no longer needed
	groupsToDelete = []
	for i in range(len(taskGroupList)):
		foundApp = 0
		for j in range(len(taskNames)):
			(n,a,icon) = getTaskAppIconName(taskNames[j])
			if (string.lower(a) == string.lower(taskGroupList[i].appName)):
				foundApp = 1
				break
		if (foundApp == 0):
			groupsToDelete.append(i)
	for i in range(len(groupsToDelete)):
		deleteTaskGroup(widget, groupsToDelete[i] - i)

	# get rid of tasks that no longer exist
	for i in range(len(taskGroupList)):
		tasksToPop = []
		for j in range(len(taskGroupList[i].taskList)):
			foundTask = 0
			for k in range(len(taskIDs)):
				if (taskIDs[k] == taskGroupList[i].taskList[j].id):
					foundTask = 1
					break
			if (foundTask == 0):
				tasksToPop.append(j)
		for j in range(len(tasksToPop)):
			karamba.removeMenuItem(widget, taskGroupList[i].menu, taskGroupList[i].taskList[tasksToPop[j] - j].menuID)
			taskGroupList[i].taskList.pop(tasksToPop[j] - j)

	# add new tasks to their proper groups, and add new task groups if necessary
	for i in range(len(taskNames)):
		(n,a,icon) = getTaskAppIconName(taskNames[i])
		foundAppAt = -1
		for j in range(len(taskGroupList)):
			if(string.lower(a) == string.lower(taskGroupList[j].appName)):
				foundAppAt = j
				break
		if (foundAppAt < 0):
			createNewTaskGroup(widget, a, icon)
			newTask = aTask()
			newTask.id = taskIDs[i]
			newTask.name = n
			newTask.menuID = karamba.addMenuItem(widget, taskGroupList[len(taskGroupList)-1].menu, newTask.name, taskGroupList[len(taskGroupList)-1].iconPath)
			taskGroupList[len(taskGroupList)-1].taskList.append(newTask)
		else:
			foundTask = 0
			for k in range(len(taskGroupList[foundAppAt].taskList)):
				if (taskIDs[i] == taskGroupList[foundAppAt].taskList[k].id):
					foundTask = 1
					break
			if (foundTask == 0):
				newTask = aTask()
				newTask.id = taskIDs[i]
				newTask.name = n
				newTask.menuID = karamba.addMenuItem(widget, taskGroupList[foundAppAt].menu, newTask.name, taskGroupList[foundAppAt].iconPath)
				taskGroupList[foundAppAt].taskList.append(newTask)

	# for the currently active task, set clickPos to that task in its taskGroup
	# also, update task names since they change a lot
	# also, update greyed out images
	# **** need to re-recreate menu item names....
	for g in taskGroupList:
		iconified = 0
		for i in range(len(g.taskList)):
			taskInfo = karamba.getTaskInfo(widget, g.taskList[i].id)
			(n,a,icon) = getTaskAppIconName(taskInfo[0])
			if (n != g.taskList[i].name):
				karamba.removeMenuItem(widget, g.menu, g.taskList[i].menuID)
				g.taskList[i].name = n
				g.taskList[i].menuID = karamba.addMenuItem(widget, g.menu, g.taskList[i].name, g.iconPath)
			if (taskInfo[7]):
				g.clickPos = i
			iconified += taskInfo[5]
		if (iconified == len(g.taskList)):
			if (g.grayed == 0):
				karamba.changeImageToGray(widget, g.iconRef, 0)
				g.grayed = 1
		else:
			if (g.grayed == 1):
				karamba.removeImageTransformations(widget, g.iconRef)
				g.grayed = 0

	karamba.redrawWidget(widget)


#############################################
#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
	global leftendSize
	global middleSize
	global iconOffset
	global iconSize
	global taskGroupList

	for i in range(len(taskGroupList)):
		if (x >= leftendSize[0] + i*(middleSize[0]) + iconOffset[0]) and \
		   (x <= leftendSize[0] + i*(middleSize[0]) + iconOffset[0] + iconSize) and \
		   (y >= iconOffset[1]) and (y <= iconOffset[1] + iconSize):
			if (button == 1):
				taskGroupList[i].clickPos += 1
				if (taskGroupList[i].clickPos == len(taskGroupList[i].taskList)):
					taskGroupList[i].clickPos = 0
				karamba.performTaskAction(widget, taskGroupList[i].taskList[taskGroupList[i].clickPos].id, 5)
#				if(taskGroupList[i].grayed == 1):
#					karamba.removeImageTransformations(widget, taskGroupList[i].iconRef)
#					taskGroupList[i].grayed = 0
#					karamba.redrawWidget(widget)
			elif (button == 2):
				karamba.popupMenu(widget, taskGroupList[i].menu, x, y)
#				for t in taskGroupList[i].taskList:
#					karamba.performTaskAction(widget, t.id, 3)
#				if (taskGroupList[i].grayed == 0):
#					karamba.changeImageToGray(widget, taskGroupList[i].iconRef, 0)
#					taskGroupList[i].grayed = 1
#					karamba.redrawWidget(widget)
			break







#############################################
#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass



def menuItemClicked(widget, menu, id):
	global taskGroupList

	# figure out which task group the menu belongs to...
	for g in taskGroupList:
		if (menu == g.menu):
			# was it the "minimize all" item?
			if (id == g.minimizeID):
				for t in g.taskList:
					karamba.performTaskAction(widget, t.id, 3)
			else:
				# figure out which task item it was
				for t in g.taskList:
					if (id == t.menuID):
						karamba.performTaskAction(widget, t.id, 5)
						break
			break





#############################################
# This will be printed when the widget loads.
print "Loaded gTaskBar python extension!"
