# KarDevices 1.2
# Written by Tian ( http://www.c-sait.net )

import karamba
import os
import glob
import ConfigParser 
import re
import time
import shutil
import urllib
from stat import *
from lang import *

#Let you select the default device
currentDevice = 0

shortcutsDir = "shortcuts/"

class Device:
    def __init__(self, widget, kdesktop, config):
        self.kdesktop = kdesktop
        self.myInit(widget, config)
        self.executeTime = time.time()

    def myInit(self, widget, config):
        self.icon = config.get("Desktop Entry", "UnmountIcon")
        self.mountIcon = config.get("Desktop Entry", "Icon")
        self.mountPoint = config.get("Desktop Entry", "MountPoint")
        try:
            self.name = config.get("Desktop Entry", "Name")
        except ConfigParser.NoOptionError:
            self.name = re.search("/([^/]*)(\.desktop)?$", self.kdesktop).group(1)
        self.image = karamba.createImage(widget, 18, 8, self.icon)
        karamba.resizeImage(widget, self.image, 64, 64)
        karamba.attachClickArea(widget,self.image)
        karamba.hideImage(widget, self.image)
        self.mountImage = karamba.createImage(widget, 18, 8, self.mountIcon)
        karamba.resizeImage(widget, self.mountImage, 64, 64)
        karamba.attachClickArea(widget,self.mountImage)
        karamba.hideImage(widget, self.mountImage)
        self.text = karamba.createRichText(widget, "<center><font size=\"4\" color=\"#fcfcfc\"><strong>" + self.name + "</strong></font></center>")
        karamba.moveRichText(widget, self.text, 0, 72)
        karamba.setRichTextWidth(widget, self.text, 100)
        karamba.hideRichText(widget, self.text)
        self.mounted = self.isMounted()

    def destroy(self, widget):
        karamba.deleteImage(widget, self.image)
        karamba.deleteImage(widget, self.mountImage)
        karamba.deleteRichText(widget, self.text)

    def setDesktopFile(self, kdesktop):
        self.kdesktop = kdesktop

    def reload(self, widget):
        self.destroy(widget)
        config = ConfigParser.ConfigParser()
        config.read(self.kdesktop)
        self.myInit(widget, config)
        self.show(widget)

    def isMounted(self):
        #if (os.system("mount | gawk '{print $3}' | grep '^" + self.mountPoint + "$' > /dev/null") == 0):
        if (os.path.ismount(self.mountPoint)):
            return 1
        else:
            return 0

    def getMeter(self):
        if (self.mounted):
            return self.mountImage
        else:
            return self.image

    def getIcon(self):
        return self.icon

    def getText(self):
        return self.text

    def getDesktop(self):
        return self.kdesktop

    def getMountPoint(self):
        return self.mountPoint

    def show(self, widget):
        if (self.mounted):
            karamba.showImage(widget, self.mountImage)
        else:
            karamba.showImage(widget, self.image)
        karamba.showRichText(widget, self.text)
    
    def hide(self, widget):
        if (self.mounted):
            karamba.hideImage(widget, self.mountImage)
        else:
            karamba.hideImage(widget, self.image)
        karamba.hideRichText(widget, self.text)

    def highlight(self, widget, time):
        karamba.changeImageIntensity(widget, self.image, 0.5, time)

    def unhighlight(self, widget):
        karamba.removeImageEffects(widget, self.image)

    def umount(self,widget):
        result = os.system("umount " + self.mountPoint)
        if (result == 0):
            self.hide(widget)
            self.mounted = 0
            self.show(widget)
            return 1
        else:
            return 0

    def mount(self,widget):
        result = os.system("mount " + self.mountPoint)
        if (result == 0):
            self.hide(widget)
            self.mounted = 1
            self.show(widget)
            return 1
        else:
            return 0

    def execute(self,widget):
        if ((time.time() - self.executeTime) > 1):
            karamba.execute("kfmclient exec '" + self.kdesktop + "'")
            time.sleep(1)
            if (self.isMounted()):
                self.hide(widget)
                self.mounted = 1
                self.show(widget)
        self.executeTime = time.time()

    def refresh(self, widget):
        if (self.mounted):
            if (self.isMounted() == 0):
                self.hide(widget)
                self.mounted = 0
                self.show(widget)
        else:
            if (self.isMounted()):
                self.hide(widget)
                self.mounted = 1
                self.show(widget)


myPopupMenu = 0
propertiesItem = 0
deleteItem = 0
unmountItem = 0
mountItem = 0
devicesList = []
maxDevice = 0
scrollButton1 = 0
scrollButton2 = 0
popupButton = 0

def updatePopupMenu(widget):
    global devicesList,currentDevice,unmountItem,mountItem, myPopupMenu
    if (devicesList[currentDevice].isMounted()):
        if (unmountItem == 0):
            unmountItem = karamba.addMenuItem(widget, myPopupMenu, lang["umountItem"], karamba.getThemePath(widget) + "images/umount.png")
            karamba.removeMenuItem(widget, myPopupMenu, mountItem)
            mountItem = 0
    else:
        if (mountItem == 0):
            mountItem = karamba.addMenuItem(widget, myPopupMenu, lang["mountItem"], karamba.getThemePath(widget) + "images/mount.png")
            karamba.removeMenuItem(widget, myPopupMenu, unmountItem)
            unmountItem = 0

def reloadPreferences(widget):
    global scrollButton1, scrollButton2, popupButton, lang
    withScrollButton = karamba.readMenuConfigOption(widget, "scrollButton")
    withPopupButton = karamba.readMenuConfigOption(widget, "popupButton")
    if (withScrollButton):
        if (scrollButton1 == 0):
            scrollButton1 = karamba.createImage(widget, 80, 8, "images/up.png")
            karamba.attachClickArea(widget,scrollButton1)
            scrollButton2 = karamba.createImage(widget, 80, 80, "images/down.png")
            karamba.attachClickArea(widget,scrollButton2)
    else:
        if (scrollButton1 != 0):
            karamba.deleteImage(widget, scrollButton1)
            karamba.deleteImage(widget, scrollButton2)
            scrollButton1 = 0
            scrollButton2 = 0

    if (withPopupButton):
        if (popupButton == 0):
            popupButton = karamba.createImage(widget, 6, 80, "images/popup.png")
            karamba.addImageTooltip(widget, popupButton, lang["menuTooltip"])
            karamba.attachClickArea(widget,popupButton)
    else:
        if (popupButton != 0):
            karamba.deleteImage(widget, popupButton)
            popupButton = 0
    karamba.redrawWidget(widget)


def scrollUp(widget):
    global devicesList, maxDevice, currentDevice
    if (maxDevice > 0):
        devicesList[currentDevice].hide(widget)
        currentDevice = currentDevice + 1
        if (currentDevice >= maxDevice):
            currentDevice = 0
        devicesList[currentDevice].show(widget)

def scrollDown(widget):
    global devicesList, maxDevice, currentDevice
    if (maxDevice > 0):
        devicesList[currentDevice].hide(widget)
        currentDevice = currentDevice - 1
        if (currentDevice < 0):
            currentDevice = maxDevice - 1
        devicesList[currentDevice].show(widget)

#this is called when your widget is initialized
def initWidget(widget):
    global devicesList, maxDevice, shortcutsDir, myPopupMenu, propertiesItem, deleteItem, unmountItem, withScrollButton, withPopupButton

    shortcutsDir = karamba.getThemePath(widget) + shortcutsDir
    desktopList = glob.glob(shortcutsDir + "*")

    files = map( lambda f: (os.stat(f)[ST_MTIME], f ), desktopList)
    files.sort()

    for fileName in desktopList:
        config = ConfigParser.ConfigParser()
        config.read(fileName)
        if (config.get("Desktop Entry", "Type") == "FSDevice"):
            devicesList.append(Device(widget, fileName, config))
            maxDevice = maxDevice + 1

    if (currentDevice < maxDevice):
        devicesList[currentDevice].show(widget)
    karamba.acceptDrops(widget)
    karamba.redrawWidget(widget)

    karamba.addMenuConfigOption(widget, "scrollButton", lang["scrollOption"] )
    karamba.addMenuConfigOption(widget, "popupButton", lang["popupOption"])

    reloadPreferences(widget)

    myPopupMenu = karamba.createMenu(widget)
    propertiesItem = karamba.addMenuItem(widget, myPopupMenu, lang["propItem"], karamba.getThemePath(widget) + "images/properties.png")
    deleteItem = karamba.addMenuItem(widget, myPopupMenu, lang["delItem"], karamba.getThemePath(widget) + "images/remove.png")
    if (currentDevice < maxDevice):
        updatePopupMenu(widget)

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
    global currentDevice, unmountItem, devicesList, myPopupMenu
    if (maxDevice > 0):
        devicesList[currentDevice].refresh(widget)
        updatePopupMenu(widget)
        karamba.redrawWidget(widget)

#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
    global devicesList, maxDevice, currentDevice, myPopupMenu, unmountItem
    if (button == 4):
        scrollUp(widget)
    elif ((button == 5) and (maxDevice > 0)):
        scrollDown(widget)
    elif (button == 2):
        karamba.popupMenu(widget, myPopupMenu, 0, 0)
    if ((button == 5) or (button == 4)):
        updatePopupMenu(widget)
        karamba.redrawWidget(widget)

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    pass

#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
    global devicesList, maxDevice, currentDevice, deleteItem, propertiesItem, unmountItem, myPopupMenu
    if (id == propertiesItem):
        result = os.system("kfmclient openProperties '" + devicesList[currentDevice].getDesktop() + "'")
        desktopList = glob.glob(shortcutsDir + "*")
        files = map( lambda f: (os.stat(f)[ST_MTIME], f ), desktopList)
        files.sort()
        files.reverse()
        if ((time.time() - files[0][0]) < 1):
            devicesList[currentDevice].setDesktopFile(files[0][1])
            devicesList[currentDevice].reload(widget)
    elif (id == deleteItem):
        devicesList[currentDevice].destroy(widget)
        os.remove(devicesList[currentDevice].getDesktop())
        devicesList.remove(devicesList[currentDevice])
        maxDevice = maxDevice - 1
        if (currentDevice == maxDevice):
            currentDevice = 0
        else:
            currentDevice = currentDevice - 1
        devicesList[currentDevice].show(widget)
    elif (id == unmountItem):
        if (devicesList[currentDevice].umount(widget)):
            updatePopupMenu(widget)
        else:
            karamba.execute("kdialog --error '" + lang["umountError"] + "'")
    elif (id == mountItem):
        if (devicesList[currentDevice].mount(widget)):
            updatePopupMenu(widget)
        else:
            karamba.execute("kdialog --error '" + lang["mountError"] + "'")
    karamba.redrawWidget(widget)
       

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
    reloadPreferences(widget)

#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
    global currentDevice, unmountItem, devicesList, myPopupMenu
    if (button == 1):
        if (meter == scrollButton1):
            scrollUp(widget)
        elif (meter == scrollButton2):
            scrollDown(widget)
        elif (meter == popupButton):
            karamba.popupMenu(widget, myPopupMenu, 6, 80)
        elif (meter == devicesList[currentDevice].getMeter()):
            devicesList[currentDevice].highlight(widget,2000)
            karamba.redrawWidget(widget)
            devicesList[currentDevice].execute(widget)
            updatePopupMenu(widget)

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
    pass

#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
def itemDropped(widget, dropText):
    global maxDevice, devicesList, currentDevice
    files = re.split("\n",dropText)
    for singleFile in files:
        fileName = urllib.unquote(singleFile[5:]).strip()
        config = ConfigParser.ConfigParser()
        try:
            config.read(fileName)
            if (config.get("Desktop Entry", "Type") == "FSDevice"):
                shutil.copy(fileName, shortcutsDir)
                devicesList.append(Device(widget, shortcutsDir + os.path.basename(fileName), config))
                maxDevice = maxDevice + 1
                devicesList[currentDevice].hide(widget)
                currentDevice = maxDevice - 1
                devicesList[currentDevice].show(widget)
                updatePopupMenu
                karamba.redrawWidget(widget)
        except:
            print lang["badDesktop"]

#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.  
def startupAdded(widget, startup):
    pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.  
def startupRemoved(widget, startup):
    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.  
def taskRemoved(widget, task):
    pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).  
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    pass

# This will be printed when the widget loads.
print lang["loadSuccess"]

