#!/bin/bash
# Installation script for the Borealis superkaramba theme ver.1.0
# Copyright 2004 - Ivica Ico Bukvic
# http://meowing.ccm.uc.edu/~ico/
# This install script is made available under GPL license
# For more info regarding this license, please see http://www.gnu.org/copyleft/gpl.html

name='Borealis superkaramba theme v.1.5'
title='by Ivica Ico Bukvic
ico@fuse.net
http://meowing.ccm.uc.edu/~ico/
'
directory="/home/$user/Karamba/Borealis"
config="/home/$USER/.kde/share/config"
homepage='http://meowing.ccm.uc.edu/~ico/'
kde_page='http://kde-look.org/content/show.php?content=13876'
nsteps='of 10:'

#==============================================================

echo "The $name $title"

# Check if we are in the proper directory
pwd=`pwd`
if [ "$pwd" != "$HOME/Karamba/Borealis" ]; then
  echo "This script needs to be run from the install folder. Exiting..."
  exit 1
fi

# Check if Borealis was already installed
if [ -w $HOME/.kde/Autostart/Borealis.sh ]; then
  upgd=1
  echo -e "Old install of the Borealis theme found. Upgrading...\n"
else
  upgd=0
fi

# Step 1
echo "Step 1 $nsteps Installing Autostart and .rc scripts..."
cp -f Autostart/Borealis.sh $HOME/.kde/Autostart/
if [ -d $HOME/.superkaramba ]; then
  echo "...Copying .rc scripts"
else
  echo "...Creating $HOME/.superkaramba directory"
  mkdir $HOME/.superkaramba
  echo "......OK"
fi
cp -f misc/*.rc $HOME/.superkaramba/
echo -e "Step 1 $nsteps completed.\n"

# Step 2
echo "Step 2 $nsteps Determining presence of the 3ddeskd..."
test=`which 3ddesktop`
if [ -z "$test" ]; then
  use3d=0
  echo "...'3ddesktop' executable not found. Falling back to the KDE implementation"

else
  echo "...3ddeskd found."
  echo "Would you like to use 3ddeskd or the default KDE implementation for virtual desktop switching? Press 'y' for 3ddeskd or any other key for KDE..."
  read text
  case $text in
    [y] | [Y] )
      use3d=1
      echo "...Configuring 3ddeskd implementation"
    ;;
    * )
      echo "...Configuring KDE implementation"
      use3d=0
    ;;
  esac  
fi

if [ $use3d == 0 ]; then
  echo "...Fixing the startup script"
  test=`grep "#3d" Startup.sh`
  if [ -n "$test" ]; then
    echo "...Startup.sh script is already patched"
  else
    cat Startup.sh | sed -e 's/3d/#3d/' | sed -e 's/sleep/#sleep/' > tmp
    mv -f tmp Startup.sh
    chmod 755 Startup.sh
  fi
  echo "......OK."
  
  echo "...Generating the desks.theme script from template"
  cat misc/desks.theme.template | sed -e 's/#kde //g' > tmp
  mv -f tmp desks.theme
  echo "......OK"
  
else
  echo "...Fixing the startup script"
  test=`grep "#3d" Startup.sh`
  if [ -z "$test" ]; then
    echo "......Startup.sh script is already patched"
  else
    cat Startup.sh | sed -e 's/#3d/3d/' | sed -e 's/#sleep/sleep/' > tmp
    mv -f tmp Startup.sh
    chmod 755 Startup.sh
  fi
  echo "......OK"
  
  echo "...Generating the desks.theme script from template"
  cat misc/desks.theme.template | sed -e 's/#3d //g' > tmp
  mv -f tmp desks.theme
  echo "......OK" 
fi
echo -e "Step 2 $nsteps completed.\n"

# Step 3
if [ $use3d == 1 ] && [ $upgd == 0 ]; then
  echo "Step 3 $nsteps The 3ddeskd patch for startkde script. We need to modify the startkde script to forcefully unload the 3ddeskd OpenGL virtual desktops daemon in order to avoid stale daemon bug (apparently a bug in the 3ddesktop project) and the resulting inaccessibility of DRI upon subsequent logins. You may need to run a portion of this step as root (su) for it to work (the script will allow you to go 'su' at an appropriate time). The old file will be backed up prior to patching. Press 'y' to continue or any other key to skip this step..."
  read text
  case $text in
    [y] | [Y] )
      kde_script=`which startkde`
      test=`grep "killall 3ddesk" $kde_script`
      if [ -n "$test" ]; then
        echo "...$kde_script appears to be already patched. Skipping this step."
      else
        test=`grep "artsshell -q terminate" $kde_script`
        if [ -z "$test" ]; then
          echo -e "$kde_script has unrecognizable structure. Please patch it manually by adding the following line at the end of the section titled 'Clean up':\n\n   killall 3ddeskd\n\nThen save the file. Aborting automatic install of the $name."
        else
          echo -e "The following step will require a superuser access. You will be asked to enter your su password and then type the following command:\n\n   sh misc/patch_kde.sh\n\nAfter the patching is complete type 'exit' in order to finish the install process.\n\nEntering su mode..."
          su
        fi
      fi
    ;;
    * )
      echo "Aborting install of the 3ddesktop patch. You may experience weird behavior during the subsequent logins using the 3ddesktop. Please report any problems to the 3ddesktop project developers."
    ;;
  esac
  echo -e "Step 3 $nsteps complete.\n"
else
  if [ $use3d == 0 ]; then
    echo -e "Step 3 $nsteps Skipping due to lack of 3ddesktop application...\n"
  else
    echo -e "Step 3 $nsteps Skipping due to upgrade...\n"
  fi
fi

# Step 4
if [ $use3d == 1 ] && [ $upgd == 0 ]; then
  echo "Step 4 $nsteps Setting up the animation settings for the 3ddesktop. This will override any settings you may already have. Your old settings will be backed up. Press 'y' to continue or any other key to skip this step..."
  read text
  case $text in
    [y] | [Y] )
      if [ -d $HOME/.3ddesktop ]; then
        if [ -e $HOME/.3ddesktop/3ddesktop.conf ]; then
	  if [ $upgd == 0 ]; then
            echo "...Backing up old settings $HOME/.3ddesktop/3ddesktop.conf to $HOME/.3ddesktop/3ddesktop.conf.old"
            cp -f $HOME/.3ddesktop/3ddesktop.conf $HOME/.3ddesktop/3ddesktop.conf.old
            echo "......OK"
	  fi
        fi
      else
        echo "...Creating the user config directory $HOME/.3ddesktop"
        mkdir $HOME/.3ddesktop
        echo "......OK"
      fi
      echo "...Copying new settings"
      cp -f misc/3ddesktop.conf $HOME/.3ddesktop/
      echo "......OK"
    ;;
    * )
      echo "...Skipping this step"
    ;;
  esac
  echo -e "Step 4 $nsteps complete.\n"
else
  if [ $use3d == 0 ]; then
    echo -e "Step 4 $nsteps Skipping due to lack of 3ddesktop application...\n"
  else
    echo -e "Step 4 $nsteps Skipping due to upgrade...\n"
  fi
fi

# Step 5
echo "Step 5 $nsteps Checking for automatic resolution detection..."
test=`which xwininfo`
if [ -z "$test" ]; then
  echo "...'xwininfo' executable not found. Please enter the HORIZONTAL size of your desktop in pixels:"
  read text
  test=`grep "desktop = 1280" desks.py`
  if [ -z "$test" ]; then
    #stuf has been already altered so let's restore from template
    cp -f misc/cpu.py.template cpu.py
    cp -f misc/desks.py.template cpu.py
    cp -f misc/discs.py.template discs.py
    cp -f misc/memory.py.template memory.py
    cp -f misc/network.py.template network.py
    cp -f misc/systray.py.template systray.py
    cp -f misc/xmms.py.template xmms.py
  fi  
  echo -n "cat cpu.py | sed -e 's/desktop = 1280/desktop = ${text}/' > tmp | mv -f tmp cpu.py" > runme
  chmod +x runme
  ./runme
  
  echo -n "cat desks.py | sed -e 's/desktop = 1280/desktop = ${text}/' > tmp | mv -f tmp desks.py" > runme
  chmod +x runme
  ./runme
    
  echo -n "cat discs.py | sed -e 's/desktop = 1280/desktop = ${text}/' > tmp | mv -f tmp discs.py" > runme
  chmod +x runme
  ./runme
    
  echo -n "cat memory.py | sed -e 's/desktop = 1280/desktop = ${text}/' > tmp | mv -f tmp memory.py" > runme
  chmod +x runme
  ./runme
    
  echo -n "cat network.py | sed -e 's/desktop = 1280/desktop = ${text}/' > tmp | mv -f tmp network.py" > runme
  chmod +x runme
  ./runme
   
  echo -n "cat systray.py | sed -e 's/desktop = 1280/desktop = ${text}/' > tmp | mv -f tmp systray.py" > runme
  chmod +x runme
  ./runme
  rm -f runme

  echo -n "cat xmms.py | sed -e 's/desktop = 1280/desktop = ${text}/' > tmp | mv -f tmp xmms.py" > runme
  chmod +x runme
  ./runme
  rm -f runme
else
  echo "...'xwininfo' found. The script will automatically reposition the theme upon startup."
fi
echo "...OK"
echo -e "Step 5 $nsteps completed.\n"

#Step 6
echo "Step 6 $nsteps Configuring KDE session to block storing superkaramba and 3ddesktop (if installed) instances as a part of the session. You can later manually edit this by opening KControl Center -> Components -> Session Manager -> Applications to be excluded from sessions."
test=`grep excludeApps=superkaramba ${HOME}/.kde/share/config/ksmserverrc`
if [ -n "$test" ]; then
  echo "...The ${HOME}/.kde/share/config/ksmserverrc appears to be already patched. Skipping this step."
else
  echo "...Backing up the ${HOME}/.kde/share/config/ksmserverrc to ${HOME}/.kde/share/config/ksmserverrc.old"
  cp -f ${HOME}/.kde/share/config/ksmserverrc ${HOME}/.kde/share/config/ksmserverrc.old
  echo "......OK"
  echo "...Configuring applications to be blocked"
  test=`grep excludeApps= ${HOME}/.kde/share/config/ksmserverrc`
  if [ $use3d == 1 ]; then
    if [ -z "$test" ]; then
      echo -n "cat ${HOME}/.kde/share/config/ksmserverrc | sed -e 's/General]/General]\nexcludeApps=superkaramba:3ddeskd:3ddesktop:3ddesk:/' > tmp | mv -f tmp ${HOME}/.kde/share/config/ksmserverrc" > runme
    else
      echo -n "cat ${HOME}/.kde/share/config/ksmserverrc | sed -e 's/excludeApps=/excludeApps=superkaramba:3ddeskd:3ddesktop:3ddesk:/' > tmp | mv -f tmp ${HOME}/.kde/share/config/ksmserverrc" > runme
    fi
  else
    if [ -z "$test" ]; then
      echo -n "cat ${HOME}/.kde/share/config/ksmserverrc | sed -e 's/General]/General]\nexcludeApps=superkaramba:/' > tmp | mv -f tmp ${HOME}/.kde/share/config/ksmserverrc" > runme 
    else
    echo -n "cat ${HOME}/.kde/share/config/ksmserverrc | sed -e 's/excludeApps=/excludeApps=superkaramba:/' > tmp | mv -f tmp ${HOME}/.kde/share/config/ksmserverrc" > runme
    fi
  fi
  chmod +x runme
  ./runme
  rm -f runme
  echo "......OK"
fi  
echo -e "Step 6 $nsteps completed.\n" 

# Step 7
echo -e "Step 7 $nsteps Configuring customizable sensors. You will now have the oportunity to configure disc and network monitoring tools. Here are the default settings:\n\ndiscs.theme: disc1:/ disc2:/home\nnetwork.theme: eth0\n\nIf you would like to edit these please press 'y'. Otherwise press any other key to use defaults..."
read text
case $text in
  [y] | [Y] )
    echo "...Configuring discs.theme"
    echo "......Restoring default config from template"
    cp misc/discs.theme.template discs.theme
    cp misc/network.theme.template network.theme
    echo ".........OK"
    
    echo "Please enter the mountpoint for the first drive (default is /):"
    read text
    echo $text > tmp
    sed -e 's/\//\\\//g' tmp > tmp2
    output=`cat tmp2`
    echo -n "cat discs.theme | sed -e 's/<disc1>/${output}/g' > tmp | mv -f tmp discs.theme" > runme
    chmod +x runme
    ./runme
    echo ".........OK"

    echo "Please enter the mountpoint for the first drive (default is /home):"
    read text
    echo $text > tmp
    sed -e 's/\//\\\//g' tmp > tmp2
    output=`cat tmp2`
    echo -n "cat discs.theme | sed -e 's/<disc2>/${output}/g' > tmp | mv -f tmp discs.theme" > runme
    chmod +x runme
    ./runme
    echo ".........OK"    

    echo "Please enter the network device to monitor (default is eth0):"
    read text
    echo $text > tmp
    sed -e 's/\//\\\//g' tmp > tmp2
    output=`cat tmp2`
    echo -n "cat network.theme | sed -e 's/<net1>/${output}/g' > tmp | mv -f tmp network.theme" > runme
    chmod +x runme
    ./runme
    rm -f runme tmp tmp2
    echo ".........OK"
  ;;
  * )
    echo "...Using default settings"
    cp misc/discs.theme.template discs.theme
    cp misc/network.theme.template network.theme
    cat discs.theme | sed -e 's/<disc1>/\//g' | sed -e 's/<disc2>/\/home/g' > tmp
    mv -f tmp discs.theme    
    cat network.theme | sed -e 's/<net1>/eth0/' > tmp
    mv -f tmp network.theme
    echo "......OK"        
  ;;
esac
echo -e "Step 7 $nsteps completed.\n"

# Step 8
echo "Step 8 $nsteps Determining whether xmmsctrl is installed which is needed for the xmms bar..."
test=`which xmmsctrl`
if [ -z "$test" ]; then
  noXmms=1
  echo -e "...'xmmsctrl' not found.\nSince Xmms bar depends upon this program, it will be disabled by default. You can obtain this program from the following website:\n\n   http://user.it.uu.se/~adavid/utils/\n\nYou may also find a prepackaged version of this program for your distribution."
else
 echo "...xmmsctrl found."
  echo "Press 'y' to enable Xmms bar or any other key to disable it..."
  read text
  case $text in
    [y] | [Y] )
      noXmms=0
      echo "...Configuring Xmms bar"
    ;;
    * )
      noXmms=1
      echo "...Disabling Xmms bar"
    ;;
  esac
fi
if [ $noXmms == 1 ]; then
  test=`grep '#' main.theme`
  if [ -n "$test" ]; then
    echo "......main.theme is already patched. Skipping this step."
  else
    cat main.theme | sed -e 's/theme path=xmms.theme/#theme path=xmms.theme/' > tmp
    mv -f tmp main.theme
    echo "......OK"
  fi
else
  test=`grep '#' main.theme`
  if [ -n "$test" ]; then
    cat main.theme | sed -e 's/#theme path=xmms.theme/theme path=xmms.theme/' > tmp
    mv -f tmp main.theme
  fi
  echo "......OK"    
fi
echo -e "Step 8 $nsteps completed.\n"

# Step 9
echo "Step 9 $nsteps Configuring cpu speed sensor..."
echo "...Restoring default config from template"
cp misc/cpu.theme.template cpu.theme
echo "......OK"
if [ -f /sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq ]; then
  echo "...Using default settings."
else
  if [ -f /proc/cpuinfo ]; then
    echo "...Using alternative #1"
    cat cpu.theme | sed -e 's/#text x=112 y=18/text x=112 y=18/' | sed -e 's/text x=90 y=18 v/#text x=90 y=18 v/' > tmp
    mv -f tmp cpu.theme    
  elif [ -f /proc/cpufreq ]; then
    echo "...Using most alternative #2"
    cat cpu.theme | sed -e 's/#text x=90 y=18 sensor=program program=\"cat \/proc\/cpuf/text x=90 y=18 sensor=program program=\"cat \/proc\/cpuf/' > tmp
    mv -f tmp cpu.theme  
  else
    echo "...WARNING! No compatible cpu speed sensors found. Please make sure that your file tree is not customized and/or that your kernel has been compiled with proper settings enabled. Disabling cpu speed sensor."
    cat cpu.theme | sed -e 's/\ntext x=90 y=18/\n#text x=90 y=18/' > tmp
    mv -f tmp cpu.theme    
  fi
  cat cpu.theme | sed -e 's/text x=90 y=18 sensor=program program=\"cat \/sys/#text x=90 y=18 sensor=program program=\"cat \/sys/' > tmp
  mv -f tmp cpu.theme  
fi
echo -e "Step 9 $nsteps completed.\n"

# Step 10
echo "Step 10 $nsteps Starting the theme. Press 'y' to start the theme now, or any other key to have the theme start on your next login..."
read text
case $text in
  [y] | [Y] )
    sh Startup.sh &
    echo "...Done. When exiting the shell, please use 'exit' command, or otherwise the process started in the shell may end up being killed together with it."
  ;;
esac
echo -e "Step 10 $nsteps completed.\n"
echo -e "\n\n\n\nCongratulations! Install complete!\n\nPlease do not forget to vote.\nYour comments and suggestions are highly appreciated!\n\nFor latest news and updates please visit\n$kde_page\n\nHave a nice day!\n"

exit 0
