# Script by Ivica Ico Bukvic (SlipStreamScapes)
# http://meowing.ccm.uc.edu/~ico/
# Distributed under the GPL license
# Version 0.1 (June 2004)
# Based on Dynsysbar by Tomas Galvez (THoMaSiN)
# (l) 2004

import karamba
import time
import os

# width & height of each Icon.

wperIcon = 25
inc = 0

# width of the desktop, width of the opened menu, and padding of the retracted menu
desktop = 1280
width = 144
padding = 36

##### Names of the popMenu Options.

menuNames = ( "Add space", "Subtract space" )
menuIcons = ( "img/add_col.png", "img/rem_col.png")

######################################

head = None
middle = None
myTray = None

menu = None
menuItems = []

width = 0
height = 32
cols = 4
vector = 0
retracted = 0
this = "systray"

def writeConfigEntryNow(widget, string, replace):
	filename = os.path.expanduser("~") + "/.superkaramba/" + this + ".rc"
	file = open( filename, "r")
	output = file.read()
	file.close()
	output = output.splitlines()
	
	i = 0
	desktop = ""
	for i in output:
		param = i.split()
		if param and param[0].find(string) != -1:
			desktop = desktop + string + "=" + replace + "\n"
		else:
			desktop = desktop + i +"\n"
	
	file = open( filename, "w")
	file.write(desktop)
	file.close

###############################################################################
## Function:    
##   detectResolution ( )
## 
## 
## Description: 
##   Runs xwininfo and sets the current XFree86 resolution to the golbal
##   variables resolutionX and resolutionY
## 
##  Thanks to Adam Geitgey for this code!
###############################################################################
def detectResolution():
	global desktop
	
	havexwininfo = os.system("which xwininfo >/dev/null")
	if (havexwininfo == 0):
		pass
	
	fp = os.popen("xwininfo -root -stats")
	output = fp.read()
	output = output.splitlines()
	
	i = 0
	for i in output:
		param = i.split()
		if (len(param) > 1):
			if param[0].find("Width:") != -1:
				desktop = int(param[1])

##########################################################
# Superkaramba API functions                             #
##########################################################
#this is called when your widget is initialized
def initWidget(widget):
	global myTray
	global head
	global middle
	global menu
	global menuItems
	global width
	global height
	global cols
	global inc
	global desktop
	global padding
	global retracted
	
	detectResolution()

	if karamba.readConfigEntry (widget, "cols") == None:
		cols = 4
		karamba.writeConfigEntry (widget, "cols", str(cols))
		retracted = 0
		karamba.writeConfigEntry (widget, "retracted", str(retracted))
	else:
		cols = karamba.readConfigEntry (widget, "cols")
		retracted = karamba.readConfigEntry (widget, "retracted")
		
	width = padding + wperIcon * (cols+1)

	position = 0

	head = karamba.createImage (widget, position, 0, "img/left.png")

	position = position + 16

	myTray = karamba.createSystray(widget, padding, 2, cols * (wperIcon + inc), height -2)
	
	middle = karamba.createImage (widget, position, 0, "img/middle_new.png")
	karamba.resizeImage (widget, middle, (cols+1) * wperIcon, 32)

	menu = karamba.createMenu ( widget )

	for i in range(len (menuNames)):
		menuItems.append (karamba.addMenuItem (widget, menu, menuNames[i], karamba.getThemePath(widget) + menuIcons[i]))

	(x, y) = karamba.getWidgetPosition(widget)
	y = karamba.readConfigEntry (widget, "widgetPosY")
	if ( retracted == 1 ):
		karamba.hideSystray(widget)
		if ( desktop - x != padding ):
			karamba.moveWidget(widget, desktop - padding, y)
	elif ( desktop - x != width ):
		karamba.moveWidget(widget, desktop - width + 17, y)

	#karamba.redrawWidget(widget)
	karamba.redrawWidgetBackground(widget)
	
#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
	global retracted

	if ( retracted != 1 ):
		pass

#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
	global menuItems
	global rows
	global cols
	global vector

	if id == menuItems[0]:
		cols = cols + 1
		vector = 1
	elif id == menuItems[1]:
		cols = cols - 1
		vector = -1
	else:
		print "error"
		return 0

	writeConfigEntryNow (widget, "cols", str(cols))

	resizeAll (widget)


#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
	global menu
	global desktop
	global padding
	global wperIcon
	global cols
	global height
	global retracted

	if (button == 2):
		karamba.popupMenu(widget, menu, x, y)
	
	if (button == 1):
		(x, y) = karamba.getWidgetPosition(widget)
		if (x >= desktop - padding ):
			karamba.resizeWidget( widget, width, height )
			karamba.showSystray(widget)
			karamba.redrawWidget (widget)
			time.sleep(0.1)
			for i in range( (wperIcon * (cols+1)) - 17 ):
				x = x - 1
				karamba.moveWidget (widget, x, y)
			retracted = 0
		else:
			for i in range( desktop - x - padding ):
				x = x + 1
				karamba.moveWidget (widget, x, y)
			time.sleep(0.1)
			karamba.hideSystray(widget)
			karamba.resizeWidget( widget, padding, height )
			retracted = 1
		writeConfigEntryNow(widget, "widgetWidth", str(width))
		writeConfigEntryNow(widget, "widgetPosX", str(x))
		writeConfigEntryNow(widget, "widgetPosY", str(y))
		writeConfigEntryNow(widget, "retracted", str(retracted))
		karamba.redrawWidget(widget)
		karamba.redrawWidgetBackground(widget)

def resizeAll (widget):
	global myTray
	global middle
	global head
	global middle
	global cols
	global inc

	center (widget)

	karamba.resizeImage (widget, middle, (int) ((cols+1)*wperIcon), 32 )

	karamba.moveSystray ( widget, padding, 1, (int) ((wperIcon + inc) * cols), height - 2)

	karamba.redrawWidget (widget)

def center (widget):
	global height
	global width
	global vector

	(x_pos, y_pos) = karamba.getWidgetPosition(widget)
	x_pos = x_pos + (int) (width / 2)
	y_pos = y_pos + (int) (height / 2)

	newWidth = padding + wperIcon * (cols+1)

	karamba.resizeWidget(widget, newWidth, height)
	karamba.moveWidget (widget, x_pos - (int) (newWidth / 2) - (int) (12 * vector), y_pos - (int) (height / 2))

	width = newWidth

def widgetClosed( widget ):
	global head
	global middle

	karamba.deleteImage (widget, head)
	karamba.deleteImage (widget, middle)
