#this import statement allows access to the karamba functions
import karamba

import os
import string
import glob
import time

# configuration settings all in one place
from Conf import *

# pack format for:
# 	buttons:
# 		icon, group, command, running, tasks (in a list), friendly name
# 	icons:
# 		icon -> next, arrow, icon0, icon1, ... (in order of size)
# 	possible:
# 		group -> (same as buttons)
# 	minimised:
# 		titlebar (also used for icon key), task, running, friendly name

init = 0
icons = { }
numPermanent = 0
currentNumTasks = 0
buttons = [ ]
taskList = [ ]
minimised = [ ]
possible = { }
currentZoomed = "##"
currentTask = -1

bar = [ ]

basePath = ""
iconSizes = [ "./", "128x128/", "64x64/", "48x48/", "32x32/", "22x22/", "16x16/" ]
iconTypes = [ "./", "actions/", "apps/", "devices/", "filesystems/", "mimetypes/" ]

#this is called when you widget is initialized
def initWidget(widget):
	global basePath, themePaths, useMyIcons, configurationFile
	global useSmoothZoom, zoomFrames
	global icons
	global bar
	global buttons, numPermanent
	global possible

	basePath = karamba.getThemePath(widget) + "/"
	if useMyIcons:
		themePaths.insert(0, basePath+iconPath)
	else:
		themePaths.append(basePath+iconPath)

	if not useSmoothZoom:
		zoomFrames = 2

	# background bar
	bar.append(karamba.createImage(widget, 0, 23, basePath+iconPath+"head-left.png"))
	bar.append(karamba.createImage(widget, 32, 23, basePath+iconPath+"middle.png"))
	bar.append(karamba.createImage(widget, 0, 0, basePath+iconPath+"divider.png"))
	bar.append(karamba.createImage(widget, 0, 0, basePath+iconPath+"middle.png"))
	bar.append(karamba.createImage(widget, 0, 0, basePath+iconPath+"head-right.png"))
	karamba.resizeImage(widget, bar[0], 32, 70)
	karamba.resizeImage(widget, bar[2], 32, 70)
	karamba.hideImage(widget, bar[2])
	karamba.hideImage(widget, bar[3])
	karamba.resizeImage(widget, bar[4], 32, 70)

	# load in application links and mappings from configuration file
	conf = file(basePath+configurationFile, "r")

	# get buttons
	line = conf.readline()
	while string.find(line, "##") == -1:
		explode = string.split(line, ",", 4)
		if explode[2][-1] == "\n":
			explode[2] = explode[2][:-1]
		buttons.append( [ explode[1], explode[0], explode[2], 0, [ ], explode[3] ] )
		icons[explode[1]] = createIcon(widget, explode[1], explode[3])
		line = conf.readline()

	numPermanent = len(buttons)

	# and possibles
	line = conf.readline()
	while string.find(line, "###") == -1:
		explode = string.split(line, ",", 3)
		if explode[1][-1] == "\n":
			explode[1] = explode[1][:-1]
		possible[explode[0]] = [ explode[1], explode[0], "", 0, [ ], explode[2] ]
		line = conf.readline()

	# just some little things
	#icons[''] = createIcon(widget, "xapp")
	#icons['unknown'] = createIcon(widget, "unknown", "Unknown")

# little helper function
def createIcon(widget, name, caption):
	global basePath, iconPath, themePaths
	global iconSizes, iconTypes
	global useMyIcons
	global useSmoothZoom, zoomFrames
	global numPermanent

	found = 0
	ret = [ ]

	print "making icon " + name

	for theme in themePaths:
		for size in iconSizes:
			for thisType in iconTypes:
				if len(glob.glob(theme + size + thisType + name + ".png")) > 0:
					print "found icon " + theme + size + thisType + name + ".png" + " for " + name
					path = theme + size + thisType + name + ".png"
					found = 1
					break
			if found:
				break
		if found:
			break

	if found:
		ret.append(karamba.createImage(widget, 0, 0, basePath+iconPath+"arrow.png"))
		ret.append(karamba.createText(widget, 0, 0, 1000, 20, caption))
		karamba.changeTextColor(widget, ret[1], 255, 255, 255)
		karamba.changeTextShadow(widget, ret[1], 1)
		karamba.changeTextSize(widget, ret[1], 9)
		karamba.changeTextFont(widget, ret[1], "Verdana")
		ret.append(caption)
		for i in range(zoomFrames):
			ret.append(karamba.createImage(widget, 0, 0, path))
	else:
		print "icon not found " + name
		ret = createIcon(widget, "xapp", caption)

	karamba.hideImage(widget, ret[0])
	karamba.resizeImage(widget, ret[0], 9, 5)
	karamba.hideText(widget, ret[1])
	for i in range(zoomFrames):
		adjustment = 16*i/(zoomFrames-1)
		karamba.hideImage(widget, ret[i+3])
		karamba.resizeImageSmooth(widget, ret[i+3], 48+adjustment, 48+adjustment)

	return ret

# another little helper
def destroyIcon(widget, key):
	global icons, zoomFrames
	print "destroying icon " + key
	karamba.deleteImage(widget, icons[key][0])
	karamba.deleteText(widget, icons[key][1])
	for i in range(zoomFrames):
		karamba.deleteImage(widget, icons[key][i+3])
	del icons[key]

# and another...
def moveIcon(widget, key, x, y):
	global icons, useSmoothZoom, zoomFrames
	karamba.moveImage(widget, icons[key][0], x+28, y+16+48)
	karamba.moveText(widget, icons[key][1], x+32-3*len(icons[key][2]), y-12)
	for i in range(zoomFrames):
		adjustment = 8 - 8*i/(zoomFrames-1)
		karamba.moveImage(widget, icons[key][i+3], x+adjustment, y+2*adjustment)
	karamba.showImage(widget, icons[key][3])

# they just keep coming!!
def zoomIcon(widget, key, smooth):
	global icons, currentZoomed
	global useSmoothZoom, zoomFrames

	#print "zoomIcon " + key

	if not smooth or not useSmoothZoom:
		if currentZoomed != key:
			if currentZoomed != "##" and currentZoomed in icons:
				karamba.hideImage(widget, icons[currentZoomed][zoomFrames+2])
				karamba.showImage(widget, icons[currentZoomed][3])
				karamba.hideText(widget, icons[currentZoomed][1])
			if key != "##" and icons.has_key(key):
				karamba.hideImage(widget, icons[key][3])
				karamba.showImage(widget, icons[key][zoomFrames+2])
				karamba.showText(widget, icons[key][1])
			currentZoomed = key

		# this is really only used by updateWidget
		elif key != "##" and icons.has_key(key):
			karamba.hideImage(widget, icons[key][3])
			karamba.showImage(widget, icons[key][zoomFrames+2])
			karamba.showText(widget, icons[key][1])

	else:
		if currentZoomed != key:
			if currentZoomed != "##" and currentZoomed in icons:
				karamba.hideText(widget, icons[currentZoomed][1])
			for frame in range(zoomFrames-1):
				if currentZoomed != "##" and currentZoomed in icons:
					karamba.hideImage(widget, icons[currentZoomed][zoomFrames-frame+2])
					karamba.showImage(widget, icons[currentZoomed][zoomFrames-frame+1])
				if key != "##" and icons.has_key(key):
					karamba.hideImage(widget, icons[key][frame+3])
					karamba.showImage(widget, icons[key][frame+4])
				karamba.redrawWidget(widget)
				time.sleep(0.05)
			if key != "##" and icons.has_key(key):
				karamba.showText(widget, icons[key][1])
			currentZoomed = key

		# this is really only used by updateWidget
		elif key != "##" and icons.has_key(key):
			karamba.hideImage(widget, icons[key][3])
			karamba.showImage(widget, icons[key][zoomFrames+2])
			karamba.showText(widget, icons[key][1])

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
	global icons
	global resX, resY
	global spaceL, spaceR
	global numPermanent, buttons, possible
	global taskList, minimised, useMinWin
	global bar
	global currentNumTasks

	# keep still... no explanation necessary :p
	keepStill = 1

	# clear running status for all
	for button in buttons:
		button[3] = 0
		button[4] = [ ]

	if useMinWin:
		#print "--------------------------------------------"
		#print len(minimised)
		for task in minimised:
			#print task[0]
			task[2] = 0
		#print "--------------------------------------------"

	# find the ones running
	taskList = karamba.getTaskList(widget)
	#print str(currentNumTasks) + ", " + str(len(taskList))
	if len(taskList) != currentNumTasks:
		currentNumTasks = len(taskList)
		keepStill = 0
	taskInfo = [ ]
	for task in taskList:
		taskInfo.append(karamba.getTaskInfo(widget, task))
		taskName = karamba.getTaskInfo(widget, task)[2]
		#print karamba.getTaskInfo(widget, task)

		# minimised
		if useMinWin:
			if karamba.getTaskInfo(widget, task)[5]:
				titleBar = karamba.getTaskInfo(widget, task)[0]

				# already in docker?
				foundMin = 0
				for minWin in minimised:
					if titleBar == minWin[0]:
						minWin[1] = task
						minWin[2] = 1
						foundMin = 1
						break

				# new minimised window
				if not foundMin:
					keepStill = 0
					print "new minimised window " + titleBar
					minimised.append( [ titleBar, task, 1 ] )
					# todo: screenshot icons
					if possible.has_key(taskName):
						icons[titleBar] = createIcon(widget, possible[taskName][0], titleBar)
					else:
						icons[titleBar] = createIcon(widget, taskName, titleBar)

		# button already in docker?
		found = 0
		for button in buttons:
			if taskName == button[1]:
				button[3] = 1
				button[4].append(task)
				found = 1
				break

		# new button needed
		if not found:
			# assign a new button
			keepStill = 0
			print "inserting button for " + taskName
			if possible.has_key(taskName):
				buttons.append(possible[taskName])
				if not icons.has_key(possible[taskName][0]):
					icons[possible[taskName][0]] = createIcon(widget, possible[taskName][0], possible[taskName][5])
			else:
				buttons.append( [taskName, taskName, "", 0, [ ] ] )
				if not icons.has_key(taskName):
					icons[taskName] = createIcon(widget, taskName, taskName)
			buttons[len(buttons)-1][3] = 1
			buttons[len(buttons)-1][4].append(task)
			pass

	# todo: remove non-running, non-permanent buttons
	# current hack: removes only the first button not there, but this works fine
	for index in range(len(buttons)-numPermanent):
		if not buttons[index+numPermanent][3]:
			print "removing button for " + buttons[index+numPermanent][0]
			destroyIcon(widget, buttons[index+numPermanent][0])
			del buttons[index+numPermanent]
			keepStill = 0
			break

	# and again with minimised windows, with same hack
	if useMinWin:
		for minWin in minimised:
			if minWin[2] == 0:
				print "window " + minWin[0] + " no longer minimised"
				destroyIcon(widget, minWin[0])
				minimised.remove(minWin)
				keepStill = 0
				break

	# clear all icons
	#for key in icons.keys():
		#hideIcon(widget, key)

	# set up chain of icons for buttons
	currentIconCount = 0
	for i in range(len(buttons)):
		currKey = buttons[i][0]
		if not icons.has_key(currKey):
			print "!!!Please contact maintainer: icon key not recognised!!! " + currKey
			currKey = "unknown"
		if not keepStill:
			moveIcon(widget, currKey, spaceL+i*64, 12)
			if buttons[i][3]:
				karamba.showImage(widget, icons[currKey][0])
			else:
				karamba.hideImage(widget, icons[currKey][0])

	currentIconCount = len(buttons)

	# and minimised windows
	if useMinWin:
		leftSpace = spaceL+currentIconCount*64+32
		for i in range(len(minimised)):
			currKey = minimised[i][0]
			if not keepStill:
				moveIcon(widget, currKey, leftSpace+i*64, 12)

	# and zoom the currently zoomed icon
	zoomIcon(widget, currentZoomed, 0)

	# position the widget and background images
	if not keepStill:
		buttonArea = 64*currentIconCount
		minArea = 64*len(minimised)
		if useMinWin == 0 or len(minimised) <= 0:
			karamba.hideImage(widget, bar[2])
			karamba.hideImage(widget, bar[3])
			widgetWidth = spaceL + buttonArea + spaceR
			karamba.resizeImage(widget, bar[1], widgetWidth - 64, 70)
		else:
			widgetWidth = spaceL + buttonArea + 32 + minArea + spaceR
			karamba.resizeImage(widget, bar[1], spaceL + buttonArea - 32, 70)
			karamba.moveImage(widget, bar[2], spaceL + buttonArea, 23)
			karamba.moveImage(widget, bar[3], spaceL + buttonArea + 32, 23)
			karamba.resizeImage(widget, bar[3], minArea + spaceR - 32, 70)
			karamba.showImage(widget, bar[2])
			karamba.showImage(widget, bar[3])

		# right curve
		karamba.moveImage(widget, bar[4], widgetWidth-32, 23)

		widgetHeight = 64+6+12
		karamba.moveWidget(widget, (resX - widgetWidth)/2, resY - widgetHeight)
		karamba.resizeWidget(widget, widgetWidth, widgetHeight)

		karamba.redrawWidget(widget)

#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, mouseButton):
	global spaceL, spaceR
	global buttons, icons, minimised
	global currentTask

	if 16 <= y <= 16+48:
		if spaceL <= x <= spaceL + 64*len(buttons):
			button = (x - spaceL) / 64

			if 0 <= button < len(buttons):

				# already running
				if buttons[button][3] and mouseButton != 2:
					# open all
					if mouseButton == 1:
						print "opening all tasks on button " + str(button)
						for task in buttons[button][4]:
							karamba.performTaskAction(widget, task, 8)
						currentTask = -1

					# prev task
					if mouseButton == 4:
						print "prev task on button " + str(button)
						if currentTask < 0:
							print "opening last task instead..."
							currentTask = len(buttons[button][4])-1
						else:
							currentTask -= 1
							currentTask = currentTask % len(buttons[button][4])
						karamba.performTaskAction(widget, buttons[button][4][currentTask], 8)

					# next task
					if mouseButton == 5:
						print "next task on button " + str(button)
						if currentTask < 0:
							print "opening first task instead..."
							currentTask = 0
						else:
							currentTask += 1
							currentTask = currentTask % len(buttons[button][4])
						karamba.performTaskAction(widget, buttons[button][4][currentTask], 8)

				# not running... yet
				elif mouseButton == 1 or mouseButton == 2:
					print "executing " + buttons[button][2]
					os.system(buttons[button][2] + " &")

		elif len(minimised) > -1 and spaceL+64*len(buttons)+32 <= x <= spaceL+64*len(buttons)+32+64*len(minimised):
			button = (x - (spaceL+64*len(buttons)+32)) / 64

			if 0 <= button < len(minimised) and mouseButton == 1:
				print "restoring minimised window " + str(button)
				karamba.performTaskAction(widget, minimised[button][1], 8)

	pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
	#Warning:  Don't do anything too intensive here
	#You don't want to run some complex piece of code everytime the mouse moves

	global spaceL, spaceR
	global buttons, minimised
	global currentZoomed, currentTask
	global icons

	if 16 <= y <= 16+48:

		if spaceL <= x < spaceL + 64*len(buttons):
			button = (x - spaceL) / 64

			if buttons[button][0] != currentZoomed:
				zoomIcon(widget, buttons[button][0], 1)
 				karamba.redrawWidget(widget)
				currentTask = -1
				pass

		elif len(minimised) > -1 and spaceL+64*len(buttons)+32 <= x < spaceL+64*len(buttons)+32+64*len(minimised):
			button = (x - (spaceL+64*len(buttons)+32)) / 64

			if currentZoomed != minimised[button][0]:
				zoomIcon(widget, minimised[button][0], 1)
				karamba.redrawWidget(widget)
				currentTask = -1

		else:
			zoomIcon(widget, "##", 1)
			karamba.redrawWidget(widget)
			currentTask = -1

	else:
			zoomIcon(widget, "##", 1)
			karamba.redrawWidget(widget)
			currentTask = -1

# This will be printed when the widget loads.
print "started OSXDocker"
