#
# import modules
#
import time, karamba, pymetar, xmms.control, glob, string, os, os.path
debug=0

###### Config Options ######

special_commands = {"home" : "kfmclient openProfile filemanagement",       # A list of special commands which are too long to be
                    "root" : "kfmclient openProfile filemanagement '/'",   # entered normally.
		    "mp3s" : "kfmclient openProfile 'MP3 Browsing'"}

task_bg_pic = "images/taskframe.png"                                       # Background pic for task list (looks ugly without).

linkfilesPath = "/home/sean/installs/PlasticBar 2.0/apps/"                 # Path to location of app icons.

weatherupdate = 45000                                                      # How often should the weather update itself? In ms.

showsystray = 1                                                            # Should the systray be shown? (1 = yes, anything else = no)

numberofdigits = 2                                                        # Number of leading digits before app names.

updateoften = 0                                                            # Should the tasked be updated more often?
                                                                           # (1 = yes, anything else = no) (May affect performance)

############################


#
# variables & constants
#
class Var:
    """
    Helper class for keeping runtime variables
    """
    def __init__(self, **kws):
        self.__dict__ = kws

#stationCode EFTU is Turku, Finland

VAR = Var(weather=None, shown=False, utime=None, timezone=0,
          stationCode='EGDL', timezonePID=None, stationCodePID=None)


METAR_TIME_FORMAT = '%Y.%m.%d %H%M %Z'
LOCAL_TIME_FORMAT = '%d %b %H:%M'
IMAGE_X, IMAGE_Y  = (1070,5)
DETAILS_X, DETAILS_Y = (1070,16)

NIGHT_PIXMAPNO = {'sun' : 31, 'suncloud' : 27, 'cloud': 26, 'rain': 45,
                  'snow': 46 , 'fog' : 29, 'storm': 47}

task_x = 470
task_y = 13
task_h = 36
task_w = 560
twidth = 0
theight = 0
task_list = []
task_oldtextlist = []
task_frametextlist = []

icons = []
shown = []
commands = []

iconCount = 0
iconsShown = 0


#
# Karamba functions
#
def initWidget(widget):
    global title
    global menu
    global menu_close
    global menu_show
    global menu_list
    global showsystray

    check_xmms(widget)
    create_images(widget)
    
    trackname = xmms.control.get_playlist_title(xmms.control.get_playlist_pos())
    title=karamba.createText(widget, 295, 15, 170, 18, trackname)
    karamba.changeTextColor(widget, title, 0, 0, 0)
    karamba.changeTextSize(widget, title, 12)
    karamba.changeTextFont(widget, title, "Verdana")
    
    VAR.stationCode = karamba.readConfigEntry(widget,'stationCode') or \
                      VAR.stationCode
    VAR.timezone = karamba.readConfigEntry(widget,'timezone') or \
                   VAR.timezone
    VAR.image = karamba.createImage(widget, IMAGE_X, IMAGE_Y,
                                    'weather_icons/small/25.png')
    karamba.attachClickArea(widget, karamba.getThemeText(widget,'MASK'))
    karamba.addMenuConfigOption(widget, 'stationcode', 'Change station code')
    karamba.setMenuConfigOption(widget, 'stationcode', '0')
    karamba.addMenuConfigOption(widget, 'timezone','Change timezone')
    karamba.setMenuConfigOption(widget, 'timezone', '0')
    karamba.moveWidget(widget, 40, 967)
    if showsystray == 1:
        karamba.createSystray(widget, 1102, 28, 57, 24)

    menu=karamba.createMenu(widget)
    menu_close=karamba.addMenuItem(widget, menu, "Close XMMS", "fileclose")
    menu_show=karamba.addMenuItem(widget, menu, "Show XMMS Window", "fileopen")
    menu_list=karamba.addMenuItem(widget, menu, "Show XMMS Playlist", "fileopen")
    
    karamba.redrawWidget(widget)
    widgetUpdated(widget)

def widgetUpdated(widget):
    update_title(widget)
    update_tasks(widget)
    if (VAR.utime is None) or (time.time()-VAR.utime > weatherupdate) or (checkflag == 1):
        VAR.weather = fetch_weather_report(VAR.stationCode)
        VAR.utime = time.time()
	checkflag = 0
    _update(widget)

def meterClicked(widget, meter, button):
    global minimize
    global checkflag
    
    if meter == minimize:
        karamba.toggleShowDesktop(widget)
    else:
        if button == 2:
	    checkflag = 1
	elif button == 1:
            if VAR.weather:
                if VAR.shown:
                    _hide_details(widget)
                else:
                    _show_details(widget)
                karamba.redrawWidget(widget)
                VAR.shown = not VAR.shown

def menuOptionChanged(widget, key, value):
    if key == 'stationcode':
        cmd = ['kdialog', '--title', 'Change station code', '--inputbox',
               'Please enter your 4-character station ICAO ID '\
               '(from http://www.nws.noaa.gov/tg/siteloc.shtml):',
               VAR.stationCode]
        karamba.setMenuConfigOption(widget, 'stationcode', '0')
        VAR.stationCodePID = karamba.executeInteractive(widget, cmd)
    elif key == 'timezone':
        cmd = ['kdialog', '--title', 'Change timezone', '--inputbox',
               'Please enter your timezone (eg. 2 for GMT+2):',
               str(VAR.timezone)]
        karamba.setMenuConfigOption(widget, 'timezone', '0')
        VAR.timezonePID = karamba.executeInteractive(widget, cmd)
               
def commandOutput(widget, pid, output):
    if pid == VAR.timezonePID:
        try:
            timezone = int(output)
            karamba.writeConfigEntry(widget,'timezone', str(timezone))
            VAR.timezone = timezone
        except:
            cmd = ['kdialog', '--title', 'Invalid Input',
                   '--error', 'Given timezone is not valid!']
            karamba.executeInteractive(widget, cmd)
            return
    elif pid == VAR.stationCodePID:
        stationCode = output.replace('\n','').strip()
        if len(stationCode) == 4:
            karamba.writeConfigEntry(widget,'stationCode', stationCode)
            VAR.stationCode = stationCode
        else:
            cmd = ['kdialog', '--title', 'Invalid Input',
                   '--error', 'Given station code is not valid!']
            karamba.executeInteractive(widget, cmd)
            return
    VAR.utime = None
    widgetUpdated(widget)
    
    
def xmms_click(widget, x, y):
    global menu
   
    if (348 < x < (348 + 13)) and (35 < y < (35 + 13)):
	check_xmms(widget)
	xmms.control.playlist_prev()
	time.sleep(0.1)
	update_title(widget)
    if ((348 + 22) < x < (348 + 29)) and (35 < y < (35 + 13)):
	check_xmms(widget)
	xmms.control.play_pause()
	time.sleep(0.1)
    if ((348 + 38) < x < (348 + 47)) and (35 < y < (35 + 13)):
	check_xmms(widget)
	xmms.control.play_pause()
	time.sleep(0.1)
    if ((348 + 57) < x < (348 + 66)) and (35 < y < (35 + 13)):
	check_xmms(widget)
	xmms.control.stop()
	time.sleep(0.1)
    if ((348 + 77) < x < (348 + 90)) and (35 < y < (35 + 13)):
	check_xmms(widget)
	xmms.control.playlist_next()
	time.sleep(0.1)
	update_title(widget)
    if (295 <= x <= (295 + 170)) and (15 <= y <= (15 + 18)):
	karamba.popupMenu(widget, menu, x, y)

def task_click(widget, x, y):
   global task_list
   global task_x
   global task_h
   global task_w
   
   tx = x - task_x
   ty = y - task_y
   listnum = (tx / twidth) * 2
   if( ty > theight ):
	listnum = listnum + 1
   karamba.performTaskAction(widget, task_list[listnum], 8)

def widgetClicked(widget, x, y, button):
   global task_x
   global task_y
   global task_h
   global task_w
   global twidth
   global theight
   
   if( (button == 1) and (295 <= x <= (348 + 90)) and (15 <= y <= (35 + 13)) ):
	xmms_click(widget, x, y)
   
   if( (button == 1) and (task_x <= x <= (task_x+task_w)) and (task_y <= y <= (task_y+task_h)) ):  
	task_click(widget, x, y)

   if( ( (button == 4) or (button == 5) ) and (100 <= x <= 272) ):
	icon_scroll(widget, button)

   pass

#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):

    if id == menu_show:
        xmms.control.main_win_toggle(1)
    if id == menu_close:
        xmms.control.quit()
    if id == menu_list:
        xmms.control.pl_win_toggle(1)

    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    if updateoften == 1:
        update_tasks(widget)

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.
def taskRemoved(widget, task):
    if updateoften == 1:
        update_tasks(widget)
    
#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    if updateoften == 1:
        update_tasks(widget)    

    
#
# XMMS functions
# 
def check_xmms(widget):
    if not xmms.control.is_running():
        karamba.executeInteractive(widget, ["kstart", "--window", "XMMS", "--alldesktops", "--skiptaskbar", "--skippager", "xmms"])
        time.sleep(100)
        xmms.control.main_win_toggle(0)
        xmms.control.pl_win_toggle(0)
        xmms.control.eq_win_toggle(0)
	xmms.control.set_main_volume(70)
	
def update_title(widget):
    trackname = xmms.control.get_playlist_title(xmms.control.get_playlist_pos())
    karamba.changeText(widget, title, trackname)
    
def icon_scroll(widget, button):
    global icons
    global shown
    global commands
    global linkfilesPath
    global firstIcon
    
    if button == 4:
        firstIcon -= 1
    elif button == 5:
        firstIcon += 1

    firstIcon = (firstIcon % len(icons))
    
    for i in range(4):
        karamba.deleteImage(widget, shown[i])

    shown = []
    
    iconCount = 0
    for x in range(firstIcon, firstIcon+4):
	z = ( x % len(icons) )
        shown.append(karamba.createImage(widget, (120+(iconCount*40)), 10, icons[z]))
	karamba.attachClickArea(widget, shown[iconCount], commands[z])
	iconCount += 1
    
def create_images(widget):
    global icons
    global shown
    global commands
    global iconCount
    global linkfilesPath
    global firstIcon
    global iconsShown
    global minimize
    
    KMenu = karamba.createImage(widget, 28, 13, 'images/kmenu.png')
    karamba.attachClickArea(widget, KMenu, "dcop kicker kicker popupKMenu 0")
    
    minimize = karamba.createImage(widget, 276, 20, 'images/down.png')
    karamba.attachClickArea(widget, minimize)
    
    fileList = glob.glob(linkfilesPath+"*")
    fileList.sort() # Thanks to genneth
    for fileName in fileList:
        
	if os.path.isfile(fileName) != 1:
	    continue
	
	fileName = os.path.split(fileName)
	fileName = fileName[1]
	fileName = string.split(fileName, '.png', 1)
	fileName = fileName[0]
	
	newcommand = check_command(fileName)
	
	if iconsShown < 4:
	    shown.append(karamba.createImage(widget, (120+(iconCount*40)), 10, (linkfilesPath+fileName+".png")))
	    karamba.attachClickArea(widget, shown[iconsShown], newcommand)
	    iconsShown += 1
	
	icons.append(linkfilesPath+fileName+".png")
	commands.append(newcommand)
	iconCount += 1
	firstIcon = 0

def check_command(oldcommand):
    global commands
    global special_commands
    if (special_commands.get(oldcommand[numberofdigits:255]) is not None):
        return special_commands.get(oldcommand[numberofdigits:255])
    else:
        return oldcommand[numberofdigits:255]

    
#
# Taskbar function
#
def update_tasks(widget):
   global task_list
   global task_oldtextlist
   global task_frametextlist
   global task_x
   global task_y
   global task_h
   global task_w
   global task_bg_pic
   global twidth
   global theight

   for this_task in task_oldtextlist:                         # cleanup all displayed tasks
       karamba.deleteText(widget, this_task)

   for this_task in task_frametextlist:                       # cleanup corresponding task pics
       karamba.deleteImage(widget, this_task)

   task_oldtextlist = []                                      # remove deleted elements from list
   task_frametextlist = []

   task_list = karamba.getTaskList(widget)                    # list of all running tasks

   taskcount = len(task_list) + (len(task_list) % 2)          # count of vert. fields
   twidth = task_w / (taskcount / 2)                          # width of a textfield
   theight = task_h / 2                                       # heigt of a textfield
   tx = task_x
   ty = task_y
   istop = 1

   for this_task in task_list:                                # display tasks
      task_info = karamba.getTaskInfo(widget,this_task)       # get info of task

      this_frame = karamba.createImage(widget, tx, ty, task_bg_pic) # build display frame
      karamba.resizeImage(widget,this_frame,twidth-5,theight-1)         # adjust frame size
      task_frametextlist.append(this_frame)
      
      this_text = karamba.createText(widget, tx+5, ty, twidth-11, theight, task_info[0])
      karamba.changeTextColor(widget, this_text, 64, 64, 64)
      karamba.changeTextSize(widget, this_text, 12)
      karamba.changeTextFont(widget, this_text, "Blue Highway")
      task_oldtextlist.append(this_text)

      if( istop == 1 ):                                       # calc coordinates for next task
         istop = 0
         ty = ty + theight
      else:
         istop = 1
         ty = task_y
         tx = tx + twidth

	 
#
# metar functions
#
def fetch_weather_report(station_code):
    try:
        #print '-- fetching report for %s --' % station_code
        rf = pymetar.ReportFetcher(station_code)
        rep = rf.FetchReport()
        #print '-- parsing report --'
        rp = pymetar.ReportParser()
        wr = rp.ParseReport(rep)
    except Exception, e:
        print 'Error fetching weather report (%s)' % e
        wr = None
    return wr

def get_localtime(time_string):
    try:
        tt = time.strptime(time_string, METAR_TIME_FORMAT)
        tsec = time.mktime(tt)
        tsec += int(VAR.timezone)*3600
        lt = time.strftime(LOCAL_TIME_FORMAT, time.localtime(tsec))
    except Exception, e:
        print 'Error parsing time (%s)' % e
        lt = 'n.a.'
    return lt

def is_night_time(localtime):
    try:
        tt = time.strptime(localtime, LOCAL_TIME_FORMAT)
        # between 9pm and 5am is night-time ;)
        return 21 <= tt[3] or tt[3] <= 5
    except:
        return False
       
	 
#
# protected functions
#
def _update(widget):
    temp = wind = ''
    pixmapno = 25
    if VAR.weather:
        wr = VAR.weather
        temp = '%s %sC' % (round(wr.getTemperatureCelsius(),1), chr(176))
        wind = '%d m/s %s' % (wr.getWindSpeed(), wr.getWindCompass())
        pixmapno = wr.getPixmapNo() or 25
        if is_night_time(get_localtime(wr.getTime())):        
            pixmapno = NIGHT_PIXMAPNO.get(wr.getPixmap(), pixmapno)
    karamba.changeText(widget, karamba.getThemeText(widget, 'TEMP'), temp)
    karamba.changeText(widget, karamba.getThemeText(widget, 'WIND'), wind)
    karamba.deleteImage(widget, VAR.image)
    VAR.image = karamba.createImage(widget, IMAGE_X, IMAGE_Y,
                                    'weather_icons/small/%s.png' % pixmapno)
    karamba.redrawWidget(widget)

def _show_details(widget):
    bg = karamba.createImage(widget, DETAILS_X, DETAILS_Y, 'images/details_bg.png')
    for i in range(115, 35, 5):
        karamba.resizeImage(widget, bg, 180, i)
        karamba.redrawWidget(widget)
    city = wtime = weather = conditions = ''
    pixmapno = 25
    if VAR.weather:
        wr = VAR.weather
        city = wr.getStationCity()
        wtime = get_localtime(wr.getTime())
        weather = wr.getWeather()
        conditions = wr.getConditions() or '-'
        pixmapno  = wr.getPixmapNo() or 25
        if is_night_time(wtime):        
            pixmapno = NIGHT_PIXMAPNO.get(wr.getPixmap(), pixmapno)
    img = karamba.createImage(widget, DETAILS_X-100, IMAGE_Y-33, 'weather_icons/large/%s.png' % pixmapno)
    titleW = karamba.createText(widget, DETAILS_X-310, DETAILS_Y-15, 260, 15, '%s %s' % (city, wtime))
    karamba.changeTextSize(widget, titleW, 11)
    karamba.changeTextColor(widget, titleW, 0, 0, 0)
    txtW = karamba.createText(widget, DETAILS_X, DETAILS_Y-15, 260, 15, '%s (%s)' % (weather, conditions))
    karamba.changeTextSize(widget, txtW, 11)
    karamba.changeTextColor(widget, txtW, 0, 0, 0)
    VAR.detailsW = (bg, img, titleW, txtW)

def _hide_details(widget):
    karamba.hideImage(widget, VAR.detailsW[0])
    karamba.hideImage(widget, VAR.detailsW[1])
    karamba.hideText(widget, VAR.detailsW[2])
    karamba.hideText(widget, VAR.detailsW[3])
    karamba.deleteImage(widget, VAR.detailsW[0])
    karamba.deleteImage(widget, VAR.detailsW[1])
    karamba.deleteText(widget, VAR.detailsW[2])
    karamba.deleteText(widget, VAR.detailsW[3])
    VAR.detailsW = None
    
    
#print "moose"