##############################################################
#                                                            #
#                       LIQUID WEATHER ++                    #
#                                                            #
#   Copyright:Matthew - dmbkiwi@yahoo.com                    #
#   License:  GPL                                            #
#   A superkaramba theme                                     #
#                                                            #
##############################################################

#this import statement allows access to the karamba functions
import karamba
import os
import string
import time
from urllib import urlopen, quote_plus
from os.path import expanduser
import copy
from threading import *

# Configuration
#
# YOU DONT NEED TO EDIT ANYTHING HERE.  DO IT FROM WITHIN THE PROGRAM
#
# I REPEAT -
#
#
# You don't need to edit ANYTHING.  Just run the widget and use the
# "Configure Theme" submenu to change everything


# SETUP URL'S TO DOWNLOAD WEATHER DATA
#
# URL's to download weather data:
url = 'http://xoap.weather.com/weather/local/NZXX0049?cc=*&dayf=5&prod=xoap&par=1003666583&key=4128909340a9b2fc'
url_base_addr = 'http://xoap.weather.com/weather/local/'
url_suffix = '?cc=*&dayf=5&prod=xoap&par=1003666583&key=4128909340a9b2fc'
url_bbc_base_addr = 'http://www.bbc.co.uk/print/weather/5day.shtml?world='
url_bbc_add_addr =  '&print'
#
# URL for webcam:
webcamurls = ['http://homepages.ihug.co.nz/%7Eseebee/webcam/wgtn_harbour.jpg']
#
#
# Where k_weather.py and the cached copy of the html page
# should be kept:
installed = '~/.superkaramba'

# SETUP SOME BASIC VARIABLES WITH DEFAULT VALUES:

metric_units = 0
days = 5
clock = 0
#
# If 1, then the high temperature will be displayed first.
# Otherwise, the low temperature will be displayed first
order = 0
#
# This changes the separator between the high and low values:
separator = "/"
#
# This is the initial background if not set in the config file:
mainback = "bg1.png"
#
# If 1, then wind arrows point towards where the wind is blowing to:
windy = 0
webcamlabels = ['Wellington Harbour']
nowebcams = 0
timegate = 0
configchange = 0
webbypic = None
satpic = None
webcams = 0
webcamdisplay = 0
whichone = 0
use_wcom = 1
wcomcode = 'NZXX0049'
satpicon = 0
sumpicon = 0
presspicon = 0
iconset = 'liquid'
iconsetpath = 'liquid/'
werethreading = 0
prefix = 'http://maps.wunderground.com/data/640x480/'
countries = ["2xus_", "2xca_", "2xsa_", "2xeu_", "2xa2_", "2xaf_", "2xau_", "2xas_"]
map_matrix = [[None, 'st', 'rh', 'ws', 'sf', 'ir', 'rd'], [None, 'st', 'rh', 'ws', 'sf'], [None, 'st', 'rh', 'ws'], [None, 'st', 'rh', 'ws', 'sf'], [None, 'st', 'rh', 'ws'], [None, 'st', 'rh', 'ws', 'sf'], [None, 'st', 'rh', 'ws'], [None, 'st', 'rh']]
continentitems = ['North America', 'Central America', 'South America', 'Europe', 'Asia', 'Africa', 'Australia', 'Russian Federation']
mapitems = ['Forecast', 'Temperature', 'Relative Humidity', 'Wind Speed', 'Fronts', 'Infrared', 'Radar']
forecastprefix = 'http://www.accuweather.com/adc_images2/english/forecast/tmpicn/640x480/'
forelocations = ['namer_todtmpicn.gif','camer_todtmpicn.gif','samer_todtmpicn.gif', 'eur_todtmpicn.gif','asia_todtmpicn.gif','afr_todtmpicn.gif','au_todtmpicn.gif','rs_todtmpicn.gif']
# This is the threading engine.

class Future:

    def __init__(self,func,*param):
        # Constructor
        self.__done=0
        self.__result=None
        self.__status='working'
        self.__excpt=None
        self.__C=Condition()   # Notify on this Condition when result is ready
        # Run the actual function in a separate thread
        self.__T=Thread(target=self.Wrapper,args=(func,param))
        self.__T.setName("FutureThread")
        self.__T.start()

    def __repr__(self):
        return '<Future at '+hex(id(self))+':'+self.__status+'>'

    def isDone(self):
        return self.__done    # Notify if the function is complete

    def __call__(self):
        self.__C.acquire()
        while self.__done==0:
            self.__C.wait()
        self.__C.release()
        # An exception was thrown in the thread, re-raise it here.
        if self.__excpt: raise self.__excpt
        # We deepcopy __result to prevent accidental tampering with it.
        a=copy.deepcopy(self.__result)
        return a

    def Wrapper(self, func, param):
        # Run the actual function, and let us housekeep around it
        self.__C.acquire()
        try:
            self.__result=func(*param)
        except Exception, e: # We got an exception object, save it.
            self.__excpt = e
        except:
            self.__result="Exception raised within Future"
        self.__done=1
        self.__status=`self.__result`
        self.__C.notify()
        self.__C.release()

# This function calls the data gathering functions, each in a separate thread
# which improves speed, and stops the theme freezing while downloading data
def getThreadedData(widget):
    global weatherthread, satpicthread, webcamthread, sumpicthread, load
    weatherthread = Future(getWeather,widget)
    webcamthread = Future(getWebcamPic,widget)
    load = Future(loadingBar,widget)    #Start progress bar in a separate thread

# Function to display progress of downloading data
def loadingBar(widget):
    global werethreading
    loadbarlist = ['nothing.png', 'half.png', 'done.png']
    progress = 0
    previous = 0
    total = 2    # This reflects 3 data threads if using BBC
    loadbar = karamba.createImage(widget, 120, 94, loadbarlist[progress])
    karamba.redrawWidget(widget)
    while progress < 2:
        progress = weatherthread.isDone() + webcamthread.isDone() # Check how many threads finished
        # Following if statement means that widget only
        # gets redrawn when a thread completes - removes ugly flicker
        if progress != previous:
            karamba.deleteImage(widget, loadbar)
            loadbar = karamba.createImage(widget, 120, 94, loadbarlist[progress])
            karamba.redrawWidget(widget)
        time.sleep(1)    # Reduce cpu load - not sure if this is necessary
        if progress == total:
            break
        previous = progress
    karamba.deleteImage(widget, loadbar)
    karamba.redrawWidget(widget)
    werethreading = 0
    widgetUpdated(widget)


# Basic function to reduce amount of typing necessary to draw text
def prettyText(widget, size, font, textname):
    karamba.changeTextSize(widget, textname, size)
    karamba.changeTextFont(widget, textname, font)
    karamba.changeTextColor(widget, textname, 0, 0, 0)

# A set of functions to display data/images on roll-down panels
def showSatPic(widget):
    global satpic, satpicon, webbypicbg
    webbypicbg = karamba.createImage(widget, 23, 25, 'webcambg.png')
    karamba.resizeImage(widget, webbypicbg, 216, 0)
    satpic = karamba.createImage(widget, 35, 34, satfile)
    karamba.attachClickArea(widget, satpic)
    karamba.resizeImage(widget, satpic, 190, 20)
    karamba.redrawWidget(widget)
    xpos = 179
    ypos = 35
    for i in range(20,140,10):
        karamba.resizeImage(widget, satpic, 190, i)
        if ypos > 12:
            ypos = ypos - 2
            karamba.moveImage(widget, satbutt, xpos, ypos)
        karamba.redrawWidget(widget)
    karamba.resizeImage(widget, satpic, 190, 140)
    karamba.resizeImage(widget, webbypicbg, 216, 166)
    karamba.redrawWidget(widget)
    satpicon = 1

def removeSatPic(widget):
    global satpicon
    karamba.deleteImage(widget, webbypicbg)
    xpos = 179
    ypos = 12
    for i in range(140,20,-10):
        karamba.resizeImage(widget, satpic, 190, i)
        if ypos < 35:
            ypos = ypos + 2
            karamba.moveImage(widget, satbutt, xpos, ypos)
        karamba.redrawWidget(widget)
    karamba.moveImage(widget, satbutt, 179, 36)
    karamba.deleteImage(widget, satpic)
    karamba.redrawWidget(widget)
    satpicon = 0

##~ def showSumPic(widget):
##~     global sumpic, sumpicon, webbypicbg
##~     webbypicbg = karamba.createImage(widget, 23, 25, 'webcambg.png')
##~     karamba.resizeImage(widget, webbypicbg, 216, 0)
##~     sumpic = karamba.createImage(widget, 35, 34, installed + "/summap.jpg")
##~     karamba.attachClickArea(widget, sumpic)
##~     karamba.resizeImage(widget, sumpic, 190, 20)
##~     karamba.redrawWidget(widget)
##~     xpos = 179
##~     ypos = 36
##~     for i in range(20,140,10):
##~         karamba.resizeImage(widget, sumpic, 190, i)
##~         if ypos > 12:
##~             ypos = ypos - 2
##~             karamba.moveImage(widget, satbutt, xpos, ypos)
##~         karamba.redrawWidget(widget)
##~     karamba.resizeImage(widget, sumpic, 190, 140)
##~     karamba.resizeImage(widget, webbypicbg, 216, 166)
##~     karamba.redrawWidget(widget)
##~     sumpicon = 1
##~ 
##~ def removeSumPic(widget):
##~     global sumpicon
##~     karamba.deleteImage(widget, webbypicbg)
##~     xpos = 179
##~     ypos = 12
##~     for i in range(140,20,-10):
##~         karamba.resizeImage(widget, sumpic, 190, i)
##~         if ypos < 40:
##~             ypos = ypos + 2
##~             karamba.moveImage(widget, satbutt, xpos, ypos)
##~         karamba.redrawWidget(widget)
##~     karamba.moveImage(widget, satbutt, 179, 36)
##~     karamba.deleteImage(widget, sumpic)
##~     karamba.redrawWidget(widget)
##~     sumpicon = 0
##~ 
##~ def showPressPic(widget):
##~     global presspic, presspicon, webbypicbg
##~     webbypicbg = karamba.createImage(widget, 23, 25, 'webcambg.png')
##~     karamba.resizeImage(widget, webbypicbg, 216, 0)
##~     presspic = karamba.createImage(widget, 35, 34, installed + "/pressmap.jpg")
##~     karamba.attachClickArea(widget, presspic)
##~     karamba.resizeImage(widget, presspic, 190, 20)
##~     karamba.redrawWidget(widget)
##~     xpos = 179
##~     ypos = 36
##~     for i in range(20,140,10):
##~         karamba.resizeImage(widget, presspic, 190, i)
##~         if ypos > 12:
##~             ypos = ypos - 2
##~             karamba.moveImage(widget, satbutt, xpos, ypos)
##~         karamba.redrawWidget(widget)
##~     karamba.resizeImage(widget, presspic, 190, 140)
##~     karamba.resizeImage(widget, webbypicbg, 216, 166)
##~     karamba.redrawWidget(widget)
##~     presspicon = 1
##~ 
##~ def removePressPic(widget):
##~     global presspicon
##~     karamba.deleteImage(widget, webbypicbg)
##~     xpos = 179
##~     ypos = 12
##~     for i in range(140,20,-10):
##~         karamba.resizeImage(widget, presspic, 190, i)
##~         if ypos < 40:
##~             ypos = ypos + 2
##~             karamba.moveImage(widget, satbutt, xpos, ypos)
##~         karamba.redrawWidget(widget)
##~     karamba.moveImage(widget, satbutt, 179, 36)
##~     karamba.deleteImage(widget, presspic)
##~     karamba.redrawWidget(widget)
##~     presspicon = 0

def showWebcamPic(widget):
    global webbypic, webpicon, webbypicbg
    webbypicbg = karamba.createImage(widget, 23, 25, 'webcambg.png')
    karamba.resizeImage(widget, webbypicbg, 216, 0)
    webbypic = karamba.createImage(widget, 35, 34, filenames[webcamdisplay])
    karamba.resizeImage(widget, webbypic, 190, 20)
    karamba.redrawWidget(widget)
    xpos = 203
    ypos = 36
    for i in range(20,140,10):
        karamba.resizeImage(widget, webbypic, 190, i)
        if ypos > 12:
            ypos = ypos - 2
            karamba.moveImage(widget, cambutt, xpos, ypos)
        karamba.redrawWidget(widget)
    karamba.resizeImage(widget, webbypic, 190, 140)
    karamba.resizeImage(widget, webbypicbg, 216, 166)
    karamba.redrawWidget(widget)
    webpicon = 1

def removeWebcamPic(widget):
    global webpicon
    karamba.deleteImage(widget, webbypicbg)
    xpos = 203
    ypos = 12
    for i in range(140,20,-10):
        karamba.resizeImage(widget, webbypic, 190, i)
        if ypos < 35:
            ypos = ypos + 2
            karamba.moveImage(widget, cambutt, xpos, ypos)
        karamba.redrawWidget(widget)
    karamba.deleteImage(widget, webbypic)
    karamba.moveImage(widget, cambutt, 203, 36)
    karamba.redrawWidget(widget)
    webpicon = 0

def createDayDetail(widget, day):
    global weather, order, separator, ddbg, setday, settemp, setrain, setsitu, ddday, ddtemp, ddrain, ddsitu, ddon, setdate, dddate, daypicrem, topBar, daypicpos, animate
    ddbg = karamba.createImage(widget, 26, 25, "background/dd_bg.png")
    karamba.resizeImage(widget, ddbg, 210, 20)
    karamba.changeImageIntensity(widget, daypicrem[day], 0.3, 0)
    karamba.redrawWidget(widget)
    flag = 0
    xpos = (daypicpos[day])
    ypos = 140
    height = 28
    width = 28
    deg = 30
    for i in range(20,140,10):
        karamba.resizeImage(widget, ddbg, 210, i)
        if animate == 0:
            karamba.rotateImage(widget, daypicrem[day], deg)
            w = karamba.getImageWidth(widget,daypicrem[day])
            h = karamba.getImageHeight(widget,daypicrem[day])
            karamba.moveImage(widget, daypicrem[day], xpos-((w-width)/2), ypos-((h-height)/2))
            deg = deg + 30
        karamba.redrawWidget(widget)

    setday = karamba.createText(widget, 50, 30, 70, 20, "Detail for:")
    prettyText(widget, 13, "arial black", setday)
    ddday = karamba.createText(widget, 125, 30, 70, 20, weather.get_forecast(day).weekday())
    prettyText(widget, 13, "arial black", ddday)
    setdate = karamba.createText(widget, 50, 50, 70, 20, "Date:")
    prettyText(widget, 10, "verdana", setdate)
    settemp = karamba.createText(widget, 50, 70, 70, 20, "Temp:")
    prettyText(widget, 10, "verdana", settemp)
    setrain = karamba.createText(widget, 50, 90, 70, 20, "Rain:")
    prettyText(widget, 10, "verdana", setrain)
    setsitu = karamba.createText(widget, 50, 110, 70, 20, "Sky:")
    prettyText(widget, 10, "verdana", setsitu)
    dddate = karamba.createText(widget, 85, 50, 50, 20, weather.get_forecast(day).date())
    prettyText(widget, 10, "verdana", dddate)
    ddtemp = karamba.createText(widget, 85, 70, 130, 20, weather.get_forecast(day).temperature_range(order, separator))
    prettyText(widget, 10, "verdana", ddtemp)
    ddrain = karamba.createText(widget, 85, 90, 50, 20, weather.get_forecast(day).rain())
    prettyText(widget, 10, "verdana", ddrain)
    ddsitu = karamba.createText(widget, 85, 110, 130, 20, weather.get_forecast(day).sky())
    prettyText(widget, 10, "verdana", ddsitu)
    ddon = 1


def deleteDayDetail(widget, day):
    global ddon, ddbg, setday, settemp, setrain, setsitu, ddday, ddtemp, ddrain, ddsitu, dddate, setdate, daypicrem, topBar, daypicpos, animate
    for i in (setday, settemp, setrain, setsitu, setdate, ddday, ddtemp, ddrain, ddsitu, dddate):
        karamba.deleteText(widget, i)
    karamba.redrawWidget(widget)
    flag = 0
    xpos = (daypicpos[day])
    ypos = 140
    height = 28
    width = 28
    deg = 330
    for i in range(130,20,-10):
        karamba.resizeImage(widget, ddbg, 210, i)
        if animate == 0:
            karamba.rotateImage(widget, daypicrem[day], deg)
            w = karamba.getImageWidth(widget,daypicrem[day])
            h = karamba.getImageHeight(widget,daypicrem[day])
            karamba.moveImage(widget, daypicrem[day], xpos-((w-width)/2), ypos-((h-height)/2))
            deg = deg - 30
        karamba.redrawWidget(widget)
    karamba.moveImage(widget, daypicrem[day], xpos, ypos)
    karamba.redrawWidget(widget)
    karamba.deleteImage(widget, ddbg)
    #if animate == 0:
        #karamba.resizeImage(widget, daypicrem[day], 28, 28)
        #karamba.moveImage(widget, daypicrem[day], daypicpos[day], 140)
    karamba.removeImageTransformations(widget, daypicrem[day])
    karamba.redrawWidget
    ddon = 0

def createCurrentDetail(widget):
    global clock, weather, ddcbg, setcday, setccity, setcdate, setctime, setcwdir, setcwstr, setcbar, setcrtmp, setchum, setcvis, setcuv, ddcday, ddccity, ddcdate, ddctime, ddcwdir, ddcwstr, ddcbar, ddcvis, ddcuv, ddcon, mainpic, clicky, animate, webpicon
    ddcbg = karamba.createImage(widget, 33, 25, "background/ddc_bg.png")
    karamba.resizeImage(widget, ddcbg, 196, 20)
    karamba.changeImageIntensity(widget, mainpic, 0.3, 0)
    karamba.redrawWidget(widget)
    #x = 108
    #y = 108
    deg = 36
    height = 120
    width = 120
    for i in range(10,140,13):
        karamba.resizeImage(widget, ddcbg, 196, i)
        if animate == 0:
            #karamba.resizeImage(widget, mainpic, x, y)
            karamba.rotateImage(widget, mainpic, deg)
            w = karamba.getImageWidth(widget,mainpic)
            h = karamba.getImageHeight(widget,mainpic)
            karamba.moveImage(widget,mainpic,0-((w-width)/2),0-((h-height)/2))
            deg = deg + 36
        karamba.redrawWidget(widget)
        #x = x - 8
        #y = y - 8
    karamba.resizeImage(widget, ddcbg, 196, 136)
    #if animate == 0:
        #karamba.resizeImage(widget, mainpic, 40, 40)
    karamba.redrawWidget(widget)
    setcday = karamba.createText(widget, 50, 30, 70, 20, "Detail for:")
    prettyText(widget, 13, "arial black", setcday)
    setccity = karamba.createText(widget, 50, 47, 70, 20, "City:")
    prettyText(widget, 10, "verdana", setccity)
    setctime = karamba.createText(widget, 50, 59, 70, 20, "Check:")
    prettyText(widget, 10, "verdana", setctime)
    setcdate = karamba.createText(widget, 50, 71, 70, 20, "Date:")
    prettyText(widget, 10, "verdana", setcdate)
    setcwdir = karamba.createText(widget, 50, 83, 70, 20, "Wind:")
    prettyText(widget, 10, "verdana", setcwdir)
    setcwstr = karamba.createText(widget, 50, 95, 70, 20, "Speed:")
    prettyText(widget, 10, "verdana", setcwstr)
    setcbar = karamba.createText(widget, 50, 107, 70, 20, "PChange:")
    prettyText(widget, 10, "verdana", setcbar)
    setcvis = karamba.createText(widget, 50, 119, 70, 20, "Vis:")
    prettyText(widget, 10, "verdana", setcvis)
    setcuv = karamba.createText(widget, 50, 131, 70, 20, "UV:")
    prettyText(widget, 10, "verdana", setcuv)
    ddcday = karamba.createText(widget, 125, 30, 100, 20, "Today " + weather.weekday())
    prettyText(widget, 13, "arial black", ddcday)
    ddccity = karamba.createText(widget, 105, 47, 120, 20, weather.place())
    prettyText(widget, 10, "verdana", ddccity)

    # showing 24h clock?
    if clock: tmptimestr = weather.time()
    else:  tmptimestr = weather.get_12h_time()
    tmptimestr +=  ' ' + weather.timezone()
    ddctime = karamba.createText(widget, 105, 59, 100, 20, tmptimestr)
    prettyText(widget, 10, "verdana", ddctime)
    ddcdate = karamba.createText(widget, 105, 71, 100, 20, weather.day() + " " + weather.month() + " " + weather.year())
    prettyText(widget, 10, "verdana", ddcdate)
    ddcwdir = karamba.createText(widget, 105, 83, 120, 20, weather.wind())
    prettyText(widget, 10, "verdana", ddcwdir)
    ddcwstr = karamba.createText(widget, 105, 95, 70, 20, weather.wind_speed())
    prettyText(widget, 10, "verdana", ddcwstr)
    ddcbar = karamba.createText(widget, 105, 107, 70, 20, weather.pressure_change())
    prettyText(widget, 10, "verdana", ddcbar)
    ddcvis = karamba.createText(widget, 105, 119, 70, 20, weather.visibility())
    prettyText(widget, 10, "verdana", ddcvis)
    ddcuv = karamba.createText(widget, 105, 131, 70, 20, weather.uv())
    prettyText(widget, 10, "verdana", ddcuv)
    ddcon = 1

def deleteCurrentDetail(widget):
    global ddcbg, setcday, setcdate, setctime, setcwdir, setcwstr, setcbar, setcvis, setcuv, ddcday, ddcdate, ddctime, ddcwdir, ddcwstr, ddcbar, ddcvis, ddcuv, ddcon, mainpic, animate
    for i in (setcday, setcdate, setccity, setctime, setcwdir, setcwstr, setcbar, setcvis, setcuv, ddcday, ddcdate, ddccity, ddctime, ddcwdir, ddcwstr, ddcbar, ddcvis, ddcuv):
        karamba.deleteText(widget, i)
    karamba.redrawWidget(widget)
    #x = 40
    #y = 40
    height = 120
    width = 120
    deg = -36
    for i in range(136,10,-13):
        karamba.resizeImage(widget, ddcbg, 196, i)
        if animate == 0:
            #karamba.resizeImage(widget, mainpic, x, y)
            karamba.rotateImage(widget, mainpic, deg)
            w = karamba.getImageWidth(widget,mainpic)
            h = karamba.getImageHeight(widget,mainpic)
            karamba.moveImage(widget,mainpic,0-((w-width)/2),0-((h-height)/2))
            deg = deg - 36
        karamba.redrawWidget(widget)
        #x = x + 8
        #y = y + 8
    karamba.deleteImage(widget, ddcbg)
    #if animate == 0:
        #karamba.resizeImage(widget, mainpic, 120, 120)
    karamba.removeImageTransformations(widget, mainpic)
    karamba.redrawWidget
    ddcon = 0

# Functions to parse weather data using regular expressions
def parseWeatherData(data, days):
    # Parses an xml page from weather.com and returns a list
    # containing the current weather conditions and forecast.

    # data: An unmodified xml page from weather.com
    # days: Number of forecast days (including today) to return (maximum 10)

    import re
    import weather
    from urllib import urlopen

    # Make sure we don't try to get any more than 5 day's forecast
    if days > 5: days = 5

    # Chop the data into more managable bits, and do a bit of manipulation
    # of data to rectify anomolies between XML and html data
    data = data.replace('CALM', 'Variable')
    data = data.replace('calm', '0')
    print data
    dcstart = data.find('<cc>')
    dcstop = data.find('</cc>')
    data_current = data[dcstart:dcstop]

    day0 = data.find('<day d="0"')
    day1 = data.find('<day d="1"')
    day2 = data.find('<day d="2"')
    day3 = data.find('<day d="3"')
    day4 = data.find('<day d="4"')
    data0 = data[day0:day1]
    data1 = data[day1:day2]
    data2 = data[day2:day3]
    data3 = data[day3:day4]
    data4 = data[day4:]


    p_report = r'''(?ix)
    .*<lsup>(?P<month>\d+?)/ # Month
    (?P<day>\d+?)/ # Day
    (?P<year>\d+?) # Year
    \s(?P<time>.+?)\s #Time
    (?P<meridian>[A|P]M) # AM or PM
    \s(?P<zone>.*?)</lsup> # time zone
    \s*<obst>(?P<city>.*?), # City
    '''




    # time and location of the weather report
    now = weather.Observation()
    match = re.search(p_report, data_current)
    print match
    print match.group('city')
    if match:
        now.year(match.group('year'))
        now.month(match.group('month'))
        now.day(match.group('day'))
        now.time(match.group('time'), match.group('meridian'))
        now.timezone(match.group('zone'))
        now.place(match.group('city'))

    p_weekday = r'''(?ix)
    <day\sd=\"0\"\st=\"(?P<weekday>.+?)\" # Day of the week
    '''

    match1 = re.search(p_weekday, data0)
    print match1
    print match1.group('weekday')
    if match1:
        now.weekday(match1.group('weekday')[:3])


    #print now
    print '==================================================='
    p_current = r'''(?isx) # Ignore case, Dot matches all, Verbose
    .*<tmp>(?P<temp>.*?)</tmp> # Temp
    \s*<flik>(?P<heat>.*?)</flik> # Heat
    \s*<t>(?P<sky>\D+?)</t> # Sky
    \s*<icon>(?P<icon>\d*?)</icon> # Icon
    \s*<bar>\s*<r>(?P<baro>.*?)</r>\s*<d>(?P<change>.*?)</d>\s*</bar>
    \s*<wind>\s*<s>(?P<speed>.*?)</s>.*<t>(?P<wind>.*?)</t>\s*</wind>
    \s*<hmid>(?P<hum>\d+)</hmid>
    \s*<vis>(?P<vis>.*?)</vis>
    \s*<uv>\s*<i>(?P<uv>.*?)</i>
    .*<dewp>(?P<dew>.*?)</dewp>
    '''


    match = re.search(p_current, data_current)
    print match
    print match.group('wind')
    if match:
        now.icon(match.group('icon'))
        now.temperature(match.group('temp'), 'F')
        now.relative_heat(match.group('heat'), 'F')
        now.sky(match.group('sky'))
        now.uv(match.group('uv'))
        now.dewpoint(match.group('dew'), 'F')
        now.humidity(match.group('hum'))
        now.visibility(match.group('vis'))
        now.pressure(match.group('baro'), 'inHg')
        now.pressure_change(match.group('change'))
        if match.group('wind') != 'CALM':
            mywind =  match.group('wind')
            gusting = mywind.find('gusting')
            if gusting != -1:
                jat = mywind.find(' at')
                mywind = mywind[:jat]
            now.wind(mywind.replace('From the ', ''))

        now.wind_speed(match.group('speed'), 'mph')
    print now


    p_today = r'''(?isx) # Ignore case, Dot matches all, Verbose
    <day\sd=\"0\".*dt=\"(?P<date>.*?)\"> # Date
    \s*?<hi>(?P<high>(.*?|N/A))</hi> # High temperature
    \s*?<low>(?P<low>(.*?|N/A))</low> # Low temperature
    .*?<icon>(?P<icon>\d*?)</icon> # Icon
    \s*?<t>(?P<sky>.*?)</t> # Sky
    .*?<ppcp>(?P<rain>\d+)</ppcp>  # Chance of rain
    '''

    today = weather.Forecast(now.weekday())
    now.add_forecast(today)
    match = re.search(p_today, data0)
    print match
    if match:
        today.icon(match.group('icon'))
        today.sky(match.group('sky'))
        today.date(match.group('date'))
        if match.group('high') != 'N/A':
            today.temperature_high(match.group('high'), 'F')
        today.temperature_low(match.group('low'), 'F')
        today.rain(match.group('rain'));

    print "+++++++++++++++++++Today\'s Forecast++++++++++++++++++++++++"
    print today
    p_forecast = r'''(?isx) # No embedded flags, since they will be added later
    .*?t=\"(?P<day>.*?)\" # Weekday
    .*?dt=\"(?P<date>.*?)\"> # Day number
    \s*?<hi>(?P<high>.*?)</hi> # High temperature
    \s*?<low>(?P<low>.*?)</low> # Low temperature
    .*?<icon>(?P<icon>\d*?)</icon> # Icon
    \s*?<t>(?P<sky>.*?)</t> # Sky
    .*?<ppcp>(?P<rain>\d+)</ppcp>  # Chance of rain
    '''
    datalist = [data1, data2, data3, data4]
    day = 1
    while day <= (days -1):
        forecast = weather.Forecast()
        now.add_forecast(forecast)
        match = re.search(p_forecast, datalist[day-1])
        if match:
            forecast.weekday(match.group('day')[:3])
            forecast.date(match.group('date'))
            forecast.icon(match.group('icon'))
            forecast.sky(match.group('sky'))
            forecast.temperature_high(match.group('high'), 'F')
            forecast.temperature_low(match.group('low'), 'F')
            forecast.rain(match.group('rain'));
        day += 1
        print forecast

    return now



def parseBBCWeatherData(data, days):
    '''Parses a page from weather.com and returns a list
    containing the current weather conditions and forecast.

    data: An unmodified html page from weather.com

    days: Number of forecast days to return (maximum 9)
    '''

    import re
    import weather

    #this maps bbc weather icons into widget icons
    iconmap = {'0':'31', '1':'32', '2':'26', '3':'28', '4':'4', '5':'22',
            '6':'21', '7':'26', '8':'26', '9':'6', '10':'9', '11':'6', '12':'6',
            '13':'12', '14':'9', '15':'12', '16':'8', '17':'8', '18':'8', '19':'15',
            '20':'15', '21':'15', '22':'14', '23':'13', '24':'14', '25':'15',
            '26':'15', '27':'15', '28':'27', '29':'17', '30':'4', '31':'2', '32':'20',
            '100':'25'}

    #'1':'31', '3':'27',
    # '32':'31', '28':'27',
    # Make sure we don't try to get any more than 5 day's forecast
    if days > 5: days = 5

    # Chop the data into more managable size
    start = data.find('forecasthead')
    stop = data.find('Nearest weather station') + 100
    data1 = data[start:stop]
    #print data1


    # The regular expressions needed to get the weather forecast

    # time and location of the weather report
    now = weather.Observation()

    p_city = r'transform:uppercase;\">(?P<city>.+?),'
    match = re.search(p_city, data1)
    if match:
            now.place(match.group('city'))
    #print now.place
    #group = group.replace('&nbsp;', '')

    p_time = r'''(?ix)
    Observed:\s(?P<time>\d+)   #Time
    \s(?P<zone>\w+) #zone
    \s(?P<weekday>.+? ) # Day of the week
    \s(?P<day>\d+)    # Day
    .(?P<month>\d+)   # Month
    .(?P<year>\d+)  # Year
    '''
    match = re.search(p_time, data1)
    if match:
        now.year(match.group('year'))
        now.month(match.group('month'))
        now.day(match.group('day'))
        now.weekday(match.group('weekday'))
        now.time(match.group('time'))
        now.timezone(match.group('zone'))

    #current observations
    p_current =  r'''(?isx)
    Nearest<br>\sObservations
    .*?images/5day/(?P<icon>\d+).*?alt=.(?P<sky>[^"]+)
    .*?images/5day/obs_(?P<wind>\w+?)\.gif.*?<strong>(?P<speed>\d+?)</strong>
    .*?images/5day/obstempc(?P<temp>\-?\d+)
    .*?Relative\sHumidity.*?<strong>(?P<hum>\d+)
    .*?<br>(?P<baro>.*?),\s(?P<change>.*?)<
    '''
    match = re.search(p_current, data1)
    if match:
        #print "obs icon:" + match.group('icon')+ '|'
        icon = iconmap[match.group('icon')] #str(match.group('icon'))
        if icon == '0' : icon = '31'
        #print 'made icon' + icon
        now.icon(icon)
        now.sky(match.group('sky'))
        now.temperature(match.group('temp'))
        now.humidity(match.group('hum'))
        now.wind_speed( match.group('speed'))
        now.wind( match.group('wind'))
        now.pressure(match.group('baro'))
        now.pressure_change(match.group('change'))
        #now.visibility(match.group('vis'))

    p_5day = r'''(?isx)
    width=\"210\".*?<strong>(?P<today>\w{3})
    .*?<strong>(?P<day1>\w{3})
    .*?<strong>(?P<day2>\w{3})
    .*?<strong>(?P<day3>\w{3})
    .*?<strong>(?P<day4>\w{3})
    '''

    match = re.search(p_5day, data1)
    if match:
        today = weather.Forecast(match.group('today'))
        day1 = weather.Forecast(match.group('day1'))
        day2 = weather.Forecast(match.group('day2'))
        day3 = weather.Forecast(match.group('day3'))
        day4 = weather.Forecast(match.group('day4'))

        now.add_forecast(today)
        now.add_forecast(day1)
        now.add_forecast(day2)
        now.add_forecast(day3)
        now.add_forecast(day4)

    if len(now.forecasts) == 0:
            print "Can not parse forecasts"
            return now

    print now

    p_5fore = r'''(?isx)
    width=\"210\"
    .*?images/5day/(?P<icon>\d+)
    .*?alt=\"(?P<desc>[^"]+)
    .*?images/5day/(?P<icon1>\d+)
    .*?alt=\"(?P<desc1>[^"]+)
    .*?images/5day/(?P<icon2>\d+)
    .*?alt=\"(?P<desc2>[^"]+)
    .*?images/5day/(?P<icon3>\d+)
    .*?alt=\"(?P<desc3>[^"]+)
    .*?images/5day/(?P<icon4>\d+)
    .*?alt=\"(?P<desc4>[^"]+)
    '''
    print 'fore------------------------------'
    match = re.search(p_5fore, data1)
    if match:
        today.icon(iconmap[match.group('icon')])
        today.sky(match.group('desc'))
        day1.icon(iconmap[match.group('icon1')])
        day1.sky(match.group('desc1'))
        day2.icon(iconmap[match.group('icon2')])
        day2.sky(match.group('desc2'))
        day3.icon(iconmap[match.group('icon3')])
        day3.sky(match.group('desc'))
        day4.icon(iconmap[match.group('icon4')])
        day4.sky(match.group('desc4'))

    #print '-------------icons---------'
    #print match.group('icon')
    #print iconmap[match.group('icon')]
    #print match.group('icon1')
    #print iconmap[match.group('icon1')]
    #print match.group('icon2')
    #print iconmap[match.group('icon2')]
    #print match.group('icon3')
    #print iconmap[match.group('icon3')]
    #print match.group('icon4')
    #print iconmap[match.group('icon4')]
    #print '-------------icons---------'

    p_5temp = r'''(?isx)
    Fahrenheit
    .*?images/5day/max_tempc(?P<max>\-?\d+) # max today
    .*?images/5day/max_tempc(?P<max1>\-?\d+) # max
    .*?images/5day/max_tempc(?P<max2>\-?\d+) # min
    .*?images/5day/max_tempc(?P<max3>\-?\d+) # max
    .*?images/5day/max_tempc(?P<max4>\-?\d+) # min
    .*?Minimum\sTemperature.*?images/5day/tempc(?P<min>\-?\d+)
    .*?images/5day/tempc(?P<min1>\-?\d+)
    .*?images/5day/tempc(?P<min2>\-?\d+)
    .*?images/5day/tempc(?P<min3>\-?\d+)
    .*?images/5day/tempc(?P<min4>\-?\d+)
    '''
    match = re.search(p_5temp, data1)
    if match:
        today.temperature_high(match.group('max'))
        today.temperature_low(match.group('min'))
        day1.temperature_high(match.group('max1'))
        day1.temperature_low(match.group('min1'))
        day2.temperature_high(match.group('max2'))
        day2.temperature_low(match.group('min2'))
        day3.temperature_high(match.group('max3'))
        day3.temperature_low(match.group('min3'))
        day4.temperature_high(match.group('max4'))
        day4.temperature_low(match.group('min4'))


    p_5pollut = r'''(?isx)
    Air\sPollution\sIndex
    .*?images/5day/air_(?P<pol>\d+)
    .*?images/5day/air_(?P<pol1>\d+)
    .*?images/5day/air_(?P<pol2>\d+)
    .*?images/5day/air_(?P<pol3>\d+)
    .*?images/5day/air_(?P<pol4>\d+)
    '''

    match = re.search(p_5pollut, data1)
    if match:
        today.pollution (match.group('pol'))
        day1.pollution (match.group('pol1'))
        day2.pollution (match.group('pol2'))
        day3.pollution (match.group('pol3'))
        day4.pollution (match.group('pol4'))

    p_5sun = r'''(?isx)
    Sun\sIndex
    .*?images/5day/sun_(?P<sun>\d+) # sun today
    .*?images/5day/sun_(?P<sun1>\d+)
    .*?images/5day/sun_(?P<sun2>\d+)
    .*?images/5day/sun_(?P<sun3>\d+)
    .*?images/5day/sun_(?P<sun4>\d+)
    '''
    match = re.search(p_5sun, data1)
    if match:
        today.uv( match.group('sun'))
        day1.uv( match.group('sun1'))
        day2.uv( match.group('sun2'))
        day3.uv( match.group('sun3'))
        day4.uv( match.group('sun4'))

    p_5wind = r'''(?isx)
    Key\sto\swind\ssymbols
    .*?images/5day/(?P<wind>\w+).*?<strong>(?P<speed>\d+)
    .*?images/5day/(?P<wind1>\w+).*?<strong>(?P<speed1>\d+)
    .*?images/5day/(?P<wind2>\w+).*?<strong>(?P<speed2>\d+)
    .*?images/5day/(?P<wind3>\w+).*?<strong>(?P<speed3>\d+)
    .*?images/5day/(?P<wind4>\w+).*?<strong>(?P<speed4>\d+)
    '''
    match = re.search(p_5wind, data1)
    # speed is in mph, we need kph
    if match:
        today.wind_speed(match.group('speed'), 'mph')
        today.wind(match.group('wind'))
        day1.wind_speed(match.group('speed1'), 'mph')
        day1.wind(match.group('wind1'))
        day2.wind_speed(match.group('speed2'), 'mph')
        day2.wind(match.group('wind2'))
        day3.wind_speed(match.group('speed3'), 'mph')
        day3.wind(match.group('wind3'))
        day4.wind_speed(match.group('speed4'), 'mph')
        day4.wind(match.group('wind4'))

##~     p_5sunrise = r'''(?isx)
##~     Sunrise
##~     .*?font\ssize="1">(?P<rise>[^\s]+) # sunrise today
##~     .*?font\ssize="1">(?P<rise1>[^\s]+)
##~     .*?font\ssize="1">(?P<rise2>[^\s]+)
##~     .*?font\ssize="1">(?P<rise3>[^\s]+)
##~     .*?font\ssize="1">(?P<rise4>[^\s]+)
##~     '''
##~     match = re.search(p_5sunrise, data1)
##~     if match:
##~         today.sunrise(match.group('rise'))
##~         day1.sunrise(match.group('rise1'))
##~         day2.sunrise(match.group('rise2'))
##~         day3.sunrise(match.group('rise3'))
##~         day4.sunrise(match.group('rise4'))
##~ 
##~     p_5sunset = r'''(?isx)
##~     Sunset
##~     .*?font\ssize="1">(?P<set>[^\s]+) # sunset today
##~     .*?font\ssize="1">(?P<set1>[^\s]+)
##~     .*?font\ssize="1">(?P<set2>[^\s]+)
##~     .*?font\ssize="1">(?P<set3>[^\s]+)
##~     .*?font\ssize="1">(?P<set4>[^\s]+)
##~     '''
##~     match = re.search(p_5sunset, data1)
##~     if match:
##~         today.sunset(match.group('set'))
##~         day1.sunset(match.group('set1'))
##~         day2.sunset(match.group('set2'))
##~         day3.sunset(match.group('set3'))
##~         day4.sunset(match.group('set4'))

    print today
    print day1
    print day2
    print day3
    print day4

    print '------------------------------fore'
    return now


def getWeather(widget):
    #setup data structures
    global weather, url, installed, metric_units, days, windy
    print "Current URL = " + url
    if installed[0] == '~':
        home = expanduser('~')
        installed = home + installed[1:]
    filename = installed + '/karamba_weather.html'
    try:
        data = urlopen(url).read()
        print data
        if data.find('BBCi') > 0:
            print "parsing BBC"
            weather = parseBBCWeatherData(data, days)
        else:
            print "parsing XML"
            weather = parseWeatherData(data, days)
        save = open(filename, 'w')
        print >> save, data
        save.close()

    except: # Maybe we're offline?
        try: # Try to load the saved weather data
            data =  open(installed + '/karamba_weather.html', 'r').read()
            if data.find('BBCi') > 0:
                weather = parseBBCWeatherData(data,  days)
            else:
                weather = parseWeatherData(data, days)
        except:
            print "Fatal Error; Cannot connect to site or load old data"

    weather.metric(metric_units)
    weather.reverse_wind_arrows(windy)

def getWebcamPic(widget):
    global filenames, nowebcams
    errorpath = "file:///" + path + "webcam_failed.png"
    filenames = []
    nowebcams = int(nowebcams)
    for i in range(nowebcams):
        try:
            picture = urlopen(webcamurls[i]).read()
        except:
            picture = urlopen(errorpath).read()
        filename = installed + '/webcampic_' + str(i) + '.jpg'
        save = open(filename, 'w')
        print >> save, picture
        save.close()
        filenames.append(filename)


def searchCodes(widget, searchurl):
    global searchsite, codespid, wcomcodespid, bbccodespid, url, configchange, werethreading
    import re
    if searchsite == 0:
        data = urlopen(searchurl).read()
        #print data
        start = data.find('<search')
        data = data[start:]
        p = re.compile('</search>')
        match = re.search(p, data)
        #print "match: " + str(match)
        if match:
            #print "no list"
            p = re.compile('</loc>')
            codes = p.findall(data)
            #print codes
            foundcity = city.replace('+',' ')
            if len(codes) == 1:
                p = re.compile('\D{4}\d{4}')
                codes = p.findall(data)
                os.system('kdialog --title "Location Selector" --msgbox "City found." &')
                url = url_base_addr + codes[0] + url_suffix
                karamba.writeConfigEntry(widget, "url", url)
                configchange = 1
                werethreading = 1
                widgetUpdated(widget)
            elif len(codes) > 1:
                p = re.compile('\D{4}\d{4}')
                q = re.compile('%s\D*</loc>' %(city), re.IGNORECASE | re.MULTILINE)
                codes = p.findall(data)
                cities = q.findall(data)
                print codes
                print cities
                for i in range(len(cities)):
                    cities[i] = cities[i].replace('</loc>', '')
                schcommand = ["kdialog", "--title", "Location Selector", "--radiolist", "Please select your location from the list:"]
                for i in range(len(codes)):
                    schcommand.append(codes[i])
                    schcommand.append(cities[i])
                    schcommand.append("off")
                wcomcodespid = karamba.executeInteractive(widget, schcommand)
        else:
            os.system('kdialog --title "Location Selector" --msgbox "City not found." &')
    if searchsite == 1:
        data = urlopen(searchurl).read()
        p = re.compile('<b>0</b>')
        match = p.findall(data)
        if len(match) == 0:
            start = data.find('cities:')
            stop = data.find('see:')
            data1 = data[start:stop]

            p = re.compile('\d{4}&links')
            codes = p.findall(data1)
            p = re.compile('%s\D*</b>' %(city), re.IGNORECASE | re.MULTILINE)
            cities = p.findall(data1)

            for i in range(len(cities)):
                cities[i] = cities[i].replace('</b>','')
            for i in range(len(codes)):
                codes[i] = codes[i].replace('&links','')

            schcommand = ["kdialog", "--title", "Location Selector", "--radiolist", "Please select your location from the list:"]

            for i in range(len(codes)):
                schcommand.append(codes[i])
                schcommand.append(cities[i])
                schcommand.append("off")

            bbccodespid = karamba.executeInteractive(widget, schcommand)

        else:
            os.system('kdialog --title "Location Selector" --msgbox "City not found." &')

def getSatPic(widget, incontinent, imap):
    global satfile
    if imap == 0:
        satpicurl = forecastprefix + forelocations[incontinent]
    else:
        satpicurl = prefix + countries[incontinent] + map_matrix[incontinent][imap] + '.gif'
    errorpath = "file:///" + path + "webcam_failed.png"
    try:
        picture = urlopen(satpicurl).read()
    except:
        picture = urlopen(errorpath).read()
    satfile = installed + '/satpic.jpg'
    save = open(satfile, 'w')
    print >> save, picture
    save.close()

##~ def getContinentPics(widget):
##~     global sumfile, pressfile, werethreading, continentdone
##~     errorpath = "file:///" + path + "webcam_failed.png"
##~     try:
##~         sumpic = urlopen(contmapsum[int(continent)]).read()
##~     except:
##~         sumpic = urlopen(errorpath).read()
##~     try:
##~         presspic = urlopen(contmappress[int(continent)]).read()
##~     except:
##~         presspic = urlopen(errorpath).read()
##~     sumfile = installed + '/summap.jpg'
##~     save = open(sumfile, 'w')
##~     print >> save, sumpic
##~     pressfile = installed + '/pressmap.jpg'
##~     save = open(pressfile, 'w')
##~     print >> save, presspic
##~     werethreading = 0
##~     if firsttime == 1:
##~         threadsfinished = 0
##~         while threadsfinished < 3:
##~             threadsfinished = weatherthread.isDone() + webcamthread.isDone() + satpicthread.isDone()
##~             time.sleep(1)
##~         continentdone = 1
##~         karamba.redrawWidget(widget)
##~         widgetUpdated(widget)

def createMapMenu(widget, cont_num):
    global webcammenu, satbuttmenu, mapmenu, selection
    print cont_num
    karamba.deleteMenu(widget, satbuttmenu)
    karamba.deleteMenu(widget, webcammenu)
    karamba.deleteMenu(widget, mapmenu)
    webcammenu = 0
    satbuttmenu = 0
    mapmenu = karamba.createMenu(widget)
    selection = {}
    for i in range(len(map_matrix[cont_num])):
        item = karamba.addMenuItem(widget, mapmenu, mapitems[i], "")
        #sorts out the id of the item clicked so it's useful
        #id seems to be negative value, convert to positive
        if item < 0:
            item = item * -1
        selection[item] = i
    print selection
    karamba.popupMenu(widget, mapmenu, 40, 10)
    
#this is called when you widget is initialized
def initWidget(widget):
    global mapmenu, weather, daytemps, daytext, daypic, daytempsrem, daytextrem, daypicrem, maintemp, pnum, hnum, rnum, situation, wind_speed, mainpic, windpic, ddon, dayshow, ddcon, buttonpressed, mainpicclick,  firsttime, background, metric_units, clock, url, mainback, clicky, daypicpos, animate, windpic_bg, windspeed, windy, webpicon, webcammenu, webcamurls, webcamlabels, nowebcams, webcamsfree, path, use_wcom, wcomcode, satpicurl, cambutt, satbutt, iconset, iconsetpath, satbuttmenu, citytext, range_font, font_range
    splash = karamba.createImage(widget, 10, 10, "splash.png")
    karamba.redrawWidget(widget)
    webcammenu = 0
    satbuttmenu = 0
    mapmenu = 0
    optchecked = mainback
    urltest = karamba.readConfigEntry(widget, "url")
    print urltest
    if (urltest != None):
        import re
        url = urltest
        p = re.compile('[A-Z]{4}\d{4}')
        code = p.findall(url)
        if len(code) == 0:
            p = re.compile('[0-9]{5}')
            code = p.findall(url)
        if len(code) > 0:
            use_wcom = 1
            wcomcode = code[0]
            satpicurl = "http://www.w3.weather.com/outlook/travel/map/" + wcomcode
    bkgdtest = karamba.readConfigEntry(widget, "bkgd")
    if (bkgdtest != None):
        mainback = bkgdtest
    webcamlabels = []
    webcamurls = []
    webcamsfree = []
    for i in range(5):
        getlabel = karamba.readConfigEntry(widget, "webcamlabel_" + str(i))
        geturl = karamba.readConfigEntry(widget, "webcamurl_" + str(i))
        if (getlabel != None) and (getlabel != ''):
            webcamlabels.append(getlabel)
        if (geturl != None) and (geturl != ''):
            webcamurls.append(geturl)
        else:
            webcamsfree.append(i)
    nowebcams = len(webcamlabels)
    if nowebcams == 0:
        nowebcams = 1
    iconsettest = karamba.readConfigEntry(widget, "iconset")
    if (iconsettest != None):
        iconset = iconsettest
        iconsetpath = iconsettest + '/'
    #print "mainback = " + mainback

    karamba.addMenuConfigOption(widget, "changeloc", "Enter your location's code")
    karamba.addMenuConfigOption(widget, "findwcom", "Find location on weather.com")
    karamba.addMenuConfigOption(widget, "findbbc", "Find location on BBC")
    karamba.addMenuConfigOption(widget, "addwebcamurl", "Add webcam URL")
    karamba.addMenuConfigOption(widget, "remwebcamurl", "Remove webcam URL")
    karamba.addMenuConfigOption(widget, "background", "Change the background")
    karamba.addMenuConfigOption(widget, "iconset", "Change the iconset")
    karamba.addMenuConfigOption(widget, "usemetric", "Use metric units (Celsius)")
    karamba.addMenuConfigOption(widget, "useclock", "Use a 24-hour clock")
    karamba.addMenuConfigOption(widget, "animation", "Disable icon animation")
    karamba.addMenuConfigOption(widget, "windoption", "Wind arrows show where wind blows to")
    karamba.addMenuConfigOption(widget, "range_font", "Use narrower font in forecast temp ranges")
    karamba.addMenuConfigOption(widget, "readme", "View readme/faq")
    metric_units = karamba.readMenuConfigOption(widget, "usemetric")
    clock = karamba.readMenuConfigOption(widget, "useclock")
    animate = karamba.readMenuConfigOption(widget, "animation")
    windy = karamba.readMenuConfigOption(widget, "windoption")
    range_font = karamba.readMenuConfigOption(widget, "range_font")
    if range_font == 1:
        font_range = "aria narrow"
    else:
        font_range = "verdana"
    #iconset = karamba.readMenuConfigOption(widget, "iconset")
    karamba.setMenuConfigOption(widget, "changeloc", "0")
    karamba.setMenuConfigOption(widget, "background", "0")
    karamba.setMenuConfigOption(widget, "readme", "0")
    karamba.setMenuConfigOption(widget, "findwcom", "0")
    karamba.setMenuConfigOption(widget, "findbbc", "0")
    karamba.setMenuConfigOption(widget, "addwebcamurl", "0")
    karamba.setMenuConfigOption(widget, "remwebcamurl", "0")
    karamba.setMenuConfigOption(widget, "iconset", "0")
    firsttime = 0
    buttonpressed = 0
    ddcon = 0
    dayshow = 5
    ddon = 0
    webpicon = 0
    path = karamba.getThemePath(widget)
    path = path + "/"
    os.chdir(path)
    #iconsetpath = "liquid/"
    #if iconset == 1:
        #iconsetpath = "um/"
    #os.system("pwd")

    #karamba.redrawWidgetBackground(widget)
    getWeather(widget)
    getWebcamPic(widget)
    karamba.deleteImage(widget, splash)
    karamba.redrawWidget(widget)
    background = karamba.createBackgroundImage(widget, 15, 15, 'background/' + mainback)
    windpic_bg = karamba.createBackgroundImage(widget, 192, 70, "wind_icons/wind_bg.png")
    #layout text
    p = karamba.createText(widget, 120, 58, 15, 12, "P:")
    prettyText(widget, 10, "verdana", p)
    tooltipp = karamba.createImage(widget, 120, 58, "tooltip.png")
    karamba.addImageTooltip(widget, tooltipp, "Air Pressure")
    h = karamba.createText(widget, 120, 70, 15, 12, "H:")
    prettyText(widget, 10, "verdana", h)
    tooltiph = karamba.createImage(widget, 120, 70, "tooltip.png")
    karamba.addImageTooltip(widget, tooltiph, "Humidity")
    r = karamba.createText(widget, 120, 82, 15, 12, "R:")
    prettyText(widget, 10, "verdana", r)
    tooltipr = karamba.createImage(widget, 120, 82, "tooltip.png")
    karamba.addImageTooltip(widget, tooltipr, "Relative Temp After Windchill")
    maintemp = karamba.createText(widget, 120, 33, 90, 30, weather.temperature())
    prettyText(widget, 20, "arial black", maintemp)
    situation = karamba.createText(widget, 37, 104, 200, 18, weather.sky())
    prettyText(widget, 12, "verdana", situation)

    mainpic = karamba.createImage(widget, 0, 0, iconsetpath + weather.icon_path())
    karamba.attachClickArea(widget, mainpic)

    satbutt = karamba.createImage(widget, 179, 36, "satbutton.png")
    karamba.attachClickArea(widget, satbutt)
    cambutt = karamba.createImage(widget, 203, 36, "webcambutton.png")
    karamba.attachClickArea(widget, cambutt)
    pnum = karamba.createText(widget, 135, 58, 75, 12, weather.pressure())
    prettyText(widget, 10, "verdana", pnum)
    hnum = karamba.createText(widget, 135, 70, 30, 12, weather.humidity())
    prettyText(widget, 10, "verdana", hnum)
    rnum = karamba.createText(widget, 135, 82, 30, 12, weather.relative_heat())
    prettyText(widget, 10, "verdana", rnum)
    windicon = weather.wind_icon_path()
    windspeed = karamba.createText(widget, 190, 94, 40, 15, weather.wind_speed())
    prettyText(widget, 9, "verdana", windspeed)
    windpic = karamba.createImage(widget, 192, 70, windicon)


    xstart = 37
    daytextrem = []
    daytempsrem = []
    daypicrem = []
    daypicpos = []
    testtime = int(weather.time()[:2])
    for i in range(0,5):
        if i == 0:
            if testtime >= 14:
                sep = weather.get_forecast(i).temp_range().find('/')
                daytext = karamba.createText(widget, xstart, 125, 40, 12, "2nite:")
                daytemps = karamba.createText(widget, xstart, 168, 40, 12, "Lo:" + weather.get_forecast(i).temp_range()[:sep])
            else:
                daytext = karamba.createText(widget, xstart, 125, 40, 12, "2day:")
                daytemps = karamba.createText(widget, xstart, 168, 40, 12, weather.get_forecast(i).temp_range())
        else:
            daytext = karamba.createText(widget, xstart, 125, 40, 12, weather.get_forecast(i).weekday() + ":")
            daytemps = karamba.createText(widget, xstart, 168, 40, 12, weather.get_forecast(i).temp_range())
        prettyText(widget, 10, "verdana", daytext)
        daytextrem.append(daytext)

        prettyText(widget, 10, font_range, daytemps)
        daytempsrem.append(daytemps)
        daypic = karamba.createImage(widget, (xstart + 5), 140, iconsetpath + weather.get_forecast(i).icon_path())
        karamba.attachClickArea(widget, daypic)
        daypicrem.append(daypic)
        daypicpos.append(xstart + 5)
        xstart = xstart + 39

    karamba.redrawWidget(widget)
#this is called everytime your widget is updated
#the update inverval is specified in the .theme filed
def widgetUpdated(widget):
    global weather, daytempsrem, daytextrem, daypicrem, maintemp, pnum, hnum, rnum, situation, wind_speed, mainpic, daytext, daytemps, daypics, windpic, windpic_bg, buttonpressed, backgroud, timegate, configchange, firsttime, clicky, daypicpos, windspeed, werethreading
    #karamba.createBackgroundImage(widget, 15, 15, "bg1.png")
    #karamba.redrawWidget(widget)
    #background = karamba.createBackgroundImage(widget, 15, 15, "bg1.png")
    #iconsetpath = "liquid/"
    #if iconset == 1:
        #iconsetpath = "um/"
    if buttonpressed == 0:
        if firsttime == 0:
            karamba.redrawWidgetBackground(widget)
            werethreading = 1
        #double check.  don't depend on INTERVAL
        #widgetUpdated can happen for other reasons
        #print (time.time() - timegate)
        if ((time.time() - timegate) > 3600.0) or (configchange == 1):
            if werethreading == 1:
                getThreadedData(widget)
            #print "Updating, button not pressed"
            #loadingtext = karamba.createText(widget, 120, 90, 200, 18, "Loading...")
            #prettyText(widget, 10, "verdana", loadingtext)
            #karamba.redrawWidget(widget)
            #karamba.redrawWidget(widget)
            #getWeather(widget)
            #print "Update weather..."
            #getWebcamPic(widget)
            #print "Update webcams..."
            #getContinentPics(widget)
            #print "Update continents..."
            #if use_wcom == 1:
                #getSatPic(widget)
                #print "Update satellite..."
            #karamba.deleteText(widget, loadingtext)
            #karamba.redrawWidget(widget)
            timegate = time.time()
            configchange = 0
        print '>>>>>>>>>>>>>>>>>>>>> widgetUpdated <<<<<<<<<<<<<<<<<<<<<<<<'
        karamba.changeText(widget, maintemp, weather.temperature())
        karamba.changeText(widget, situation, weather.sky())
        karamba.changeText(widget, windspeed, weather.wind_speed())

        if (ddon == 0) and (ddcon == 0):
            karamba.deleteImage(widget, mainpic)
            #karamba.deleteImage(widget, windpic_bg)
            #print today[6]
            mainpic = karamba.createImage(widget, 0, 0, iconsetpath + weather.icon_path())
            karamba.attachClickArea(widget, mainpic)
            windicon = weather.wind_icon_path()
            karamba.deleteImage(widget, windpic)
            karamba.redrawWidget(widget)
            windpic = karamba.createImage(widget, 192, 70, windicon)
            #windpic_bg = karamba.createImage(widget, 192, 70, "wind_icons/wind_bg.png")
            karamba.redrawWidget(widget)
            #print mainpic
            xstart = 37
            testtime = int(weather.time()[:2])
            for i in range(0,5):
                print 'aaaaaaaaaaaaaa===arraylen:'+ str(i)+ ':'
                if i == 0:
                    if testtime >= 14:
                        sep = weather.get_forecast(i).temp_range().find('/')
                        karamba.changeText(widget, daytextrem[i], "2nite:")
                        karamba.changeText(widget, daytempsrem[i], "Lo:" + weather.get_forecast(i).temp_range()[:sep])
                    else:
                        karamba.changeText(widget, daytextrem[i], "2day:")
                        karamba.changeText(widget, daytempsrem[i], weather.get_forecast(i).temp_range())
                else:
                    karamba.changeText(widget, daytextrem[i], weather.get_forecast(i).weekday() + ":")
                    karamba.changeText(widget, daytempsrem[i], weather.get_forecast(i).temp_range())
                prettyText(widget, 10, font_range, daytempsrem[i])
                #karamba.redrawWidget(widget)
                karamba.deleteImage(widget, daypicrem[i])
                daypic = karamba.createImage(widget, (xstart + 5), 140, iconsetpath + weather.get_forecast(i).icon_path())
                karamba.attachClickArea(widget, daypic)
                karamba.redrawWidget(widget)
                daypicrem[i] = daypic
                daypicpos[i] = (xstart + 5)
                xstart +=  39
        karamba.changeText(widget, pnum, weather.pressure())
        karamba.changeText(widget, hnum, weather.humidity())
        karamba.changeText(widget, rnum, weather.relative_heat())

        #karamba.changeText(widget, wind_speed, today[9])
        karamba.redrawWidget(widget)
    buttonpressed = 0
    firsttime = 1
    werethreading = 1

#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
    global clicky, dayshow, buttonpressed, oldday
    pass
#
#   if ((x > 0) and (x < 120) and (y > 0) and (y < 120)) and (button == 1):
#       if ddcon == 0:
#           if ddon == 1:
#               deleteDayDetail(widget, dayshow)
#               karamba.redrawWidget(widget)
#           createCurrentDetail(widget)
#           karamba.redrawWidget(widget)
#       else:
#           deleteCurrentDetail(widget)



#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass

def meterClicked(widget, meter, button):
    global dayshow, daypicrem, ddon, ddcon, buttonpressed, clicky, mapmenu, selection, webcammenu, satbuttmenu, satfilenames
    satfilenames = [installed + '/satpic.jpg', installed + '/summap.jpg', installed + '/pressmap.jpg']
    selection = {}
    print "meterclick: " + str(meter) + 'dayshow:' + str(dayshow) + ' ddon:' + str(ddon) + ' ddcon:' + str(ddcon)
    if (meter == cambutt) and (button == 2) and (webpicon == 1):
        karamba.changeImageIntensity(widget, webbypic, 0.4, 300)
        karamba.redrawWidget(widget)
        os.system('kuickshow %s &' %(filenames[webcamdisplay]))
    elif (meter == satbutt) and (button == 2) and (satpicon == 1):
        karamba.changeImageIntensity(widget, satpic, 0.4, 300)
        karamba.redrawWidget(widget)
        os.system('kuickshow %s &' %(satfile))
    elif (meter == cambutt) and (button == 1):
        if (webpicon == 0) and (satpicon == 0) and (ddcon == 0) and (ddon == 0):
            karamba.deleteMenu(widget, webcammenu)
            karamba.deleteMenu(widget, satbuttmenu)
            satbuttmenu = 0
            webcammenu = karamba.createMenu(widget)
            for i in range(len(webcamlabels)):
                item = karamba.addMenuItem(widget, webcammenu, webcamlabels[i], "")
                #sorts out the id of the item clicked so it's useful
                #id seems to be negative value, convert to positive
                if item < 0:
                    item = item * -1
                selection[item] = i
            karamba.popupMenu(widget, webcammenu, 40, 10)
        elif (webpicon == 1):
            removeWebcamPic(widget)
            karamba.redrawWidget(widget)
        else:
            pass
    elif (meter == satbutt) and (button == 1):
        if (webpicon == 0) and (satpicon == 0) and (ddcon == 0) and (ddon == 0):
            karamba.deleteMenu(widget, satbuttmenu)
            karamba.deleteMenu(widget, webcammenu)
            webcammenu = 0
            mapmenu = 0
            satbuttmenu = karamba.createMenu(widget)
            for i in range(len(continentitems)):
                item = karamba.addMenuItem(widget, satbuttmenu, continentitems[i], "")
                #sorts out the id of the item clicked so it's useful
                #id seems to be negative value, convert to positive
                if item < 0:
                    item = item * -1
                selection[item] = i
            karamba.popupMenu(widget, satbuttmenu, 40, 10)
        elif (webpicon == 1):
            removeWebcamPic(widget)
            karamba.redrawWidget(widget)
        elif (satpicon == 1):
            removeSatPic(widget)
            karamba.redrawWidget(widget)

    elif (meter == mainpic) and (button == 1):
        if ddcon == 0:
            if ddon == 1:
                deleteDayDetail(widget, dayshow)
                karamba.redrawWidget(widget)
            elif webpicon == 1:
                removeWebcamPic(widget)
                karamba.redrawWidget(widget)
            elif satpicon == 1:
                removeSatPic(widget)
                karamba.redrawWidget(widget)
            createCurrentDetail(widget)
            karamba.redrawWidget(widget)
        else:
            deleteCurrentDetail(widget)
    #elif (meter == mainpic) and (button == 2):
        #if webpicon == 0:
            #if ddcon == 1:
                #deleteCurrentDetail(widget)
                #karamba.redrawWidget(widget)
            #elif ddon == 1:
                #deleteDayDetail(widget, dayshow)
                #karamba.redrawWidget(widget)
            #showWebcamPic(widget)
            #karamba.redrawWidget(widget)
        #else:
            #removeWebcamPic(widget)
    else:
        for i in range(0,5):
            if (meter == daypicrem[i]) and (button == 1) and (dayshow != i):
                print "buttonpressed: day" + str(i)
                if ddon == 1:
                    deleteDayDetail(widget, dayshow)
                    karamba.redrawWidget(widget)
                elif ddcon == 1:
                    deleteCurrentDetail(widget)
                    karamba.redrawWidget(widget)
                elif webpicon == 1:
                    removeWebcamPic(widget)
                    karamba.redrawWidget(widget)
                elif satpicon == 1:
                    removeSatPic(widget)
                    karamba.redrawWidget(widget)
                createDayDetail(widget, i)
                karamba.redrawWidget(widget)
                dayshow = i
                break
            if (meter == daypicrem[i]) and (button == 1) and (dayshow == i):
                print "deleting ... buttonpressed: day" + str(i)
                if ddon == 1:
                    deleteDayDetail(widget, dayshow)
                    karamba.redrawWidget(widget)
                elif ddon == 0:
                    createDayDetail(widget, i)
                    karamba.redrawWidget(widget)
                elif ddcon == 1:
                    deleteCurrentDetail(widget)
                    karamba.redrawWidget(widget)
                elif webpicon == 1:
                    removeWebcamPic(widget)
                    karamba.redrawWidget(widget)
                elif satpicon == 1:
                    removeSatPic(widget)
                    karamba.redrawWidget(widget)
                dayshow = i
                break
    buttonpressed = 1

def menuItemClicked(widget, menu, id):
    global webcamdisplay, whichone, whichcontinent
    if menu == webcammenu:
        index = id * -1
        webcamdisplay = selection[index]
        showWebcamPic(widget)
    if menu == satbuttmenu:
        index = id * -1
        whichone = selection[index]
        print whichone
        createMapMenu(widget, whichone)
        whichcontinent = whichone
    if menu == mapmenu:
        index = id * -1
        whichone = selection[index]
        print whichone
        getSatPic(widget, whichcontinent, whichone)
        showSatPic(widget)
#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
    global clock, metric_units, url, url_base_addr, configchange, mainback, background, animate, bkgdpid, locpid, wcompid, bbcpid, searchsite, wcomcodespid, bbccodespid, windy, webcamurlpid, remwebcampid, webcamlabelpid, iconset, contpid, werethreading, iconpid, range_font, font_range
    contpid = None
    bkgdpid = None
    locpid = None
    wcompid = None
    bbcpid = None
    wcomcodespid = None
    bbccodespid = None
    webcampid = None
    remwebcampid = None
    webcamurlpid = None
    webcamlabelpid = None
    iconpid = None
    print "Config Clicked"
    iconcommand = ["kdialog", "--title", "Iconset Selector", "--radiolist", "Please select an iconset to use:", "liquid", "Original Liquid Icons", "off", "um", "Umicons", "off", "weather.com", "Weather.com Icons", "off"]
    bkgdcommand = ["kdialog", "--title", "Background Selector", "--radiolist", "Please select a background to use:", "bg0.png", "Stippled White Raised", "off", "bg1.png", "Plain White Raised (1)", "off", "bg2.png", "Plain White Flat (1)", "off", "bg3.png", "Plain White Raised (2)", "off", "bg4.png", "Plain White Flat (2)", "off", "bg5.png", "Plain Gray Raised (1)", "off", "bg6.png", "Plain Gray Flat (1)", "off", "bg7.png", "Plain Gray Raised (2)", "off", "bg8.png", "Plain Gray Flat (2)", "off"]
    loccommand = ["kdialog", "--title", "Location Configuration", "--inputbox", "Please enter your US Zip Code, Weather.com Location ID\n or BBC 4 digit Weather Code (from www.bbc.co.uk/weather):"]
    findwcomcommand = ["kdialog", "--title", "Find location on weather.com", "--inputbox", "Enter the name of the city or town you want to search for:"]
    findbbccommand = ["kdialog", "--title", "Find location on BBC", "--inputbox", "Enter the name of the city or town you want to search for:"]
    addwebcamurlcommand1 = ["kdialog", "--title", "Add webcam URL", "--inputbox", "Enter a URL that points to the desired webcam:"]
    if (key == "background"):
        karamba.setMenuConfigOption(widget, "background", "0")
        #pipe = os.popen('kdialog --title "Background Selector" --radiolist "Please select a background to use:" bg0.png "Stippled White Raised" "off" bg1.png "Plain White Raised (1)" "off" bg2.png "Plain White Flat (1)" "off" bg3.png "Plain White Raised (2)" "off" bg4.png "Plain White Flat (2)" "off" bg5.png "Plain Gray Raised (1)" "off" bg6.png "Plain Gray Flat (1)" "off" bg7.png "Plain Gray Raised (2)" "off" bg8.png "Plain Gray Flat (2)" "off"', 'r')
        bkgdpid = karamba.executeInteractive(widget, bkgdcommand)
        #changeback = pipe.readline()
        #changeback = changeback.strip('\n')
        #print changeback
        #mainback = changeback
        #karamba.writeConfigEntry(widget, "bkgd", mainback)
        #karamba.deleteImage(widget, background)
        #background = karamba.createBackgroundImage(widget, 15, 15, mainback)
        #karamba.redrawWidgetBackground(widget)

    elif (key == "animation"):
        animate = karamba.readMenuConfigOption(widget, "animation")

    elif (key == "usemetric"):
        metric_units = karamba.readMenuConfigOption(widget, "usemetric")
        configchange = 1
        werethreading = 1
        widgetUpdated(widget)

    elif (key == "useclock"):
        clock = karamba.readMenuConfigOption(widget, "useclock")
        configchange = 1
        werethreading = 1
        widgetUpdated(widget)

    elif (key == "windoption"):
        windy = karamba.readMenuConfigOption(widget, "windoption")
        configchange = 1
        widgetUpdated(widget)

    elif (key == "range_font"):
        range_font = karamba.readMenuConfigOption(widget, "range_font")
        if range_font == 1:
            font_range = "arial narrow"
        else:
            font_range = "verdana"
        configchange = 0
        widgetUpdated(widget)

    elif (key == "iconset"):
        karamba.setMenuConfigOption(widget, "iconset", "0")
        iconpid = karamba.executeInteractive(widget, iconcommand)

    elif (key == "changeloc"):
        karamba.setMenuConfigOption(widget, "changeloc", "0")
        #pipe = os.popen("kdialog --title \"Location Configuration\" --inputbox \"Please enter your US Zip Code, Weather.com Location ID\n or BBC 4 digit Weather Code (from www.bbc.co.uk/weather):\"", 'r')
        locpid = karamba.executeInteractive(widget, loccommand)
        #rawarea = pipe.readlines()
        #zipcode = rawarea[0].strip()
        #if (zipcode != ""):
            #if len(zipcode) > 4 :
                #url = url_base_addr + zipcode
            #else :
                #url= url_bbc_base_addr + zipcode + url_bbc_add_addr
            #karamba.writeConfigEntry(widget, "url", url)
            #widgetUpdated(widget)
        #configchange = 1

    elif (key == "readme"):
        karamba.setMenuConfigOption(widget, "readme", "0")
        os.system('kfmclient openURL ./readme_faq.html &')

    elif (key == "findwcom"):
        karamba.setMenuConfigOption(widget, "findwcom", "0")
        wcompid = karamba.executeInteractive(widget, findwcomcommand)

    elif (key == "findbbc"):
        karamba.setMenuConfigOption(widget, "findbbc", "0")
        bbcpid = karamba.executeInteractive(widget, findbbccommand)

    elif (key == "addwebcamurl"):
        karamba.setMenuConfigOption(widget, "addwebcamurl", "0")
        if len(webcamsfree) == 0:
            os.system('kdialog --title "Error!" --msgbox "You already have 5 webcams loaded.\nYou need to delete one first." &')
        else:
            webcamurlpid = karamba.executeInteractive(widget, addwebcamurlcommand1)

    elif (key == "remwebcamurl"):
        karamba.setMenuConfigOption(widget, "remwebcamurl", "0")
        remwebcamcommand = ["kdialog", "--title", "Remove A Webcam", "--radiolist", "Choose the webcam to be removed:"]
        for i in range(nowebcams):
            remwebcamcommand.append(str(i))
            remwebcamcommand.append(webcamlabels[i])
            remwebcamcommand.append("off")
        remwebcampid = karamba.executeInteractive(widget, remwebcamcommand)

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
    global bkgdpid, locpid, wcompid, bbcpid, mainback, background, configchange, url, url_base_addr, url_bbc_base_addr, searchsite, city, url_webcam, webcamlabelpid, nowebcams, webcamlabels, webcamurls, webcamsfree, use_wcom, wcomcode, satpicurl, werethreading, iconset, iconsetpath

    if pid == iconpid:
        iconset = output.replace('\n','')
        iconsetpath = iconset + '/'
        karamba.writeConfigEntry(widget, "iconset", iconset)
        configchange = 0
        widgetUpdated(widget)

    elif pid == bkgdpid:
        mainback = output.replace('\n','')
        karamba.writeConfigEntry(widget, "bkgd", mainback)
        karamba.deleteImage(widget, background)
        background = karamba.createBackgroundImage(widget, 15, 15, 'background/' + mainback)
        karamba.redrawWidgetBackground(widget)

    elif pid == locpid:
        zipcode = output.replace('\n','')
        if (zipcode != ""):
            if len(zipcode) > 4 :
                url = url_base_addr + zipcode + url_suffix
                use_wcom = 1
                wcomcode = zipcode
                satpicurl = "http://www.w3.weather.com/outlook/travel/map/" + zipcode
            else:
                url= url_bbc_base_addr + zipcode + url_bbc_add_addr
                use_wcom = 0
            karamba.writeConfigEntry(widget, "url", url)
            widgetUpdated(widget)
        configchange = 1
        werethreading = 1
        widgetUpdated(widget)

    elif pid == wcompid:
        city = output.replace('\n','')
        city1 = quote_plus(city)
        wcomurl =  "http://xoap.weather.com/search/search?where=" + city1
        #wcomurl = quote_plus(wcomurl)
        searchsite = 0
        searchCodes(widget, wcomurl)

    elif pid == bbcpid:
        city = output.replace('\n','')
        city1 = quote_plus(city)
        bbcurl =  'http://www.bbc.co.uk/cgi-bin/weather/world_location/location.cgi?wlocation=' + city1 + '&imageField.x=0&imageField.y=0'
        searchsite = 1
        searchCodes(widget, bbcurl)

    elif pid == wcomcodespid:
        zipcode = output.replace('\n','')
        print zipcode
        url = url_base_addr + zipcode + url_suffix
        karamba.writeConfigEntry(widget, "url", url)
        use_wcom = 1
        wcomcode = zipcode
        satpicurl = "http://www.w3.weather.com/outlook/travel/map/" + zipcode
        configchange = 1
        werethreading = 1
        widgetUpdated(widget)

    elif pid == bbccodespid:
        zipcode = output.replace('\n','')
        url= url_bbc_base_addr + zipcode + url_bbc_add_addr
        karamba.writeConfigEntry(widget, "url", url)
        use_wcom = 0
        configchange = 1
        werethreading = 1
        widgetUpdated(widget)

    elif pid == webcamurlpid:
        zipcode = output.replace('\n','')
        if len(zipcode) != 0:
            from mimetypes import guess_type
            try:
                test = urlopen(zipcode).read()
                save = open(installed + '/test.jpg', 'w')
                print >> save, test
                save.close()
                imagetest = guess_type(installed + '/test.jpg')
                imagetester = imagetest[0].find('image')
                if imagetester == 0:
                    errorflag = 0
                else:
                    errorflag = 1
                if errorflag == 0:
                    #url_webcam = zipcode
                    nowebcams = len(webcamlabels)
                    if (nowebcams == None):
                        nowebcams = 0
                    karamba.writeConfigEntry(widget, "webcamurl_" + str(webcamsfree[0]), zipcode)
                    webcamurls.append(zipcode)
                    #configchange = 1
                    addwebcamlabelcommand = ["kdialog", "--title", "Add Webcam Label", "--inputbox", "Enter a label for this webcam:"]
                    webcamlabelpid = karamba.executeInteractive(widget, addwebcamlabelcommand)
                else:
                    #print "gone haywire"
                    os.system('kdialog --title "Error!" --msgbox "URL does not point to an image" &')

            except:
                #print "gone haywire except"
                os.system('kdialog --title "Error!" --msgbox "URL does not point to an image" &')

    elif pid == webcamlabelpid:
        labely = output.replace('\n','')
        karamba.writeConfigEntry(widget, "webcamlabel_" + str(webcamsfree[0]), labely)
        webcamlabels.append(labely)
        nowebcams = len(webcamlabels)
        webcamsfree.pop(0)
        #loadingtext = karamba.createText(widget, 120, 90, 200, 18, "Loading...")
        #prettyText(widget, 10, "verdana", loadingtext)
        #karamba.redrawWidget(widget)
        #getWebcamPic(widget)
        #karamba.deleteText(widget, loadingtext)
        #karamba.redrawWidget(widget)
        configchange = 1
        werethreading = 1
        widgetUpdated(widget)

    elif pid == remwebcampid:
        index = output.replace('\n','')
        index = int(index)
        webcamurls.pop(index)
        webcamlabels.pop(index)
        nowebcams = len(webcamlabels)
        karamba.writeConfigEntry(widget, "webcamurl_" + str(index), '')
        karamba.writeConfigEntry(widget, "webcamlabel_" + str(index), '')
        webcamsfree.append(index)
        #loadingtext = karamba.createText(widget, 120, 90, 200, 18, "Loading...")
        #prettyText(widget, 10, "verdana", loadingtext)
        #karamba.redrawWidget(widget)
        #getWebcamPic(widget)
        #karamba.deleteText(widget, loadingtext)
        #karamba.redrawWidget(widget)
        configchange = 1
        werethreading = 1
        widgetUpdated(widget)
