############################################################################
#    Copyright (C) 2004 by Bobby Z.                                        #
#    ziuchkov@uiuc.edu                                                     #
#    Modifications by Brad De la Salle                                     #
#    brad-d"at"canada.com                                                  #
#                                                                          #
#    This program is free software; you can redistribute it and or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

import karamba
import os
import string
UPDATEDELAY = 1000

def initWidget(widget):
   # Globals and button handles
   global updatedelay
   updatedelay = UPDATEDELAY # This is to offset the player time correctly
   global player, artist, song, timetextpos, timetextlen, bar, barW, barH, playerpid, seekpid
   global offset, prevbtn, prevskbtn, playbtn, pausebtn, stopbtn, nextskbtn, nextbtn, plistbtn, show

   playerpid = 0
   seekpid = 0
   show = 0
   karamba.addMenuConfigOption(widget, "playercfg", "Change the monitored media player")
   karamba.addMenuConfigOption(widget, "seekcfg", "Change the seek offset")
   karamba.setMenuConfigOption(widget, "playercfg", "0")
   karamba.setMenuConfigOption(widget, "seekcfg", "0")

   # Create the modular Player object and set the seek offset
   player = getPlayer(widget)
   offset = getSeekOffset(widget)

   # Grab button images from theme file and attach them
   prevbtn = karamba.getThemeImage(widget, 'SEEKPREV')
   prevskbtn = karamba.getThemeImage(widget, 'SEEKPREVOFFSET')
   playbtn = karamba.getThemeImage(widget, 'PLAY')
   pausebtn = karamba.getThemeImage(widget, 'PAUSE')
   stopbtn = karamba.getThemeImage(widget, 'STOP')
   nextskbtn = karamba.getThemeImage(widget, 'SEEKNEXTOFFEST')
   nextbtn = karamba.getThemeImage(widget, 'SEEKNEXT')
   plistbtn = karamba.getThemeImage(widget, 'PLAYLIST')
   karamba.attachClickArea(widget, prevbtn)
   karamba.attachClickArea(widget, prevskbtn)
   karamba.attachClickArea(widget, playbtn)
   karamba.attachClickArea(widget, pausebtn)
   karamba.attachClickArea(widget, stopbtn)
   karamba.attachClickArea(widget, nextskbtn)
   karamba.attachClickArea(widget, nextbtn)
   karamba.attachClickArea(widget, plistbtn)

   # Grab text handles from theme file
   artist = karamba.getThemeText(widget,'ARTISTSONG')
   # song = karamba.getThemeText(widget,'SONG')
   timetextpos = karamba.getThemeText(widget,'POSTIME')
   timetextlen = karamba.getThemeText(widget,'LENGTHTIME')

   # Grab bar info from theme file
   bar = karamba.getThemeBar(widget, 'PROGRESS_BAR')
   karamba.createSystray(widget, 880, 11, 120, 48)

def widgetUpdated(widget):
   songtext = str(player.getSong())

   #Split song string into artist and title values and update widget
   # artisttext = songtext[0:string.find(str(songtext), "- ")]
   # titletext = songtext[(string.find(songtext, "- ")+1):]
   # karamba.changeText(widget, artist, artisttext)
   # karamba.changeText(widget, song, titletext)
   songtext = str(player.getSong())
   karamba.changeText(widget, artist, songtext)
   updateTime(widget)

def menuOptionChanged(widget, key, value):
   # Called when a user clicks on a menu config option
   global playerpid, seekpid

   # The following uses kdialog and creates a dynamic list of values according to the available
   # Player class modules in the theme dir
   playermods = findPlayerModules(widget)
   playercmd = ["kdialog", "--title", "Player Configuration", "--radiolist", \
      "Please select your media player:"]
   # Generate the list of available modules for kdialog
   for idx in range(0, len(playermods), 2):
      playercmd.append(playermods[idx])
      playercmd.append(playermods[idx+1])
      playercmd.append("off")

   seekcmd = ["kdialog", "--title", "Seek Offset Configuration", "--inputbox", \
       "Please enter the amound of time (seconds) you would like the seek buttons to jump:"]

   if (key == "playercfg"):
      karamba.setMenuConfigOption(widget, "playercfg", "0")
      playerpid = karamba.executeInteractive(widget, playercmd)

   if (key == "seekcfg"):
      karamba.setMenuConfigOption(widget, "seekcfg", "0")
      seekpid = karamba.executeInteractive(widget, seekcmd)


def commandOutput(widget, pid, output):
   # This is called a "karamba.executeInteractive()" call returns a value, i.e. player/offset change
   global playername, playerpid, seekpid, seekoffset, player

   if pid == playerpid:
      playername = output.replace('\n','')
      karamba.writeConfigEntry(widget, "playername", playername)
      player = getPlayer(widget)

   elif pid == seekpid:
      seekoffset = output.replace('\n','')
      karamba.writeConfigEntry(widget, "seekoffset", seekoffset)

   widgetUpdated(widget)


def updateTime(widget):
   if player.getPosition() >= 0:
      position = player.getPosition() + updatedelay / 1000 # Adjust player time according to delay
   else:
      position = 0
   if player.getLength() >= 0:
      length = player.getLength()
   else:
      length = 0
   karamba.setBarMinMax(widget, bar, 0, length)
   karamba.setBarValue(widget, bar, position)
   posstring = "%d:%02d" %(divmod(position, 60))
   lenstring = "%d:%02d" %(divmod(length, 60))
   karamba.changeText(widget, timetextpos, posstring)
   karamba.changeText(widget, timetextlen, lenstring)

def meterClicked(widget, meter, button):
   global show
   offset = getSeekOffset(widget)
   if meter == prevbtn:
      player.seekPrev()
   elif meter == prevskbtn:
      player.seekPrevOffset(offset)
   elif meter == playbtn:
      player.cmdPlay()
   elif meter == pausebtn:
      player.cmdPause()
   elif meter == stopbtn:
      player.cmdStop()
   elif meter == nextskbtn:
      player.seekNextOffset(offset)
   elif meter == nextbtn:
      player.seekNext()
   elif meter == plistbtn:
      if show == 0:
         player.listShow()
         show = 1
      else:
         player.listHide()
         show = 0

def widgetClicked(widget, x, y, button):
   # This function is called every time a user clicks anywhere on the widget, so we're using this
   # to determine if a user clicks on our progress bar, and if so we'll seek to a new song position.
   # "x" and "y" are the mouse coordinates passed to the function when the widget is clicked.
   barx, bary = karamba.getBarPos(widget, bar)
   sizex, sizey = karamba.getBarSize(widget, bar)
   if (barx <= x <= barx + sizex) and (bary <= y <= bary + sizey):
      # We need to change the behavior if the bar is vertically oriented
      if (karamba.getBarVertical(widget, bar)):
         percent = float(1.0 - (1.0 * (y - bary) / sizey))
      else:
         percent = float(1.0 * (x - barx) / sizex)
      length = float(player.getLength())
      position = int(length * percent)
      player.seekTime(position)
      widgetUpdated(widget)

def getPlayer(widget):
   # This function is special because it allows people to easily write their own
   # player classes (especially dcop-enabled players with the included helper functions
   # and examples) and the script will create the appropriate player object
   playername = karamba.readConfigEntry(widget, "playername")
   playermods = findPlayerModules(widget)
   if playername != None:
      try:
         idx = playermods.index(playername)
         playermodule = playermods[idx]
         playerclass = playermods[idx+1]
      except ValueError:
         playermodule = playermods[0]
         playerclass = playermods[1]
   else:
      playermodule = playermods[0]
      playerclass = playermods[1]
   player = createPlayerObject(playermodule, playerclass)
   return player

def createPlayerObject(playermodule, playerclass):
   # This function creates the dynamic code objects needed to import the proper player modules.
   codeobj = compile ("import %s" %(playermodule), '<string>', 'exec')
   exec(codeobj)
   codeobj = compile ("player = %s.%s()" %(playermodule, playerclass), '<string>', 'exec')
   exec(codeobj)
   return player

def findPlayerModules(widget):
   # This function searchs the themepath directory for all .py files that extend the Player class
   playermods = []
   themepath = karamba.getThemePath(widget)
   results = os.popen("cd %s && grep -r --include '*.py' --exclude 'panel.py*' \
      'class .*(player.Player)' ./" %(themepath), 'r')
   lines = results.readlines()
   # Grab <modulename>.<classname> from the grep results
   for entry in lines:
      playermodule = entry[2:string.find((entry), ".py")]
      playerclass = entry[(string.find(entry, "class ")+6):string.find(entry, "(player.Player)")]
      playermods.append(playermodule)
      playermods.append(playerclass)
   results.close()
   return playermods


def getSeekOffset(widget):
   global offset
   offset = karamba.readConfigEntry(widget, "seekoffset")
   if offset == None:
      offset = 5
   return offset