######################################################################
# Programmer: Ryan Nickell
# Filename: doomCpu.py
# Date: June 13, 2004
# Release version: 0.4
# Notes: This is a re-write of a Dock App called 'wmdl'. 
#  It takes the health images from the game Doom and 
#  displays them on your dekstop using superkaramba.
#  The face gets more bloody based on your cpu's load
#  on your computer.
#
# Original author of WMdl is Ben Cohen, buddog@aztec.asu.edu
# and it can be downloaded either at rpmfind.net or by visiting
# http://web.cs.mun.ca/~gstarkes/wmaker/dockapps/
#
# Requirements:
#	Qt: 3.1.1
#	KDE: 3.1.0
#	Superkaramba: 0.32 (it may work on older versions, untested)
#
######################################################################
import karamba

images0=["images/face_0c.xpm","images/face_0l.xpm","images/face_0r.xpm"]
images1=["images/face_1c.xpm","images/face_1l.xpm","images/face_1r.xpm"]
images2=["images/face_2c.xpm","images/face_2l.xpm","images/face_2r.xpm"]
images3=["images/face_3c.xpm","images/face_3l.xpm","images/face_3r.xpm"]
images4=["images/face_4c.xpm","images/face_4l.xpm","images/face_4r.xpm"]
names=[]
preProg,prevIdle = 0,0
i = 0 #rotation of images when updated
debug = 0
fontSize = 8
font = "arial"
fontColor = "white"
shadow = 0
kBgPid = 0
kFontPid = 0
kNiceHonored = 0
debug = 0
no_nice = 1
################################################
# cpuLoad(): generates the cpu load by querying
# /proc/stat and calculating the difference
################################################
def cpuLoad():
	global cpu,preProg,prevIdle,debug,ignoreNice
        try:
            #kernel 2.4 format
            (junk,user,nice,system,idle) = file('/proc/stat').readline().split()
        except:
            #kernel 2.6 format
            (junk,user,nice,system,idle,junk2,junk3,junk4) = file('/proc/stat').readline().split()
	#credit to David A. Panariti for the nice'd processes patch below
	if(ignoreNice == 'False'):
		# ignore nice, pretend it's idle time to keep all time
		# accounted for.
		idle = long(idle) + long(nice)
		nice = '0'
	prog = long(user)+long(nice)+long(system)
	idle = long(idle)
	#this had to be abs()'d b/c the moment when it changes
	#from checking the menu for nice'd processes, it would
	#get thrown way off for a second
	idleChange = abs(idle - prevIdle)
	progChange = abs(prog - preProg)
	load = progChange * 100.0 / (idleChange + progChange)
	preProg, prevIdle = prog, idle
	cpu = load
	if(debug >=1):
		print "cpuLoad(): load:",load,"idleChange:",idleChange,"progChange:",progChange
		print "cpuLoad(): junk:",junk,"user",user,"nice:",nice,"system:",system,"idle:",idle
		print "cpuLoad(): cpu=",progChange,"/",idleChange,"+",progChange,"=>",load

####################################################
#text modification
#More colors will be added later.. right now we have
#support for: white, black, blue, red, green, yellow, purple, tan
####################################################
def prettyText(widget, size, font, textname, fontColor, shadow):
	karamba.changeTextSize(widget, textname, size)
	karamba.changeTextFont(widget, textname, font)
	if(fontColor == "white"):
		karamba.changeTextColor(widget, textname, 255, 255, 255)
	elif(fontColor == "black"):
		karamba.changeTextColor(widget, textname, 0, 0, 0)
	elif(fontColor == "blue"):
		karamba.changeTextColor(widget, textname, 0, 0, 255)
	elif(fontColor == "red"):
		karamba.changeTextColor(widget, textname, 255, 0, 0)
	elif(fontColor == "green"):
		karamba.changeTextColor(widget, textname, 0, 255, 0)
	elif(fontColor == "yellow"):
		karamba.changeTextColor(widget, textname, 255, 255, 0)
	elif(fontColor == "purple"):
		karamba.changeTextColor(widget, textname, 127, 0, 127)
	elif(fontColor == "tan"):
		karamba.changeTextColor(widget, textname, 127, 127, 0)
	else:
		karamba.changeTextColor(widget, textname, 0,0,0)
	karamba.changeTextShadow(widget, textname, shadow)

###############################################
# initWidget(): this is called when you widget 
#  is initialized, in our case it takes care
#  of loading the background and first image
#  then transfers control over to widgetUpdated()
###############################################
def initWidget(widget):
	global image,i,system,cpu,background,debug,fontColor,ignoreNice
	cpu = 0
	if (karamba.readConfigEntry(widget, "firstTime") != "No"):
		karamba.writeConfigEntry(widget, "firstTime", "No")
		karamba.writeConfigEntry(widget, "background", "images/backdrop.xpm")
		karamba.writeConfigEntry(widget, "fontColor","white")
		karamba.writeConfigEntry(widget, "niceHonored","withOutNice")
	#menu options in the configure right click menu
	#to add icons and remove icons
	karamba.addMenuConfigOption(widget, "backgroundColor", "Background Frame Color")
	karamba.addMenuConfigOption(widget, "fontColor", "Font Color")
	karamba.addMenuConfigOption(widget, "ignoreNice", "Ignore nice'd Processes")
	karamba.setMenuConfigOption(widget, "backgroundColor", "0")
	karamba.setMenuConfigOption(widget, "fontColor", "0")
	ignoreNice = karamba.readConfigEntry(widget, "ignoreNice")
	niceValue = karamba.readMenuConfigOption(widget, "ignoreNice")
	if( debug >= 1 ):
		print "initWidget(): ignoreNice: ",ignoreNice
		print "initWidget(): niceValue: ",niceValue

	bg = karamba.readConfigEntry(widget,"background")
	background = karamba.createBackgroundImage(widget, 0, 0, bg)

	image = karamba.createImage(widget,5,5,images0[i])
	text = "%03i" % cpu
	system = karamba.createText(widget, 13, 50, 40, 20, "cpu%: "+text)
	fontColor = karamba.readConfigEntry(widget,"fontColor")
	prettyText(widget, fontSize, font, system, fontColor, shadow)
	karamba.redrawWidget(widget)

###############################################
# widgetUpdated(): this is called everytime your
#  widget is updated the update inverval is
#  specified in the doomCpu.theme file
###############################################
def widgetUpdated(widget):
	global cpu,image,i,debug,system
	cpuLoad()
	karamba.deleteText(widget, system)
	if(i >=3):
		i = 0
	#use global variable i to randomize the 3 images to display
	if(cpu < 10):
		karamba.deleteImage(widget,image)
		image = karamba.createImage(widget,5,5,images0[i])
	elif(cpu < 30):
		karamba.deleteImage(widget,image)
		image = karamba.createImage(widget,5,5,images1[i])
	elif(cpu < 50):
		karamba.deleteImage(widget,image)
		image = karamba.createImage(widget,5,5,images2[i])
	elif(cpu < 75):
		karamba.deleteImage(widget,image)
		image = karamba.createImage(widget,5,5,images3[i])
	elif(cpu <= 100):
		karamba.deleteImage(widget,image)
		image = karamba.createImage(widget,5,5,images4[i])
	else:
		karamba.deleteImage(widget,image)
		image = karamba.createImage(widget,5,5,"images/face_god.xpm")
	i += 1
	if(debug >=1):
		print "widgetUpdated(): cpu:",cpu
	text = "%03i" % cpu
	system = karamba.createText(widget, 13, 50, 40, 20, "cpu%: "+text)
	prettyText(widget, fontSize, font, system, fontColor, shadow)
	karamba.redrawWidget(widget)

##################################################################
#menuOptionChanged() detects what you selected in the right click
#configure theme menu.
# backgroundColor - brings up kdialog to select the color of the frame
##################################################################
def menuOptionChanged(widget, key, value):
	global kBgPid,kFontPid,kNiceHonored,ignoreNice
	if (key == "backgroundColor"):
		karamba.setMenuConfigOption(widget, "backgroundColor", "0")
		kdialog = ["kdialog", "--radiolist", "Pick your frame color:", "0" ,"original", "on", "1", "blue", "off", "2", "red","off","3","green","off","4","none","off"]
		kBgPid = karamba.executeInteractive(widget,kdialog)
		if( debug >= 1 ):
			print "menuOptionChanged(): backgroundColor-",kBgPid

	if (key == "fontColor"):
		karamba.setMenuConfigOption(widget, "fontColor","0")
		kdialog = ["kdialog","--radiolist","Pick your font color:","white","white","on","black","black","off","blue","blue","off","red","red","off","green","green","off","yellow","yellow","off","purple","purple","off","tan","tan","off"]
		kFontPid =  karamba.executeInteractive(widget,kdialog)
		if( debug >= 1 ):
			print "menuOptionChanged(): fontColor-",kFontPid

	if (key == "ignoreNice"):
		niceValue = karamba.readMenuConfigOption(widget,"ignoreNice")
		if( niceValue == 0 ):
			ignoreNice = 'True'
			karamba.writeConfigEntry(widget,"ignoreNice","True")
		elif( niceValue == 1 ):
			ignoreNice = 'False'
			karamba.writeConfigEntry(widget,"ignoreNice","False")
		if( debug >= 1 ):
			print "menuOptionChanged(): ignoreNice: ",ignoreNice
			print "menuOptionChanged(): niceValue: ",niceValue

	karamba.redrawWidget(widget)

##################################################################
#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
##################################################################
def commandOutput(widget, pid, output):
	global background,debug,fontColor,niceHonored
	if(pid == kBgPid):
		output = output.strip()
		if(debug>=1):
			print "commandOutput(): output:", output
		if(output == "0"):
			if(debug>=1):
				print "commandOutput(): do nothing or revert to original frame color"
			karamba.deleteImage(widget,background)
			background = karamba.createBackgroundImage(widget, 0, 0, "images/backdrop.xpm")
			karamba.writeConfigEntry(widget, "background", "images/backdrop.xpm")
		elif(output == "1"):
			if(debug>=1):
				print "commandOutput(): change frame to blue"
			karamba.deleteImage(widget,background)
			background = karamba.createBackgroundImage(widget, 0, 0, "images/backdrop_blue.xpm")
			karamba.writeConfigEntry(widget, "background", "images/backdrop_blue.xpm")
		elif(output == "2"):
			if(debug>=1):
				print "commandOutput(): change frame to red"
			karamba.deleteImage(widget,background)
			background = karamba.createBackgroundImage(widget, 0, 0, "images/backdrop_red.xpm")
			karamba.writeConfigEntry(widget, "background","images/backdrop_red.xpm")
		elif(output == "3"):
			if(debug>=1):
				print "commandOutput(): change frame to green"
			karamba.deleteImage(widget,background)
			background = karamba.createBackgroundImage(widget, 0, 0, "images/backdrop_green.xpm")
			karamba.writeConfigEntry(widget, "background","images/backdrop_green.xpm")
		elif(output == "4"):
			if(debug>=1):
				print "commandOutput(): change frame to none"
			karamba.deleteImage(widget,background)
			background = karamba.createBackgroundImage(widget, 0, 0, "images/backdrop_none.xpm")
			karamba.writeConfigEntry(widget, "background","images/backdrop_none.xpm")

	if(pid == kFontPid):
		output = output.strip()
		fontColor = output
		karamba.writeConfigEntry(widget, "fontColor", fontColor)

	karamba.redrawWidgetBackground(widget)
	karamba.redrawWidget(widget)

print "Loaded doomCpu version 0.4!"

