# Evolution Python interface
# Copyright (C) 2003  Selan at users dot sf dot net, jesse at uky dot edu
#
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA

import os
import string
import re

class folders:
    ''' creates an iterater to the mail folders in Evolution, given
    the directory which mail is stored. '''

    def __init__( self, basePath = '~/evolution/local/Inbox' ):

        self.path = os.path.expanduser( basePath )
        self.folders = []
        os.path.walk( self.path, self.__add, None )

    def __add( self, arg, directory, files ):
        if 'mbox' in files:
            name = '/'.join( ['Inbox'] + directory.split('/subfolders/')[1:])
            self.folders.append( [name, directory] )

    # iterator support
    def __getitem__( self, x ):
        return self.folders.__getitem__( x )

class imapfolders:
    ''' creates an iterater to the imap mail folders in Evolution, given
    the directory which mail is stored. '''

    def __init__( self, basePath = '~/evolution/mail/imap' ):

        self.path = os.path.expanduser( basePath )
        self.folders = []
        os.path.walk( self.path, self.__add, None )

    def __add( self, arg, directory, files ):
        if 'summary' in files:
            #imap folder paths should be in the format
            #~/evolution/mail/imap/<name>/folders/<folder>/subfolders/<subfolder>....
            #TODO there should definitely be a better way to parse this
            name=directory.split('imap/')[1:][0].split('folders/')[:1]
            name = '/'.join(name+directory.split('folders/')[1:])
            name=name.replace('/sub','')
            name=name.replace('//','/')
            self.folders.append( [name, directory] )

    # iterator support
    def __getitem__( self, x ):
        return self.folders.__getitem__( x )



#--------------------------------------------------------------------
# so we don't continously GREP through the mbox files, we will
# keep a record of the modification times of the mboxes, and store
# the old results, so we can just return last result if the file
# has not been modified ( checked via os.stat( file ).st_mtime )
__cached = {} # ex: __cached['.../Inbox/mbox'] = [ mtime, 3 ]  .. time, msgs

def check_mbox( mbox ):
  ''' this parses through the passed mbox returns the number
      of new messages.  If there is an error, it returns -1 '''

  try:
    modTime = os.stat( mbox ).st_mtime


    # if we have a cache for the mbox and the file has not changed, return it
    if __cached.has_key( mbox ):
        if __cached[mbox][0] == modTime:
            #print "using cache"
            return __cached[mbox][1]

    # otherwise we need to grep through the mbox :(
    #print "grepping " + mbox
    data = os.popen( "grep -c '^X-Evolution: [0-9a-f-]\{11\}[02]' '%s'" % mbox )
    count = int( data.read() )
    data.close()

    # and cache it for next time
    __cached[mbox] = [ modTime, count ]

    return count

  except:  # we should really check for different types of errors ...
    if __cached.has_key( mbox ):
        del __cached[mbox]  # when there is an error, remove the cached data
    return -1

def list_new_mbox(mbox):
    tmp=file(mbox,'r')
    data=tmp.read()
    tmp.close()
    pattern=re.compile('X-Evolution: [0-9a-f-]{11}[02]')
    headers=data.split('\n\n')
    new=[]
    for header in headers:
        if pattern.search(header):
            new.append(re.search('^From:.*?$',header,re.M+re.I).group()+'\n'+re.search('^Subject:.*?$',header,re.M+re.I).group())
    return new

def check_imap( imap ):
    ''' this parses through the passed imap summary and returns the number
        of new messages.  If there is an error, it returns -1 '''

    try:
        modTime = os.stat( imap ).st_mtime


        # if we have a cache for the summary and the file has not changed, return it
        if __cached.has_key( imap ):
            if __cached[imap][0] == modTime:
                #print "using cache"
                return __cached[imap][1]

        # otherwise we need to parse the imap summary
        #print "parsing " + imap
        i=file(imap,'r')
        data=i.read()
        count=data.count('\x80\x00\x80')+data.count('\x88\x00\x80')
        i.close()

        # and cache it for next time
        __cached[imap] = [ modTime, count ]

        return count

    except:  # we should really check for different types of errors ...
        if __cached.has_key( imap ):
            del __cached[imap]  # when there is an error, remove the cached data
        return -1

def list_new_imap(imap):
    tmp=file(imap,'r')
    data=tmp.read()
    tmp.close()
    headers=data.split('\x00\x80')
    new=[]
    for header in headers:
        header+='\x00\x80'
        if (header.find('\x80\x00\x80')>-1) or (header.find('\x88\x00\x80')>-1):
            #get list of new messages
            new.append(re.search('\d+',header).group())
    summary=[]
    for mail in new:
        #parse the mail.HEADER file
        mailfile=os.path.dirname(imap)+'/'+mail+'.HEADER'
        tmp=file(mailfile,'r')
        data=tmp.read()
        tmp.close()
        summary.append(re.search('^From:.*?$',data,re.M+re.I).group()+'\n'+re.search('^Subject:.*?$',data,re.M+re.I).group())            
    return summary

if __name__ == '__main__':
#     for f,l in folders():
#         print '%s is located %s has %s new messages' % (f,l, check_mbox(l+'/mbox'))
#     for f,l in imapfolders():
#         print '%s is located %s has %s new messages' % (f,l, check_imap(l+'/summary'))
    pass
