# Evolution Biff v0.4.1
# http://evolutionbiff.sourceforge.net
# selan at users dot sf dot net
#
# An email notification widget for evolution mail files.
#    Copyright (C) 2003
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA

#this import statement allows access to the karamba functions
import karamba
import evo
import os

class evomail:

    def __init__(self,widget):
        self.setup(widget)

    def check(self):
        if self.mode==0:
            #mbox
            new=evo.check_mbox(self.file)
        else:
            #imap
            new=evo.check_imap(self.file)
        self.count=new
        return new

    def setup(self,widget):
        file = karamba.readConfigEntry(widget, "file")
        if file==None:
            file=os.path.expanduser("~/evolution/local/Inbox/mbox")
        mode = karamba.readConfigEntry(widget,"mode")
        if mode==None:
            mode=0
        self.file=file
        self.mode=mode
        self.count=0

class mailsummary:

    def __init__(self,widget):
        global maillabel,nextlabel,prevlabel,next,prev

        maillabel=karamba.getThemeText(widget,"mail")
        nextlabel=karamba.getThemeText(widget,"next")
        prevlabel=karamba.getThemeText(widget,"prev")
        next=karamba.getThemeImage(widget,"nextarrow")
        prev=karamba.getThemeImage(widget,"prevarrow")
        karamba.attachClickArea(widget,next)
        karamba.attachClickArea(widget,prev)
        karamba.hideText(widget,maillabel)
        karamba.hideText(widget,nextlabel)
        karamba.hideText(widget,prevlabel)
        karamba.hideImage(widget,next)
        karamba.hideImage(widget,prev)
        karamba.redrawWidget(widget)
        self.on=0
        self.index=0
        self.headers=[]

    def showsummary(self,widget):
        self.index=0
        if mailfile.mode==0:
            self.headers=evo.list_new_mbox(mailfile.file)
        else:
            self.headers=evo.list_new_imap(mailfile.file)
        #only show if new mail exists
        if len(self.headers) > 0:
            self.drawminus(widget)
            karamba.showText(widget,maillabel)
            #don't redraw widget until it's populated
            self.show(widget)
            self.on=1

    def hidesummary(self,widget):
        self.headers=[]
        if mailfile.count > 0:
            self.drawplus(widget)
        else:
            self.drawnone(widget)
        karamba.hideText(widget,maillabel)
        karamba.hideText(widget,nextlabel)
        karamba.hideImage(widget,next)
        karamba.hideText(widget,prevlabel)
        karamba.hideImage(widget,prev)
        karamba.redrawWidget(widget)
        self.index=0
        self.on=0

    def show(self,widget):
        #print self.headers[self.index]
        karamba.changeText(widget,maillabel,self.headers[self.index])
        #only show prev if there is a prev
        if self.index!=0:
            karamba.showText(widget,prevlabel)
            karamba.showImage(widget,prev)
        else:
            karamba.hideText(widget,prevlabel)
            karamba.hideImage(widget,prev)
        #only show next if there is a next
        if self.index < (len(self.headers)-1):
            karamba.showText(widget,nextlabel)
            karamba.showImage(widget,next)
        else:
            karamba.hideText(widget,nextlabel)
            karamba.hideImage(widget,next)
        karamba.redrawWidget(widget)

    def shownext(self,widget):
        self.index+=1
        self.show(widget)

    def showprev(self,widget):
        self.index-=1
        self.show(widget)

    def drawplus(self,widget):
        karamba.showImage(widget,hide)
        karamba.showImage(widget,show)

    def drawminus(self,widget):
        karamba.hideImage(widget,show)
        karamba.showImage(widget,hide)

    def drawnone(self,widget):
        karamba.hideImage(widget,show)
        karamba.hideImage(widget,hide)

def deleteIcons(widget):
    karamba.deleteImage(widget,newmail)
    karamba.deleteImage(widget,nomail)
    
def createIcons(widget,size):
    global newmail,nomail
    if size=='Larger':
        newmail=karamba.createImage(widget,0,0,"pics/queue.png")
        nomail=karamba.createImage(widget,0,0,"pics/mail_generic.png")
    elif size=='Smaller':
        newmail=karamba.createImage(widget,0,5,"pics/queue_22.png")
        nomail=karamba.createImage(widget,0,5,"pics/mail_generic_22.png")
    karamba.attachClickArea(widget,newmail)
    karamba.attachClickArea(widget,nomail)

def initWidget(widget):
    '''this is called when your widget is initialized'''
    global newmail,nomail,show,hide,label,mailfile,mailsum

    mailfile=evomail(widget)
    mailsum=mailsummary(widget)

    karamba.addMenuConfigOption(widget,"file","Change mail folder")
    karamba.addMenuConfigOption(widget,"icon","Change mail icon size")
    karamba.setMenuConfigOption(widget, "file", 0)
    karamba.setMenuConfigOption(widget, "icon", 0)
    label=karamba.getThemeText(widget,"label")
    icon=karamba.readConfigEntry(widget,"icon")
    if icon==None:
        icon="Larger"
    createIcons(widget,icon)
    show=karamba.getThemeImage(widget,"show")
    hide=karamba.getThemeImage(widget,"hide")
    karamba.hideImage(widget,newmail)
    karamba.hideImage(widget,nomail)
    karamba.hideImage(widget,show)
    karamba.hideImage(widget,hide)
    karamba.attachClickArea(widget,hide)
    karamba.addImageTooltip(widget,newmail,"Click here to go to Evolution")
    karamba.addImageTooltip(widget,nomail,"Click here to go to Evolution")
    karamba.redrawWidget(widget)
    widgetUpdated(widget)

def widgetUpdated(widget):
    '''this is called everytime your widget is updated
    the update inverval is specified in the .theme file'''

    try:
        count=mailfile.check()

        if count==-1:
            karamba.changeText(widget,label,"mail file not found")
        elif count==0:
            karamba.hideImage(widget,newmail)
            karamba.showImage(widget,nomail)
            mailsum.drawnone(widget)
            if mailsum.on==1:
                mailsum.hidesummary(widget)
        else:
            karamba.hideImage(widget,nomail)
            karamba.showImage(widget,newmail)
            if mailsum.on==0:
                mailsum.drawplus(widget)

        s="s"
        if count==1:
            s=""
        tip=str(count)+" new message"+s

        karamba.changeText(widget,label,tip)
        karamba.redrawWidget(widget)
    except ValueError:
        karamba.changeText(widget,label,"error")
        karamba.redrawWidget(widget)


def widgetClicked(widget, x, y, button):
    '''This gets called everytime our widget is clicked.
    Notes:
    widget = reference to our widget
    x = x position (relative to our widget)
    y = y position (relative to our widget)
    botton = button clicked:
                        1 = Left Mouse Button
                        2 = Middle Mouse Button
                        3 = Right Mouse Button, but this will never happen
                            because the right mouse button brings up the
                            Karamba menu.
                        4,5 = Scroll wheel up and down'''
    pass

#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
    if (meter==nomail) or (meter==newmail):
        #get task class names and references
        tasks={}
        for task in karamba.getTaskList(widget):
            tasks[karamba.getTaskInfo(widget,task)[2]]=karamba.getTaskInfo(widget,task)[8]
        #if evolution is open, raise it
        if tasks.has_key("evolution"):
            karamba.performTaskAction(widget,tasks["evolution"],8)
    elif meter==hide:
        if mailsum.on==0:
            mailsum.showsummary(widget)
        else:
            mailsum.hidesummary(widget)
    elif meter==next:
        mailsum.shownext(widget)
    elif meter==prev:
        mailsum.showprev(widget)



def widgetMouseMoved(widget, x, y, button):
    '''This gets called everytime our widget is clicked.
    Notes
    widget = reference to our widget
    x = x position (relative to our widget)
    y = y position (relative to our widget)
    botton = button being held:
                        0 = No Mouse Button
                        1 = Left Mouse Button
                        2 = Middle Mouse Button
                        3 = Right Mouse Button, but this will never happen
                            because the right mouse button brings up the
                            Karamba menu.
    Warning:  Don't do anything too intensive here
    You don't want to run some complex piece of code everytime the mouse moves'''
    pass



def menuOptionChanged(widget, key, value):
    '''This gets called when an item is clicked in the theme CONFIGURATION menu,
    not the popup menus that you create.
    key = the reference to the configuration key that was changed
    value = the new value (true or false) that was selected'''

    if key=="file":
        mboxlist=" ".join(["'%s/mbox\n0' '%s'" % (path,mbox) for mbox,path in evo.folders()])
        imaplist=" ".join(["'%s/summary\n1' '%s'" % (path,imap) for imap,path in evo.imapfolders()])
        maillist=mboxlist+" "+imaplist
        karamba.setMenuConfigOption(widget, "file", 0)
        temp=os.popen("kdialog --caption \"Evolution Biff\" --menu \"Which mail folder would you like to monitor?\" %s" % maillist)
        mailtmp=temp.read()
        temp.close()
        
        mailtmp=mailtmp.strip().split('\n')
        mailnew=mailtmp[0]
        if mailnew!='':
            modenew=mailtmp[1]
            mailnew=os.path.expanduser(mailnew)
            karamba.writeConfigEntry(widget, "file", mailnew)
            karamba.writeConfigEntry(widget, "mode", modenew)
            mailfile.setup(widget)
        widgetUpdated(widget)

    if key=="icon":
        karamba.setMenuConfigOption(widget, "icon", 0)
        temp=os.popen("kdialog --caption \"Evolution Biff\" --combobox \"Which size do you prefer for the mail icon?\" \"Larger\" \"Smaller\"")
        icontmp=temp.read()
        temp.close()
        
        icon=icontmp.strip()
        if icon!='':
            karamba.writeConfigEntry(widget, "icon", icon)
            deleteIcons(widget)
            createIcons(widget,icon)
        widgetUpdated(widget)

# This will be printed when the widget loads.
print """
Evolution Biff version 0.4.1, Copyright (C) 2003 selan at users dot sf dot net
This software comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions. See the included COPYING file for details.
"""
