#Programmer: Ryan Nickell    ryan_s_nickell@hotmail.com
#Date: 04/17/2004
#Filename: iLaunch.py
#Version: 0.14
#
#Requirements:  Qt: 3.1.1
#		KDE: 3.1.3
#		superkaramba: 0.33
#		kdialog 0.9
#		wheel mouse (for scrolling through icons)
#		'No Wheel Mouse' option available for mice without a wheel
#
#	Copyright 2004 Ryan Nickell
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##########################################################################

import karamba
import string, re, os, urllib
import applications

##################################################
# Add a few safety variables that people can edit
# if they need to in the .py file
##################################################
conf = {
	"font" : "ariel" ,
	"fontColor" : "white" ,
	"fontSize" : 10 ,
	"debug" : 0 ,
	"shadow" : 1,
	"kdePath" : "" ,
	"iconDirectory" : "icons/" ,
	"bg" : "bg.png" ,
	"currentIconEffect" : "highlight" ,
}
####################################################
# Global Variables to check user input and such
# against the rc file information
####################################################
debug = conf['debug']
maxApps = 0
appObjs = [] #this is the array of applications objects from applications.py
apps = [] #this holds the applications that are written to the rc file
icons = [] #this holds the icons that will be used
names = [] #this is the command line execution including arguments
displayText = [] #this is to hold the display text for each application
app0 = ""
i = 0 #application count
fontSize = conf['fontSize']
font = conf['font']
fontColor = conf['fontColor']
shadow = conf['shadow']
kdeIconPath = ""
bg = conf['bg']
bgChange = 0
wmChange = 0
iconchange = 0
fromRemPid = 0
fromAddPid = 0
kAddPid = 0
kRemPid = 0
kIconPid = 0
kDisplayPid = 0
kUnique = 0
koutput = ""
bgImage = 0
iconMenu = 0
onIcon = 0
onUp = 0
onDown = 0
count = 0
haveWheelMouse = ""
upImage = 0
downImage = 0
resize = 1

##################################################################
#text modification
#More colors will be added later.. right now we have
#support for: white, black, blue, red, green, yellow, purple, tan
#If there is no fontColor match, the default will be black
##################################################################
def prettyText(widget, size, font, textname, fontColor, shadow):
	karamba.changeTextSize(widget, textname, size)
	karamba.changeTextFont(widget, textname, font)
	if(fontColor == "white"):
		karamba.changeTextColor(widget, textname, 255, 255, 255)
	elif(fontColor == "black"):
		karamba.changeTextColor(widget, textname, 0, 0, 0)
	elif(fontColor == "blue"):
		karamba.changeTextColor(widget, textname, 0, 0, 255)
	elif(fontColor == "red"):
		karamba.changeTextColor(widget, textname, 255, 0, 0)
	elif(fontColor == "green"):
		karamba.changeTextColor(widget, textname, 0, 255, 0)
	elif(fontColor == "yellow"):
		karamba.changeTextColor(widget, textname, 255, 255, 0)
	elif(fontColor == "purple"):
		karamba.changeTextColor(widget, textname, 127, 0, 127)
	elif(fontColor == "tan"):
		karamba.changeTextColor(widget, textname, 127, 127, 0)
	else:
		karamba.changeTextColor(widget, textname, 0,0,0)
	karamba.changeTextShadow(widget, textname, shadow)

####################################################
# Configuration in iLaunch.conf
# uses the ~/.superkaramba/iLaunch.rc file
####################################################
def rcFileCheck(widget):
	global maxApps,debug
	#initialize the rc file with some stuff to use later
	karamba.writeConfigEntry(widget, "Version", "0.14")
	if (karamba.readConfigEntry(widget, "firstTime") != "No"):
		firstTimeCheck(widget)
	if(debug >= 1):
		print "rcSetup(): rc file updated!"
	karamba.redrawWidget(widget)

####################################################
# Check for if it's the first time to load iLaunch
# and load the first icon into the rc file for
# initApps() and the icon loading callbacks
####################################################
def firstTimeCheck(widget):
	global maxApps,appObjs
	#FIXME: this needs to use executeInteractive() instead of os.open
	# but it presents problems for initWidget() if we don't use os.open
	pipe = os.popen("kdialog --title \"Add Icon\" --inputbox \"Please enter the application you want to add:\"", 'r')
	rawarea = pipe.readlines()
	addicon = rawarea[0].strip()
	#if you enter an application with a space in front, this breaks the code later
	# so don't do that on the first launch
	if( addicon != "" ):
		karamba.writeConfigEntry(widget, "app0", addicon)
		maxApps = maxApps + 1
		pipe = os.popen("kdialog --title \"Add Display Text\" --inputbox \"Please enter the display text for this application:\"",'r')
		rawarea= pipe.readlines()
		addtext = rawarea[0].strip()
		if( addtext != "" ):
			karamba.writeConfigEntry(widget,"display0",addtext)
			temp = re.split(r'(\s)',addicon,1)
			karamba.writeConfigEntry(widget,"icon0",temp[0]+'.png')
		karamba.writeConfigEntry(widget, "firstTime", "No")
		karamba.writeConfigEntry(widget, "maxApps", str(maxApps))
		setStandardTheme(widget)
		karamba.writeConfigEntry(widget,"iconTheme0",karamba.readConfigEntry(widget, "iconDir"))
	if(debug >= 1):
		print "firstTimeCheck(): loading default kde icon theme..."



#################################################################
# Loads default icons matching the kde theme icons you have setup
# FIXME: this isn't setup to avoid overwriting the individual
#  selected appObjs IconTheme
#################################################################
def setStandardTheme(widget):
	global conf,kdeIconPath,debug,appObjs
	themePath = karamba.getThemePath(widget)
	#New implementation for getting the icon directory
	#from the theme you are using found in ~/.kde/share/config/kdeglobals file
	#as some distributions don't use the $KDEDIR environment variable (ie Mandrake)
	if( os.path.isfile(os.environ["HOME"]+"/.kde/share/config/kdeglobals") ):
		themeFile = os.environ["HOME"]+"/.kde/share/config/kdeglobals"
		if(debug >= 1):
			print "setTheme(): Theme file is",themeFile
		file = open(themeFile,"r")
		text = file.readlines()
		file.close()
		keyword = re.compile(r"Theme")
		for line in text:
			if keyword.search(line):
				Theme = line
				Theme = Theme.replace('\n','')
				(junk,Theme) = re.split('=',Theme)
				break
			#on a new install, Theme isn't set in kdeglobals
			#this else needed added just in case
			else:
				Theme = "kdeclassic"
		KDEDIR = os.getenv('KDEDIR')
		if( os.path.exists("/usr/share/icons/"+Theme+"/32x32/apps/") ):
			kdeIconPath = "/usr/share/icons/"+Theme+"/32x32/apps/"
			karamba.writeConfigEntry(widget, "iconDir", kdeIconPath)
		#just in case the user doesn't have the /usr/share/icons or KDEDIR directories
		#load the included icons to be on the safe side
		#KDEDIR is not tested as I don't have that set in Mandrake so this is just a guess
		elif( os.getenv('KDEDIR') ):
		#if( os.path.exists(os.getenv("KDEDIR")+"/share/icons/"+Theme+"/32x32/apps/") ):
			kdeIconPath = os.environ["KDEDIR"]+"/share/icons/"+Theme+"/32x32/apps/"
			karamba.writeConfigEntry(widget, "iconDir", kdeIconPath)
		else:
			kdeIconPath = themePath+"/icons/"
			karamba.writeConfigEntry(widget, "iconDir", kdeIconPath)
		karamba.writeConfigEntry(widget,"kdePath","none")
		if(debug >= 1):
			print "setTheme(): kdeIconPath -",kdeIconPath
	else:
		kdeIconPath = themePath+"/icons/"
		karamba.writeConfigEntry(widget, "iconDir", kdeIconPath)
		if(debug >= 1):
			print "setTheme(): Using default 'icons/' directory for icons"
		karamba.writeConfigEntry(widget,"kdePath","none")
	#set all the appObjs theme paths with the final result
	# FIXME: when we implement individual icons, update this to check for overwriting
	# maybe write a function later called setAllIconsToDefaultTheme()

#################################################################
# Load the custom icons selected when changing the menu option
# to the full path of the icons you want to load
# FIXME: this isn't setup to avoid overwriting the individual
#  selected appObjs IconTheme
#################################################################
def setCustomTheme(widget):
	global kdeIconPath,debug
	#check for menu added icon directory and update accordingly
	if(karamba.readConfigEntry(widget,"kdePath") == "custom"):
		kdeIconPath = karamba.readConfigEntry(widget,"iconDir")
		if(debug >= 1):
			print "setCustomTheme(): Icon directory updated to ",kdeIconPath
	else:
		if(debug >= 1):
			print "setCustomTheme(): Icon directory not changed because 'custom' was not set in the rc file"
	#set all the appObjs theme paths with the new kdeIconPath
	# FIXME: when we implement individual icons, update this to check for overwriting
	# maybe write a function later called setAllIconsToDefaultTheme()

#################################################################
# Calculate the widgetsize, icon placement, display text placement,
#  icon highlight boundaries
# this includes:
#   widgetXsize, widgetYsize, midXmax,xIconCenter,
#   yIconCenter, yMax, xIconLeftBounds, xIconRightBounds,
#   yIconTopBounds, yIconBottomBounds
#################################################################
def getAppLocations(widget):
	global xIconLeftBounds,xIconRightBounds,yIconTopBounds,yIconBottomBounds,midXmax,widgetXsize,widgetYsize
	global background,image,kdeIconPath,debug,xIconCenter,yIconCenter,maxApps,yMax,bgImage
	height = karamba.getImageHeight(widget, image)
	width = karamba.getImageWidth(widget, image)
	karamba.deleteImage(widget, image)
	widgetXsize = width+50
	widgetYsize = height+50
	#FIXME: implement this when the resizing works correctly
	if(resize == 1):
		karamba.resizeWidget(widget,widgetXsize,widgetYsize)
		karamba.resizeImage(widget,background,widgetXsize,widgetYsize)
                karamba.redrawWidgetBackground(widget)
	if(debug >= 1):
		print "widget X:",widgetXsize,"Y:",widgetYsize
	midXmax = karamba.getImageWidth(widget, background) / 2
	xIconCenter = karamba.getImageWidth(widget, background) / 2 - 20
	yIconCenter = karamba.getImageHeight(widget, background) - 80
	if(debug >= 1):
		print "getAppLocations(): X: ",xIconCenter," Y: ",yIconCenter
		print "getAppLocations(): Width: ",karamba.getImageWidth(widget, background)," Height: ",karamba.getImageHeight(widget, background)
	#do some funky stuff to reload the correct size for varying icons such as 22x22 and 128x128
	xIconCenter = karamba.getImageWidth(widget, background) / 2 - width / 2
	yIconCenter = karamba.getImageHeight(widget, background) / 2 - height/2 -5
	#text placement
	yMax = yIconCenter + height
	#calculate icon's boundaries, might want to separate this later
	xIconLeftBounds = xIconCenter
	xIconRightBounds = xIconCenter + width
	yIconTopBounds = yIconCenter
	yIconBottomBounds = yIconCenter + height

#########################################################
#function to calculate the up/down arrow image locations
# and the highlight code boundaries & reloads the arrows
#########################################################
def getArrowsLocations(widget):
	global upCenter, downCenter, haveWheelMouse,upImage,downImage,upBounds,downBounds
	global background,yIconTopBounds,yIconBottomBounds,debug
	#get the image sizes if you don't have them displayed..but this is a crude hack for the up/down arrows
	# this is for the widgetUpdated() callback, where the image doesn't exist to get the sizes we need for upBounds/downBounds
	if(haveWheelMouse == "Yes"):
		upImage = karamba.createImage(widget,0,0,"up.png")
		downImage = karamba.createImage(widget,0,0,"down.png")
	#load up arrow image bounds in order of (left,right,top,bottom) for widgetMouseMoved() callback
	upBounds = []
	upBounds.append((karamba.getImageWidth(widget,background)-karamba.getImageWidth(widget,upImage)))
	upBounds.append(karamba.getImageWidth(widget,background))
	upBounds.append(yIconTopBounds)
	upBounds.append(yIconTopBounds+karamba.getImageHeight(widget,upImage))
	downBounds = []
	downBounds.append((karamba.getImageWidth(widget,background)-karamba.getImageWidth(widget,downImage)))
	downBounds.append(karamba.getImageWidth(widget,background))
	downBounds.append(yIconBottomBounds)
	downBounds.append(yIconBottomBounds+karamba.getImageHeight(widget,upImage))

	if(haveWheelMouse == "Yes"):
		karamba.deleteImage(widget, upImage)
		karamba.deleteImage(widget, downImage)
		upImage = 0
		downImage = 0
	upCenter = [upBounds[0],upBounds[2]]
	downCenter = [downBounds[0],downBounds[2]]
	if(debug >= 1):
		print "getArrowsLocations(): Bounds: UP:",upBounds,"DOWN:",downBounds
		print "getArrowsLocations(): upCenter: ",upCenter,"downCenter: ",downCenter

####################################################
#function to sort through the .rc file to get all
# the applications that were added
####################################################
def initApps(widget):
	global apps,maxApps,kdeIconPath,debug
	maxApps = karamba.readConfigEntry(widget,"maxApps")
	if(debug >= 1):
		print "initApps(): maxApps-",maxApps, " ", karamba.readConfigEntry(widget,"maxApps")
	#load up the apps from the rc file and the appObjs[] tuple for execution of applications
	for x in range( 0, karamba.readConfigEntry(widget,"maxApps") ):
		name = karamba.readConfigEntry(widget,"app"+str(x))
		name = re.sub("&","",name)
		displayName = karamba.readConfigEntry(widget,"display"+str(x))
		#the icon and iconTheme are used by the uniqueIcon stuff
		icon = karamba.readConfigEntry(widget,"icon"+str(x))
		iconTheme = karamba.readConfigEntry(widget,"iconTheme"+str(x))
		#make a new applications object instance
		appObjs.append( applications.Application(name,displayName,widget) )
		if(debug >= 1):
			print "initApps(): iconTheme:",iconTheme+icon,"kdeIconPath:",kdeIconPath+appObjs[x].getIcon()
		if( karamba.readConfigEntry(widget,"unique"+str(x)) == 'Yes' ):
			appObjs[x].setUniqueIcon(icon)
			appObjs[x].setIconTheme(iconTheme)
		else:
			appObjs[x].setIconTheme(kdeIconPath)
		appObjs[x].refreshIcon()
		if( (os.path.exists( (appObjs[x].getFullIcon()) )) == False):
			print "initApps() Couldn't find the path and icon: ",appObjs[x].getFullIcon()
			appObjs[x].setIcon('unknown.png')
			appObjs[x].setIconTheme('icons/')
			appObjs[x].refreshIcon()
		if(debug >= 1):
			print "initApps(): Icon:",x,appObjs[x].getIcon()
			print "initApps(): Application:",x,appObjs[x].getName()
			print "initApps(): Display text:",x,appObjs[x].getDisplayName()
			print "initApps(): Full Path to Icon:",x, appObjs[x].getFullIcon()
			print "initApps(): Icon Theme:",x, appObjs[x].getIconTheme()

#######################################################
# Reinitializes icons for when you change the icon
# directory from the menu immediately refreshing
# it is only called when changing icon directories
# in the menuOptionChanged() callback
#######################################################
def loadIcons(widget):
	global apps,maxApps,icons,names,apps,displayText,kdeIconPath,i,image,system,fontSize, font,fontColor, shadowx,IconCenter, yIconCenter
	global xIconLeftBounds,xIconRightBounds,yIconTopBounds,yIconBottomBounds
	global upCenter,downCenter,haveWheelMouse,upImage,downImage,upBounds,downBounds,widgetXsize,widgetYsize,appObjs
	if(debug >= 1):
		print "loadIcons(): maxApps-",maxApps, " ", karamba.readConfigEntry(widget,"maxApps")

	karamba.deleteImage(widget, image)
	#reinitialize them now
	for x in range(0,maxApps):
		iconTheme = karamba.readConfigEntry(widget,"iconTheme"+str(x))
		#check for unknown types, and reset them to be recalculated with the new icon directory
		if(appObjs[x].getIcon() == 'unknown.png'):
			temp = re.split(r'(\s)',appObjs[x].getName(),1)
			appObjs[x].setIcon(temp[0]+'.png')
		if(appObjs[x].isIconUnique() == 'Yes'):
			if(debug >= 1):
				print "loadIcons(): Icon is unique:",appObjs[x].getFullIcon()
			appObjs[x].setIconTheme(iconTheme)
		else:
			appObjs[x].setIconTheme(kdeIconPath)
			if(debug >= 1):
				print "loadIcons(): Icon is NOT unique:",appObjs[x].getFullIcon()
		appObjs[x].refreshIcon()
		if( (os.path.exists( (appObjs[x].getFullIcon()) )) == False):
			print "loadIcons(): Couldn't find the path and icon: ",appObjs[x].getFullIcon()
			appObjs[x].setIcon('unknown.png')
			appObjs[x].setIconTheme('icons/')
			appObjs[x].refreshIcon()
	#for the dynamic resizeable background, get image size and calculate
	image = karamba.createImage(widget, 0, 0, appObjs[i].getFullIcon())
	getAppLocations(widget)

	#FIXME: when we implement resizeWidget() do this
	if(resize == 1):
		karamba.resizeWidget(widget,widgetXsize,widgetYsize)
		karamba.resizeImage(widget,background,widgetXsize,widgetYsize)
		karamba.redrawWidgetBackground(widget)

	#redraw the icon that is currently showing
	karamba.deleteText(widget, system)
	karamba.deleteImage(widget, image)
	image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[i].getFullIcon())
	getAppLocations(widget)

	if(haveWheelMouse == "No"):
		#add the alternate image navigation here with the new location
		getArrowsLocations(widget)
		karamba.moveImage(widget,upImage,upCenter[0],upCenter[1])
		karamba.moveImage(widget,downImage,downCenter[0],downCenter[1])
		if(debug >= 1):
			print "loadIcons(): if: haveWheelMouse:",haveWheelMouse

	karamba.deleteImage(widget, image)
	karamba.deleteText(widget, system)
	image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[i].getFullIcon())
	system = karamba.createText(widget, midXmax - len(appObjs[i].getDisplayName())*3, yMax, 80, 20, appObjs[i].getDisplayName())
	prettyText(widget, fontSize, font, system, fontColor, shadow)
	karamba.attachClickArea(widget,image)

	#FIXME: when we implement resizeWidget() do this
	if(resize == 1):
		karamba.resizeImage(widget,background,widgetXsize,widgetYsize)
		karamba.redrawWidget(widget)
		if(karamba.getImageWidth(widget,background) == widgetXsize and karamba.getImageHeight(widget,background) == widgetYsize):
			karamba.resizeWidget(widget,widgetXsize,widgetYsize)
			if(debug >= 1):
			    print "Resized widget now that the background is done.."

	if(debug >= 1):
		print "loadIcons(): Applications: ", apps, "Names: ", names, "Icons: ", icons
	karamba.redrawWidget(widget)
	karamba.redrawWidget(widget)
	#karamba.redrawWidgetBackground(widget)
	if(debug >= 1):
		print "loadIcons(): kdeIconPath-",kdeIconPath

####################################################
# initialize our widget and setup iLaunch
####################################################
def initWidget(widget):
	global background,image,app0,kdeIconPath,debug,xIconCenter,yIconCenter,maxApps,system,bg,yMax,bgImage,displayText
	global xIconLeftBounds, xIconRightBounds, yIconTopBounds, yIconBottomBounds,widgetXsize,widgetYsize,midXmax
	global upCenter, downCenter, haveWheelMouse,upImage,downImage,upBounds,downBounds
	#check for first time ran to ask for first application
	rcFileCheck(widget)
	#FIXME: make this background dynamic, than just none or bg
	if(karamba.readConfigEntry(widget,"background") != "none.png"):
		background = karamba.createBackgroundImage(widget, 0, 0, bg)
	else:
		bg = karamba.readConfigEntry(widget,"background")
		background = karamba.createBackgroundImage(widget, 0, 0, bg)

	#figure out if we have a wheel mouse or not and save it
	if(karamba.readConfigEntry(widget,"wheelMouse") != "No"):
		haveWheelMouse = "Yes"
		if(debug >= 1):
			print "initWidget(): if: haveWheelMouse:",haveWheelMouse
	else:
		haveWheelMouse = karamba.readConfigEntry(widget,"wheelMouse")
		if(debug >= 1):
			print "initWidget(): else: haveWheelMouse:",haveWheelMouse
	#setup the paths to find icons and theme directory
	#check for menu added icon directory and update accordingly
	if(karamba.readConfigEntry(widget,"kdePath") != "custom"):
		setStandardTheme(widget)
		if(debug >= 1):
			print "initWidget(): kdeIconPath-",kdeIconPath
	elif(karamba.readConfigEntry(widget,"kdePath") == "custom"):
		setCustomTheme(widget)
		if(debug >= 1):
			print "initWidget(): kdeIconPath-",kdeIconPath
	#get the application names, display text, icons, icon themes, etc.
	initApps(widget)
	#menu options in the configure right click menu
	#to add icons and remove icons
	karamba.addMenuConfigOption(widget, "addicon", "Add a New Icon")
	karamba.addMenuConfigOption(widget, "remicon", "Remove Last Icon")
	karamba.addMenuConfigOption(widget, "remCurrentIcon", "Remove Current Icon")
	karamba.addMenuConfigOption(widget, "uniqueIcon","Set Unique Icon")
	karamba.addMenuConfigOption(widget, "clearUniqueIcon","Clear Unique Icon")
	karamba.addMenuConfigOption(widget, "iconDir", "Custom Icon Directory")
	karamba.addMenuConfigOption(widget, "bgImage", "No Background")
	karamba.addMenuConfigOption(widget, "wheelMouse", "No Wheel Mouse")
	karamba.setMenuConfigOption(widget, "addicon", 0)
	karamba.setMenuConfigOption(widget, "remicon", 0)
	karamba.setMenuConfigOption(widget, "remCurrentIcon", 0)
	karamba.setMenuConfigOption(widget, "uniqueIcon",0)
	karamba.setMenuConfigOption(widget, "clearUniqueIcon",0)
	iconMenu = karamba.readMenuConfigOption(widget, "iconDir")
	bgImage = karamba.readMenuConfigOption(widget, "bgImage")
	wheelMouse = karamba.readMenuConfigOption(widget, "wheelMouse")

	#setup first icon in our stack
	shadow = conf['shadow']
	font = conf['font']
	fontColor = conf['fontColor']
	fontSize = conf['fontSize']
	image = karamba.createImage( widget, 0, 0, appObjs[i].getFullIcon() )
	getAppLocations(widget)

	#display or not to display the alternate image navigation after calculating the coordinates
	if(haveWheelMouse == "Yes"):
		#remove the alternate image navigation here - not used and might need moved later
		pass
	elif(haveWheelMouse == "No"):
		#begin calculating the up/down arrows location and highlight code
		upImage = karamba.createImage(widget,0,0,"up.png")
		downImage = karamba.createImage(widget,0,20,"down.png")
		getArrowsLocations(widget)
		karamba.moveImage(widget,upImage,upCenter[0],upCenter[1])
		karamba.moveImage(widget,downImage,downCenter[0],downCenter[1])
		karamba.attachClickArea(widget,upImage)
		karamba.attachClickArea(widget,downImage)
		if(debug >= 1):
			print "initWidget(): Bounds: UP:",upBounds,"DOWN:",downBounds

	image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[i].getFullIcon())
	system = karamba.createText(widget, midXmax - len(appObjs[i].getDisplayName())*3, yMax, 80, 20, appObjs[i].getDisplayName())
	prettyText(widget, fontSize, font, system, fontColor, shadow)

	karamba.acceptDrops(widget)
	karamba.attachClickArea(widget,image)
	karamba.redrawWidgetBackground(widget)
	karamba.redrawWidget(widget)

####################################################
#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
####################################################
def widgetUpdated(widget):
	global bgChange,background,bg,debug,wmChange,haveWheelMouse,upImage,downImage,upCenter,downCenter
	global widgetXsize,widgetYsize
	#handle the background image toggling here
	if (bgChange == 1):
		if(debug >= 1):
			print "widgetUpdated(): deleting and redisplaying new background"
		karamba.deleteImage(widget,background)
		if (bg == "none.png"):
			bg = "bg.png"
		elif (bg == "bg.png"):
			bg = "none.png"
		karamba.writeConfigEntry(widget, "background", bg)
		background = karamba.createBackgroundImage(widget, 0, 0, bg)
		if(resize == 1):
			karamba.resizeImage(widget,background,widgetXsize,widgetYsize)
		karamba.redrawWidgetBackground(widget)
		karamba.redrawWidget(widget)
		bgChange = 0

	if(wmChange == 1):
		if(haveWheelMouse == "Yes"):
			#load our new bounds for the arrow highlights to work
			getArrowsLocations(widget)
			upImage = karamba.createImage(widget,0,0,"up.png")
			downImage = karamba.createImage(widget,0,20,"down.png")
			karamba.moveImage(widget,upImage,upCenter[0],upCenter[1])
			karamba.moveImage(widget,downImage,downCenter[0],downCenter[1])
			karamba.attachClickArea(widget,upImage)
			karamba.attachClickArea(widget,downImage)
			haveWheelMouse = "No"
			if(debug >= 1):
				print "widgetUpdated(): if: haveWheelMouse:",haveWheelMouse
				print "widgetUpdated(): finished arrows loading.."
		elif(haveWheelMouse == "No"):
			#remove the alternate image navigation here
			karamba.deleteImage(widget,upImage)
			karamba.deleteImage(widget,downImage)
			upImage = 0
			downImage = 0
			haveWheelMouse = "Yes"
			if(debug >= 1):
				print "widgetUpdated(): elif: haveWheelMouse:",haveWheelMouse
		karamba.writeConfigEntry(widget, "wheelMouse", haveWheelMouse)
		wmChange = 0

		if(debug >= 1):
			print "widgetUpdated(): toggling the image arrows"

####################################################
#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
####################################################
def widgetClicked(widget, x, y, button):
	#global variables
	global i,image,last,system,conf,font, fontColor, fontSize,debug,background,xMax,yMax,maxApps
	global shadow,icons,apps,names,xIconCenter,yIconCenter,xIconLeftBounds,xIconRightBounds,yIconTopBounds,yIconBottomBounds
	global upCenter, downCenter, haveWheelMouse,widgetXsize,widgetYsize,appOjbs,onIcon
	font = conf['font']
	fontColor = conf['fontColor']
	fontSize = conf['fontSize']
	debug = conf['debug']
	shadow = conf['shadow']

	if(debug >= 2):
		print "widgetClicked(): ", x, y,currentIconEffect, "Image return: ",image
	#Display, in logical forward order, the stack on mouse button 4
	#Detect if you mouse is in the area of the karamba widget
	#and scroll through applications and display them
	if ((x > 0) and (y > 0) and (x < widgetXsize) and (y < widgetYsize)) and (button == 4) and (haveWheelMouse == "Yes"):
		if(debug >= 1):
			print "widgetClicked(): UP:", appOjbs[i].getFullIcon(), button, i, maxApps
		#cycle from app1 up to app#, and set back to beginning
		i = i+1
		if (debug >= 2):
			print "widgetClicked(): UP:", i
		#set maxApps to dynamically setup applications
		if ( i >= maxApps ):
			i = 0
		#keep text from continuous overwriting (making it really bold)
		karamba.deleteText(widget, system)
		karamba.deleteImage(widget, image)
		image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[i].getFullIcon())
		karamba.attachClickArea(widget,image)
		#detect if you are on icon or not to update the iconEffect stuff
		#since we are in a different callback than widgetMouseMoved() it won't update without this
		#if you are over the icon
		if ((x > xIconLeftBounds) and (y > yIconTopBounds) and (x < xIconRightBounds) and (y < yIconBottomBounds)):
				if (currentIconEffect == "highlight"):
					karamba.changeImageIntensity(widget, image, 0.3, 0)
				if (currentIconEffect == "gray"):
					karamba.changeImageToGray(widget, image, 0)
		else:
			if (currentIconEffect != "none"):
				karamba.removeImageEffects(widget, image)
		system = karamba.createText(widget, midXmax - len(appObjs[i].getDisplayName())*3, yMax, 80, 20, appObjs[i].getDisplayName())
		prettyText(widget, fontSize, font, system, fontColor, shadow)
		karamba.redrawWidget(widget)
		if(debug >= 1):
			print "widgetClicked(): Icon: "+appOjbs[i].getFullIcon(), "Button: "+str(button), "interation: "+str(i)
			if(debug >= 2):
				print "widgetClicked(): Font: ",font, "FontColor: ",fontColor

	#Display in reverse on mouse button 5
	#Detect if you mouse is in the area of the karamba widget
	#and scroll through applications and display them
	if ((x > 0) and (y > 0) and (x < widgetXsize) and (y < widgetYsize)) and (button == 5) and (haveWheelMouse == "Yes"):
		if (debug >= 1):
			print "widgetClicked(): DOWN:", appOjbs[i].getFullIcon(), button, i, maxApps
		#cycle back from app# to app0 and then back to end of stack
		i = i-1
		if (debug >= 2):
			print "widgetClicked(): DOWN:", i
		#take care of cycling back to 0 and then to maxApps
		if ( i <= (0-1) ):
			i = (maxApps-1)
		#keep text from continuous overwriting (making it really bold, not desireable)
		karamba.deleteText(widget, system)
		karamba.deleteImage(widget, image)
		image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[i].getFullIcon())
		karamba.attachClickArea(widget,image)
		#detect if you are on icon or not to update the iconEffect stuff
		#since we are in a different callback than widgetMouseMoved() it won't update without this
		#if you are over the icon
		if ((x > xIconLeftBounds) and (y > yIconTopBounds) and (x < xIconRightBounds) and (y < yIconBottomBounds)):
        		if (currentIconEffect != "none"):
				if (currentIconEffect == "highlight"):
					karamba.changeImageIntensity(widget, image, 0.3, 0)
				if (currentIconEffect == "gray"):
					karamba.changeImageToGray(widget, image, 0)
		else:
			if (currentIconEffect != "none"):
				karamba.removeImageEffects(widget, image)
		system = karamba.createText(widget, midXmax - len(appObjs[i].getDisplayName())*3, yMax, 80, 20, appObjs[i].getDisplayName())
		prettyText(widget, fontSize, font, system, fontColor, shadow)
		karamba.redrawWidget(widget)
		if(debug >= 1):
			print "widgetClicked(): Icon: ",appOjbs[i].getFullIcon(), "Button: ",button, "interation: ",i
			if(debug >= 2):
				print "widgetClicked(): Font: ",font, "FontColor: ",fontColor
	if(debug >= 1):
		print "widgetClicked(): maxApps:",maxApps, "Config maxApps:", karamba.readConfigEntry(widget,"maxApps")
		print "widgetClicked(): i: ",i
		print "widgetClicked(): Icon:",i,appObjs[i].getIcon()
		print "widgetClicked(): Application:",i,appObjs[i].getName()
		print "widgetClicked(): Display text:",i,appObjs[i].getDisplayName()
		print "widgetClicked(): Full Path to Icon:",i, appObjs[i].getFullIcon()
		print "widgetClicked(): Icon Theme:",i, appObjs[i].getIconTheme()

	if ( ((x > 0) and (y > 0) and (x < widgetXsize) and (y < widgetYsize)) and (button == 2) ):
		karamba.removeImageEffects(widget, image)
		onIcon = 0
		karamba.redrawWidget(widget)
		appObjs[i].showPopup()

####################################################
#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
####################################################
def widgetMouseMoved(widget, x, y, button):
	#This is where the highliting code will go when you pass
	# over the clickable area
	## Apply current icon effects if any
	global image,xIconCenter,yIconCenter,currentIconEffect
	currentIconEffect = conf['currentIconEffect']
	global xIconLeftBounds,xIconRightBounds,yIconTopBounds,yIconBottomBounds,onIcon
	global upCenter,downCenter,haveWheelMouse,upImage,downImage,upBounds,downBounds,onUp,onDown
	#use all the icon barriers based on earlier initialization
	#localize the highlight to the icon only instead of whole karamba region
	#this now works for any size icon used, tested up to 64x64 pixel icons
	#this also minimizes the cpu usage checking for entry only once
	if ((onIcon == 0) and (x > xIconLeftBounds) and (y > yIconTopBounds) and (x < xIconRightBounds) and (y < yIconBottomBounds)):
        	if (currentIconEffect != "none"):
			if (currentIconEffect == "highlight"):
				karamba.changeImageIntensity(widget, image, 0.3, 0)
			if (currentIconEffect == "gray"):
				karamba.changeImageToGray(widget, image, 0)
		onIcon = 1
		karamba.redrawWidget(widget)
		if(debug >= 2):
			print "widgetMouseMoved(): ", "X-",x, "Y-",y, "icon effect-",currentIconEffect, "Image return: ",image
	elif((onIcon == 1) and ( (x < xIconLeftBounds) or (y < yIconTopBounds) or (x > xIconRightBounds) or (y > yIconBottomBounds)) ):
		if (currentIconEffect != "none"):
			karamba.removeImageEffects(widget, image)
		onIcon = 0
		karamba.redrawWidget(widget)
		if(debug >= 2):
			print "widgetMouseMoved(): ", "X-",x, "Y-",y, "icon effect-",currentIconEffect, "Image return: ",image
	#highlight code for the up/down arrows when not using a wheel mouse
	#print "widgetMouseMoved(): Bounds: UP:",upBounds,"DOWN:",downBounds, "X:",x,"Y:",y,"onUp:",onUp
	if( (onUp == 0) and (haveWheelMouse == "No") and ( (x >= upBounds[0]) and (x <= upBounds[1]-1) and (y >= upBounds[2]) and (y <= upBounds[3]-1) ) ):
		if(debug >= 1):
			print "widgetMouseMoved(): highlight the UP arrow!"
		if (currentIconEffect != "none"):
			if (currentIconEffect == "highlight"):
				karamba.changeImageIntensity(widget, upImage, 0.3, 0)
			if (currentIconEffect == "gray"):
				karamba.changeImageToGray(widget, upImage, 0)
		onUp = 1
		karamba.redrawWidget(widget)
	elif( (onUp == 1) and (haveWheelMouse == "No") and ( (x < upBounds[0]) or (x >= upBounds[1]) or (y < upBounds[2]) or (y >= upBounds[3]) ) ):
		if (currentIconEffect != "none"):
			karamba.removeImageEffects(widget, upImage)
		onUp = 0
		karamba.redrawWidget(widget)
	#highlight code for the down arrow when not using a wheel mouse
	#print "widgetMouseMoved(): Bounds: UP:",upBounds,"DOWN:",downBounds, "X:",x,"Y:",y,"onDown:",onDown
	if( (onDown == 0) and (haveWheelMouse == "No") and ( (x >= downBounds[0]) and (x <= downBounds[1]-1) and (y >= downBounds[2]) and (y <= downBounds[3]-1) ) ):
		if(debug >= 1):
			print "widgetMouseMoved(): highlight the DOWN arrow!"
		if (currentIconEffect != "none"):
			if (currentIconEffect == "highlight"):
				karamba.changeImageIntensity(widget, downImage, 0.3, 0)
			if (currentIconEffect == "gray"):
				karamba.changeImageToGray(widget, downImage, 0)
		onDown = 1
		karamba.redrawWidget(widget)
	elif( (onDown == 1) and (haveWheelMouse == "No") and ( (x < downBounds[0]) or (x >= downBounds[1]) or (y < downBounds[2]) or (y >= downBounds[3]) ) ):
		if (currentIconEffect != "none"):
			karamba.removeImageEffects(widget, downImage)
		onDown = 0
		karamba.redrawWidget(widget)

####################################################
#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
####################################################
def meterClicked(widget, meter, button):
	global i,image,last,system,conf,font, fontColor, fontSize,debug,background,xMax,yMax,maxApps
	global shadow,icons,apps,names,xIconCenter,yIconCenter,xIconLeftBounds,xIconRightBounds,yIconTopBounds,yIconBottomBounds
	global upImage,downImage,midXmax,appObjs

	font = conf['font']
	fontColor = conf['fontColor']
	fontSize = conf['fontSize']
	debug = conf['debug']
	shadow = conf['shadow']
	#Applications - launch when clicked on by left mouse button
	#This takes care of all loaded applications and updates for each application
	#Launching into the background with either arguments if supplied in .rc file
	# or just by itself in the background without arguments.
	if((button == 1) and (meter == image)):
		name = re.split(r'(\s)',appObjs[i].getName(),1)
		if(len(name) >= 2):
			blah = name[2]
			blah = str(blah)
			myCommand = blah.strip().split(" ")
			myCommand.insert(0,name[0])
			karamba.executeInteractive(widget, myCommand)
			if(debug >= 1):
				print "meterClicked(): (if)","name: ",name,"myCommand: ",myCommand, "\nString: ",blah,"Meter:",meter,"Image:",image
		else:
			myCommand = [name[0]]
			blah = myCommand
			karamba.executeInteractive(widget, myCommand)
			if(debug >= 1):
				print "meterClicked(): (else)","name: ",name,"myCommand: ",myCommand, "\nString: ",blah,"Meter:",meter,"Image:",image
		karamba.redrawWidget(widget)
	#use the alternate up arrow to navigate
	if((meter == upImage) and (button == 1)):
		#do the up navigation here
		if(debug >= 1):
			print "widgetClicked(): UP:", appObjs[i].getFullIcon(), button, i, maxApps
		#cycle from app1 up to app#, and set back to beginning
		i = i+1
		if (debug >= 2):
			print "widgetClicked(): UP:", i
		#set maxApps to dynamically setup applications
		if ( i >= maxApps ):
			i = 0
		#keep text from continuous overwriting (making it really bold)
		karamba.deleteText(widget, system)
		karamba.deleteImage(widget, image)
		image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[i].getFullIcon())
		karamba.attachClickArea(widget,image)
		system = karamba.createText(widget, midXmax - len(appObjs[i].getDisplayName())*3, yMax, 80, 20, appObjs[i].getDisplayName())
		prettyText(widget, fontSize, font, system, fontColor, shadow)
		karamba.redrawWidget(widget)
		if(debug >= 1):
			print "widgetClicked(): Icon: "+str(appObjs[i].getDisplayName()), "Button: "+str(button), "interation: "+str(i)
			if(debug >= 2):
				print "widgetClicked(): Font: ",font, "FontColor: ",fontColor
	#use the alternate down arrow to navigate
	if((meter == downImage) and (button == 1)):
		#do the down navigation here
		if (debug >= 1):
			print "widgetClicked(): DOWN:", appObjs[i].getFullIcon(), button, i, maxApps
		#cycle back from app# to app0 and then back to end of stack
		i = i-1
		if (debug >= 2):
			print "widgetClicked(): DOWN:", i
		#take care of cycling back to 0 and then to maxApps
		if ( i <= (0-1) ):
			i = (maxApps-1)
		#keep text from continuous overwriting (making it really bold, not desireable)
		karamba.deleteText(widget, system)
		karamba.deleteImage(widget, image)
		image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[i].getFullIcon())
		karamba.attachClickArea(widget,image)
		system = karamba.createText(widget, midXmax - len(appObjs[i].getDisplayName())*3, yMax, 80, 20, appObjs[i].getDisplayName())
		prettyText(widget, fontSize, font, system, fontColor, shadow)
		karamba.redrawWidget(widget)
		if(debug >= 1):
			print "widgetClicked(): Icon: "+str(appObjs[i].getDisplayName()), "Button: "+str(button), "interation: "+str(i)
			if(debug >= 2):
				print "widgetClicked(): Font: ",font, "FontColor: ",fontColor

##################################################################
#adding new icons using the right click menu - configure theme
#Add an Icon - adds new icons to your current stack
#Remove an Icon - removes icons from the end of your stack
#Icon Directory - enter in the full path to the icons you want to use
#Background - turn background on or off
##################################################################
def menuOptionChanged(widget, key, value):
	global system, image, background,i,maxApps,kdeIconPath,names,xIconCenter, yIconCenter, icons,fontSize, font
	global yMax, bgChange,bgImage,kAddPid,kRemPid,kIconPid,haveWheelMouse,wmChange,midXmax,widgetXsize,widgetYsize,kUnique
	global xIconCenter,yIconCenter,upImage,downImage

	#add a new icon menu work
	if (key == "addicon"):
		karamba.setMenuConfigOption(widget, "addicon", 0)
		kdialog = ["kdialog","--inputbox","Please enter the application you want to add:"]
		kAddPid = karamba.executeInteractive(widget,kdialog)
		if(debug >= 1):
			print "menuOptionChanged(): addicon-",kAddPid
		karamba.redrawWidget(widget)
		if(debug >= 1):
			print "menuOptionChanged(): Apps: ",apps, "Names: ", names, "Icons: ", icons

	#remove last icon work
	if (key == "remicon"):
		karamba.setMenuConfigOption(widget, "remicon", 0)
		if(debug >= 1):
			print "menuOptionChanged(): i:",i,"maxApps:",karamba.readConfigEntry(widget,"maxApps")
		#test to be sure you don't get zero maxApps
		if( karamba.readConfigEntry(widget,"maxApps") > 1 ):
			appObjs.pop()
			maxApps = maxApps - 1
			karamba.writeConfigEntry(widget, "maxApps", str(maxApps))
			if(debug >= 1):
				print "menuOptionChanged(): removed icon",icons[maxApps],"maxApps:",karamba.readConfigEntry(widget,"maxApps")
		else:
			if(debug >= 1):
				print "menuOptionChanged(): You can not have zero applications in your stack!"
				print "menuOptionChanged(): But, you can re-choose your first application"
			karamba.setMenuConfigOption(widget, "addicon", 0)
			kRemDialog = ["kdialog","--inputbox", "Please enter a new first application you want to add:"]
			kRemPid = karamba.executeInteractive(widget,kRemDialog)
		if(debug >= 1):
			print "menuOptionChanged(): remicon-",maxApps
		karamba.attachClickArea(widget,image)
		karamba.redrawWidget(widget)

	#remove current icon work
	#FIXME: this is not complete in any way yet!!
	if (key == "remCurrentIcon"):
		karamba.setMenuConfigOption(widget, "remCurrentIcon", 0)
		if(debug >= 1):
			print "menuOptionChanged(): i:",i,"maxApps:",karamba.readConfigEntry(widget,"maxApps")
		#test to be sure you don't get zero maxApps
		if( karamba.readConfigEntry(widget,"maxApps") > 1 ):
			#get a reference to the current application object
			currentObj = appObjs[i]
			if currentObj.isIconUnique() == 'Yes':
				karamba.writeConfigEntry(widget,"unique"+str(i), "No")
			appObjs.remove(currentObj)
			del currentObj
			#this is so that we show the previous icon in our stack
			i -= 1
			#update our max apps
			maxApps = maxApps - 1
			karamba.writeConfigEntry(widget, "maxApps", str(maxApps))
			#rewrite our config file with the new list of applications
			for index in range(0,len(appObjs)):
				karamba.writeConfigEntry(widget, "app"+str(index), str(appObjs[index].getName()))
				karamba.writeConfigEntry(widget, "display"+str(index), str(appObjs[index].getDisplayName()))
				if appObjs[index].isIconUnique == 'Yes':
					temp = re.split(r'(\s)',karamba.readConfigEntry(widget,"app"+str(index)),1)
					karamba.writeConfigEntry(widget,"icon"+str(index),temp[0]+'.png')
					karamba.writeConfigEntry(widget,"unique"+str(index), "Yes")
					karamba.writeConfigEntry(widget,"iconTheme"+str(index),str(appObjs[index].getIconTheme()))
			if(debug >= 1):
				print "menuOptionChanged(): appObjs: ",appObjs[i].getName(),appObjs[i+1].getName()

			#Now display the previous application b/c if we show the next one, it may be out of range
			karamba.deleteImage(widget, image)
			#b/c i becomes -1 at some point.. set it to the last one in our list
			if i == -1:
				i = maxApps-1
			image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[i].getFullIcon())
			karamba.attachClickArea(widget,image)

			karamba.deleteText(widget, system)
			system = karamba.createText(widget, midXmax - len(appObjs[i].getDisplayName())*3, yMax, 80, 20, appObjs[i].getDisplayName())
			prettyText(widget, fontSize, font, system, fontColor, shadow)
			karamba.redrawWidget(widget)

			if(debug >= 1):
				print "menuOptionChanged(): removed current icon",str(appObjs[i].getDisplayName()),"maxApps:",karamba.readConfigEntry(widget,"maxApps")
		else:
			if(debug >= 1):
				print "menuOptionChanged(): You can not have zero applications in your stack!"
				print "menuOptionChanged(): But, you can re-choose your first application"
			karamba.setMenuConfigOption(widget, "addicon", 0)
			kRemDialog = ["kdialog","--inputbox", "Please enter a new first application you want to add:"]
			kRemPid = karamba.executeInteractive(widget,kRemDialog)
		if(debug >= 1):
			print "menuOptionChanged(): remCurrentIcon-",maxApps
		karamba.attachClickArea(widget,image)
		karamba.redrawWidget(widget)

	#unique icon work
	if(key == "uniqueIcon"):
		if(debug >= 1):
			print "Setting unique icon now..."
		karamba.setMenuConfigOption(widget, "uniqueIcon", 0)
		kUniqueDialog = ["kdialog","--inputbox", "Please enter a unique icon for the showing application. \nFormat: /path/to/the/icon.png"]
		kUnique = karamba.executeInteractive(widget,kUniqueDialog)
		if(debug >= 1):
			print "menuOptionChanged(): kUnique should have changed", kUnique

	#clear unique icon work
	if(key == "clearUniqueIcon"):
		if(debug >= 1):
			print "Clearing unique icon, and setting to default..."
		karamba.setMenuConfigOption(widget, "clearUniqueIcon",0)
		karamba.writeConfigEntry(widget,"iconTheme"+str(i),kdeIconPath)
		appObjs[i].setIconTheme(kdeIconPath)
		appObjs[i].clearUniqueIcon()
		appObjs[i].refreshIcon()
		if( (os.path.exists( (appObjs[i].getFullIcon()) )) == False):
			print "commandOutput() Couldn't find the path and icon: ",appObjs[i].getFullIcon()
			appObjs[i].setIcon('unknown.png')
			appObjs[i].refreshIcon()
		karamba.writeConfigEntry(widget,"icon"+str(i),appObjs[i].getIcon())
		karamba.writeConfigEntry(widget,"unique"+str(i),'No')
		karamba.deleteImage(widget,image)
		image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[i].getFullIcon())
		karamba.attachClickArea(widget,image)
		if(debug >= 1):
			print "commandOutput(): Icon:",i,appObjs[i].getIcon()
			print "commandOutput(): Application:",i,appObjs[i].getName()
			print "commandOutput(): Display text:",i,appObjs[i].getDisplayName()
			print "commandOutput(): Full Path to Icon:",i, appObjs[i].getFullIcon()
			print "commandOutput(): Icon Theme:",i, appObjs[i].getIconTheme()
		karamba.redrawWidget(widget)

	#icon directory work
	if(key == "iconDir" ):
		iconMenu = karamba.readMenuConfigOption(widget, "iconDir")
		if(karamba.readConfigEntry(widget, "kdePath") != "custom"):
			karamba.writeConfigEntry(widget, "kdePath", "custom")
			iconDialog = ["kdialog","--inputbox","Please enter directory of the icons you want to use:"]
			kIconPid = karamba.executeInteractive(widget,iconDialog)
			if(debug >= 1):
				print "menuOptionChanged(): iconDir should have brought up kdialog now"
		elif(karamba.readConfigEntry(widget,"kdePath") == "custom"):
			setStandardTheme(widget)
			loadIcons(widget)
			if(debug >= 1):
				print "menuOptionChanged(): iconDir should have been set to default theme now"
		if(debug >= 1):
			print "menuOptionChanged(): kdePath-",karamba.readConfigEntry(widget, "kdePath")
			print "menuOptionChanged(): value-", value
			print "menuOptionChanged(): The Icon Directory option should have changed.",iconMenu

	#toggle background image work
	if(key == "bgImage"):
		if (debug >= 1):
			print "menuOptionChanged(): bgImage called from menu"
		#FIXME: use callback to widgetUpdated() b/c the comparison isn't 'bool' on the return of other functions
		bgChange = 1
		bgImage = karamba.readMenuConfigOption(widget, "bgImage")
		widgetUpdated(widget)
		karamba.redrawWidget(widget)
		if(debug >= 1):
			print "menuOptionChanged(): bgImage should have changed", bgImage

	#toggle the arrows for navigation without a wheel mouse
	if(key == "wheelMouse"):
		wmChange = 1
		widgetUpdated(widget)
		karamba.redrawWidget(widget)
		if(debug >= 1):
			print "menuOptionChanged(): haveWheelMouse should have changed",haveWheelMouse

####################################################
#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
####################################################
def commandOutput(widget, pid, output):
	global kAddPid,koutput,kRemPid,kIconPid,kFirstPid,kfirstoutput,kDisplayPid,fromRemPid,fromAddPid,kUnique
	global system, image, background,i,maxApps,kdeIconPath,names,xIconCenter, yIconCenter, icons,fontSize, font,yMax, configchange,bgImage,kAddPid,kRemPid,iconchange
	global upImage, downImage, debug, midXmax

	#add icon work
	if( pid == kAddPid ):
		koutput = output
		koutput = koutput.strip()
		if (koutput != ""):
			koutput = re.sub("&","",koutput)
			karamba.writeConfigEntry(widget, "app"+str(maxApps), koutput)
			fromAddPid = 1
			displayDialog = ["kdialog","--inputbox","Please enter name you want displayed for this application:"]
			kDisplayPid = karamba.executeInteractive(widget,displayDialog)

	#remove very first icon and replace it, updating text and rc file work
	if( pid == kRemPid ):
		koutput = output
		koutput = koutput.strip()
		if (koutput != ""):
			appObjs.pop()
			koutput = re.sub("&","",koutput)
			karamba.writeConfigEntry(widget, "app0", koutput)
			fromRemPid = 1
			displayDialog = ["kdialog","--inputbox","Please enter name you want displayed for this application:"]
			kDisplayPid = karamba.executeInteractive(widget,displayDialog)

	if(pid == kUnique ):
		koutput = output
		koutput = koutput.strip()
		if (koutput != ""):
			#split the ending icon name from the path
			fullPathAndIcon = re.split("/",koutput)
			icon = fullPathAndIcon[-1]
			path = re.sub(icon,"",koutput)
			karamba.writeConfigEntry(widget,"iconTheme"+str(i),path)
			if(debug >= 1):
				print "Icon:",icon,"\nPath:",path
			#set the appObjs[i] with the new icon and iconTheme
			appObjs[i].setIconTheme(path)
			appObjs[i].setUniqueIcon(icon)
			appObjs[i].refreshIcon()
			if( (os.path.exists( (appObjs[i].getFullIcon()) )) == False):
				print "commandOutput() Couldn't find the path and icon: ",appObjs[i].getFullIcon()
				appObjs[i].setIcon('unknown.png')
				appObjs[i].setIconTheme('icons/')
				appObjs[i].refreshIcon()
			karamba.writeConfigEntry(widget,"icon"+str(i),appObjs[i].getIcon())
			karamba.writeConfigEntry(widget,"unique"+str(i),'Yes')
			karamba.deleteImage(widget,image)
			image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[i].getFullIcon())
			karamba.attachClickArea(widget,image)
			if(debug >= 1):
				print "commandOutput(): Icon:",i,appObjs[i].getIcon()
				print "commandOutput(): Application:",i,appObjs[i].getName()
				print "commandOutput(): Display text:",i,appObjs[i].getDisplayName()
				print "commandOutput(): Full Path to Icon:",i, appObjs[i].getFullIcon()
				print "commandOutput(): Icon Theme:",i, appObjs[i].getIconTheme()
			karamba.redrawWidget(widget)

	#icon directory work
	if( pid == kIconPid ):
		koutput = output
		koutput = koutput.strip()
		iconDir = koutput
		if( koutput != "" ):
			length = len(iconDir)
			last = iconDir[length-1]
			if(last != "/"):
				last = "/"
				iconDir = iconDir + last
			kdeIconPath = iconDir
			karamba.writeConfigEntry(widget, "iconDir", iconDir)
			karamba.writeConfigEntry(widget, "kdePath", "custom")
			if(debug >= 1):
				print "commandOutput(): icon directory",karamba.readConfigEntry(widget,"iconDir")
			loadIcons(widget)
			if(debug >= 1):
				print "commandOutput(): Redrew the text","Icon's center:","(",xIconCenter,",",yIconCenter,")","yMax:",yMax
		karamba.redrawWidget(widget)

	#display text work
	if( pid == kDisplayPid ):
		koutput = output
		koutput = koutput.strip()
		if ( koutput != "" ):
			#detect where we are coming from ie. kRemPid
			if( fromRemPid == 1):
				#since this is the first application, all place holders will be zero
				karamba.writeConfigEntry(widget, "display0", koutput)
				temp = re.split(r'(\s)',karamba.readConfigEntry(widget,"app0"),1)
				karamba.writeConfigEntry(widget,"icon0",temp[0]+'.png')
				karamba.writeConfigEntry(widget,"iconTheme0",karamba.readConfigEntry(widget, "iconDir"))
				name = karamba.readConfigEntry(widget,"app0")
				#print "commandOutput(): name:",name
				displayName = karamba.readConfigEntry(widget,"display0")
				icon = karamba.readConfigEntry(widget,"icon0")
				fromRemPid = 0
				if( debug >= 1):
					print "commandOutput(): name-",name, kdeIconPath+name[0]+".png"
				appObjs.append( applications.Application(name,displayName,widget) )
				#FIXME: rework this to support unique icons
				appObjs[0].setIconTheme(kdeIconPath)
				appObjs[0].refreshIcon()
				if( (os.path.exists( (appObjs[0].getFullIcon()) )) == False):
					print "commandOutput() Couldn't find the path and icon: ",appObjs[0].getFullIcon()
					appObjs[0].setIcon('unknown.png')
					appObjs[0].refreshIcon()
				if(debug >= 1):
					print "commandOutput(): Icon:",0,appObjs[0].getIcon()
					print "commandOutput(): Application:",0,appObjs[0].getName()
					print "commandOutput(): Display text:",0,appObjs[0].getDisplayName()
					print "commandOutput(): Full Path to Icon:",0, appObjs[0].getFullIcon()
					print "commandOutput(): Icon Theme:",0, appObjs[0].getIconTheme()
				#delete the first showing application
				karamba.deleteText(widget, system)
				karamba.deleteImage(widget, image)
				#now display the new first application
				image = karamba.createImage(widget, xIconCenter, yIconCenter, appObjs[0].getFullIcon())
				system = karamba.createText(widget, midXmax - len(appObjs[0].getDisplayName())*3, yMax, 80, 20, appObjs[0].getDisplayName())
				prettyText(widget, fontSize, font, system, fontColor, shadow)
				karamba.attachClickArea(widget,image)
				if( debug >= 1 ):
					print "commandOutput(): text entered-",koutput
					print "commandOutput(): displayText-", displayName
			#detect where we are coming from ie. kAddPid
			if( fromAddPid == 1 ):
				karamba.writeConfigEntry(widget, "display"+str(maxApps), koutput)
				temp = re.split(r'(\s)',karamba.readConfigEntry(widget,"app"+str(maxApps)),1)
				karamba.writeConfigEntry(widget,"icon"+str(maxApps),temp[0]+'.png')
				karamba.writeConfigEntry(widget,"iconTheme"+str(maxApps),karamba.readConfigEntry(widget, "iconDir"))
				fromAddPid = 0
				#do all the needed work from kAddPid that needed moved here
				name = karamba.readConfigEntry(widget,"app"+str(maxApps))
				displayName = karamba.readConfigEntry(widget,"display"+str(maxApps))
				icon = karamba.readConfigEntry(widget,"icon"+str(maxApps))
				#make a new applications object instance
				appObjs.append( applications.Application(name,displayName,widget) )
				#FIXME: rework this to support unique icons
				appObjs[maxApps].setIconTheme(kdeIconPath)
				appObjs[maxApps].refreshIcon()
				if( (os.path.exists( (appObjs[maxApps].getFullIcon()) )) == False):
					print "commandOutput() Couldn't find the path and icon: ",appObjs[maxApps].getFullIcon()
					appObjs[maxApps].setIcon('unknown.png')
					appObjs[maxApps].refreshIcon()
					if(debug >= 1):
						print "commandOutput(): Icon:",maxApps,appObjs[maxApps].getIcon()
						print "commandOutput(): Application:",maxApps,appObjs[maxApps].getName()
						print "commandOutput(): Display text:",maxApps,appObjs[maxApps].getDisplayName()
						print "commandOutput(): Full Path to Icon:",maxApps, appObjs[maxApps].getFullIcon()
						print "commandOutput(): Icon Theme:",maxApps, appObjs[maxApps].getIconTheme()
				maxApps = maxApps + 1
				karamba.writeConfigEntry(widget, "maxApps",str(maxApps))
				if(debug >= 1):
					print "commandOutput(): added new icon",icons[maxApps-1],"maxApps:",karamba.readConfigEntry(widget,"maxApps")
					print "commandOutput():",koutput, kAddPid, maxApps
					print "commandOutput(): names:",names
		karamba.redrawWidget(widget)

####################################################
#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
#  all urls are getting decoded w/ urllib.unquote() b/c
#  you can't use them when opening files in paths with special characters
####################################################
def itemDropped(widget, dropText):
	global kdeIconPath,maxApps,apps,names,icons,count,appObjs,debug
	import mimetypes
	#preliminary work for the dropped text
	#print "itemDropped() dropText:",dropText
	url = dropText
	dropText = urllib.unquote(dropText)
	fileName = ""
	fileExtension = ""
	Exec = []
	ExecName = ""
	#print "itemDropped() dropText:",dropText
	dropCount=0
	#count the number of dropped items for multifile drops
	for char in dropText:
		if( char == ':' ):
			dropCount += 1
	#separate each file now
	items = re.split("\n",dropText)
	drops=[]
	objects=[]
	desktopFiles=[]
	desktopFilesExt=[]
	#calculate multiple dropped items
	for dropItem in range(0,dropCount):
		preObjects = re.split(":",items[dropItem])
		preObjects[1] = preObjects[1].strip()
		(path,desktopFile) = os.path.split(preObjects[1])
		desktopFile = desktopFile.strip()
		desktopFiles.append(desktopFile)
		desktopFileExt = re.split("\.",desktopFile)
		desktopFilesExt.append(desktopFileExt)
		objects.append(preObjects[1])
		#print "preObjects: ",preObjects
		objType = preObjects[0]
		drops.append(items[dropItem])
		#print "items:",items,"dropCount:",dropCount,"drops:",drops
		#FIXME: put a check for matching extensions on all the drops[] list
		# if it fails, bail out of this callback completely, and don't try to open the file
		# this check has to incorporate mimetypes compatibility for the showing application

	# if we want the application to open it
	# and only handle file:// drops for now, url:// will be implemented later
	if(objType == "file"):
		#add a check here to see if this is a .desktop file as well as if it is a file or not
		if( desktopFileExt[-1] != "desktop"):
			#it seems that links to applications on the desktop
			# are just files so they need parsed here
			if(debug >= 1):
				print "itemDropped(): file was dropped"
			#FIXME: experimental application launching of dropped files
			name = re.split(r'(\s)',appObjs[i].getName(),1)
			if(len(name) >= 2):
				blah = name[2]
				blah = str(blah)
				myCommand = blah.strip().split(" ") #command line execution, spaces, -options
				myCommand.insert(0,name[0])
				#multi-file drop support loop
				for x in range(0,dropCount):
					if( os.path.isfile(objects[x]) ):
						myCommand.append(objects[x]) #now add the file(s) to the command line execution
				#FIXME: applications running when you close iLaunch halt unexpectedly, at least mplayer did
				#myCommand.append(object+" > /dev/null 2> /dev/null") #<-this doesn't workaround it
				karamba.executeInteractive(widget, myCommand)
				if(debug >= 1):
					print "itemDropped(): (if)","name: ",name,"myCommand: ",myCommand, "\nString: ",blah,"Image:",image
			else:
				myCommand = [name[0]] #without -options
				#multi-file drop support loop
				for x in range(0,dropCount):
					if( os.path.isfile(objects[x]) ):
						myCommand.append(objects[x]) #now add the file(s) to the command line exection
				#FIXME: applications running when you close iLaunch halt unexpectedly, at least mplayer did
				#myCommand.append(" > /dev/null 2> /dev/null") #<-this doesn't workaround it
				blah = myCommand
				karamba.executeInteractive(widget, myCommand)
				if(debug >= 1):
					print "itemDropped(): (else)","name: ",name,"myCommand: ",myCommand, "\nString: ",blah,"Image:",image
			karamba.redrawWidget(widget)
		#don't handle directories yet
		#FIXME: objects[0] is the first dropped item, rework for multiple dropped desktop icons
		elif( os.path.isdir(objects[0]) ):
			#FIXME: do something constructive with a directory
			kErrorDialog = ["kdialog","--error", "Directories are not yet supported for drag-n-drop."]
			kErrorPid = karamba.executeInteractive(widget,kErrorDialog)
			if(debug >= 1):
				print "itemDropped(): directory was dropped"
		#handle .desktop files and add them to the iLaunch list of applications
		elif( desktopFileExt[-1] == "desktop" ):
			#handles all the .desktop files in your Kmenu and kicker
			#and adds them to your stack of icons in iLaunch
			#FIXME: objects[0] is the first dropped item, rework for multiple dropped desktop icons
			object = os.path.normpath(objects[0])
			#1)process the .desktop file for exec and icon parameters now
			#take care of whitespace & newlines
			dFile = object.strip()
			dFile = dFile.replace('\n','') #newlines
			file = open(dFile,"r")
			text = file.readlines()
			file.close()
			keyword = re.compile(r"Exec")
			keyword2 = re.compile(r"Icon")
			keyword3 = re.compile(r"Name")
			#find the icon and execution command
			count = 0
			for line in text:
				if keyword.search(line):
					Exec.append(line)
					Exec[count] = Exec[count].replace('\n','')
					(junk,Exec[count]) = re.split('=',Exec[count])
					Exec[count] = re.split(r'(\s)',Exec[count],1)
					temp = Exec[count][0]
					#resolve the 'soundwrapper' being first problem
					#FIXME: temporary fix though b/c we might want soundwrapper later
					if( temp == 'soundwrapper' ):
						Exec[count].remove('soundwrapper')
						Exec[count].remove(' ')
						split = Exec[count]
						nameString = ""
						for part in split:
							nameString += part
						nameString = nameString.replace('%u','')
						nameString = nameString.strip()
						temp = nameString
						Exec[count][0] = os.path.basename(temp)
					else:
						Exec[count][0] = os.path.basename(temp)
					if( debug >= 1):
						print "itemDropped(): Exec:", Exec,"temp:",temp
						print "itemDropped(): Final png icon:",kdeIconPath+Exec[0][0]+".png"
					count += 1
				if keyword2.search(line):
					Icon = line
					Icon = Icon.replace('\n','')
					(junk,Icon) = re.split('=',Icon)
				if keyword3.search(line):
					dName = line
					dName = dName.replace('\n','')
					(junk,dName) = re.split('=',dName)
			#2)check to see if there is an icon for it and give error if not(optional kdialog entry)
			#3)then figure out how to add it to the stack
			try:
				#build the string needed for 'names' and 'writeConfigEntry()'
				for test in Exec[0]:
					ExecName += test
				#experimental and might not include all rarities in the kmenu
				#that doesn't work from command line
				ExecName = ExecName.replace('-caption','')
				ExecName = ExecName.replace('%f','')
				ExecName = ExecName.replace('%m','')
				ExecName = ExecName.replace('\"%c\"','')
				ExecName = ExecName.replace('%i','')
				ExecName = ExecName.replace('%u','')
				ExecName = ExecName.replace('%U','')
				ExecName = ExecName.strip()
				karamba.writeConfigEntry(widget, "app"+str(maxApps), ExecName)
				karamba.writeConfigEntry(widget, "display"+str(maxApps), dName)
				temp = re.split(r'(\s)',ExecName,1)
				karamba.writeConfigEntry(widget,"icon"+str(maxApps),temp[0]+'.png')
				karamba.writeConfigEntry(widget,"iconTheme"+str(maxApps),karamba.readConfigEntry(widget, "iconDir"))
				name = karamba.readConfigEntry(widget,"app"+str(maxApps))
				displayName = karamba.readConfigEntry(widget,"display"+str(maxApps))
				icon = karamba.readConfigEntry(widget,"icon"+str(maxApps))
				#make a new applications object instance
				appObjs.append( applications.Application(name,displayName,widget) )
				#FIXME: rework this to support unique icons
				appObjs[maxApps].setIconTheme(kdeIconPath)
				appObjs[maxApps].refreshIcon()
				if( (os.path.exists( (appObjs[maxApps].getFullIcon()) )) == False):
					print "itemDropped() Couldn't find the path and icon: ",appObjs[maxApps].getFullIcon()
					appObjs[maxApps].setIcon('unknown.png')
					if(os.path.exists((kdeIconPath+Exec[0][0]+".png")) == False):
						appObjs[maxApps].setIconTheme('icons/')
					appObjs[maxApps].refreshIcon()
					if(debug >= 1):
						print "itemDropped(): Icon:",maxApps,appObjs[maxApps].getIcon()
						print "itemDropped(): Application:",maxApps,appObjs[maxApps].getName()
						print "itemDropped(): Display text:",maxApps,appObjs[maxApps].getDisplayName()
						print "itemDropped(): Full Path to Icon:",maxApps, appObjs[maxApps].getFullIcon()
						print "itemDropped(): Icon Theme:",maxApps, appObjs[maxApps].getIconTheme()
				maxApps = maxApps + 1
				karamba.writeConfigEntry(widget, "maxApps",str(maxApps))
				if(debug >= 1):
					print "itemDropped(): icon found- ",kdeIconPath+Exec[0]+".png"
			except:
				print "Error locating an icon or directory for the dropped application"
			if(debug >= 1):
				print "itemDropped(): .desktop file was dropped"
				print "itemDropped(): Exec-",Exec,"Icon-",Icon
				print "itemDropped(): ExecName:",ExecName,"Exec:",Exec
	elif(objType == "http" or objType == "https"):
		#browser = os.getenv("BROWSER")
		# should execute in the form of: <browser> <url> (ie. mozilla http://slashdot.org)
		#karamba.executeInteractive(widget,[browser,dropText])
		print "itemDropped():Url drops are not supported currently: ",url
	if(debug >= 1):
		print "itemDropped(): dropText-",dropText,"Object:",object,"DesktopFile:",desktopFile

####################################################
# callbacks that I don't use for iLaunch
####################################################
def widgetClosed(widget):
    pass
def menuItemClicked(widget, menu, id):
    pass

def startupAdded(widget, startup):
    pass

def startupRemoved(widget, startup):
    pass

def taskAdded(widget, task):
    pass

def taskRemoved(widget, task):
    pass

def activeTaskChanged(widget, task):
    pass

# This will be printed when the widget loads.
print "Loaded iLaunch version 0.15!"

