#!/usr/bin/python2
# This file is part of the pymoontool application
# This file is part of the karmaba_moontool application
# Written by Eric J. Gerney <gerney@att.com>
# Adapted from xmoontool by John Walker
# Distributed under GPL version 2.

# This file defines functions for converting time between
# UTC (Gregorian) and Juilian and is used by the moontool applications.

# Import the math module
import math

def jtime(t):
	# Helper function to assist in converting UTC to Julian
	# Argument(s): struct_time
	# Returns: float Julian time
	return utc2jd(t.tm_year, int(t.tm_mon), t.tm_mday, int(t.tm_hour), \
		int(t.tm_min), int(t.tm_sec))

def utctojd(y,m,d,h,mn,s):
	# Convert UTC date and time to astronomical Julian time
	# Argument(s): year,month,day,hour,minute,second)
	# Returns: float Julian time
	if ( y == 0):
		print "INVALID - No Zero year in Julian"
		return 0
	if ( y == 1582 & m == 10 & d > 4 & d < 15):
		print "INVALID The dates 5 through 14 October, 1582, do not exist in the Gregorian system!"
		return 0

	if (m > 2):
		jy = y
		jm = m + 1
	else:
		jy = y - 1
		jm = m + 13

	d = d - 1
	intgr = math.floor( math.floor(365.25*jy) + math.floor(30.6001*jm) + d + 1720995 )
	gregcal = 15 + 31*( 10 + 12*1582 )
	if ( d + 31*(m + 12*y) >= gregcal ):
		ja = math.floor(0.01*jy)
		intgr += 2 - ja + math.floor(0.25*ja)

	dayfrac = h/24.0 - 0.5
	if (dayfrac < 0.0 ):
		dayfrac += 1.0
		--intgr

	frac = dayfrac + (mn + s/60.0)/60.0/24.0

	jd0 = (intgr + frac)*100000
	jd = math.floor(jd0)
	if (jd0 - jd >0.5 ):
		++jd
	return (jd/100000)

def jdtoutc(jd):
	# Convert Julian date to UTC
	# Argument(s): float julian date
	# Returns: year, month, day, hour, minute, second integers
	# get the date from the Julian day number
	intgr   = math.floor(jd)
	frac    = jd - intgr
	gregjd  = 2299161
	if (intgr >= gregjd ):		# Gregorian calendar correction
		tmp = math.floor( ( (intgr - 1867216) - 0.25 ) / 36524.25 )
		j1 = intgr + 1 + tmp - math.floor(0.25*tmp)
	else:
		j1 = intgr

	# correction for half day offset
	dayfrac = frac + 0.5
	if (dayfrac >= 1.0 ):
		dayfrac -= 1.0
		++j1

	j2 = j1 + 1524;
	j3 = math.floor( 6680.0 + ( (j2 - 2439870) - 122.1 )/365.25 )
	j4 = math.floor(j3*365.25)
	j5 = math.floor( (j2 - j4)/30.6001 )

	d = math.floor(j2 - j4 - math.floor(j5*30.6001))
	m = math.floor(j5 - 1)
	if( m > 12 ):
		m -= 12
	y = math.floor(j3 - 4715)
	if( m > 2 ):
		--y
	if( y <= 0 ):
		--y

	# get time of day from day fraction
	hr  = math.floor(dayfrac * 24.0)
	mn  = math.floor((dayfrac*24.0 - hr)*60.0)
	f  = ((dayfrac*24.0 - hr)*60.0 - mn)*60.0
	sc  = math.floor(f)
	f -= sc
	if( f > 0.5 ):
		++sc
	if( sc == 60 ):
		sc = 0
		++mn
	if( mn == 60 ):
		mn = 0
		++hr
	if( hr == 24 ):
		hr = 0
		++d				# this could cause a bug, but probably will never happen in practice

	return (int(y),int(m),int(d),int(hr),int(mn),int(sc+1))

def jyear(td):
	td += 0.5					# Astronomical to civil
	j = math.floor(td)
	j = j - 1721119.0
	y = math.floor(((4 * j) - 1) / 146097.0)
	j = (j * 4.0) - (1.0 + (146097.0 * y))
	d = math.floor(j / 4.0)
	j = math.floor(((4.0 * d) + 3.0) / 1461.0)
	d = ((4.0 * d) + 3.0) - (1461.0 * j)
	d = math.floor((d + 4.0) / 4.0)
	m = math.floor(((5.0 * d) - 3) / 153.0)
	d = (5.0 * d) - (3.0 + (153.0 * m))
	d = math.floor((d + 5.0) / 5.0)
	y = (100.0 * y) + j
	if (m < 10.0):
		m = m + 3
	else:
		m = m - 9
		y = y + 1

	return (y, m, d)

def jhms(jd):
	jd += 0.5				# Astronomical to civil
	ij = (jd - math.floor(jd)) * 86400.0
	h = ij / 3600
	m = (ij/60) % 60
	s = ij % 60

	return (h,m,s)

def utc2jd(y,m,d,h,mn,s):
	extra = 100.0*y + m - 190002.5
	rjd = 367.0*y

	rjd -= math.floor(7.0*(y+math.floor((m+9.0)/12.0))/4.0)
	rjd += math.floor(275.0*m/9.0)
	rjd += d
	rjd += (h + (mn + s/60.0)/60.0)/24.0
	rjd += 1721013.5
	rjd -= 0.5*extra/math.fabs(extra)
	rjd += 0.5

	return rjd
