#!/usr/bin/python2
# This file is part of the karmaba_moontool application
# Written by Eric J. Gerney <gerney@att.com>
# Adapted from xmoontool by John Walker
# Distributed under GPL version 2.

# This is the main application called by SuperKarmaba for the
# karamba_moontool theme.

# Import the Karamba, time, math, & string modules along with our own functions.
import karamba, time, math, string
import constants, julian, cus_math, mooncalc, drawmoon

# This tool should display the current phase of the moon graphically (picture)
# When left clicked, detailed information about the moon should appear.
# When right clicked, detailed lunation information should be displayed.

init 		= 0		# Init var
textual 	= 0		# 0 means yes: 1 means no
graphical 	= 0		# 0 means yes: 1 means no
detail 		= 1		# 0 means yes: 1 means no
boxwidth 	= 301		# Our theme size (width)
boxheight 	= 140		# Our theme size (height)
mimagewidth	= 130		# Our graphic size (width)
mimageheight	= 130		# Our graphic size (height)
fontsize 	= 10		# Our fontsize
font 		= "verdana"	# Our font
oldph		= 0		# Previous value of moon phase fraction
moonorig	= "moonpics/fullmoon320.png"
moonfile	= "moonpics/current-moon.png"
# Initial values if not found in the config file.
mrotation	= 45
fillvalue	= "#00000080"
dis_miles	= 0		# default to kilometers.

# Function to format our text.  Taken from liquid_weather++
def prettyText(widget, size, font, textname):
	karamba.changeTextSize(widget, textname, size)
	karamba.changeTextFont(widget, textname, font)
	karamba.changeTextColor(widget, textname, 255,255,255)

# This is called when you widget is initialized
def initWidget(widget):
	pass

# This is called everytime your widget is updated
# the update inverval is specified in the .theme file
def widgetUpdated(widget):
	global init, graphical, textual, detail, fillvalue, mrotation, dis_miles
	#global init, fontsize, boxwidth, boxheight, textual, graphical, detail
	#global strjuliantime, struniversaltime, strlocaltime, straom, strmoonphase
	#global strmoondist, strmoonsub, strsundist, strsunsub
	global mytime, utc, lutc, jd, p, pphase, mage, mdist, mangdia, sudist, oldph
	global swangdia, aom_d, aom_h, aom_m
	# Determine the current date.
	mytime = time.time()
	# Get UTC
	utc = time.gmtime(mytime)
	# Local
	lutc = time.localtime(mytime)
	# Julian time
	jd = julian.jtime(utc)
	# Compute the phase of the moon.
	(p,pphase,mage,mdist,mangdia,sudist,swangdia) = mooncalc.phase(jd)
	# Process the Age of the moon.
	(aom_d, aom_h, aom_m) = mooncalc.aom(mage)

	# Screen Updates
	if init == 0:
		# Initial Draw
        # Add config menus......
		karamba.addMenuConfigOption(widget, "fillvalue", "Change moon shade color")
		karamba.addMenuConfigOption(widget, "mrotation", "Change moon rotation")
		karamba.addMenuConfigOption(widget, "dis_miles", "Display Miles instead of Kilometers")

		# Grab some configs.....
		filltest = karamba.readConfigEntry(widget, "shadecolor")
		if (filltest != None):
			fillvalue = filltest
		mrottest = karamba.readConfigEntry(widget, "moonrotation")
		if (mrottest != None):
			mrotation = int(mrottest)
		dis_miles = karamba.readMenuConfigOption(widget, "dis_miles")

		# Set menu's that have dialogs to have no checkmarks....
		karamba.setMenuConfigOption(widget, "fillvalue", "0")
		karamba.setMenuConfigOption(widget, "mrotation", "0")

		# Should we show the moon?
		if graphical == 0:
			# The Moon
			drawmoon.drawmoon(p,karamba.getThemePath(widget)+moonorig, \
					karamba.getThemePath(widget)+moonfile, \
					str(fillvalue))
			moontoolgraphic(widget)
			# Store the value of p into oldph for later....
			oldph = round(p,3)
		# Should we show the text?
		if textual == 0:
			moontoolInitTextDraw(widget)

		# Set global init is complete
		init = 1
	else:
		# We need to update the screen
		# if the moon is visible, update it.
		if graphical == 0:
			# The Moon
			# Only update if p is different.....
			if (oldph != round(p,3)):
				# Update oldph with p for later......
				oldph = round(p,3)
				drawmoon.drawmoon(p,karamba.getThemePath(widget)+moonorig, \
						karamba.getThemePath(widget)+moonfile, \
						str(fillvalue))
				moontoolgraphic(widget)
				# We need to redo the text if shown......
				if textual == 0:
					moontoolDelTextDraw(widget)
					moontoolInitTextDraw(widget)
				if detail == 0:
					moontooldetailDelTextDraw(widget)
					moontooldetailInitTextDraw(widget, jd)
		# if the text is shown, update it.
		if textual == 0:
			moontoolUpdTextDraw(widget)

	# Force the screen refresh
	karamba.redrawWidget(widget)

# This gets called everytime our widget is clicked.
# Notes:
#   widget = reference to our widget
#   x = x position (relative to our widget)
#   y = y position (relative to our widget)
#   botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
	global textual, detail, jd
	# If the button is left, then we'll toggle the "Normal" text
	if button == 1:
		# If the text is on, we want to remove it.
		if textual == 0:
			moontoolDelTextDraw(widget)
			textual = 1
			return
		# Else, the text is off, we'll turn it on.
		else:
			# First if detail is on, turn it off.
			if detail == 0:
				moontooldetailDelTextDraw(widget)
				detail = 1
			# Update the moon so it's behind the text....
			moontoolgraphic(widget)
			moontoolInitTextDraw(widget)
			textual = 0

	# If the button is middle, then we'll toggle the "detail" text
	if button == 2:
		# If the detail is on, we want to remove it.
		if detail == 0:
			moontooldetailDelTextDraw(widget)
			detail = 1
			return
		# Else, the text is off, we'll turn it on.
		else:
			detail = 0
			# First if detail is on, turn it off.
			if textual == 0:
				moontoolDelTextDraw(widget)
				textual = 1
			# Update the moon so it's behind the text....
			moontoolgraphic(widget)
			moontooldetailInitTextDraw(widget, jd)
	return

def widgetMouseMoved(widget, x, y, button):
	# Warning:  Don't do anything too intensive here.
	#You don't want to run some complex piece of code everytime
	#the mouse moves
	pass

def moontoolInitTextDraw(widget):
	# Draw a box with the textual information of the moontool.
	# Only call this once per invokation of the textual information.
	global fontsize, boxwidth, boxheight
	global strjuliantime, struniversaltime, strlocaltime, straom, strmoonphase
	global strmoondist, strmoonsub, strsundist, strsunsub, strheader
	global backgroundimageI
	# Background Graphic
	backgroundimageI = karamba.createImage(widget, 1, 1, \
			   karamba.getThemePath(widget)+"pics/backgnd.png")
	# Box Header
	strheader = karamba.createText(widget, 1, 1, boxwidth, 11,		\
		"Moontool Information:")
	prettyText(widget, fontsize, font, strheader)
	# Julian Time
	strjuliantime = karamba.createText(widget, 1, 11, boxwidth, 11, 	\
		"Julian Time:\t\t" +  str("%.5f" % jd))
	prettyText(widget, fontsize, font, strjuliantime)
	# Universal Time
	struniversaltime = karamba.createText(widget, 1, 22, boxwidth, 11, 	\
		"Universal Time:\t" + str(time.strftime("%H:%M:%S - %d %B %Y (%a)",utc)))
	prettyText(widget, fontsize, font, struniversaltime)
	# Local Time
	strlocaltime = karamba.createText(widget, 1, 33, boxwidth, 11, 		\
		"Local Time:\t\t" + str(time.strftime("%H:%M:%S - %d %B %Y (%a)",lutc)))
	prettyText(widget, fontsize, font, strlocaltime)
	# Age of the moon
	straom = karamba.createText(widget, 1, 55, boxwidth, 11, 		\
		"Age of Moon:\t" + str(aom_d) + " days " + str(aom_h) 		\
		+ " hours " + str(aom_m) + " minutes")
	prettyText(widget, fontsize, font, straom)
	# Moon Phase
	strmoonphase = karamba.createText(widget, 1, 66, boxwidth, 11,		\
		"Moon Phase:\t" + str(int(pphase*100)) + "% [0% = New, 100% = Full]")
	prettyText(widget, fontsize, font, strmoonphase)
	# Moon's distance
	if dis_miles: strmoondist = karamba.createText(widget, 1, 77, boxwidth, 11, \
				"Moon's Distance:\t" + str(int(mdist*0.62137119)) + " miles, "    \
				+ str(round(mdist/constants.earthrad,1)) + " Earth radii.")
	else: strmoondist = karamba.createText(widget, 1, 77, boxwidth, 11, \
		"Moon's Distance:\t" + str(int(mdist)) + " kilometers, "	\
		+ str(round(mdist/constants.earthrad,1)) + " Earth radii.")
	prettyText(widget, fontsize, font, strmoondist)
	# Moon subtends
	strmoonsub = karamba.createText(widget, 1, 88, boxwidth, 11,		\
		"Moon subtends:\t" + str(round(mangdia,4)) + " degrees.")
	prettyText(widget, fontsize, font, strmoonsub)
	# Sun's distance
	if dis_miles: strsundist = karamba.createText(widget, 1, 110, boxwidth, 11,       \
			"Sun's distance:\t" + str(int(sudist*0.62137119)) + " miles, "    \
			+ str(round(sudist/constants.sunsmax,3)) + " au.")
	else: strsundist = karamba.createText(widget, 1, 110, boxwidth, 11,		\
		"Sun's distance:\t" + str(int(sudist)) + " kilometers, "	\
		+ str(round(sudist/constants.sunsmax,3)) + " au.")
	prettyText(widget, fontsize, font, strsundist)
	# Sun subtends
	strsunsub = karamba.createText(widget, 1, 121, boxwidth, 11,		\
		"Sun subtends:\t" + str(round(swangdia,4)) + " degrees.")
	prettyText(widget, fontsize, font, strsunsub)

def moontooldetailInitTextDraw(widget, jd):
	global fontsize, font, boxwidth, boxheight
	global moontooldetailinformation, backgroundimageD
	# Compute the phase information
	(ph1,ph2,ph3,ph4,ph5) = mooncalc.phasehunt(jd)
	lunation = mooncalc.lunation(ph1)
	# Draw the background
	backgroundimageD = karamba.createImage(widget, 1, 1, \
			   karamba.getThemePath(widget) + "pics/backgnd.png")
	# Draw the text
	moontooldetailinformation = karamba.createText(widget, 1, 1, boxwidth, 11,	\
		"Moontool Detail:\n" 	+					\
		"\tLunation:\t" + str(lunation) + "\n" +				\
		"Last new moon:\t" + mooncalc.fmt_phase_time(ph1) + "\n" +	\
		"First quarter:\t" + mooncalc.fmt_phase_time(ph2) + "\n" +	\
		"Full moon:\t\t" + mooncalc.fmt_phase_time(ph3) + "\n" +	\
		"Last quarter:\t" + mooncalc.fmt_phase_time(ph4) + "\n" +	\
		"Next new moon:\t" + mooncalc.fmt_phase_time(ph5) + "\n" +	\
		"\tLunation:\t" + str(lunation+1))
	prettyText(widget, fontsize, font, moontooldetailinformation)

def moontoolUpdTextDraw(widget):
	# Update the textual information of the moontool.
	# Julian time
	karamba.changeText(widget, strjuliantime, "Julian Time:\t\t"		\
		+ str("%.5f" % jd))
	# Universal time
	karamba.changeText(widget, struniversaltime, "Universal Time:\t" 	\
		+ str(time.strftime("%H:%M:%S - %d %B %Y (%a)",utc)))
	# Local time
	karamba.changeText(widget, strlocaltime, "Local Time:\t\t" 		\
		+ str(time.strftime("%H:%M:%S - %d %B %Y (%a)",lutc)))
	# Age of the moon.
	karamba.changeText(widget, straom, 					\
        	"Age of Moon:\t" + str(aom_d) + " days " + str(aom_h) 		\
		+ " hours " + str(aom_m) + " minutes")
	# Moon Phase
	karamba.changeText(widget, strmoonphase,				\
		"Moon Phase:\t" + str(int(pphase*100))				\
		 + "% [0% = New, 100% = Full]")
	# Moon's distance
	if dis_miles: karamba.changeText(widget, strmoondist,                 \
	        "Moon's Distance:\t" + str(int(mdist*0.62137119)) + " miles, "    \
			        + str(round(mdist/constants.earthrad,1)) + " Earth radii.")
	else: karamba.changeText(widget, strmoondist,					\
		"Moon's Distance:\t" + str(int(mdist)) + " kilometers, "	\
		+ str(round(mdist/constants.earthrad,1)) + " Earth radii.")
	# Moon subtends
	karamba.changeText(widget, strmoonsub,					\
		"Moon subtends:\t" + str(round(mangdia,4)) + " degrees.")
	# Sun's distance
	if dis_miles: karamba.changeText(widget, strsundist,                  \
				"Sun's distance:\t" + str(int(sudist*0.62137119)) + " miles, "    \
				+ str(round(sudist/constants.sunsmax,3)) + " au.")
	else: karamba.changeText(widget, strsundist,					\
		"Sun's distance:\t" + str(int(sudist)) + " kilometers, "	\
		+ str(round(sudist/constants.sunsmax,3)) + " au.")
	# Sun subtends
	karamba.changeText(widget, strsunsub,					\
		"Sun subtends:\t" + str(round(swangdia,4)) + " degrees.")

def moontoolDelTextDraw(widget):
	# Remove the text objects and the box.
	for i in (strjuliantime, struniversaltime, strlocaltime, straom,	\
			strmoonphase, strmoondist, strmoonsub, strsundist,	\
			strsunsub, strheader):
		karamba.deleteText(widget, i)
	karamba.deleteImage(widget, backgroundimageI)
	karamba.redrawWidget(widget)

def moontooldetailDelTextDraw(widget):
#	for i in (strheaderd, strlastmoond, strfirstqtrd, strfullmoond,		\
#			strlastqtrd, strnextnewmoond):
#	karamba.deleteText(widget, i)
	karamba.deleteText(widget, moontooldetailinformation)
	karamba.deleteImage(widget, backgroundimageD)
	karamba.redrawWidget(widget)

def moontoolgraphic(widget):
	global moonimage, mrotation
	try:
		karamba.deleteImage(widget, moonimage)
	except NameError:
		pass
	moonimage = karamba.createImage(widget, (boxwidth-mimagewidth)/2,	\
			(boxheight-mimageheight)/2,				\
			karamba.getThemePath(widget)+moonfile)
	karamba.rotateImage(widget, moonimage, int(mrotation))
	karamba.resizeImage(widget, moonimage, mimagewidth, mimageheight)
	karamba.moveImage(widget, moonimage,					\
			(boxwidth-karamba.getImageWidth(widget, moonimage))/2,	\
			(boxheight-karamba.getImageHeight(widget, moonimage))/2)
	karamba.redrawWidget(widget)

def menuOptionChanged(widget, key, value):
	global fillpid, fillvalue, mrotpid, mrotation, dis_miles
	fillpid = None
	mrotpid = None
	fillcmd = ["kdialog", "--title", "Moon Shading Color", "--inputbox", "Enter the color in HEX 'RRGGBBAA' for\n the shading of themoon:\n#00000080 = transparent black", fillvalue]
	mrotcmd = ["kdialog", "--title","Moon Rotation","--inputbox", "Enter the rotation of the moon:", str(mrotation)]
	
	if (key == "fillvalue"):
		karamba.setMenuConfigOption(widget, "fillvalue", "0")
		fillpid = karamba.executeInteractive(widget, fillcmd)

	elif (key == "mrotation"):
		karamba.setMenuConfigOption(widget, "mrotation", "0")
		mrotpid = karamba.executeInteractive(widget, mrotcmd)

	elif (key == "dis_miles"):
		dis_miles = karamba.readMenuConfigOption(widget, "dis_miles")
		widgetUpdated(widget)

def commandOutput(widget, pid, output):
	global fillpid, fillvalue, mrotpid, mrotation, oldph
	if pid == fillpid:
		fillvalue = output.replace('\n','')
		karamba.writeConfigEntry(widget, "shadecolor", str(fillvalue))
		oldph = 0
		widgetUpdated(widget)

	if pid == mrotpid:
		mrotation = str(output.replace('\n',''))
		mrotation = int(mrotation)
		karamba.writeConfigEntry(widget, "moonrotation", str(mrotation))
		oldph = 0
		widgetUpdated(widget)

# This will be printed when the widget loads.
print "Loaded Moontool theme by Eric Gerney <egerney@grci.com>."
