#!/usr/bin/python

# Number of processes to show
num_procs = 5

# Bath to top binary
path_top = "/usr/bin/top"

# List process to ignore, like if you were running SETI@home you could put that in here
# Currently I listed some noisey programs here, you can leave this blank
#ignore_progs = 'top, kdeinit'

#Uncomment belowe to not ignore any programs
ignore_progs = ''

bg_img = "images/background.png"
title_tile = "images/tile.png"
top_tile = "images/tile_big.png"
icon = "images/ksysguard.png"
icon_pause = "images/pause.png"
icon_play = "images/play.png"


# Font to use for displaying the processes. Cheq is a fixed width font
procs_font = "Cheq"

# The variables below should not have to be tweaked unless your top binary is displaying a different output than mine

# Number of lines top generates before actual process listing
top_headers = 7

# Number of columns until %CPU, count starts at 0
cpu_col = 8

# Number of columns until Command, count starts at 0
command_col = 11

# Number of columns until RES, count starts at 0
mem_col = 5

# Comand line arguments for top
# -b Batch mode operation, -n Number of iterations, set to 1; see 'man top'
# Again this should not have to be modified unless your top binary is different than mine
top_args = "-b -n 1"

#############
# KaramTop 0.21
# Will only run when called from SuperKaramba
#############

import karamba
import os
import string
import getopt, sys

#You should not need to modify the script

ktop_text = None
kicon_img = None
ignores_dict = None

paused = 0

# Needed for top to run when running KDE from KDM instead of startx
os.putenv('TERM','xterm') 

def initWidget(widget):
    #Define all the images
    global ktop_text, kicon_img, kicon_pause, kicon_play, top_command
    kbg_img = karamba.createImage(widget, 2, 2, bg_img)
    ktitle_img = karamba.createImage(widget, 15, 12, title_tile)
    ktop_img = karamba.createImage(widget, 15, 54, top_tile)
    
    kicon_img = karamba.createImage(widget, 25, 22, icon)
    karamba.addImageTooltip(widget, kicon_img, "Launch KSysGuard Processes")
    karamba.attachClickArea(widget, kicon_img)
    
    kicon_pause = karamba.createImage(widget, 80, 22, icon_pause)
    karamba.addImageTooltip(widget, kicon_pause, "Pause Karamtop")
    karamba.attachClickArea(widget, kicon_pause)
    
    kicon_play = karamba.createImage(widget, 105, 22, icon_play)
    karamba.addImageTooltip(widget, kicon_play, "Resume Karamtop")
    karamba.attachClickArea(widget, kicon_play)
    karamba.changeImageToGray(widget, kicon_play, 0)
    
    #Define the text
    ktitle_txt = karamba.createText(widget, 135, 24, 60, 30, "Top") #Title text
    karamba.changeTextColor(widget, ktitle_txt, 0, 0, 0)
    karamba.changeTextSize(widget, ktitle_txt, 12)
    
    ktop_text = karamba.createText(widget, 20, 60, 175, 145, "Loading Processes...") #Process listing text
    karamba.changeTextColor(widget, ktop_text, 0, 0, 0)
    karamba.changeTextFont(widget, ktop_text, procs_font)
    karamba.changeTextSize(widget, ktop_text, 9)    
    karamba.redrawWidget(widget)
    buildDict() # Function to build dictionary from ignore_progs
    
    # Below prepares top command for executeInteractive
    top_command = [path_top]
    top_args_lst = top_args.split(' ')
    top_command.extend(top_args_lst)
    
    print "init: KaramTop by Jeff Glover [ http://jeffg.dnsalias.com ]"
    
    
def widgetUpdated(widget):
    global ktop_text, kicon_img, toppid, paused, top_command
    if not paused:
        toppid = karamba.executeInteractive(widget, top_command)

def meterClicked(widget, meter, button):
    #global ktop_text, kicon_img, kicon_pause, kicon_play
    global paused
    if meter == kicon_img:
        karamba.execute("ksysguard --showprocesses")
        karamba.changeImageIntensity(widget, kicon_img, 0.3, 300)
    elif meter == kicon_pause:
	karamba.changeImageToGray(widget, kicon_pause, 0)
	karamba.removeImageTransformations(widget, kicon_play)
        paused = 1
    elif meter == kicon_play:
	karamba.changeImageToGray(widget, kicon_play, 0)
	karamba.removeImageTransformations(widget, kicon_pause)
        paused = 0


def widgetClicked(widget, x, y, button):
    pass
    
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass

def buildDict(): # Function to build dictionary from ignore_progs
    global ignores_dict
    ignores_dict = []
    ignores_lst = ignore_progs.split(',')
    
    for ignore in ignores_lst:
    	ignore = ignore.strip() #strip whitespaces
    	ignores_dict.append((ignore, 0))

    ignores_dict = dict(ignores_dict)

def commandOutput(widget, pid, output):
    global ktop_text, kicon_img, toppid
    
    if pid == toppid:
        topprocs_out = topprocs(output)
	if topprocs_out != "":
	    karamba.changeText(widget, ktop_text, topprocs_out)
            karamba.redrawWidget(widget)

def topprocs(top_out_str):
    global ignores_dict
    cnt = 0
    cpu = ""
    mem = ""
    procs = ""
    
    top_out_lst = top_out_str.splitlines()
    for l in top_out_lst:
    	if cnt >= (num_procs + top_headers):  #need to compensate for the headers with the count
    		return procs    # Count has gone past the num_procs variable, we're done

    	cnt = cnt + 1
    
    	if cnt <= top_headers:  #skip the extra junk generated from top
    		continue

	l = l.strip()  # remove whitespaces that could potentialy confuse things
    	
	if l == "":
		#found a blank line, this should only happen at the end of top
		#continue
		return procs

    	fields = string.split(l)
	
	# A possible issue related with superkaramba's commandoutput
	# or psyco (import command removed)
	
	# so far after removing the use of the psyco module, this bug appears
	#to be gone, left commented code, just in case.
# 	if len(fields) != 12:
# 		print top_out_str
# 		return procs
		
    	if fields[cpu_col] == "0.0": 
    		return procs # End here, because if cpu is 0, then everything below will be 0

	if ignores_dict.has_key(fields[command_col]): #skip process from ignores
		cnt -= 1 # need to put the count back down
		continue
    	
    	cpu = str(int(round(float(fields[cpu_col])))) # float it, round it, int it, then string it ;) probably a better way to do this?
	mem = fields[mem_col]
	if not mem.endswith("m"):
		mem = mem + "b"    # When size is in bytes top leaves off the b, I add it
	
	procs = procs + cpu + "% " + fields[command_col] + " [" + mem + "]\n"

    return procs #or if end of for loop is reached first