###############################################################################
##  Kroller - v0.94 sidebar
###############################################################################
##  
##  Brandon Zehm <caspian at dotconf.net>
##  
##  Kroller is an extension for SuperKaramba to make a MacOS-X like
##  menu bar.
##  It was originally based on tuxbar, but now only a few lines of
##  code from it remain.
##  
##  This SuperKaramba extension was designed to be flexible.
##  You can add new icons, or change existing ones by simply
##  editing the lines in the "User Configurable Section" in this
##  file.  You can use icons of any size you want.  All you need
##  to do is make sure that all the "small" icons are the same
##  size, and all the "big" icons are the same size, and you set
##  those sizes in the imageSize and imageSizeBig variables.
##  
##  Feel free to email me with comments or suggestions!
##  
###############################################################################

## This import statement allows access to the karamba functions
import karamba

## Import a few standard python modules
import string, re, os

## The time module causes problems on some systems (i.e. Redhat 9)
## so we are a little cautious loading it.  If it fails we redefine
## the time function with our own implementation.
try:
    import time
except ImportError:
    print "WARNING: The time module failed to load.  Switching to fallback implementation.  Some time related functions (like maxFPS) may not function properly!"
    class MyTime:
        def time(self):
            os.times()[-1]
        
        def sleep(self, time=0):
            return
        
    time = MyTime()



############################
##  Global Configuration  ##
############################
conf = {
    
    ## Configuration file location, DO NOT NORMALLY EDIT THIS!  Recent enhancements
    ## in SuperKaramba allow us to locate the conf file on our own.  If you
    ## do hard-code a path here it will still work, but you shouldn't need to.
    "configurationFile"      : "kroller.conf",
    
    "version"                : "v0.94",
    
    ## The following variables are overwritten by the configuration file
    "animateOnClick"         : "none",                    ## Animate icons when clicked
    "detectResolution"       : 0,                         ## Auto detection of resolution
    "resolutionX"            : 0,                         ## Manually specified X resolution
    "resolutionY"            : 0,                         ## Manually specified Y resolution
    "autoPosition"           : 0,                         ## Automatically position kroller or not
    "maxFPS"                 : 0,                         ## Maximum widget updates / second.  0 == no limit.
    "smallImageSize"         : 0,                         ## Size of small images
    "largeImageSize"         : 0,                         ## Size of large images
    "iconSeparation"         : "default",                 ## Icon Separation
    "currentIconEffect"      : "none",                    ## Effect for current icon
    "useBackgroundBar"       : 0,                         ## Enable/disable background bar image
    "barImages"              : "barimages/",              ## Directory where to find bar images
    "backgroundBarColor"     : "",                        ## Color or background bar.. this need to be a directory under the barImages dir!
    "autoHide"               : 0,                         ## Enable/disable auto-hiding
    "autoHideDelay"          : 0,                         ## Seconds of idle time to activate autoHide
    "virtualDesktopSelector" : 0,                         ## Enable/disable virtual desktop selector
    "numVirtualDesktops"     : 0,                         ## Number of virtual desktops to display
    "virtualDesktopIconX"    : 0,                         ## Pixels wide the virtual desktop image is
    "virtualDesktopIconY"    : 0,                         ## Pixels tall the virtual desktop image is
    "debug"                  : 0,                         ## Debug level
    
}


## Define some more global variables
## The values are not set here!

## Icon related vars
commands         = []
imageFiles       = []
images           = []
imagesBig        = []
imageSize        = 0
imageSizeBig     = 0
numIcons         = 0
currentIconEffect= "none"
animateOnClick   = "none"
kdeIconPaths     = []

## Icon placement variables
iconSeparation   = 0
centerSmallIcon  = 0
iconFloor        = 0
raiseIcons       = 0
xZoomBoundaryMin = 0
xZoomBoundaryMax = 0
yZoomBoundaryMin = 0
yZoomBoundaryMax = 0


## Widget parameters
borderLeft       = 0
borderTop        = 0
borderBottom     = 0
widgetWidth      = 0
widgetHeight     = 0
resolutionX      = 0
resolutionY      = 0

## Icon resizing related parameters
sizeDiff         = 0
sizeRatio        = 0

## Auto Hiding
autoHide         = 0
autoHideDelay    = 0

## Time related vars
sleepTime        = 0
lastActiveTime   = 0

## Variables for background bar support
useBackBar       = 0
backBarTop       = 0
backBarCenter    = 0
backBarBottom    = 0
backBarBorder    = 4

## These are used for storing current icon placement info
currentXPosition = 0
currentYPosition = 0
currentSize      = 0

## Virtual Desktop Related
virtualDesktop   = []
xVdBoundaryMin   = 0
xVdBoundaryMax   = 0
yVdBoundaryMin   = 0
yVdBoundaryMax   = 0

## Misc
bounceHeight     = 0    ## How high to bounce icons when an icon effect of "bounce" is chosen
isReset          = 0    ## To improve CPU efficiency, we use this to keep track of the state of the widget
debug            = 0    ## Debugging level












###############################################################################
##  Function:
##    initWidget(widget)
##  
##  Description:
##    This function is buggy.. I think it gets called too early, because
##    if you try to create an image it crashes karamba.
###############################################################################
def initWidget(widget):
    """Initialize Widget"""    
    
    global conf
    global debug
    global images
    global imagesBig
    global commands
    global imageFiles
    global images
    global imagesBig
    global imageSize
    global imageSizeBig
    global numIcons
    global currentIconEffect
    global animateOnClick
    global iconSeparation
    global centerSmallIcon
    global borderLeft
    global borderTop
    global borderBottom
    global sizeDiff
    global sizeRatio
    global autoHide
    global autoHideDelay
    global iconFloor
    global raiseIcons
    global bounceHeight
    global xZoomBoundaryMin
    global xZoomBoundaryMax
    global yZoomBoundaryMin
    global yZoomBoundaryMax
    global widgetHeight
    global widgetWidth
    global sleepTime
    global lastActiveTime
    global useBackBar
    global backBarTop
    global backBarCenter
    global backBarBottom
    global resolutionX
    global resolutionY
    global virtualDesktop
    global xVdBoundaryMin
    global xVdBoundaryMax
    global yVdBoundaryMin
    global yVdBoundaryMax
    
    ###################################
    ##  Read the configuration file  ##
    ###################################
    ## Try hard to find the configuration file, otherwise exit with an error message
    themePath = karamba.getThemePath(widget)
    if (os.path.exists(os.getenv("HOME") + "/.superkaramba/" + conf["configurationFile"])):
        print "Loading configuration from home directory: ", os.getenv("HOME") + "/.superkaramba/" + conf["configurationFile"]
        readConfigurationFile(os.getenv("HOME") + "/.superkaramba/" + conf["configurationFile"])
    elif (os.path.exists(karamba.getThemePath(widget) + "/" + conf["configurationFile"])):
        print "Loading configuration from theme directory: ", karamba.getThemePath(widget) + "/" + conf["configurationFile"]
        readConfigurationFile(karamba.getThemePath(widget) + "/" + conf["configurationFile"])
    elif (os.path.exists(conf["configurationFile"])):
        print "Loading configuration from manually specified location: ", conf["configurationFile"]
        readConfigurationFile(conf["configurationFile"])
    else:
        print
        print "ERROR - Loading the configuration file failed!  The configuration file could not be located!"
        print "Hint: try putting kroller.conf in ~/.superkaramba/"
        print
        ## FIXME: sys.exit(1)
    
    
    ## Make sure numeric configuration parameters are set that way
    imageSize    = int(conf['smallImageSize'])
    imageSizeBig = int(conf['largeImageSize'])
    debug = int(conf['debug'])
    currentIconEffect = conf['currentIconEffect']
    autoHide = int(conf['autoHide'])
    autoHideDelay = int(conf['autoHideDelay'])
    animateOnClick = conf['animateOnClick']
    conf["virtualDesktopSelector"] = int(conf["virtualDesktopSelector"])
    conf["virtualDesktopIconX"] = int(conf["virtualDesktopIconX"])
    conf["virtualDesktopIconY"] = int(conf["virtualDesktopIconY"])
    useBackBar = int(conf['useBackgroundBar'])
    bounceHeight = int(conf['bounceHeight'])
    
    ## If numVirtualDesktops is "auto" find out the current value from the users
    ## KDE config files.
    if (conf["numVirtualDesktops"] == "auto"):
        ## Try and find a good kdeglobals file
        if (os.path.exists(os.getenv("HOME") + "/.kde/share/config/kdeglobals")):
            conf["numVirtualDesktops"] = getKDEConfigValue(os.getenv("HOME") + "/.kde/share/config/kdeglobals", "Desktops", "Number")
        elif (os.path.exists(os.getenv("KDEDIR") + "/share/config/kdeglobals")):
            conf["numVirtualDesktops"] = getKDEConfigValue(os.getenv("KDEDIR") + "/share/config/kdeglobals", "Desktops", "Number")
        
        if ( (conf["numVirtualDesktops"] == "auto") or (conf["numVirtualDesktops"] == -1) ):
            print
            print "ERROR - Could not automatically determin the number of virtual desktops in use!"
            print "Hint: Try manually specifying the \"numVirtualDesktops\" setting in kroller.conf"
            print
            conf["virtualDesktopSelector"] = 0
            conf["numVirtualDesktops"] = 0
            ## FIXME: sys.exit(1)
    
    ## Make sure its an int
    conf["numVirtualDesktops"] = int(conf["numVirtualDesktops"])
    
    
    ## <calculations>
    
    ## Calculate some other important stuff
    if (re.search("default", str(conf['iconSeparation']))):
        iconSeparation = (imageSize + imageSizeBig)/2            ## iconSeparation is the size of the "cells" where icons go
        if (iconSeparation > (2*imageSize)):
            iconSeparation = (2*imageSize)
    else:
        if (re.search("\D", str(conf['iconSeparation']))):
            print "ERROR - The iconSeparation parameter in the configuration file is not \"default\" or a numeric value!"
        else:
            iconSeparation = int(conf['iconSeparation'])
            iconSeparation = iconSeparation + imageSize
    
    numIcons = len(imageFiles)                               ## Self explanitory
    centerSmallIcon = ((iconSeparation/2)-(imageSize/2))     ## The number of pixels to center each small icon in it's "cell"
    borderLeft = 10                                          ## How much "white" space on the left
    borderTop  = iconSeparation                              ## How much "white" space on the top   
    borderBottom = iconSeparation                            ## How much "white" space on the bottom
    sizeDiff = imageSizeBig - imageSize                      ## How many pixels are there from the small icon to the big icon
    sizeRatio = ((sizeDiff + 0.0) / imageSizeBig)            ## Ratio of small icon to big icon
    raiseIcons = 0                                           ## How far to "raise" icons up when icon is at its largest size
    
    ## Calculate widget size
    widgetWidth = imageSizeBig + borderLeft
#    widgetHeight = imageSizeBig + raiseIcons + borderBottom   ## changed this since we don't "raise" icons anymore.
    widgetHeight = (borderTop + (iconSeparation * numIcons) + borderBottom)
        
    ## Figure out where the bottom of the icons should be (the iconFloor)
    iconFloor = (widgetWidth - imageSize - borderLeft)
    ##iconFloor = borderLeft
    
    ## Boundaries in which we do icon zooming
    ## These will probably have to become more complex formulas in the future
    xZoomBoundaryMin = 0
    xZoomBoundaryMax = widgetWidth
    yZoomBoundaryMin = (borderTop-iconSeparation)
    yZoomBoundaryMax = (borderTop+(iconSeparation*(numIcons+1)))
    
    ## Sleep time based on maxFPS from configuration file
    if (conf['maxFPS'] != 0):
        sleepTime = (1.0 / conf['maxFPS'])
    
    ## If we're in debugging mode display the current kroller environment
    if (debug >= 1):
        print "numIcons: ", numIcons
        print "imageSize: ", imageSize
        print "imageSizeBig:", imageSizeBig
        print "centerSmallIcon: ", centerSmallIcon
        print "iconSeparation: ", iconSeparation
        print "iconFloor: ", iconFloor
        print "sizeDiff: ", sizeDiff
        print "sizeRatio: ", sizeRatio
        print "borderLeft: ", borderLeft
        print "borderTop: ", borderTop
        print "borderBottom: ", borderBottom
        print "raiseIcons: ", raiseIcons
        print "widgetWidth: ", widgetWidth
        print "widgetHeight: ", widgetHeight
        print "sleepTime: ", sleepTime
        
    ## </calculations>
    
    ###################################
    ##  Detect current X resolution  ##
    ###################################
    if (int(conf['detectResolution']) == 1):
        detectResolution()
    else:
        resolutionX = int(conf['resolutionX'])
        resolutionY = int(conf['resolutionY'])
        print "Using specified resolution: " + str(resolutionX) + "x" + str(resolutionY)
    
    ## Set the size of our widget since we're just starting
    karamba.resizeWidget(widget, widgetWidth, widgetHeight)
    
    
    ################################
    ##  Setup the background bar  ##
    ################################
    if (useBackBar == 1):
        ## Create image references
        backBarTop =   (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-top.png"))
        backBarCenter = (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-center.png"))
        backBarBottom =  (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-bottom.png"))
        ## Set initial sizes
        karamba.resizeImageSmooth(widget, backBarTop, imageSize + (backBarBorder * 2), imageSize + (backBarBorder * 2))
        karamba.resizeImage(widget, backBarCenter, imageSize + (backBarBorder * 2), widgetHeight - borderTop - borderBottom)
        karamba.resizeImageSmooth(widget, backBarBottom, imageSize + (backBarBorder * 2), imageSize + (backBarBorder * 2))
        ## Set initial positions
        karamba.moveImage(widget, backBarTop, iconFloor - backBarBorder, borderTop - imageSize - (backBarBorder * 2))
        karamba.moveImage(widget, backBarCenter, iconFloor - backBarBorder, borderTop)
        karamba.moveImage(widget, backBarBottom, iconFloor - backBarBorder, borderTop + (numIcons * iconSeparation))
    
	
	    
    ##########################################
    ##  Setup All the normal rolling icons  ##
    ##########################################
    for i in range (0,numIcons):
        
        ## First do some work to find the icon the user wants.  It may be manually specified path, or 
        ## a generic name like "konsole".  If it's a generic name we need to find where the icon they
        ## want is really located from the currnetly selected KDE icon theme.
        imageFiles[i] = findIcon(widget, imageFiles[i], str(imageSizeBig) + "x" + str(imageSizeBig))
        
        ## If the findIcon function returned -1, then use the "unknown" icon
        if (imageFiles[i] == -1):
            imageFiles[i] = findIcon(widget, "unknown", str(imageSizeBig) + "x" + str(imageSizeBig))
        
        ## Add small image
        images.append(karamba.createImage(widget, iconFloor, borderTop + centerSmallIcon + (i*iconSeparation), imageFiles[i]))
        ## Add large image
        imagesBig.append(karamba.createImage(widget, widgetWidth, borderTop + (i*iconSeparation), imageFiles[i]))
        ## Add clickable area that runs a command -- FIXME, maybe this should be a system command later?
        karamba.createClickArea(widget, (widgetWidth - imageSizeBig - borderLeft - raiseIcons), borderTop + (i*iconSeparation), imageSizeBig + (raiseIcons/2), iconSeparation, commands[i])
        ## Resize the small image to the small image size
        karamba.resizeImageSmooth(widget, images[i], imageSize, imageSize)
    
    
    #######################################################
    ##  Place widget at the right center of the screen  ##
    #######################################################
    if (int(conf['autoPosition']) == 1):
    
        karamba.moveWidget(widget, (resolutionX - widgetWidth),((resolutionY/2) - (widgetHeight/2)) )
    
    ## Update last active time if we're autoHiding (without this autoHiding never starts)
    if (autoHide == 1):
        lastActiveTime = time.time()
    
    karamba.redrawWidget(widget)
    print "Kroller-" + conf['version'] + " successfully initiated!"










###############################################################################
##  Function: 
##    widgetClicked(widget, x, y, button)
##  
##  This gets called everytime our widget is clicked.
##  Notes:
##    widget = reference to our widget
##    x = x position (relative to our widget)
##    y = y position (relative to our widget)
##    botton = button clicked:
##                    1 = Left Mouse Button
##                    2 = Middle Mouse Button
##                    3 = Right Mouse Button, but this will never happen
##                        because the right mouse button brings up the
##                        Karamba menu.
###############################################################################
def widgetClicked(widget, x, y, button):
    """widgetClicked gets called when our widget is clicked"""
    global currentXPosition
    global currentYPosition
    global currentSize
    global lastActiveTime
    
    if (debug >= 2):
        print "widgetClicked() got called"
    
    ## Unhide if we need to
    if (lastActiveTime):
        if ((time.time() - lastActiveTime) > autoHideDelay):
            raiseWidget(widget)
        
        ## Update last active time
        lastActiveTime = time.time()
    
    
    ## LEFT CLICK
    ## Animate icon when an application is launched
    if (button == 1):
        iconNum = (y - borderTop) / iconSeparation
        if ((iconNum >= 0) and (iconNum < numIcons) and (animateOnClick != "none")):
            
            ## Bounce Effect
            if (animateOnClick == "bounce"):
                ## Going up
                for up in range (1,bounceHeight,2):
                    karamba.moveImage(widget, imagesBig[iconNum], currentXPosition, currentYPosition - up)
                    karamba.redrawWidget(widget)
                    ## Sleep a bit to accomadate the maxFPS setting
                    if (sleepTime):
                        time.sleep(sleepTime)
                ## Going down
                for down in range (1,bounceHeight,2):
                    karamba.moveImage(widget, imagesBig[iconNum], currentXPosition, currentYPosition - bounceHeight + down)
                    karamba.redrawWidget(widget)
                    ## Sleep a bit to accomadate the maxFPS setting
                    if (sleepTime):
                        time.sleep(sleepTime)
            
            ## Highlight Effect
            elif (animateOnClick == "highlight"):
                karamba.changeImageIntensity(widget, imagesBig[iconNum], 0.7, 150)
                karamba.redrawWidget(widget)
                
            ## Gray Effect
            elif (animateOnClick == "gray"):
                karamba.changeImageToGray(widget, imagesBig[iconNum], 200)
                karamba.redrawWidget(widget)
                
            ## Click Effect
            elif (animateOnClick == "click"):
                ## Go down and to the right (and highlight)
                karamba.moveImage(widget, imagesBig[iconNum], currentXPosition + 6, currentYPosition + 6)
                karamba.changeImageIntensity(widget, imagesBig[iconNum], 0.5, 0)
                for i in range(1, conf['maxFPS'], 10):
                    karamba.redrawWidget(widget)
                    time.sleep(sleepTime)
                karamba.removeImageEffects(widget, imagesBig[iconNum])
                ## Go up and to the left
                karamba.moveImage(widget, imagesBig[iconNum], currentXPosition, currentYPosition)
                karamba.redrawWidget(widget)
                
            
###############################################################################
##  Function:
##    widgetUpdated(widget)
##  
##  Description:
##    Right now this function is only used for initializing the widget
##
###############################################################################
def widgetUpdated(widget):
    """widgetUpdated gets called at program init and every few seconds"""    
    
    
    if (debug >= 2):
        print "widgetUpdate got called"
    
    ## Check to see if we need to autoHide
    if (lastActiveTime):
        if ((time.time() - lastActiveTime) >= autoHideDelay):
            resetWidget(widget)
            hideWidget(widget)
    
    
        












###############################################################################
##  Function:   
##    widgetMouseMoved(widget, x, y, button)
##  
##  This gets called everytime the mouse is moved over our widget
##  Notes
##    widget = reference to our widget
##    x = x position (relative to our widget)
##    y = y position (relative to our widget)
##    botton = button being held:
##                    0 = No Mouse Button
##                    1 = Left Mouse Button
##                    2 = Middle Mouse Button
##                    3 = Right Mouse Button, but this will never happen
##                        because the right mouse button brings up the
##                        Karamba menu.
###############################################################################
def widgetMouseMoved(widget, x, y, button):
    """widgetMouseMoved gets called everytime the mouse gets moved over our widget"""    
    global currentXPosition
    global currentYPosition
    global currentSize
    global lastActiveTime
    global isReset
    
    ## Unhide if we need to
    if (lastActiveTime):
        if ((time.time() - lastActiveTime) >= autoHideDelay):
            raiseWidget(widget)
        
        ## Update last active time
        lastActiveTime = time.time()
    
    
    iconNum = -2
    
    ## If the mouse is in the target area
    if (x >= xZoomBoundaryMin) and (x < xZoomBoundaryMax) and (y >= yZoomBoundaryMin) and (y <= yZoomBoundaryMax):
        
        if (debug >= 2):
            print "processing.."
        
        ## We'll be resizing things, so we're not in a reset state anymore
        isReset = 0
        
        ## Which icon is the mouse over?
        iconNum = (y - borderTop) / iconSeparation   ## borderTop is initial white-space, and iconSeparation pixels between each icon
        
        ## Unless the current icon comes up as a non-valid number
        if (-2 < iconNum < numIcons+1):
            
            ## This returns a number between 0 and iconSeparation depending on how far through the image you are
            offset              = (y - borderTop) % iconSeparation          ## 0 - iconSeparation
            offsetRatio         = (offset/(iconSeparation + 0.0) )          ## Ratio from 0-1 of the current offset
            offsetImageSizeBig  = (offsetRatio * (imageSizeBig/2))          ## 0 - (imageSizeBig/2)
            offsetSizeDiff      = (offsetRatio * sizeDiff)                  ## 0 - sizeDiff
            ## yRatio lets us do smooth scrolling from the top and bottom
            ## You will see it throughout the script for this reason
            ## If y is in the top half of the icon
            if (x < widgetWidth - borderLeft - raiseIcons - (imageSizeBig/2)):
                xRatio          = ((x + 0.0) / (widgetWidth - borderLeft - raiseIcons - (imageSizeBig/2)))
            else: 
                xRatio = 1
            
            ## If we're positioning the first or last icon, do a special formula
            if (iconNum == -1):
                yShiftUp = (offset - iconSeparation) * xRatio
                yShiftDown = 0
            elif (iconNum == numIcons):
                yShiftDown = offset * xRatio
                yShiftUp = 0
            else:
                yShiftUp = 0
                yShiftDown = 0
                
            
            ## Resize the background bar if we're using it
            if (useBackBar == 1):
                karamba.moveImage(widget, backBarTop, iconFloor - backBarBorder, borderTop - imageSize - (backBarBorder * 2) - ((iconSeparation/2) * xRatio) - (yShiftUp/2) + (yShiftDown/2))
                karamba.moveImage(widget, backBarCenter, iconFloor - backBarBorder, borderTop - ((iconSeparation/2) * xRatio) + (yShiftDown/2) - (yShiftUp/2))
                karamba.moveImage(widget, backBarBottom, iconFloor - backBarBorder, borderTop + (numIcons * iconSeparation)    + ((iconSeparation/2) * xRatio) - (yShiftDown/2) + (yShiftUp/2))
                karamba.resizeImage(widget, backBarCenter, imageSize + (backBarBorder * 2), widgetHeight - borderTop - borderBottom + (iconSeparation * xRatio) + yShiftUp - yShiftDown + 1)
            
            
            ################################
            ## Size and position iconNum-1
            ################################
            
            ## Unless we're on the first image, resize the icon up
            if iconNum > 0:
                ## Calculate stuff
                ## Size is getting smaller
                size = imageSize + (xRatio * ((sizeDiff/2) - (offsetSizeDiff/2)))
                ## Icon is going up
                ## If we're positioning the last icon, do a special formula
                if (iconNum == numIcons):
                    ypos = ( ((iconNum-1-xRatio) * iconSeparation) + borderTop + ((iconSeparation/2)-(size/2)) + (xRatio * (iconSeparation/2)) + (xRatio * ((iconSeparation/2) * offsetRatio)) )
                else:
                    ##        Initial placement                                    center it in it's cell          start half way accross and come back to initial placement
                    ypos = ( ((iconNum-1-xRatio) * iconSeparation) + borderTop + ((iconSeparation/2)-(size/2)) + (xRatio * (iconSeparation/2)) - (xRatio * ((iconSeparation/2) * offsetRatio)) )
                ## Icon is going left
                ##     center icon                 start from half way up   and go half way down
                xpos = (iconFloor + imageSize - size) - (xRatio * (raiseIcons/2)) + ((xRatio * raiseIcons * offsetRatio)/2)
                
                
                ## Place stuff
                karamba.resizeImage(widget, imagesBig[iconNum-1], size, size)
                karamba.moveImage(widget, imagesBig[iconNum-1], xpos, ypos)
                
                ## Make sure there are no left-over image effects on the image
                if (currentIconEffect != "none"):
                    karamba.removeImageEffects(widget, imagesBig[iconNum-1])
                
                ## Make sure the current small icon is off the screen
                karamba.moveImage(widget, images[iconNum-1], widgetWidth, ((iconNum-1)*iconSeparation)+imageSizeBig)
                
                ## Make sure all the previous icons are setup normally
                
                for i in range (0, iconNum - 1):
                    karamba.moveImage(widget, images[i], iconFloor, ((i-xRatio)*(iconSeparation)) + borderTop + centerSmallIcon + yShiftDown)
                    karamba.moveImage(widget, imagesBig[i], widgetWidth, ((i-1)*(iconSeparation)) + borderTop)
                
                if (debug >= 2):
                    print "iconNum: ", iconNum-1, "\toffset: ", offset, "\toffsetSizeDiff: ", offsetSizeDiff, "\toffsetImageSizeBig: ", offsetImageSizeBig, "\tyRatio: ", xRatio, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
            
            ################################
            ## Size and position iconNum
            ################################
            
            if ( (iconNum > -1) and (iconNum < numIcons) ):
                ## If we're below the icon
                if offset < (iconSeparation/2):
                    ## Calculate stuff
                    ## Size is getting larger
                    size = imageSize + (xRatio * ((sizeDiff/2) + offsetSizeDiff))
                    ## Icon is moving Down
                    ypos = ( (iconNum * iconSeparation) + borderTop + ((iconSeparation/2)-(size/2)) + (xRatio * (iconSeparation/2)) - (xRatio * offset) )
                    ## Icon is going right
                    xpos = (iconFloor + imageSize - size) - (xRatio * (raiseIcons/2)) - ((xRatio * raiseIcons * offsetRatio)/2)
                    
                    ## Place stuff
                    karamba.resizeImage(widget, imagesBig[iconNum], size, size)
                    karamba.moveImage(widget, imagesBig[iconNum], xpos, ypos)
                
                
                ## If we're above the icon
                else:
                    ## Calculate stuff
                    ## Size is getting smaller
                    size = imageSize + (xRatio * (sizeDiff + (sizeDiff/2) - offsetSizeDiff))
                    ##        initial placement                         center it                       go half way accross and come back offset pixels
                    ypos = ( (iconNum * iconSeparation) + borderTop + ((iconSeparation/2)-(size/2)) + (xRatio * (iconSeparation/2)) - (xRatio * offset) )
                    ## icon is going down
                    xpos = (iconFloor + imageSize - size) - (xRatio * raiseIcons) + ((xRatio * raiseIcons * offsetRatio)/2)
                    
                    ## Place stuff
                    karamba.resizeImage(widget, imagesBig[iconNum], size, size)
                    karamba.moveImage(widget, imagesBig[iconNum], xpos, ypos)
                
                ## Apply current icon effects if any
                if (currentIconEffect != "none"):
                    if (currentIconEffect == "highlight"):
                        karamba.changeImageIntensity(widget, imagesBig[iconNum], 0.2, 0)
                    if (currentIconEffect == "gray"):
                        karamba.changeImageToGray(widget, imagesBig[iconNum], 0)
                
                ## Move the current small image off the widget
                karamba.moveImage(widget, images[iconNum], widgetWidth, (iconNum*iconSeparation) + borderTop + centerSmallIcon)
                
                ## These are used later if the icon is clicked
                currentXPosition = xpos
                currentYPosition = ypos
                currentSize = size
                
                if (debug >= 2):
                    print "iconNum: ", iconNum, "\toffset: ", offset, "\toffsetSizeDiff: ", offsetSizeDiff, "\toffsetImageSizeBig: ", offsetImageSizeBig, "\tyRatio: ", yRatio, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
                
            
            
            
            
            ################################
            ## Size and position iconNum+1
            ################################
            
            ## Unless we're on the last image, resize the upper icon
            if (iconNum < numIcons-1):
                ## Calculate stuff
                size = imageSize + (xRatio * ((offsetSizeDiff/2)))
                ## If we're positioning the first icon, do a special formula
                if (iconNum == -1):
                    ypos = ( ((iconNum+1) * iconSeparation) + borderTop + ((iconSeparation/2)-(size/2)) + (xRatio * (iconSeparation/2) * offsetRatio) )
                else:
                    ## icon is going left
                    ypos = ( ((iconNum+1+xRatio) * iconSeparation) + borderTop + ((iconSeparation/2)-(size/2)) - (xRatio * (iconSeparation/2) * offsetRatio) )   ## Moves to the left as offset increases
                ## icon is going up
                xpos = (iconFloor + imageSize - size) - ((xRatio * raiseIcons * offsetRatio)/2)
                
                
                ## Place stuff
                karamba.resizeImage(widget, imagesBig[iconNum+1], size, size)
                karamba.moveImage(widget, imagesBig[iconNum+1], xpos, ypos)
                
                ## Make sure there are no left-over image effects on the image
                if (currentIconEffect != "none"):
                    karamba.removeImageEffects(widget, imagesBig[iconNum+1])
                
                ## Make sure the current small icon is off the screen
                karamba.moveImage(widget, images[iconNum+1], widgetWidth, ((iconNum+1)*iconSeparation)+imageSizeBig)
                
                ## Make sure all the following icons are setup normally
                
                for i in range (iconNum+2, numIcons):
                    karamba.moveImage(widget, images[i], iconFloor, ((i-1+xRatio)*iconSeparation) + iconSeparation + borderTop + centerSmallIcon + yShiftUp)
                    karamba.moveImage(widget, imagesBig[i], widgetWidth, ((i)*iconSeparation) + iconSeparation + borderTop)
                    
                if (debug >= 2):
                    print "iconNum: ", iconNum+1, "\toffset: ", offset, "\toffsetSizeDiff: ", offsetSizeDiff, "\toffsetImageSizeBig: ", offsetImageSizeBig, "\txRatio: ", xRatio, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
            if (debug >= 2):
                print
            
            
            ## Update the screen with the changes we've just made
            karamba.redrawWidget(widget)
                        
    
    ## If mouse is to far right or left reset everything to normal
    else:
        if (debug >= 1):
            print "cursor is out of bounds. x: ", x, " y: ", y
        resetWidget(widget)
        
    
    ## Sleep a bit to accomadate the maxFPS setting
    if (sleepTime):
        time.sleep(sleepTime)
    










#############################################################################
##  Function:
##    resetWidget(widget)
##  
##  Description:
##    Resets the widget to it's normal state, i.e. no icons zoomed
##  
#############################################################################
def resetWidget(widget):
    """Resets widget to standby state.. i.e. no icons zoomed."""    
    global isReset
    
    if (isReset == 0):
        for i in range (0,numIcons):
            karamba.moveImage(widget, images[i], iconFloor, (i*iconSeparation) + borderTop + centerSmallIcon)
            karamba.moveImage(widget, imagesBig[i], widgetWidth, (i*iconSeparation) + borderTop)
        
        if (useBackBar == 1):
	
	    karamba.moveImage(widget, backBarTop, iconFloor - backBarBorder, borderTop - imageSize - (backBarBorder * 2))
            karamba.moveImage(widget, backBarCenter, iconFloor - backBarBorder, borderTop)
            karamba.moveImage(widget, backBarBottom, iconFloor - backBarBorder, borderTop + (numIcons * iconSeparation))
	    karamba.resizeImage(widget, backBarCenter, imageSize + (backBarBorder * 2), widgetHeight - borderTop - borderBottom)
    
        karamba.redrawWidget(widget)
        isReset = 1









###############################################################################
## Function:    
##   readConfigurationFile ( string $filename )
## 
## 
## Description: 
##   Reads the file $filename, and stores information from the file into 
##   the global hash %conf.  This version of the function has been modified 
##   specifically to accomadate the needs of Kroller.
## 
## Example: 
##   readConfigurationFile("/etc/configuration.conf");
##
###############################################################################
def readConfigurationFile(filename):
    """Read a configuration file and store name:value pairs into the global variable conf"""
    global conf
    global commands
    global imageFiles
    
    ## Open the configuration file
    try:
        file = open(filename,"r")
    except IOError:
        print
        print "ERROR While opening the configuration file [" + filename + "]"
        print "Hint: try putting kroller.conf in ~/.superkaramba/"
        print
    
    ## Read through each line of the file
    while 1:
        line = file.readline()                        ## Read a line from the file
        if len(line) == 0:                            ## Break this while loop if we're at the end of the file
            break
        
        line = re.sub("#.*$", "", line)               ## Remove comments
        line = re.sub("//.*$", "", line)              ## Remove comments
        line = line.strip()                           ## Remove line ending characters, and extra whitespace
        if re.search("\s+=>\s+", line):               ## If it looks like a valid configuration line
            
            ## Debugging
            if (int(conf['debug']) >= 6):
                print "Line is: ", line
            
            ## Extract the name/value pair
            (name, value) = re.split("\s+=>\s+", line, 1)
            
            ## Add a command
            if (name == "addCommand"):
                commands.append(value)
            
            ## Add an icon name
            elif (name == "addIcon"):
                imageFiles.append(value)
            
            ## Must be a generic configuration parameter, store it in conf
            else:
                if (re.match("^\d+$", value)):
                    conf[name] = int(value)
                else:
                    conf[name] = value
        
    ## Close the coniguration file
    file.close()
    
    
    ## DEBUGGING:
    if (int(conf['debug']) >= 3):
        print "Configuration: "
        for key in conf.keys():
            print key, "\t=>\t", conf[key]










###############################################################################
## Function:    
##   getKDEConfigValue ( string $filename, string $section, string $option )
## 
## 
## Description: 
##   Reads the file $filename, searches for the section $section (each section
##   is delimeted by  "[section name]" in KDE config files, finds the option
##   $option and returns the value.
## 
## Example: 
##   num = getKDEConfigValue("/root/.kde/share/kdeglobals", "Desktops", "Number");
##
###############################################################################
def getKDEConfigValue(filename, section, option):
    """Read a KDE config file and return the value for option under the specified section"""
    
    ## FIXME: Should this function be replaced with calls to 
    ## pythons ConfigParser?
    
    ## Open the configuration file
    try:
        file = open(filename,"r")
    except IOError:
        print
        print "ERROR While opening the configuration file [" + filename + "]"
        print
    
    ## Read through each line of the file
    while 1:
        line = file.readline()                        ## Read a line from the file
        if len(line) == 0:                            ## Break this while loop if we're at the end of the file
            break
        
        line = re.sub("#.*$", "", line)               ## Remove comments
        line = re.sub("//.*$", "", line)              ## Remove comments
        line = line.strip()                           ## Remove line ending characters, and extra whitespace
        if re.search("^\[" + section + "\]", line):           ## If we're entering the right section
            while 1:
                line = file.readline()
                if len(line) == 0:                            ## Break this while loop if we're at the end of the file
                    break
                line = line.strip()
                if re.search("^" + option + "=", line):
                    (name, value) = re.split("=", line, 1)
                    file.close
                    return(str(value))
                    
    ## Close the coniguration file and return an error if we didn't find what we were looking for
    file.close()
    return(-1)
    










###############################################################################
## Function:    
##   findIcon ( widget, string $iconName, string $size )
## 
## 
## Description: 
##   Trys really hard to find an icon based on the text given in $icon.
##   If $icon is an absolute path to an icon $icon will be returned 
##   unchanged.  Otherwise it will look for an icon in various KDE icon
##   directories that matches the name given.
##   If a $size is specified it will try to find one of that size in the
##   KDE dirs.  
##  
## Example: 
##   iconPath = findIcon("konsole", "96x96");
##
###############################################################################
def findIcon(widget, icon, size):
    """Try to find an icon"""
    global kdeIconPaths
    
    ## Try to build the kdeIconPaths array only once to save CPU
    if (not kdeIconPaths):
        
        ## Find the KDE icon theme
        kdeIconTheme = getKDEConfigValue(os.getenv("HOME") + "/.kde/share/config/kdeglobals", "Icons", "Theme")
        if ( (kdeIconTheme == -1) or (kdeIconTheme == "") or (re.match("^\d+$", kdeIconTheme)) ):
            print "KDE has no icon theme configured.  Using default.kde"
            kdeIconTheme = "default.kde"
        print "Current KDE Icon Theme: " + kdeIconTheme
        
        ## Get the paths that KDE is using for icons
        fp = os.popen("kde-config --path icon")
        output = []
        if (fp):
            output = fp.read()
            ## Strip newlines
            output = output.replace('\n', '')
            output = output.split(":")
        
        ## KDEDIR should probably be removed from future kroller versions
        ## and rely solely on the kde-config command, since KDEDIR var
        ## is being depricated.
        kdedir = os.getenv("KDEDIR")
        if (not kdedir):
            fp = os.popen("kde-config --prefix")
            if (fp):
                tmp = fp.read()
                tmp = tmp.replace('\n', '')
                kdedir = tmp
                del tmp
        if (not kdedir):
            kdedir = "/opt/kde"
        print "Detected KDEDIR: " + str(kdedir)
        if (not os.path.exists(kdedir)):
            print "ERROR - Could not locate the KDE installation directory!"
        
        
        ## Now look in KDE's icon paths
        kdeIconPaths = []
        
        ## These few entries *should* find everyones icons
        for entry in output:
            kdeIconPaths.append(entry + kdeIconTheme)
        
        ## Make sure to check the kroller theme dir for icons
        kdeIconPaths.append(karamba.getThemePath(widget))
        kdeIconPaths.append(karamba.getThemePath(widget) + "/icons")
        
        ## These should have already been added from the kde-config command, but just in case
        kdeIconPaths.append(os.getenv("HOME") + "/.kde/share/icons/" + kdeIconTheme)
        kdeIconPaths.append(kdedir + "/share/icons/" + kdeIconTheme)
        
        ## As a fallback check the default.kde and hicolor dirs
        for entry in output:
            kdeIconPaths.append(entry + "default.kde")
        for entry in output:
            kdeIconPaths.append(entry + "hicolor")
        
        ## Add a few more search paths just for paranoia's sake
        kdeIconPaths.append("/usr/share/icons" + kdeIconTheme)
        kdeIconPaths.append("/usr/share/icons")
        kdeIconPaths.append(kdedir + "/share/icons/default.kde")
        kdeIconPaths.append(kdedir + "/share/icons/hicolor")
        
        if (debug >= 1):
            print "Icon Search Paths:"
            for path in kdeIconPaths:
                print "    " + str(path)
    
    
    sizes = []
    if (size):
        sizes.append(size)
    sizes.append("128x128")
    sizes.append("96x96")
    sizes.append("64x64")
    sizes.append("48x48")
    sizes.append("32x32")
    sizes.append("22x22")
    sizes.append("16x16")
    
    types = []
    types.append("apps")
    types.append("devices")
    types.append("filesystems")
    types.append("mimetypes")
    
    
    ## If an absolute path was defined use that
    if (os.path.exists(icon)):
        if (debug >= 1):
            print "Found icon for: " + icon + " at path: " + icon
        return(icon)
    
    ## Look for the icon in all of the search paths discovered above
    for path in kdeIconPaths:
        if (os.path.exists(path + "/" + icon + ".png")):
            if (debug >= 1):
                print "Found icon for: " + icon + " at path: " + path + "/" + icon + ".png"
            return(path + "/" + icon + ".png")
        if (os.path.exists(path + "/" + icon)):
            if (debug >= 1):
                print "Found icon for: " + icon + " at path: " + path + "/" + icon
            return(path + "/" + icon)
        for size in sizes:
            for type in types:
                if (os.path.exists(path + "/" + size + "/" + type + "/" + icon + ".png")):
                    if (debug >= 1):
                        print "Found icon for: " + icon + " size: " + size + " at path: " + path + "/" + size + "/" + type + "/" + icon + ".png"
                    return(path + "/" + size + "/" + type + "/" + icon + ".png")
                if (os.path.exists(path + "/" + size + "/" + type + "/" + icon)):
                    if (debug >= 1):
                        print "Found icon for: " + icon + " size: " + size + " at path: " + path + "/" + size + "/" + type + "/" + icon
                    return(path + "/" + size + "/" + type + "/" + icon)
    
    ## Return -1 if we couldn't find the icon
    print "ERROR - Coundn't find an icon for \"" + icon + "\""
    return(-1)
    










###############################################################################
## Function:    
##   detectResolution ( )
## 
## 
## Description: 
##   Runs xwininfo and sets the current XFree86 resolution to the golbal
##   variables resolutionX and resolutionY
## 
##  Thanks to Adam Geitgey for this code!
###############################################################################
def detectResolution():
    """Detect Current XFree86 Resolution"""
    global resolutionX
    global resolutionY
    
    havexwininfo = os.system("which xwininfo >/dev/null")
    
    if (havexwininfo == 0):
        pass
    else:
        resolutionX = 1024
        resolutionY = 768
        print "\nCan't find xwininfo in your path!  Guessing resolution is 1024x768."
    
    fp = os.popen("xwininfo -root -stats")
    output = fp.read()
    output = output.splitlines()
    i = 0
    for x in output:
        param = x.split()
        if (len(param) > 1):
            if param[0].find("Width:") != -1:
                resolutionX = int(param[1])
            if param[0].find("Height:") != -1:
                resolutionY = int(param[1])
    
    print "Detected resolution: " + str(resolutionX) + "x" + str(resolutionY)









###############################################################################
## Function:    
##   hideWidget ( widget )
## 
## 
## Description: 
##   Hides kroller
## 
###############################################################################
def hideWidget(widget):
    
    karamba.moveWidget(widget, widgetWidth, ((resolutionY/2) - (widgetHeight/2)))
    
    
    







###############################################################################
## Function:    
##   raiseWidget ( widget )
## 
## 
## Description: 
##   Shows kroller
## 
###############################################################################
def raiseWidget(widget):
    
    karamba.moveWidget(widget, (resolutionX-widgetWidth), ((resolutionY/2) - (widgetHeight/2)))










# This will be printed when the widget loads.
print "Kroller python module loaded!"

