#!/usr/bin/python

def parseWeatherData(days, data):
	'''Parses an xml page from weather.com and returns a list
	containing the current weather conditions and forecast.

	data: An unmodified xml page from weather.com
	days: Number of forecast days (including today) to return (maximum 10)
	'''

	import re
	import weather
	from urllib import urlopen

	# Make sure we don't try to get any more than 5 day's forecast
	if days > 5: days = 5

	# Chop the data into more managable bits
	data = urlopen('http://xoap.weather.com/weather/local/NZXX0049?cc=*&dayf=5&prod=xoap&par=1003666583&key=4128909340a9b2fc').read()
	#print data1
	# The regular expressions needed to get the weather forecast
	# listed roughly in the order that they occur

	dcstart = data.find('<cc>')
	dcstop = data.find('</cc>')
	data_current = data[dcstart:dcstop]

	day0 = data.find('<day d="0"')
	day1 = data.find('<day d="1"')
	day2 = data.find('<day d="2"')
	day3 = data.find('<day d="3"')
	day4 = data.find('<day d="4"')
	data0 = data[day0:day1]
	data1 = data[day1:day2]
	data2 = data[day2:day3]
	data3 = data[day3:day4]
	data4 = data[day4:]


	p_report = r'''(?ix)
	.*<lsup>(?P<month>\d+?)/ # Month
	(?P<day>\d+?)/ # Day
	(?P<year>\d+?) # Year
	\s(?P<time>.+?)\s #Time
	(?P<meridian>[A|P]M) # AM or PM
	\s(?P<zone>.*?)</lsup> # time zone
	\s*<obst>(?P<city>.*?), # City
	'''




	# time and location of the weather report
	now = weather.Observation()
	match = re.search(p_report, data_current)
	print match
	print match.group('city')
	if match:
		now.year(match.group('year'))
		now.month(match.group('month'))
		now.day(match.group('day'))
		now.time(match.group('time'), match.group('meridian'))
		now.timezone(match.group('zone'))
		now.place(match.group('city'))

	p_weekday = r'''(?ix)
	<day\sd=\"0\"\st=\"(?P<weekday>.+?)\" # Day of the week
	'''

	match1 = re.search(p_weekday, data0)
	print match1
	print match1.group('weekday')
	if match1:
		now.weekday(match1.group('weekday')[:3])


	#print now
	print '==================================================='
	p_current = r'''(?isx) # Ignore case, Dot matches all, Verbose
	.*<tmp>(?P<temp>\d*?)</tmp> # Temp
	\s*<flik>(?P<heat>.*?)</flik> # Heat
	\s*<t>(?P<sky>\D+?)</t> # Sky
	\s*<icon>(?P<icon>\d*?)</icon> # Icon
	\s*<bar>\s*<r>(?P<baro>.*?)</r>\s*<d>(?P<change>.*?)</d>\s*</bar>
	\s*<wind>\s*<s>(?P<speed>\d*?)</s>.*<t>(?P<wind>.*?)</t>\s*</wind>
	\s*<hmid>(?P<hum>\d+)</hmid>
	\s*<vis>(?P<vis>.*?)</vis>
	\s*<uv>\s*<i>(?P<uv>.*?)</i>
	.*<dewp>(?P<dew>.*?)</dewp>
	'''


	match = re.search(p_current, data_current)
	print match
	print match.group('dew')
	if match:
		now.icon(match.group('icon'))
		now.temperature(match.group('temp'), 'F')
		now.relative_heat(match.group('heat'), 'F')
		now.sky(match.group('sky'))
		now.uv(match.group('uv'))
		now.dewpoint(match.group('dew'), 'F')
		now.humidity(match.group('hum'))
		now.visibility(match.group('vis'))
		now.pressure(match.group('baro'), 'inHg')
		now.pressure_change(match.group('change'))
		mywind =  match.group('wind')
		gusting = mywind.find('gusting')
		if gusting != -1:
			jat = mywind.find(' at')
			mywind = mywind[:jat]
		now.wind(mywind.replace('From the ', ''))

		now.wind_speed(match.group('speed'), 'mph')
	print now


	p_today = r'''(?isx) # Ignore case, Dot matches all, Verbose
	<day\sd=\"0\".*dt=\"(?P<date>.*?)\"> # Date
	\s*?<hi>(?P<high>\d*?)</hi> # High temperature
	\s*?<low>(?P<low>\d*?)</low> # Low temperature
	.*?<icon>(?P<icon>\d*?)</icon> # Icon
	\s*?<t>(?P<sky>.*?)</t> # Sky
	.*?<ppcp>(?P<rain>\d+)</ppcp>  # Chance of rain
	'''

	today = weather.Forecast(now.weekday())
	now.add_forecast(today)
	match = re.search(p_today, data0)
	if match:
		today.icon(match.group('icon'))
		today.sky(match.group('sky'))
		today.date(match.group('date'))
		today.temperature_high(match.group('high'), 'F')
		today.temperature_low(match.group('low'), 'F')
		today.rain(match.group('rain'));


	p_forecast = r'''(?isx) # No embedded flags, since they will be added later
	.*?t=\"(?P<day>.*?)\" # Weekday
	.*?dt=\"(?P<date>.*?)\"> # Day number
	\s*?<hi>(?P<high>\d*?)</hi> # High temperature
	\s*?<low>(?P<low>\d*?)</low> # Low temperature
	.*?<icon>(?P<icon>\d*?)</icon> # Icon
	\s*?<t>(?P<sky>.*?)</t> # Sky
	.*?<ppcp>(?P<rain>\d+)</ppcp>  # Chance of rain
	'''
	datalist = [data1, data2, data3, data4]
	day = 1
	while day <= (days -1):
		forecast = weather.Forecast()
		now.add_forecast(forecast)
		match = re.search(p_forecast, datalist[day-1])
		if match:
			forecast.weekday(match.group('day'))
			forecast.date(match.group('date'))
			forecast.icon(match.group('icon'))
			forecast.sky(match.group('sky'))
			forecast.temperature_high(match.group('high'), 'F')
			forecast.temperature_low(match.group('low'), 'F')
			forecast.rain(match.group('rain'));
		day += 1
		print forecast

	return now

parseWeatherData(5)
