#this import statement allows access to the karamba functions
import karamba
import string

CONTENT_FONT_SIZE = 12
CONTENT_FONT_NAME = "utopia"
CONTENT_COLOR_RED = 27
CONTENT_COLOR_GREEN = 249
CONTENT_COLOR_BLUE = 235

FILENAME_FONT_SIZE = 14
FILENAME_FONT_NAME = "proof"
FILENAME_COLOR_RED = 150
FILENAME_COLOR_GREEN = 150
FILENAME_COLOR_BLUE = 0
FILENAME_SEPARATOR = 20

CURRENT_COLOR_RED = 0
CURRENT_COLOR_GREEN = 255
CURRENT_COLOR_BLUE = 0

MAX_LINES = 6
MAX_COLUMNS = 80

###### End of customizable section ######

logfilelist = []
textmeterlist = []
current = -1
logtext = ""

tailpid = 0
addlogpid = 0
#dellogpid = 0

#This is an empirical function
def getWidth(columns, fontsize):
	return columns * (fontsize - 4)

#This is an empirical function
def getHeight(lines, fontsize):
	return lines * (fontsize + 5)

def getFilename(str):
	return str[string.rfind(str, "/") + 1 : len(str)]

def setTextStyle(widget, text, fontname, fontsize, r, g, b, shadow):
	karamba.changeTextSize(widget, text, fontsize)
	karamba.changeTextFont(widget, text, fontname)
	karamba.changeTextColor(widget, text, r, g, b)
	karamba.changeTextShadow(widget, text, shadow)

#this is called when you widget is initialized
def initWidget(widget):
	global logtext, textmeterlist, current

	karamba.resizeWidget(widget, getWidth(MAX_COLUMNS, CONTENT_FONT_SIZE), getHeight(MAX_LINES, CONTENT_FONT_SIZE) + getHeight(1, FILENAME_FONT_SIZE))

	karamba.addMenuConfigOption(widget, "addlog", "Add a log...")
	karamba.setMenuConfigOption(widget, "addlog", "0")
	#karamba.addMenuConfigOption(widget, "dellog", "Remove a log...")
	#karamba.setMenuConfigOption(widget, "dellog", "0")

	logtext = karamba.createText(widget, 0, getHeight(1, FILENAME_FONT_SIZE), getWidth(MAX_COLUMNS, CONTENT_FONT_SIZE), getHeight(MAX_LINES, CONTENT_FONT_SIZE), "Please wait...")
	setTextStyle(widget, logtext, CONTENT_FONT_NAME, CONTENT_FONT_SIZE, CONTENT_COLOR_RED, CONTENT_COLOR_GREEN, CONTENT_COLOR_BLUE, 1)

	print "LogWatch: Reading file names"
	i = 0
	file_temp = karamba.readConfigEntry(widget, "file" + str(i))
	while file_temp != None:
		print "LogWatch: " + file_temp
		logfilelist.append(file_temp)
		i = i + 1
		file_temp = karamba.readConfigEntry(widget, "file" + str(i))
	current = karamba.readConfigEntry(widget, "index")
	print "LogWatch: Reading last viewed index (" + str(current) + ")"

	W, w = 0, 0
	for i in range(len(logfilelist)):
		filename = getFilename(logfilelist[i])
		w = getWidth(len(filename), FILENAME_FONT_SIZE)
		textmeterlist.append(karamba.createText(widget, W, 0, w, getHeight(1, FILENAME_FONT_SIZE), filename))
		if i == current:
			setTextStyle(widget, textmeterlist[i], FILENAME_FONT_NAME, FILENAME_FONT_SIZE, CURRENT_COLOR_RED, CURRENT_COLOR_GREEN, CURRENT_COLOR_BLUE, 1)
		else:
			setTextStyle(widget, textmeterlist[i], FILENAME_FONT_NAME, FILENAME_FONT_SIZE, FILENAME_COLOR_RED, FILENAME_COLOR_GREEN, FILENAME_COLOR_BLUE, 1)
		karamba.attachClickArea(widget, textmeterlist[i])
		W = W + w + FILENAME_SEPARATOR

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
	global text, tailpid

	if current != None and len(logfilelist) > 0:
		if logfilelist[current] != None and logfilelist[current] != "":
			#print "LogWatch: Executing tail"
			command = ["/usr/bin/tail", "--lines=" + str(MAX_LINES), logfilelist[current]]
			tailpid = karamba.executeInteractive(widget, command)
	else:
		karamba.changeText(widget, logtext, "Select a log file to view.")

#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#					1 = Left Mouse Button
#					2 = Middle Mouse Button
#					3 = Right Mouse Button, but this will never happen
#						because the right mouse button brings up the
#						Karamba menu.
#					4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
	pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#					0 = No Mouse Button
#					1 = Left Mouse Button
#					2 = Middle Mouse Button
#					3 = Right Mouse Button, but this will never happen
#						because the right mouse button brings up the
#						Karamba menu.
def widgetMouseMoved(widget, x, y, button):
	#Warning:  Don't do anything too intensive here
	#You don't want to run some complex piece of code everytime the mouse moves
	pass

#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
	pass

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
	global addlogpid#, dellogpid

	if key == "addlog":
		karamba.setMenuConfigOption(widget, "addlog", "0")
		command = ["kdialog", "--title", "Enter the log file to view", "--inputbox", "Path to Logfile", "/var/log/"]
		addlogpid = karamba.executeInteractive(widget, command)
#	elif key == "dellog":
#		karamba.setMenuConfigOption(widget, "dellog", "0")
#		command = ["kdialog", "--title", "Select the log file to remove", "--separate-output", "--checklist", "File name"]
#		for i in range(len(logfilelist)):
#			command.append(str(i))
#			command.append(logfilelist[i])
#			command.append("0")
#		dellogpid = karamba.executeInteractive(widget, command)

#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
	global current

	try:
		i = textmeterlist.index(meter)
	except:
		i = -1
	if i >= 0:
		current = i

	for i in range(len(logfilelist)):
		if i == current:
			setTextStyle(widget, textmeterlist[i], FILENAME_FONT_NAME, FILENAME_FONT_SIZE, CURRENT_COLOR_RED, CURRENT_COLOR_GREEN, CURRENT_COLOR_BLUE, 1)
		else:
			setTextStyle(widget, textmeterlist[i], FILENAME_FONT_NAME, FILENAME_FONT_SIZE, FILENAME_COLOR_RED, FILENAME_COLOR_GREEN, FILENAME_COLOR_BLUE, 1)
	karamba.writeConfigEntry(widget, "index", str(current))

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
	global current

	if pid == tailpid:
		#print "LogWatch: Updating text"
		karamba.changeText(widget, logtext, output)
	elif pid == addlogpid:
		filename = output.strip("\n")
		if filename != "":
			W = 0
			old = current
			for i in range(len(logfilelist)):
				W = W + getWidth(len(getFilename(logfilelist[i])), FILENAME_FONT_SIZE) + FILENAME_SEPARATOR
			try:
				current = logfilelist.index(filename)
			except:
				logfilelist.append(filename)
				current = len(logfilelist) - 1
				filename = getFilename(filename)
				w = getWidth(len(filename), FILENAME_FONT_SIZE)
				textmeterlist.append(karamba.createText(widget, W, 0, w, getHeight(1, FILENAME_FONT_SIZE), filename))
				karamba.attachClickArea(widget, textmeterlist[current])

			setTextStyle(widget, textmeterlist[current], FILENAME_FONT_NAME, FILENAME_FONT_SIZE, CURRENT_COLOR_RED, CURRENT_COLOR_GREEN, CURRENT_COLOR_BLUE, 1)
			if old != current and old >= 0:
				setTextStyle(widget, textmeterlist[old], FILENAME_FONT_NAME, FILENAME_FONT_SIZE, FILENAME_COLOR_RED, FILENAME_COLOR_GREEN, FILENAME_COLOR_BLUE, 1)

			print "LogWatch: Writing file names"
			for i in range(len(logfilelist)):
				print "LogWatch: " + logfilelist[i]
				karamba.writeConfigEntry(widget, "file" + str(i), logfilelist[i])
			print "LogWatch: Writing last viewed index"
			karamba.writeConfigEntry(widget, "index", str(current))
#	elif pid == dellogpid:
#		for i in range(len(output)):
#			print "[" + output[0] + "]"
#			del textmeterlist[int(output[0])]
#			del logfilelist[int(output[0])]

#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  drologtext = the text of the dropped item (probably a URL to it's location in KDE)
def itemDropped(widget, drologtext):
	pass


#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.
def startupAdded(widget, startup):
	pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.  
def startupRemoved(widget, startup):
	pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#		 to get the name, etc of this new task.
def taskAdded(widget, task):
	pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.  
def taskRemoved(widget, task):
	pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).  
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#		 to get the name, etc of this new task.
def activeTaskChanged(widget, task):
	pass

# This will be printed when the widget loads.
print "Loaded LogWatch extension!"
