#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "mylib.h"


/**************************************************
***********    Funzioni generiche   ***************
**************************************************/



/*************************************
* Chiude 1 file aperto in precedenza * 
*************************************/
void ChiudiFile(FILE *file_punt)
{
  fclose(file_punt);
}


/*************************************************
***********  Funzioni per Seti@home   ************
**************************************************/

/***************************************
 * Cerca il valore corrispondente alla *
 * label indicata nel file 'state.sah' *
 * della dir del SETI                  *
 ***************************************/

double SetiState(char label[],char f_in[])
{
  FILE  *fp_in;

  int    i=0; /* indice della stringa che contiene ogni linea file in lettura */
  int    j=0; /* indice della stringa che contiene il valore interessato*/
  char  *p; /* puntatore che riceve l'uscita di 'strstr' */
  char   line[MAXLINE]; /* contiene ogni linea del file */
  char   prog[MAXLINE]; /* contiene il valore interessato */
  double n_val; /* contiene il valore convertito in double*/
  
  /* Apre il file in lettura */
  fp_in  = fopen(f_in,"r");

  do 
    {
      if((p=strstr(line,label))!= NULL) /* se questa linea file contiene la stringa 'label' */
	{
	  while(*p++ != '=') /* j conta la posizione del primo car dopo l'uguale */
	    { j++;}
	  for (i=j+1;i<=MAXLINE;i++) /* copia il val che trova dopo '=' nella */	
	    {prog[i-(j+1)]=line[i];} /* variabile 'prog' */ 
	}      
    } while (fgets(line,MAXLINE,fp_in) != NULL); /* esegue per tutte le linee del file */

  ChiudiFile(fp_in);

  n_val=atof(prog); /* converte il val ascii in val double */
  
  return (n_val);   /* ritorna il valore calcolato */ 
}


/*****************************************
 * Cerca il valore corrispondente alla   *
 * label indicata nel file 'outfile.sah' *
 * della dir del SETI                    *
 ****************************************/

int SetiOutfile(char label[],char f_in[])
{
  FILE  *fp_in;

  int    i_g=0; 
  int    i_s=0; 
  int    i_t=0; 
  int    i_p=0; 
  char   line[MAXLINE]="OK"; /* contiene ogni linea del file */
  char  *p; /* puntatore che riceve l'uscita di 'strstr' */
  int    n_val=0; /* contiene il valore convertito in double*/
  
  /* Apre il file in lettura */
  fp_in  = fopen(f_in,"r");

  do 
    {
      if((p=strstr(line,label))!= NULL) 
	{ 
	  if (*p=='g')
	    {	    
	      i_g++;
	      n_val=i_g;
	    }
	  else if (*p=='s')
	    {	    
	      i_s++;
	      n_val=i_s;
	    }
	  else if (*p=='t')
	    {	    
	      i_t++;
	      n_val=i_t;
	    }
	  else if (*p=='p')
	    {	    
	      i_p++;
	      n_val=i_p;
	    }
	  else
	    ;
	}
	    	  
    } while (fgets(line,MAXLINE,fp_in) != NULL); /* per tutte le linee del file */
 
  ChiudiFile(fp_in);

  return (n_val);   /* ritorna il valore calcolato */ 
}




void SetiWork(char label[],char f_in[],char p_time[])
{
  FILE  *fp_in;

  int    i=0; /* indice della stringa che contiene ogni linea file in lettura */
  int    j=0; /* indice della stringa che contiene il valore interessato*/
  char  *p; /* puntatore che riceve l'uscita di 'strstr' */
  char   line[MAXLINE]; /* contiene ogni linea del file */
  
  /* Apre il file in lettura */
  fp_in  = fopen(f_in,"r");

  do 
    {
      if((p=strstr(line,label))!= NULL) /* se questa linea file contiene la stringa 'label' */
	{


	  while(*p++ != '(') /* j conta la posizione del primo car dopo l'uguale */
	    { j++;}
	  for (i=j+1;i<=MAXLINE;i++) /* copia il val che trova dopo '=' nella */	
	    {p_time[i-(j+1)]=line[i];} /* variabile 'prog' */ 
	}      
    } while (fgets(line,MAXLINE,fp_in) != NULL); /* esegue per tutte le linee del file */
 
  ChiudiFile(fp_in);
}



void SetiUser (char label[],char f_in[],char p_info[])
{
  FILE  *fp_in;

  int    i=0; /* indice della stringa che contiene ogni linea file in lettura */
  int    j=0; /* indice della stringa che contiene il valore interessato*/
  char  *p; /* puntatore che riceve l'uscita di 'strstr' */
  char   line[MAXLINE]; /* contiene ogni linea del file */
  
  /* Apre il file in lettura */
  fp_in  = fopen(f_in,"r");

  do 
    {
      if((p=strstr(line,label))!= NULL && (*(p-1)!='_')) 
	{
	  while(*p++ != '=') /* j conta la posizione del primo car dopo l'uguale */
	    { j++;}
	  for (i=j+1;i<=MAXLINE;i++) /* copia il val che trova dopo '=' nella */	
	    {p_info[i-(j+1)]=line[i];} /* variabile 'prog' */ 
	}      
    } while (fgets(line,MAXLINE,fp_in) != NULL); /* esegue per tutte le linee del file */
 
  ChiudiFile(fp_in);
}


/*****************************************
 * Converte un valore double dato in sec *
 * e ritorna 3 valori int in ore,min,sec *
 ****************************************/

void TimeConvSec (double time_in,int *p_h,int *p_m,int *p_s)
{
  double tmp_h;
  double tmp_m;
  double tmp_s;

  tmp_h=time_in/3600.0;
  *p_h=(int)tmp_h;
  tmp_m=(tmp_h-(double)*p_h)*60.0;
  *p_m=(int)tmp_m;
  tmp_s=(tmp_m-(double)*p_m)*60.0;
  *p_s=(int)tmp_s;
}



/*****************************************
 * Converte un valore double dato in ore *
 * e ritorna 3 valori interi in ore, min *
 ****************************************/

void TimeConvHour (double time_in,int *p_h,int *p_m)
{
  double tmp_m;

  *p_h=(int)time_in;
  tmp_m=(time_in-(double)*p_h)*60.0;
  *p_m=(int)tmp_m;
}
