###############################################################
#  Copyright by Charles Wienken
#  Licensed Under GPL
###############################################################

###############################################################
#  This import statement allows access to the karamba functions
###############################################################
import karamba
import commands
import glob
import string
import time
import os
import sys
 
###############################################################
#  Globals
###############################################################
defaultHeight = 70 # Default Height
screenWidth = 0 # Width of screen
widgetYpos = 0  # Y position of widget

dateFontSize = 14 # Date font size
dateSpacing = 220 # Space for time and date
dateRef = "" # Reference to date

iconSize = 48 # Size of the icons ( resize icons to 48 by 48 )
iconBigSize = 64 # Size of the Zoomed icons
iconSpacing = 10 # Spacing between icons
iconZoomed = -1 # The index of the Zoomed icon
zoomIcons = 0 # Zoom Icons
numOfIcons = 1 # Number of Icons
sideSize = 32 # Size of the round bars on the sides

menuName = ["enableclock", "enablezoom" ] # "enablekaffeine"] # config menu options names 
menuText = ["Enable Date and Time", "Enable Zoom" ] # "Enable Kaffeine" ] # config menu options text

icons =  [ ] # Icon image paths
cmd = [ "dcop kicker kicker popupKMenu 0" ] # Icon command
tip = [ "" ] # Icon tip
xpos = [ sideSize-iconSpacing ] # Icon X position
xposBig = [ sideSize-iconSpacing*1.5 ] # Icon X Big end position 
iconRef = [ ] # Icon reference
iconBigRef = [ ] # Icon Big reference

autoWidth = dateSpacing + iconSize + numOfIcons * ( iconSpacing+iconSize ) # default size for filled space

themePath = ["", "", ""] # Icon theme paths to check

barRef = [ 0, 0, 0 ] # image bar reference when clock is disabled

###############################################################
#  Set the screen width and widget location
###############################################################
def setWidthAndLocation( widget ):
    global screenWidth
    global defaultHeight
    global widgetYpos	

    resHeight = 0
    
    #  Get config file width
    karambaPathCmd = "cat " + karamba.getThemePath(widget) +  "xZoomBar.theme"
    configFP = commands.getoutput( karambaPathCmd )
    if ( configFP ):
	pass
    else:
	print "\nCannot find your karamba theme path\n"

    for line in configFP.splitlines():
	param = line.split()
	for i in range( len(param) ):
		if param[i].find( "BOTTOMBAR") <> -1:
			bBar = string.upper( param[i].strip( "BOTTOMBAR" ) )

    #  Detect resolution
    xwinFP = commands.getoutput("xwininfo -root -stats")
    if ( xwinFP ):
	pass
    else:
	print "\nCannot find xwininfo in your path.\n"

    for line in xwinFP.splitlines():
	param = line.split()
	if (len(param) > 1) and param[0].find("Width:") != -1:
		screenWidth = int(param[1])
	if (len(param) > 1) and param[0].find("Height:") != -1:
		resHeight = int(param[1])

    if bBar == "=TRUE":
    	# set widget Y position
	widgetYpos = resHeight - defaultHeight
    	

###############################################################
#  Set the Icon Theme Paths
###############################################################
def setThemePaths( widget ): 
    global themePath
    global iconPath

    karambaPath = karamba.getThemePath(widget) 
    iconPath = karambaPath + "Icons/xZoomBar/"
    configPath = karambaPath + ".config"
    checkPath = glob.glob(configPath)
    
    iconGetPath = commands.getoutput( "kde-config --path icon")
    param = iconGetPath.split( ":" )
    pIndex = len( param ) - 1
    iconGetPath = param[pIndex] 

    if( checkPath <> [] ):
    	configFP = open( configPath )
    	configOut = configFP.read()
    	for line in configOut.splitlines():
		if string.find( line, "iconThemePath" ) <> -1:
			themePath[0] = line.lstrip("iconThemePath=")
	configFP.close()
	
    haveKDialog = commands.getoutput( "which kdialog" )
    if( themePath[0] == "" and haveKDialog ):
	kDialogCmd="kdialog --inputbox 'Please enter the path to your icon theme with a trailing /' "+iconGetPath+" > .config"
	commands.getoutput( kDialogCmd )
	configFP = open(configPath, 'r' )
	themePath[0] = configFP.read()
	configFP.close()
	configFP = open( configPath, 'w' )	
	configFP.write( "iconThemePath=" + themePath[0] )
	configFP.close() 
    elif haveKDialog == "": 
	print "\nPlease set icon path manually."

    kdeGlobalFP = commands.getoutput("cat $HOME/.kde/share/config/kdeglobals")
    for line in kdeGlobalFP.splitlines():
	if string.find( line, "Theme" ) <> -1:
		themePath[1] = iconGetPath + line.lstrip("Theme=") + "/64x64/apps/" 

    themePath[0] = themePath[0] + "/64x64/apps/"
    themePath[2] = "/usr/share/pixmaps/"

###############################################################
#  This is called when an option is selected
###############################################################
def menuOptionChanged( widget, key, value ):
    menuConfigOptions( widget, key )

###############################################################
#  This is called to recreate the widget
###############################################################
def recreateWidget( widget ):
    global autoWidth
    global screenWidth
    global defaultHeight
    global widgetYpos
    global menuName
    global numOfIcons
    global barRef
    global iconRef

    # delete the bars
    for i in range( len( barRef ) ):
    	karamba.deleteImage( widget, barRef[i] )   
    
    for i in range( len( iconRef ) ):
	karamba.deleteImage( widget, iconRef[i] )	
	karamba.deleteImage( widget, iconBigRef[i] ) 

    # resize the widget and center
    karamba.resizeWidget( widget, autoWidth, defaultHeight  ) 
    midX = ( screenWidth - autoWidth ) / 2
    karamba.moveWidget( widget , midX , widgetYpos )

    # create the bar
    createWidgetBg( widget, autoWidth, sideSize )
    
    # create the icons
    createWidgetIcons( widget )

    # redraw the widget 
    karamba.redrawWidget(widget)

###############################################################
#  This is called to configure the menu Options
############################################################### 
def menuConfigOptions( widget, key ):
    global menuName
    global dateRef
    global dateSpacing
    global iconSpacing
    global iconSize
    global autoWidth
    global numOfIcons
    global zoomIcons

    # read Menu Configuration 
    value = karamba.readMenuConfigOption(widget, key)
    
    # clock Key
    if  key == menuName[0]:
	if value == 1:
		# reset the width and recreate the widget
		autoWidth = dateSpacing + sideSize + numOfIcons * ( iconSpacing+iconSize )
		recreateWidget( widget )
		# set the time
		dateTime = time.strftime('%b %d, %Y   %I:%M %P', time.localtime())  # Get the time Now()
    		karamba.changeText( widget, dateRef, dateTime )  # Set the time 
		# move the Text to original position
		karamba.moveText( widget, dateRef, autoWidth-dateSpacing, 20 )
	else:
		# change clock Text to ""
		karamba.changeText( widget, dateRef, "" )
		# reset the width and recreate the widget
		autoWidth = sideSize + numOfIcons * ( iconSpacing+iconSize )
		recreateWidget( widget )
		# move clock Text off the screen
    		karamba.moveText( widget, dateRef, -250, 20 )
    if key == menuName[ 1 ]:
	zoomIcons = value
	
###############################################################
#  This is called to create the Widget Background
############################################################### 
def createWidgetBg( widget, autoWidth, sideSize ):
    global barRef

    #  Left port
    barRef[0] = karamba.createImage( widget, 0, 0, "Images/xZoomBar/round_bar_left.png" )
  
    #  Middle port
    barRef[1] = karamba.createImage( widget, sideSize, 0, "Images/xZoomBar/round_bar_middle.png" )
    karamba.resizeImageSmooth( widget, barRef[1], autoWidth - (sideSize*2), defaultHeight ) 

    #  Right port
    barRef[2] = karamba.createImage( widget, autoWidth-sideSize, 0, "Images/xZoomBar/round_bar_right.png" ) 

###############################################################
#  This is called to create the Icons
############################################################### 
def createWidgetIcons( widget, createClickArea = 0 ):
    global icons
    global cmd
    global tip
    global xpos
    global xposBig
    global iconRef
    global iconBigRef
    global themePath
    global iconPath
    global iconSize
    global iconSpacing


   
    # Cycle through the icons
    iconList = glob.glob( iconPath+"*")

    iconCount = 0    
    iconClickSize = ( iconSize + iconBigSize )  / 2
    icons.insert( iconCount, themePath[0] + "kmenu.png" )  
    iconRef.insert( iconCount, karamba.createImage(widget, xpos[iconCount], 8, icons[iconCount]))
    karamba.resizeImageSmooth(widget, iconRef[iconCount], iconSize, iconSize )  
    iconBigRef.insert( iconCount, karamba.createImage(widget, xposBig[iconCount], 2, icons[iconCount]))
    karamba.hideImage( widget, iconBigRef[iconCount] )
    karamba.resizeImageSmooth(widget, iconBigRef[iconCount], iconBigSize, iconBigSize )
    if( createClickArea == 1 ):
		karamba.createClickArea(widget, xpos[iconCount] + iconSpacing / 4, 2, iconClickSize, iconClickSize, cmd[iconCount]) 

    iconCount += 1
    iconList.sort() # Put these icons in order 1, 2, 3, etc.
    
    for iconName in iconList:
    	if os.path.isfile(iconName) != 1:
		continue
        iconFile = open(iconName, 'r')
        iconLine = iconFile.readline()
        if string.count(iconLine, "[Desktop Entry]"):
                for iconLine in iconFile.readlines():
                	if iconLine[:4] == "Name":
                        	strPos = string.find(iconLine, "=") + 1
                        	tip.insert( iconCount, iconLine[strPos:-1] )
			if iconLine[:4] == "Exec":
                        	strPos = string.find(iconLine, "=") + 1
                        	cmd.insert( iconCount, iconLine[strPos:-1] )
                        if iconLine[:4] == "Icon":
                        	iconLine = string.replace(iconLine, ".png", '')
                        	strPos = string.find(iconLine, '=') + 1
				if glob.glob(iconLine[strPos:-1]+".png") <> []:
					icons.insert( iconCount, iconLine[strPos:-1]+".png" )
				elif glob.glob(themePath[0] + iconLine[strPos:-1]+".png") <> []:
					icons.insert( iconCount, themePath[0] + iconLine[strPos:-1]+".png" )
				elif glob.glob(themePath[1] + iconLine[strPos:-1]+".png") <> []:
					icons.insert( iconCount, themePath[1] + iconLine[strPos:-1]+".png" )
				else:
					icons.insert( iconCount, themePath[2] + iconLine[strPos:-1]+".png" )

	xpos.insert( iconCount,  xpos[0] + iconCount  * ( iconSpacing + iconSize ) )
	xposBig.insert( iconCount, xpos[iconCount] - iconSpacing / 2 )
	iconRef.insert( iconCount, karamba.createImage(widget, xpos[iconCount], 8, icons[iconCount]))
	karamba.resizeImageSmooth(widget, iconRef[iconCount], iconSize, iconSize )
	karamba.addImageTooltip(widget, iconRef[iconCount], tip[iconCount] )
        iconBigRef.insert( iconCount, karamba.createImage(widget, xposBig[iconCount] , 2, icons[iconCount]))
	karamba.hideImage( widget, iconBigRef[iconCount] )
	karamba.resizeImageSmooth(widget, iconBigRef[iconCount], iconBigSize, iconBigSize )	
	if( createClickArea == 1 ):
		karamba.createClickArea(widget, xpos[iconCount] + iconSpacing / 4, 2, iconClickSize, iconClickSize, cmd[iconCount]) 
	iconCount += 1

###############################################################
#  This is called when you widget is initialized
###############################################################
def initWidget(widget):
    global iconPath
    global iconSize
    global iconSpacing
    global dateSpacing
    global dateRef
    global dateFontSize
    global sideSize
    global menuName
    global menuText
    global autoWidth
    global defaultHeight
    global numOfIcons

    #  Set width, location, and theme path
    setWidthAndLocation( widget )
    setThemePaths(widget)    

    # Get the number of icons 
    iconList = glob.glob( iconPath+"*")
    numOfIcons = len( iconList ) + 1
    autoWidth = dateSpacing + sideSize + numOfIcons * ( iconSpacing+iconSize ) 

    # Create Bars
    createWidgetBg( widget, autoWidth, sideSize )

    # Create Widget Icons
    createWidgetIcons( widget, 1 )

    #  Date and time
    dateRef = karamba.createText( widget, autoWidth-dateSpacing, 20, dateSpacing, defaultHeight, "" )
    karamba.attachClickArea(widget, dateRef, "sudo kcmshell clock", "", "")  
    karamba.changeTextColor(widget, dateRef,255,255,255 )
    karamba.changeTextSize(widget, dateRef,dateFontSize )
    karamba.changeTextShadow(widget, dateRef, 1 ) 

    #  Configure the option menu
    for i in range(len(menuName)):
    	karamba.addMenuConfigOption(widget, menuName[i], menuText[i])
	menuConfigOptions( widget, menuName[i] )    
        
###############################################################
#  This is called everytime your widget is updated
#  NOTE: The update inverval is specified in the .theme file
###############################################################

def widgetUpdated(widget):
    global menuName
    global dateRef
    
    #  Get date and time if not hidden
    if karamba.readMenuConfigOption(widget, menuName[0]) == 1:
    	dateTime = time.strftime('%b %d, %Y   %I:%M %P', time.localtime())  # Get the time Now()
    	karamba.changeText( widget, dateRef, dateTime )  # Set the time 
 
###############################################################
#  This gets called everytime our widget is clicked.
#  NOTE:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#  1 = Left Mouse Button
#  2 = Middle Mouse Button
#  3 = Right Mouse Button, but this will never happen
#      because the right mouse button brings up the
#      Karamba menu.
#  4,5 = Scroll wheel up and down
###############################################################
def widgetClicked(widget, x, y, button):
	pass

###############################################################
#  This gets called everytime our widget is clicked.
#  NOTE:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#  0 = No Mouse Button
#  1 = Left Mouse Button
#  2 = Middle Mouse Button
#  3 = Right Mouse Button, but this will never happen
#      because the right mouse button brings up the
#      Karamba menu.
###############################################################
def widgetMouseMoved(widget, x, y, button):
    ###########################################################
    #  WARNING:  Don't do anything too intensive here
    #  You don't want to run some complex piece of code 
    #  everytime the mouse moves
    ###########################################################
    global xpos
    global xposBig
    global numOfIcons
    global sideSize
    global iconRef
    global iconBigRef
    global iconSize
    global iconBigSize
    global iconZoomed
    global zoomIcons

    if( zoomIcons == 1 ):
    	xStart = xposBig[ 0 ]
    	xEnd = xposBig[ numOfIcons-1 ] + iconBigSize
    	iconIndex = -1
    	if xStart <= x <= xEnd  and 8 <= y <= 56: 
		for i in range(numOfIcons):
 			if( xpos[i] < x and x < ( xposBig[i] + iconBigSize ) ):
				iconIndex = i
		if 0 <= iconIndex <= numOfIcons and iconZoomed == -1 and iconZoomed <> iconIndex:
                	karamba.hideImage(widget, iconRef[iconIndex])
			karamba.showImage(widget, iconBigRef[iconIndex])
			karamba.redrawWidget(widget)
			iconZoomed = iconIndex	   	
		elif iconZoomed <> iconIndex:
			karamba.hideImage(widget, iconBigRef[iconZoomed])
			karamba.showImage(widget, iconRef[iconZoomed])
			karamba.redrawWidget(widget)
			iconZoomed = -1	
    	elif iconZoomed <> iconIndex:
		karamba.hideImage(widget, iconBigRef[iconZoomed])
		karamba.showImage(widget, iconRef[iconZoomed])
		karamba.redrawWidget(widget)
		iconZoomed = -1	    
